package com.qunzhixinxi.hnqz.common.core.util;

import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Document;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.AcroFields;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfCopy;
import com.itextpdf.text.pdf.PdfImportedPage;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfStamper;
import com.itextpdf.text.pdf.PdfWriter;
import lombok.experimental.UtilityClass;

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.util.Map;

/**
 * pdf工具
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/04/14 12:33
 */
@UtilityClass
public final class PdfUtils {

    public static void resumeGenerate(String templatePath, String outFilePath, String font, Map<String, String> params) {

        PdfReader reader;
        FileOutputStream out;
        ByteArrayOutputStream bos;
        PdfStamper stamper;
        try {

            out = new FileOutputStream(outFilePath);
            reader = new PdfReader(templatePath);
            bos = new ByteArrayOutputStream();
            stamper = new PdfStamper(reader, bos);
            // 获取pdf表单合集
            AcroFields form = stamper.getAcroFields();

            // 中文字体
            BaseFont baseFont = BaseFont.createFont(font + ",0", BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
            form.addSubstitutionFont(baseFont);

            for (Map.Entry<String, String> entry : params.entrySet()) {
                form.setField(entry.getKey(), entry.getValue());
            }

            // pdf不能编辑
            stamper.setFormFlattening(true);
            stamper.close();

            // 文档对象
            Document doc = new Document(PageSize.A4);
            PdfCopy copy = new PdfCopy(doc, out);
            doc.open();

            for (int i = 1; i <= 12; i++) {
                PdfImportedPage importedPage = copy.getImportedPage(new PdfReader(bos.toByteArray()), i);
                copy.addPage(importedPage);
            }
            doc.close();


        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    public static void generateSupervisionReport(String targetPath, Rectangle pageSize, boolean rotate, String f, Object[][] datas) {
        FileOutputStream fos;
        PdfWriter writer;
        Font font;

        int numberColumns = 14;

        if (pageSize == null) {
            pageSize = PageSize.A4;
        }


        try {
            fos = new FileOutputStream(targetPath);

            Rectangle rectangle = rotate ? new Rectangle(pageSize).rotate() : new Rectangle(pageSize);

            Document document = new Document(rectangle);
            writer = PdfWriter.getInstance(document, fos);
            writer.setViewerPreferences(PdfWriter.PageModeUseThumbs);
            document.open();

            PdfPTable table = new PdfPTable(numberColumns);
            table.setLockedWidth(true);
            table.setTotalWidth(780);
            table.setHorizontalAlignment(Element.ALIGN_LEFT);

            // 设置单元格文本字体样式
            font = getPdfChineseFont(f);

            for (int i = 0; i < datas.length; i++) {
                for (int j = 0; j < datas[i].length; j++) {
                    PdfPCell pdfCell = new PdfPCell();
                    pdfCell.setMinimumHeight(30);

                    // 设置单元格样式
                    pdfCell.setHorizontalAlignment(Element.ALIGN_CENTER);
                    pdfCell.setVerticalAlignment(Element.ALIGN_MIDDLE);


                    // 设置表头背景色
                    if (i <= 1 && j <= numberColumns) {
                        pdfCell.setBackgroundColor(BaseColor.GRAY);
                    }

                    pdfCell.setBorder(1);
                    pdfCell.setBorderWidthTop(0.1f);
                    pdfCell.setBorderWidthBottom(0.1f);
                    pdfCell.setBorderWidthLeft(0.1f);
                    pdfCell.setBorderWidthRight(0.1f);
                    pdfCell.setBorderColorBottom(BaseColor.BLACK);
                    pdfCell.setBorderColorLeft(BaseColor.BLACK);
                    pdfCell.setBorderColorRight(BaseColor.BLACK);
                    pdfCell.setBorderColorTop(BaseColor.BLACK);

                    // // 设置单元格文本字体样式
                    // Font font = getPdfChineseFont(font);

                    // 合并单元格
                    if (i == 0 && j == 0) {
                        pdfCell.setRowspan(1);
                        pdfCell.setColspan(numberColumns);
                    }

                    Paragraph paragraph = new Paragraph(String.valueOf(datas[i][j]), font);
                    pdfCell.setPhrase(paragraph);

                    table.addCell(pdfCell);
                }
            }

            document.add(table);
            document.close();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static Font getPdfChineseFont(String font) throws Exception {
        // BaseFont bfChinese = BaseFont.createFont("STSongStd-Light", "UniGB-UCS2-H",
        //         BaseFont.NOT_EMBEDDED);
        BaseFont bfChinese = BaseFont.createFont(font + ",0", BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
        Font fontChinese = new Font(bfChinese, 6, Font.NORMAL);
        return fontChinese;
    }

    public static void main(String[] args) throws Exception {

        String targetPath = "/Users/jimmy/table8.pdf";

        // Object[][] datas = new Object[3][12] {
        //         {"2024年第四季度监督结果"},
        //         {"服务ID", "服务类型", "代表名称", "产品名称", "生产企业", "执行包名称", "周期", "监督结果", "监督意见", "部门负责人意见", "分管领导意见", "总经理意见"} };
        //         // {"江苏省" , 9045,  2256},
        //         // {"广东省", 3000, 690}};
        // Object[][] datas ;

        Object[][] datas = new Object[4][];
        for (int i = 0; i < datas.length; i++) {
            // 列的长度每次都变化。每次都要重新申请空间(长度)
            datas[i] = new Object[i == 0 ? 1 : 12];

            if (i== 0){
                datas[0][0] = "区域产品销售额";
                continue;
            }

            if (i == 1){
                datas[1][0] = "服务ID";
                datas[1][1] = "服务类型";
                datas[1][2] = "代表名称";
                datas[1][3] = "产品名称";
                datas[1][4] = "生产企业";
                datas[1][5] = "执行包名称";
                datas[1][6] = "周期";
                datas[1][7] = "监督结果";
                datas[1][8] = "监督意见";
                datas[1][9] = "部门负责人意见";
                datas[1][10] = "分管领导意见";
                datas[1][11] = "总经理意见";
                continue;
            }


            for (int j = 0; j < datas[i].length; j++){
                datas[i][j] = i + j;
            }
        }
        // for(int i=0; i<arr3.length; ++i) {
        //     for(int j=0; j<arr3[i].length; ++j)
        //         System.out.print(arr3[i][j]);
        //     System.out.print("\n");
        // }


        // Object[][] datas = new Object[2][12];

        // for (int i = 0; i < datas.length; i++) {
        //     datas[i] = new Object[i];
        //     for (int j = 0; j < datas[i].length; j++) {
        //         datas[i][j] = "区域产品销售额";
        //     }
        // }


        // datas[0][0] = "区域产品销售额";
        // datas[0][1] = "";
        // datas[0][2] = "";
        // datas[0][3] = "";
        // datas[0][4] = "";
        // datas[0][5] = "";
        // datas[0][6] = "";
        // datas[0][7] = "";
        // datas[0][8] = "";
        // datas[0][9] = "";
        // datas[0][10] = "";
        // datas[0][11] = "";
        // datas[1][0] = "服务ID";
        // datas[1][1] = "服务类型";
        // datas[1][2] = "代表名称";
        // datas[1][3] = "产品名称";
        // datas[1][4] = "生产企业";
        // datas[1][5] = "执行包名称";
        // datas[1][6] = "周期";
        // datas[1][7] = "监督结果";
        // datas[1][8] = "监督意见";
        // datas[1][9] = "部门负责人意见";
        // datas[1][10] = "分管领导意见";
        // datas[1][11] = "总经理意见";

        // datas[3][0] = "服务ID";
        // datas[3][1] = "服务类型";
        // datas[3][2] = "代表名称";
        // datas[3][3] = "产品名称";
        // datas[3][4] = "生产企业";
        // datas[3][5] = "执行包名称";
        // datas[3][6] = "周期";
        // datas[3][7] = "监督结果";
        // datas[3][8] = "监督意见";
        // datas[3][9] = "部门负责人意见";
        // datas[3][10] = "分管领导意见";
        // datas[3][11] = "总经理意见";

        // datas[2][0] = "服务ID";
        // datas[2][1] = "服务类型";
        // datas[2][2] = "代表名称";
        // datas[2][3] = "产品名称";
        // datas[2][4] = "生产企业";
        // datas[2][5] = "执行包名称";
        // datas[2][6] = "周期";
        // datas[2][7] = "监督结果";
        // datas[2][8] = "监督意见";
        // datas[2][9] = "部门负责人意见";
        // datas[2][10] = "分管领导意见";
        // datas[2][11] = "总经理意见";


        generateSupervisionReport(targetPath, PageSize.A4, true, "/Users/jimmy/Desktop/SimSun.ttc", datas);

        // String templatePath = "/Users/jimmy/pkg_temp_v19_1.pdf";
        // String outFilePath = "/Users/jimmy/Desktop/" + UUID.randomUUID() + ".pdf";
        // Map<String, String> params = new HashMap<>();
        // params.put("party_a_name", "中国生物水痘生产企业");
        // params.put("party_a_address", "北京市朝阳区双桥路123号中国生物基地10-12号");
        // params.put("party_a_resp_name", "张三");
        // params.put("party_a_contact_name", "买还买提～阿卜杜拉～买买提");
        // params.put("party_a_contact_phone", "13388889999");
        //
        // params.put("party_b_name", "长春生物药品科技有限公司");
        // params.put("party_b_address", "吉林省长春市光驱路123号生物医药基地123楼");
        // params.put("party_b_resp_name", "里斯");
        // params.put("party_b_contact_name", "王五");
        // params.put("party_b_contact_phone", "18877778888");
        //
        // params.put("issue_date", DateTimeFormatter.ofPattern(DatePattern.CHINESE_DATE_PATTERN).format(LocalDate.of(2024, 1, 1)));
        // params.put("expiry_date", DateTimeFormatter.ofPattern(DatePattern.CHINESE_DATE_PATTERN).format(LocalDate.of(2024, 3, 31)));
        //
        // params.put("s1_name", "市场信息手机");
        // params.put("s1_qty", "1");
        // params.put("s1_price", "3000");
        // params.put("s1_rate", "19%");
        //
        // params.put("s2_name", "会议举办");
        // params.put("s2_qty", "1000");
        // params.put("s2_price", "30000");
        // params.put("s2_rate", "100%");
        //
        // params.put("s3_name", "会议举办");
        // params.put("s3_qty", "1000");
        // params.put("s3_price", "30000");
        // params.put("s3_rate", "100%");
        //
        // params.put("s4_name", "会议举办");
        // params.put("s4_qty", "1000");
        // params.put("s4_price", "30000");
        // params.put("s4_rate", "100%");
        //
        //
        // params.put("sign_date", DateTimeFormatter.ofPattern(DatePattern.CHINESE_DATE_PATTERN).format(LocalDate.now()));
        // resumeGenerate(templatePath, outFilePath, "/Users/jimmy/Desktop/SimSun.ttc", params);
    }
}
