package com.qunzhixinxi.hnqz.admin.api.dto;

import com.alibaba.excel.annotation.ExcelProperty;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.LockEnum;
import lombok.AccessLevel;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 执行计划dto
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/19 11:16
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class SysImplementPlanDTO {

    @Data
    public static class OnBatchCreate {

        /**
         * 计划名称
         */
        @ExcelProperty(value = "计划名称")
        private String planName;

        /**
         * 产品
         */
        @ExcelProperty(value = "产品名称")
        private String skuName;

        /**
         * 计划类型
         */
        @ExcelProperty(value = "计划类型")
        private String planType;

        /**
         * 计划值
         */
        @ExcelProperty(value = "计划值")
        private String planScore;

        /**
         * 所属年度
         */
        @ExcelProperty(value = "所属年度")
        private String year;

        /**
         * 所属季度
         */
        @ExcelProperty(value = "所属季度")
        private String quarter;

        /**
         * 承接人
         */
        @ExcelProperty(value = "承接对象")
        private String consignee;

    }

    @Data
    public static class OnUpdate {

        @NotNull(message = "被修改计划ID必填")
        private Integer planId;

        /**
         * 计划名称
         */
        private String planName;

        /**
         * 计划值
         */
        private Long planScore;

        /**
         * 产品ID
         */
        private Integer skuId;

        /**
         * 受托人ID
         */
        private Long consigneeId;

        /**
         * 签发时间
         */
        private LocalDate planIssue;

        /**
         * 截止日期
         */
        private LocalDate planExpiry;

    }

    @Data
    public static class OnCreate {

        /**
         * 计划名称
         */
        @NotBlank(message = "计划名称必填")
        private String planName;

        /**
         * 计划类型
         */
        @NotNull(message = "计划类型必填")
        private UpmsType.ImplementPlanType planType;

        /**
         * 父ID
         */
        @NotNull(message = "关联计划ID必填")
        private Integer parentId;

        /**
         * 计划值
         */
        @NotNull(message = "计划值必填")
        @Min(value = 0L, message = "计划值必须大于{value}")
        private Long planScore;

        /**
         * 产品ID
         */
        @NotNull(message = "产品ID必填")
        private Integer skuId;

        /**
         * 受托人ID
         */
        @NotNull(message = "受托人ID必填")
        private Long consigneeId;

        /**
         * 签发时间
         */
        @NotNull(message = "签发时间必填")
        private LocalDate planIssue;

        /**
         * 截止日期
         */
        @NotNull(message = "截止日期必填")
        private LocalDate planExpiry;

        /**
         * 锁定标记
         */
        @NotNull(message = "锁定标记必填")
        private LockEnum locFlag;

    }

    @Data
    public static class OnPage {

        /**
         * 计划名称
         */
        private String planName;

        /**
         * 服务计划类型
         */
        private UpmsType.ImplementPlanType planType;

        /**
         * 实际可见状态
         */
        private UpmsState.ImplementPlanActionState actionState;

        /**
         * 受托人ID
         */
        private Integer consigneeId;

        /**
         * 周期
         */
        private LocalDate[] period;

        /**
         * 创建时间
         */
        private LocalDateTime[] createTime;


        @NotNull(message = "页码必填")
        @Min(value = 1, message = "页码最小为{value}")
        private Integer current;

        @NotNull(message = "分页数量必填")
        @Min(value = 10, message = "分页数量最小为{value}")
        @Max(value = 100, message = "分页数量最大为{value}")
        private Integer size;

        private Integer planId;
    }

    @Data
    public static class OnCheck {

        @NotNull(message = "计划ID必填")
        private Integer planId;

        @NotNull(message = "审核结果必填")
        private Boolean result;

        // @NotBlank(message = "审核意见必填")
        private String msg;
    }

    @Data
    public static class OnBatchCheck {

        @NotEmpty(message = "批量审核计划ID缺失")
        private List<Integer> planIds;

        @NotNull(message = "审核结果必填")
        private Boolean result;

        // @NotBlank(message = "审核意见必填")
        private String msg;
    }

    @Data
    public static class OnBatchCheckInfo {

        @ExcelProperty("计划ID")
        private Integer planId;

        @ExcelProperty(value = "审核意见")
        private String result;

        @ExcelProperty(value = "审核说明")
        private String msg;
    }
}
