package com.qunzhixinxi.hnqz.admin.api.dto;

import com.alibaba.excel.annotation.ExcelProperty;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import lombok.AccessLevel;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 计划详情
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/19 16:04
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class SysImplementPlanDetailsDTO {

    @Data
    public static class OnCreate {

        @NotNull(message = "计划ID必填")
        private Integer planId;

        @NotNull(message = "总数必填")
        private Integer qty;

        @NotNull(message = "总值必填")
        private Long total;

        @Valid
        @NotEmpty(message = "计划项信息")
        private List<OnItemCreate> items;

    }

    @Data
    public static class OnItemCreate {

        /**
         * 任务类型ID
         */
        @NotNull(message = "任务类型ID必填")
        private Integer taskTypeId;

        /**
         * 任务类型数量
         */
        @NotNull(message = "任务类型数量必填")
        @Min(value = 0, message = "任务类型数量最小值为{value}")
        private Integer qty;

        /**
         * 任务类型名称
         */
        @NotBlank(message = "任务类型名称必填")
        private String taskTypeName;

        /**
         * 单位得分
         */
        @NotNull(message = "单位得分必填")
        private Integer score;

        /**
         * 任务类型小记
         */
        @NotNull(message = "任务类型小记必填")
        @Min(value = 0, message = "任务类型小记最小值为{value}")
        private Long subtotal;

    }

    @Data
    public static class OnCheck {

        @NotNull(message = "计划详情ID必填")
        private Integer detailsId;

        @NotNull(message = "审核结果必填")
        private Boolean result;

        // @NotBlank(message = "审核意见必填")
        private String msg;

        @NotNull(message = "当前状态必填")
        private UpmsState.ImplPlanDetailsState currentState;
    }

    @Data
    public static class OnBatchCheck {

        @ExcelProperty(value = "详情ID")
        private Integer detailsId;

        @ExcelProperty(value = "审核结果")
        private String result;

        @ExcelProperty(value = "审核意见")
        private String msg;

        @ExcelProperty(value = "当前审核节点状态（系统用）")
        private String currentState;
    }

    @Data
    public static class OnExport{

    }
}
