package com.qunzhixinxi.hnqz.admin.api.dto;

import com.fasterxml.jackson.annotation.JsonAlias;
import com.qunzhixinxi.hnqz.admin.api.entity.WmAgentComplianceDocument;
import com.qunzhixinxi.hnqz.admin.api.model.WmEvaluationResultItemModel;
import lombok.AccessLevel;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.Future;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.util.List;

@NoArgsConstructor(access = AccessLevel.NONE)
public final class WmDaAgentDTO {

    @Data
    public static class OnEntAgentCreate {

        @NotEmpty(message = "简称必填")
        private String abbr;

        @NotEmpty(message = "服务省份简称必填")
        private String provAbbr;

        /**
         * 所属区域
         */
        @NotEmpty(message = "所属区域必填")
        private List<Long> areaCodes;

        /**
         * 启用标记
         */
        @NotBlank(message = "启用标记必填")
        private String enableFlag;

        /**
         * 所属药企
         */
        @NotNull(message = "所属药企必填")
        private Integer extDrugEnt;

        /**
         * 身份证背面图片
         */
        @JsonAlias(value = "frsfzzmBm")
        @NotBlank(message = "身份证背面图片必填")
        private String idCardBackImg;

        /**
         * 身份证正面图片
         */
        @JsonAlias(value = "frsfzzmZm")
        @NotBlank(message = "身份证正面图片必填")
        private String idCardFrontImg;

        /**
         * 手持身份证照片
         */
        @JsonAlias(value = "frsfzzmSc")
        @NotBlank(message = "手持身份证照片必填")
        private String idCardHolderImg;

        /**
         * 企业名称
         */
        @JsonAlias(value = "gsmc")
        @NotBlank(message = "企业名称必填")
        private String agentName;

        /**
         * 营业执照图片
         */
        @JsonAlias(value = "yyzz")
        @NotBlank(message = "营业执照图片必填")
        private String bizLicenceImg;

        /**
         * 统一信用代码
         */
        @JsonAlias(value = "zjNumber")
        @NotBlank(message = "统一信用代码必填")
        private String regCode;

        /**
         * 社保缴纳证明
         */
        @JsonAlias(value = "sbjnzm")
        @NotBlank(message = "社保缴纳证明必填")
        private String socialInsuranceImg;

        /**
         * 近12个月的依法纳税缴纳证明
         */
        @NotBlank(message = "近12个月的依法纳税缴纳证明必填")
        private String taxRecordIn12m;

        /**
         * 公示系统查询结果
         */
        @NotBlank(message = "公示系统查询结果必填")
        private String publicitySystemQueryResults;

        /**
         * 市场服务协议
         */
        @NotBlank(message = "市场服务协议必填")
        private String marketServicesAgreement;

        /**
         * 服务合同
         */
        @NotBlank(message = "服务合同必填")
        private String serviceContract;

        /**
         * 服务截止时间
         */
        @NotNull(message = "服务截止时间必填")
        @Future(message = "截止时间必须晚于当前日期")
        private LocalDate serviceDeadline;

        /**
         * 服务产品
         */
        @NotEmpty(message = "服务产品必填")
        private List<Integer> skuIds;

    }

    @Data
    public static class OnEval {

        @NotNull(message = "企业ID必填")
        private Integer deptId;

        @NotBlank(message = "评价类型必填")
        private String evalType;

        @NotBlank(message = "响应人名称必填")
        private String targetName;

        @NotNull(message = "评价时间必填")
        private LocalDate evalTime;

        @NotEmpty(message = "评审项信息必填")
        private List<WmEvaluationResultItemModel> items;

        @NotBlank(message = "评审人姓名必填")
        private String adminName;

        @NotNull(message = "签约日期必填")
        private LocalDate signDate;
    }

    @Data
    public static class OnCompDocCreate {

        private Integer documentId;

        @NotNull(message = "企业ID必填")
        public Integer deptId;

        /**
         * 员工规范手册
         */
        private List<WmAgentComplianceDocument.DocumentItem> employeeCodeHandbook;

        /**
         * 内部控制制度手册
         */
        private List<WmAgentComplianceDocument.DocumentItem> controlSystemManual;

        /**
         * 内部审计文件
         */
        private List<WmAgentComplianceDocument.DocumentItem> internalAuditDocuments;

        /**
         * 合规制度管理
         */
        private List<WmAgentComplianceDocument.DocumentItem> complianceMantSystem;

        /**
         * 反腐败和反贿赂制度
         */
        private List<WmAgentComplianceDocument.DocumentItem> antiCorrAndAntiBriManSys;

        /**
         * 员工招待管理制度
         */
        private List<WmAgentComplianceDocument.DocumentItem> employeeEntertainManSys;

        /**
         * 合规承诺函
         */
        private List<WmAgentComplianceDocument.DocumentItem> complianceCommitLet;

        /**
         * 内部合规培训材料
         */
        private List<WmAgentComplianceDocument.DocumentItem> internalComTrainMate;

    }
}
