package com.qunzhixinxi.hnqz.admin.api.dto;

import com.alibaba.excel.annotation.ExcelProperty;
import com.qunzhixinxi.hnqz.admin.api.model.WmPkgRedemptionNoticeItemModel;
import com.qunzhixinxi.hnqz.admin.api.model.WmQuizResultModel;
import lombok.AccessLevel;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 积分包操作dto
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-10-24 16:08
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class WmPkgDTO {

    @Data
    public static class OnEntPkgBatchCreate {

        @ExcelProperty(value = "执行包名称")
        private String pkgName;


        @ExcelProperty(value = "计划包名称")
        private String planName;

    }


    @Data
    public static class OnPkgPickup {

        @NotBlank(message = "积分包ID必填")
        private String pkgId;

        private WmQuizResultModel[] quizResults;

    }

    @Data
    public static class OnRedemptionNoticeLaunch {
        @NotNull(message = "兑付通知ID必填")
        private Integer redemptionNoticeId;
    }

    @Data
    public static class OnRedemptionNoticeCheck {
        @NotNull(message = "兑付通知ID必填")
        private Integer redemptionNoticeId;

        @NotNull(message = "审核结果必填")
        private Boolean checkResult;

        private String checkRemarks;
    }


    @Data
    public static class OnRedemptionNoticeCreate {

        @NotNull(message = "执行包ID必填")
        private Integer packageId;

        @NotNull(message = "通知步进值必填")
        @Max(value = 3, message = "步进值最大为${value}")
        @Min(value = 1, message = "步进值最小为${value}")
        private Integer step;

        @NotNull(message = "发起方ID必填")
        private Integer fromEntId;

        @NotNull(message = "接收方ID必填")
        private Integer toEntId;

        @NotEmpty(message = "通知项必填")
        private List<WmPkgRedemptionNoticeItemModel> items;

        @NotNull(message = "本次兑付分值必填")
        private Long currentRedemptionScore;

        @NotBlank(message = "本次兑付比例必填")
        private String currentRedemptionRate;

        // @NotBlank(message = "本次兑付权重必填")
        // private String currentRedemptionWeight;

        @NotBlank(message = "兑付总值必填")
        private String totalAmount;

        @NotBlank(message = "评价总值必填")
        private String currentRedemptionTotal;

    }

    @Data
    public static class OnRedemptionNoticePage {

        @NotNull(message = "页码必填")
        @Min(value = 1, message = "页码最小为{value}")
        private Integer current;

        @NotNull(message = "分页数量必填")
        @Min(value = 10, message = "分页数量最小为{value}")
        @Max(value = 100, message = "分页数量最大为{value}")
        private Integer size;
    }

    @Data
    public static class OnNoticeRead {

        @NotNull(message = "兑付通知必填")
        private Integer noticeId;
    }

    @Data
    public static class OnAcceptNotice {

        @NotNull(message = "兑付通知必填")
        private Integer noticeId;

        @NotBlank(message = "确认结果必填")
        private String state;

        @NotBlank(message = "确认信息必填")
        private String remarks;
    }
}
