package com.qunzhixinxi.hnqz.admin.api.dto;

import lombok.AccessLevel;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.util.Collection;
import java.util.List;

/**
 * 监督dto
 *
 * @author jimmy
 * @date 11/07/2024 16:57
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class WmSupervisionDto {

    @Data
    public static class OnPkgPage {

        private String pkgName;

        private List<Long> areaCode;

        private String entName;

        private Boolean supervision;

        @NotNull(message = "页码必填")
        @Min(value = 1, message = "页码最小为{value}")
        private Integer current;

        @NotNull(message = "分页数量必填")
        @Min(value = 10, message = "分页数量最小为{value}")
        @Max(value = 100, message = "分页数量最大为{value}")
        private Integer size;
    }

    @Data
    public static class OnB2BPkgPage {

        private String pkgName;

        private List<Long> areaCode;

        private String entName;

        private LocalDate[] period;

        private Boolean supervision;

        @NotNull(message = "页码必填")
        @Min(value = 1, message = "页码最小为{value}")
        private Integer current;

        @NotNull(message = "分页数量必填")
        @Min(value = 10, message = "分页数量最小为{value}")
        @Max(value = 100, message = "分页数量最大为{value}")
        private Integer size;
    }


    @Data
    public static class OnPkgSupervision {

        @NotNull(message = "执行包ID必填")
        private Integer pkgId;

        /**
         * 监督意见
         */
        @NotNull(message = "监督意见必填")
        private Boolean supervision;

        /**
         * 备注
         */
        // @NotBlank(message = "监督说明")
        private String remarks;

        @NotBlank(message = "监督结果必填")
        private String resultInfo;

    }

    @Data
    public static class OnTaskSupervision {

        @NotNull(message = "服务ID必填")
        private Integer taskId;

        @NotNull(message = "执行包ID必填")
        private Integer pkgId;

        /**
         * 监督意见
         */
        @NotNull(message = "监督意见必填")
        private Boolean supervision;

        /**
         * 备注
         */
        @NotBlank(message = "监督说明")
        private String remarks;

    }

    @Data
    public static class OnTaskSupervisionBatch {

        @NotEmpty(message = "监督ID必填")
        private Collection<Long> supervisionIds;

        /**
         * 监督意见
         */
        @NotNull(message = "监督意见必填")
        private Boolean supervision;

        /**
         * 备注
         */
        private String remarks;

    }

    @Data
    public static class OnPkgSupervisionCheck {

        @NotNull(message = "监督ID必填")
        private Integer supervisionId;

        /**
         * 监督意见
         */
        @NotNull(message = "监督意见必填")
        private Boolean supervision;

        /**
         * 备注
         */
        private String remarks;

    }

    @Data
    public static class OnTaskSupervisionCheck {

        @NotNull(message = "监督ID必填")
        private Integer supervisionId;

        /**
         * 监督意见
         */
        @NotNull(message = "监督意见必填")
        private Boolean supervision;

        /**
         * 备注
         */
        private String remarks;

    }


    @Data
    public static class OnWarnRulePage {

        @NotNull(message = "页码必填")
        @Min(value = 1, message = "页码最小为{value}")
        private Integer current;

        @NotNull(message = "分页数量必填")
        @Min(value = 10, message = "分页数量最小为{value}")
        @Max(value = 100, message = "分页数量最大为{value}")
        private Integer size;
    }

    @Data
    public static class OnWarnRulCreate {

        /**
         * 角色ID
         */
        @NotNull(message = "角色ID必填")
        private Integer roleId;

        /**
         * 预警类型
         */
        @NotBlank(message = "预警类型必填")
        private String warnType;

        /**
         * 预警逻辑
         */
        @NotBlank(message = "预警逻辑必填")
        private String warnLogic;

        /**
         * 预警值
         */
        @NotNull(message = "预警值必填")
        private Integer value;

        /**
         * 预警消息模板
         */
        @NotBlank(message = "预警消息模板必填")
        private String template;

    }

    @Data
    public static class OnWarnRulUpdate {

        /**
         * 规则ID
         */
        @NotNull(message = "规则ID必填")
        private Integer ruleId;

        /**
         * 预警逻辑
         */
        private String warnLogic;

        /**
         * 预警值
         */
        private Integer value;

        /**
         * 预警消息模板
         */
        private String template;
    }

    @Data
    public static class OnWarnPage {

        @NotNull(message = "页码必填")
        @Min(value = 1, message = "页码最小为{value}")
        private Integer current;

        @NotNull(message = "分页数量必填")
        @Min(value = 10, message = "分页数量最小为{value}")
        @Max(value = 100, message = "分页数量最大为{value}")
        private Integer size;
    }

    @Data
    public static class OnWarnCreate {

        @NotNull(message = "警告信息必填")
        private Integer warnId;
    }

    @Data
    public static class OnTaskSupervisionPage {

        private String taskId;

        private String taskNumber;

        private String taskTypeId;

        private String pkgName;

        private LocalDate[] pkgPeriod;

        private String skuName;

        private String mahName;

        private String memberName;


        @NotNull(message = "查询服务商必填")
        private Integer deptId;

        @NotNull(message = "复查标记必填")
        private Boolean review;

        @NotNull(message = "页码必填")
        @Min(value = 1, message = "页码最小为{value}")
        private Integer current;

        @NotNull(message = "分页数量必填")
        @Min(value = 10, message = "分页数量最小为{value}")
        @Max(value = 100, message = "分页数量最大为{value}")
        private Integer size;
    }

}
