package com.qunzhixinxi.hnqz.admin.config;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaHospital;
import com.qunzhixinxi.hnqz.admin.service.WmDaHospitalService;
import com.qunzhixinxi.hnqz.common.core.constant.CacheConstants;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.gateway.support.DynamicRouteInitEvent;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.web.context.WebServerInitializedEvent;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.event.EventListener;
import org.springframework.core.annotation.Order;
import org.springframework.data.geo.Point;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Async;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 地理位置初始化
 *
 * @author jimmy
 * @date 2024-12-18 15:54
 */
@Slf4j
@Configuration
@AllArgsConstructor
public class GeoInitRunner {


    private final RedisTemplate redisTemplate;

    private final WmDaHospitalService hospitalService;

    @Async
    @Order
    @EventListener({WebServerInitializedEvent.class, DynamicRouteInitEvent.class})
    public void initRoute() {

        log.info("======> 初始化医院地理位置开始 <======");
        Boolean result = redisTemplate.delete(CacheConstants.HOSPITAL_GEO_KEY);
        log.info("======> 删除历史医院地理位置：【{}】 ", result);

        // 获取全部的医院信息
        List<WmDaHospital> hospitals = hospitalService.list(Wrappers.<WmDaHospital>lambdaQuery()
                .eq(WmDaHospital::getEnableFlag, CommonConstants.STATUS_NORMAL)
                .eq(WmDaHospital::getDelFlag, CommonConstants.STATUS_NORMAL));

        // 过滤全部的可用医院信息
        List<WmDaHospital> avail = hospitals.stream().filter(h -> StrUtil.isAllNotBlank(h.getLatitude(), h.getLongitude())).collect(Collectors.toList());
        log.info("======> 获取医院地理位置：全部【{}】，可用【{}】 ", hospitals.size(), avail.size());


        if (CollUtil.isEmpty(avail)) {
            log.info("======> 可用医院地理位置为0，结束初始化");

        } else {

            avail.forEach(h -> {

                // 纬度
                double latitude = Double.parseDouble(h.getLatitude());

                // 经度
                double longitude = Double.parseDouble(h.getLongitude());

                log.info("医院地理位置家在：ID：【{}】名称【{}】经度【{}】 纬度【{}】", h.getId(), h.getYymc(), longitude, latitude);
                redisTemplate.opsForGeo().add(CacheConstants.HOSPITAL_GEO_KEY, new Point(longitude, latitude), h.getId());

            });

        }
        log.debug("======> 初始化医院地理位置结束 <======");
    }
}
