package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.util.StrUtil;
import com.qunzhixinxi.hnqz.admin.manager.WmPackageManager;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.annotation.Inner;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * 小程序前端控制器
 *
 * @author jimmy
 * @date 2024-10-30 20:49
 */
@Slf4j
@RestController
@AllArgsConstructor
public class ApiControllerV2 {

    private final RedisTemplate<String, Object> redisTemplate;
    private final WmPackageManager packageManager;

    @Inner(false)
    @PostMapping(value = "/api/v2/tool/del/cache")
    private R<Boolean> delCache(@Validated @RequestBody Map<String, String> cacheKey) {

        String s = cacheKey.get("key");
        String v1 = cacheKey.get("v1");
        String v2 = cacheKey.get("v2");
        String ck = String.format("%s:%s:%s", s, v1, v2);

        log.info("key_info:{}", ck);
        if (StrUtil.isBlank(ck)) {
            return R.failed("缓存key不存在");
        }

        redisTemplate.delete(ck);

        return R.ok();

    }


    /**
     * 计算两点中间的距离
     *
     * @return 两点之间的距离
     */
    @Inner(false)
    @GetMapping(value = "/api/v2/tool/calc/distance")
    public R<Double> calcDistance(double long1, double lat1, double long2, double lat2) {
        double a, b, r;
        // 地球半径
        r = 6378137;
        lat1 = lat1 * Math.PI / 180.0;
        lat2 = lat2 * Math.PI / 180.0;
        a = lat1 - lat2;
        b = (long1 - long2) * Math.PI / 180.0;
        double d;
        double sa2, sb2;
        sa2 = Math.sin(a / 2.0);
        sb2 = Math.sin(b / 2.0);
        d = 2 * r * Math.asin(Math.sqrt(sa2 * sa2 + Math.cos(lat1) * Math.cos(lat2) * sb2 * sb2));
        return R.ok(d);
    }

    /**
     * 获取当前积分包任务信息详情
     *
     * @param pkgId 积分包ID
     * @return 任务详情列表
     */
    @GetMapping(value = "/api/v2/getPkgTaskDetails")
    public R<List<?>> listPkgTaskDetails(@RequestParam("pkgId") Integer pkgId) {
        return R.ok(packageManager.listPkgTaskDetails(pkgId));
    }
}
