package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.MedicalEquipment;
import com.qunzhixinxi.hnqz.common.core.entity.BaseEntity;
import com.qunzhixinxi.hnqz.admin.service.MedicalEquipmentService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 医疗企业前端控制器
 *
 * @author lixuesong
 * @date 2022/11/08 10:07
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/medical-equipment")
public class MedicalEquipmentController {

    private final MedicalEquipmentService medicalEquipmentService;
    private final SysDeptService sysDeptService;

    /**
     * 新增结果
     *
     * @param medicalEquipment 医疗器械信息
     * @return 新增结果
     */
    @SysLog(value = "新增医疗器械")
    @PostMapping
    public R<?> save(@Validated(BaseEntity.Create.class) @RequestBody MedicalEquipment medicalEquipment) {
        int count = medicalEquipmentService.count(Wrappers.<MedicalEquipment>lambdaQuery()
                .eq(MedicalEquipment::getDrugEntId, medicalEquipment.getDrugEntId())
                .eq(MedicalEquipment::getName, medicalEquipment.getName()));
        if (count > 0) {
            return R.failed("所属企业下已存在");
        }

        SysDept sysDept = sysDeptService.getById(medicalEquipment.getDrugEntId());
        if (sysDept != null) {
            medicalEquipment.setDrugEntName(sysDept.getName());
        }

        medicalEquipmentService.save(medicalEquipment);
        return R.ok();
    }

    /**
     * 修改医疗器械
     *
     * @param medicalEquipment 修改信息
     * @return 修改结果
     */
    @SysLog(value = "修改医疗器械")
    @PutMapping
    public R<?> update(@Validated(BaseEntity.Update.class) @RequestBody MedicalEquipment medicalEquipment) {
        int count = medicalEquipmentService.count(Wrappers.<MedicalEquipment>lambdaQuery()
                .eq(MedicalEquipment::getDrugEntId, medicalEquipment.getDrugEntId())
                .eq(MedicalEquipment::getName, medicalEquipment.getName())
                .ne(MedicalEquipment::getId, medicalEquipment.getId()));
        if (count > 0) {
            return R.failed("所属企业下已存在");
        }

        SysDept sysDept = sysDeptService.getById(medicalEquipment.getDrugEntId());
        if (sysDept != null) {
            medicalEquipment.setDrugEntName(sysDept.getName());
        }

        medicalEquipmentService.updateById(medicalEquipment);
        return R.ok();
    }

    /**
     * 删除医疗器械
     *
     * @param id 器械ID
     * @return 删除结果
     */
    @SysLog("删除医疗器械")
    @DeleteMapping("/{id}")
    public R<?> remove(@PathVariable("id") Integer id) {
        medicalEquipmentService.removeById(id);
        return R.ok();
    }

    /**
     * 分页查询
     *
     * @param page             分页信息
     * @param medicalEquipment 分页条件
     * @return 分页结果
     */
    @GetMapping("/page")
    public R<?> page(Page<MedicalEquipment> page, MedicalEquipment medicalEquipment) {
        LambdaQueryWrapper<MedicalEquipment> queryWrapper = Wrappers.lambdaQuery();
        if (StrUtil.isNotBlank(medicalEquipment.getName())) {
            queryWrapper.like(MedicalEquipment::getName, medicalEquipment.getName());
        }
        if (StrUtil.isNotBlank(medicalEquipment.getManufacturer())) {
            queryWrapper.like(MedicalEquipment::getManufacturer, medicalEquipment.getManufacturer());
        }
        if (StrUtil.isNotBlank(medicalEquipment.getDrugEntName())) {
            queryWrapper.like(MedicalEquipment::getDrugEntName, medicalEquipment.getDrugEntName());
        }
        Page<MedicalEquipment> equipmentPage = medicalEquipmentService.page(page, queryWrapper);
        return R.ok(equipmentPage);
    }

}
