package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.util.ArrayUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.pig4cloud.plugin.excel.annotation.ResponseExcel;
import com.qunzhixinxi.hnqz.admin.api.entity.SysAreaEntity;
import com.qunzhixinxi.hnqz.admin.service.SysAreaEntityService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 行政区划 前端控制器
 *
 * @author lbw
 * @date 2024-02-16 22:40:06
 */
@RestController
@RequiredArgsConstructor
public class SysAreaController {

    private final SysAreaEntityService sysAreaEntityService;

    /**
     * 分页查询
     *
     * @param page    分页对象
     * @param sysArea 行政区划
     * @return 区域分页
     */
    @GetMapping("/sysArea/page")
    public R<Page<SysAreaEntity>> getSysAreaPage(Page<SysAreaEntity> page, SysAreaEntity sysArea) {
        return R.ok(sysAreaEntityService.selectPage(page, sysArea));
    }

    /**
     * 前端联动组件需要数据
     *
     * @param sysArea 查询条件
     * @return tree
     */
    @GetMapping("/sysArea/tree")
    public R<List<Tree<Long>>> getSysAreaTree(SysAreaEntity sysArea) {
        return R.ok(sysAreaEntityService.selectTree(sysArea));
    }

    /**
     * 根据parentId获取区域列表
     *
     * @param parentId 父ID
     * @return tree
     */
    @GetMapping("/sysArea/list")
    public R<List<SysAreaEntity>> listAreas(@RequestParam(value = "parentId", defaultValue = "0") Long parentId) {
        return R.ok(sysAreaEntityService.listAreas(parentId));
    }


    /**
     * 通过id查询行政区划
     *
     * @param sysArea 查询条件
     * @return R
     */
    @GetMapping("/sysArea/details}")
    public R<SysAreaEntity> getDetails(SysAreaEntity sysArea) {
        return R.ok(sysAreaEntityService.getOne(Wrappers.query(sysArea)));
    }

    /**
     * 新增行政区划
     *
     * @param sysArea 行政区划
     * @return R
     */
    @SysLog("新增行政区划")
    @PostMapping("/sysArea/create")
    public R<Boolean> save(@RequestBody SysAreaEntity sysArea) {
        return R.ok(sysAreaEntityService.save(sysArea));
    }

    /**
     * 修改行政区划
     *
     * @param sysArea 行政区划
     * @return R
     */
    @SysLog("修改行政区划")
    @PostMapping("/sysArea/update")
    public R<Boolean> updateById(@RequestBody SysAreaEntity sysArea) {
        return R.ok(sysAreaEntityService.updateById(sysArea));
    }

    /**
     * 通过id删除行政区划
     *
     * @param ids id列表
     * @return R
     */
    @SysLog("通过id删除行政区划")
    @PostMapping("/sysArea/del")
    public R<Boolean> removeById(@RequestBody Long[] ids) {
        return R.ok(sysAreaEntityService.removeByIds(CollUtil.toList(ids)));
    }


    /**
     * 导出excel 表格
     *
     * @param sysArea 查询条件
     * @param ids     导出指定ID
     * @return excel 文件流
     */
    @ResponseExcel
    @GetMapping("/sysArea/export")
    public List<SysAreaEntity> export(SysAreaEntity sysArea, Long[] ids) {
        return sysAreaEntityService.list(Wrappers.lambdaQuery(sysArea).in(ArrayUtil.isNotEmpty(ids), SysAreaEntity::getAreaId, ids));
    }
}
