package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.dto.DeptTree;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptCsm;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptRelation;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaAgent;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEnt;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDeptRegulations;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.DeptLevelEnum;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.EnableEnum;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptCsmMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptRelationMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserMapper;
import com.qunzhixinxi.hnqz.admin.service.SysCsmUserRelationService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptRelationService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysEnterpriseAreaService;
import com.qunzhixinxi.hnqz.admin.service.WmDaAgentService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.admin.service.WmDeptRegulationsService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 部门管理 前端控制器
 * </p>
 *
 * @author hnqz
 * @since 2018-01-20
 */
@RestController
@AllArgsConstructor
@RequestMapping("/dept")
public class SysDeptController {

    private final SysDeptRelationService relationService;

    private final SysDeptService sysDeptService;

    private final SysDeptMapper sysDeptMapper;

    private final WmDaDrugEntService wmDaDrugEntService;

    private final WmDaAgentService wmDaAgentService;

    private final WmDeptRegulationsService wmDeptRegulationsService;

    private final SysDeptCsmMapper sysDeptCsmMapper;

    private final SysUserMapper sysUserMapper;

    private final SysDeptRelationMapper sysDeptRelationMapper;

    private final SysEnterpriseAreaService enterpriseAreaService;

    private final SysCsmUserRelationService sysCsmUserRelationService;

    /**
     * 按照等级获取企业列表
     *
     * @param lv 企业等级
     * @return 企业列表
     */
    @GetMapping(value = "/list/{level}")
    public R<List<SysDept>> listDeptsByLv(@PathVariable("level") Integer lv) {
        List<SysDept> list = sysDeptService.list(Wrappers.<SysDept>lambdaQuery().eq(SysDept::getLevel, lv));

        if (CollUtil.isNotEmpty(list)) {
            Set<Long> collect = list.stream().mapToLong(SysDept::getDeptId).boxed().collect(Collectors.toSet());

            Map<Long, List<Long>> longListMap = enterpriseAreaService.listEntAreas1(collect);

            list.forEach(d -> {
                        List<Long> longs = longListMap.get(Long.valueOf(d.getDeptId()));
                        d.setAreaCodes(CollUtil.isNotEmpty(longs) ? longs : Collections.emptyList());
                    }
            );

        }

        return R.ok(list);
    }

    /**
     * 查收子级列表
     *
     * @return 返回子级
     */
    @GetMapping(value = "/list/descendant")
    public R<List<SysDept>> listDescendants(@RequestParam(value = "deptId", required = false) Integer deptId) {

        if (deptId == null) {
            HnqzUser user = SecurityUtils.getUser();
            deptId = user.getDeptId();
        }

        List<SysDeptRelation> list = relationService.list(Wrappers.<SysDeptRelation>lambdaQuery()
                .ne(SysDeptRelation::getDescendant, deptId)
                .eq(SysDeptRelation::getAncestor, deptId));

        if (CollUtil.isNotEmpty(list)) {

            Set<Integer> collect = list.stream().map(SysDeptRelation::getDescendant).collect(Collectors.toSet());

            List<SysDept> sysDepts = sysDeptService.listByIds(collect);

            return R.ok(sysDepts);

        }


        return R.ok(Collections.emptyList());
    }


    /**
     * 查收父级列表
     *
     * @return 返回子级
     */
    @GetMapping(value = "/list/ancestor")
    public R<List<SysDept>> listAncestors(@RequestParam(value = "deptId", required = false) Integer deptId,
                                          @RequestParam(value = "onlySelf", required = false, defaultValue = "false") Boolean onlySelf) {



        if (deptId == null) {
            HnqzUser user = SecurityUtils.getUser();
            deptId = user.getDeptId();
        }

        if (onlySelf){
            SysDept service = sysDeptService.getById(deptId);
            return R.ok(Collections.singletonList(service));
        }

        List<SysDeptRelation> list = relationService.list(Wrappers.<SysDeptRelation>lambdaQuery()
                .ne(SysDeptRelation::getAncestor, deptId)
                .eq(SysDeptRelation::getDescendant, deptId));

        if (CollUtil.isNotEmpty(list)) {

            Set<Integer> collect = list.stream().map(SysDeptRelation::getAncestor).collect(Collectors.toSet());

            List<SysDept> sysDepts = sysDeptService.listByIds(collect);

            return R.ok(sysDepts);

        }


        return R.ok(Collections.emptyList());
    }

    /**
     * 获取登录用户企业的有效期
     *
     * @return 有效期信息
     */
    @GetMapping("/expiry")
    public R<Map<String, Object>> checkDeptExpiryDate() {

        HnqzUser user = SecurityUtils.getUser();

        SysDept sysDept = sysDeptService.getById(user.getDeptId());

        LocalDate expiryDate = (sysDept != null && sysDept.getExpiryDate() != null) ? sysDept.getExpiryDate() : LocalDate.of(2099, 12, 31);
        long remainDays = Duration.between(LocalDateTime.now(), LocalDateTime.of(expiryDate, LocalTime.MAX)).toDays();

        Map<String, Object> res = new HashMap<>(3);

        res.put("expiryDate", expiryDate);
        res.put("remainDays", remainDays);

        return R.ok(res);
    }

    /**
     * 通过ID查询
     *
     * @param id ID
     * @return SysDept
     */
    @GetMapping("/{id}")
    public R<SysDept> getById(@PathVariable Integer id) {
        return R.ok(sysDeptService.getById(id));
    }

    /**
     * 返回树形菜单集合
     *
     * @return 树形菜单
     */
    @GetMapping(value = "/tree")
    public R<List<DeptTree>> getTree(SysDept sysDept) {
        return R.ok(sysDeptService.selectTree(sysDept));
    }


    /**
     * 返回树形菜单集合
     *
     * @return 树形菜单
     */
    @GetMapping(value = "/treeAddUser")
    public R<List<DeptTree>> getTreeAddUser(SysDept sysDept) {
        return R.ok(sysDeptService.selectTreeAddUser(sysDept));
    }

    /**
     * 添加
     *
     * @param sysDept 实体
     * @return success/false
     */
    @SysLog("添加部门")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('sys_dept_add')")
    public R<Boolean> save(@Valid @RequestBody SysDept sysDept) {
        int level = sysDept.getLevel();
        SysDept selDept = new SysDept();
        if (level == 2) {
            selDept.setLevel1Id(sysDept.getLevel1Id());
        } else if (level == 3) {
            if (null != sysDept.getParentId()) {
                SysDept le2 = sysDeptService.getById(sysDept.getParentId());
                if (null != le2) {
                    sysDept.setLevel1Id(le2.getLevel1Id());
                    sysDept.setLevel1Name(le2.getLevel1Name());
                    selDept.setLevel1Id(le2.getLevel1Id());
                }
            }
            selDept.setLevel2Id(sysDept.getLevel2Id());
        } else if (level == 4) {
            if (null != sysDept.getParentId()) {
                SysDept le3 = sysDeptService.getById(sysDept.getParentId());
                if (null != le3) {
                    sysDept.setLevel2Id(le3.getLevel2Id());
                    sysDept.setLevel2Name(le3.getLevel2Name());
                    selDept.setLevel1Id(le3.getLevel1Id());
                }

                if (null != le3.getParentId()) {
                    SysDept le2 = sysDeptService.getById(le3.getParentId());
                    if (null != le2) {
                        sysDept.setLevel1Id(le2.getLevel1Id());
                        sysDept.setLevel1Name(le2.getLevel1Name());
                    }
                }
            }


            selDept.setLevel3Id(sysDept.getLevel3Id());
        } else {
            return R.failed("不允许录入的组织机构");
        }

        if (null == sysDept.getDeptId()) {
            return R.failed("组织机构信息只能选择，不能手工输入");
        }

        List<SysDept> list = sysDeptService.list(Wrappers.query(selDept));
        if (CollUtil.isNotEmpty(list)) {
            return R.failed("机构信息已录入组织架构");
        } else {
            Integer deptId = sysDeptService.saveDept(sysDept);
            if (deptId > 0) {
                if (level == 2) {
                    WmDaDrugEnt wmDaDrugEnt = wmDaDrugEntService.getById(sysDept.getLevel1Id());
                    if (null != wmDaDrugEnt) {
                        wmDaDrugEnt.setDeptId(deptId);
                        wmDaDrugEntService.updateById(wmDaDrugEnt);
                    }
                } else if (level == 3) {
                    WmDaAgent wmDaAgent = wmDaAgentService.getById(sysDept.getLevel2Id());
                    if (null != wmDaAgent) {
                        wmDaAgent.setDeptId(deptId);
                        wmDaAgentService.updateById(wmDaAgent);
                    }
                } else if (level == 4) {
                    WmDaAgent wmDaAgent = wmDaAgentService.getById(sysDept.getLevel3Id());
                    if (null != wmDaAgent) {
                        wmDaAgent.setDeptId(deptId);
                        wmDaAgentService.updateById(wmDaAgent);
                    }
                }
            }
            return R.ok(deptId > 0);
        }
    }

    /**
     * 删除
     *
     * @param id ID
     * @return success/false
     */
    @SysLog("删除部门")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('sys_dept_del')")
    public R<Boolean> removeById(@PathVariable Integer id) {
        return R.ok(sysDeptService.removeDeptById(id));
    }

    /**
     * 编辑
     *
     * @param sysDept 实体
     * @return success/false
     */
    @SysLog("编辑部门")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('sys_dept_edit')")
    public R<Boolean> update(@Valid @RequestBody SysDept sysDept) {
        sysDept.setUpdateTime(LocalDateTime.now());
        return R.ok(sysDeptService.updateDeptById(sysDept));
    }

    /**
     * 查收子级列表
     *
     * @return 返回子级
     */
    @GetMapping(value = "/getDescendantList/{deptId}")
    public R<List<SysDeptRelation>> getDescendantList(@PathVariable Integer deptId) {
        return R.ok(relationService.list(Wrappers.<SysDeptRelation>lambdaQuery().eq(SysDeptRelation::getAncestor, deptId)));
    }


    /**
     * 修改个人信息
     *
     * @return success/false
     */
    @GetMapping("/list")
    public R<List<Map<String, Object>>> listUserInfo() {

        List<SysDept> list = sysDeptService.list();
        List<Map<String, Object>> answerList = new ArrayList<>(list.size());
        if (CollUtil.isNotEmpty(list)) {
            for (SysDept sysUser : list) {
                Map<String, Object> map = new HashMap<>(2);
                map.put("dept_id", sysUser.getDeptId());
                map.put("name", sysUser.getName());
                answerList.add(map);
            }

        }
        return R.ok(answerList);
    }


    /**
     * 修改个人信息
     *
     * @return success/false
     */
    @GetMapping("/listScope")
    public R<List<Map<String, Object>>> listScope(SysDept sysDept) {

        List<SysDept> list = sysDeptService.listScope(sysDept);
        List<Map<String, Object>> answerList = new ArrayList<>(list.size());
        if (CollUtil.isNotEmpty(list)) {
            for (SysDept sysUser : list) {
                Map<String, Object> map = new HashMap<>(2);
                map.put("deptId", sysUser.getDeptId());
                map.put("name", sysUser.getName());
                answerList.add(map);
            }

        }
        return R.ok(answerList);
    }

    @GetMapping("/selectDeptByName")
    public R<List<Map<String, Object>>> selectDeptByName(SysDept sysDept) {

        List<SysDept> list = sysDeptService.selectDeptByName(sysDept);
        List<Map<String, Object>> answerList = new ArrayList<>(list.size());
        if (CollUtil.isNotEmpty(list)) {
            for (SysDept sysUser : list) {
                Map<String, Object> map = new HashMap<>(2);
                map.put("deptId", sysUser.getDeptId());
                map.put("name", sysUser.getName());
                answerList.add(map);
            }

        }
        return R.ok(answerList);
    }


    /**
     * 修改个人信息
     *
     * @return success/false
     */
    @SysLog("获取组织架构完整名称")
    @GetMapping("/getDeptBuildNameById")
    public R<String> getDeptBuildNameById(String deptId, String type) {

        SysDept sysDept = sysDeptService.getById(deptId);
        String buildName = "";
        if ("agent".equals(type)) {
            if (null != sysDept) {
                sysDept = sysDeptService.getById(sysDept.getParentId());
            }
        }

        if (null != sysDept && sysDept.getLevel() == 3) {

            if (null != sysDept.getLevel1Id()) {
                WmDaDrugEnt wmDaDrugEnt = wmDaDrugEntService.getById(sysDept.getLevel1Id());
                if (null != wmDaDrugEnt) {
                    buildName = wmDaDrugEnt.getEntname();
                }
            }


            if (null != sysDept.getLevel2Id()) {
                WmDaAgent wmDaAgent = wmDaAgentService.getById(sysDept.getLevel2Id());
                if (null != wmDaAgent) {
                    if (!"".equals(buildName)) {
                        buildName = buildName + "-" + wmDaAgent.getGsmc();
                    } else {
                        buildName = wmDaAgent.getGsmc();
                    }
                }
            }


        }

        if (null != sysDept && sysDept.getLevel() == 2) {
            if (null != sysDept.getLevel1Id()) {
                WmDaDrugEnt wmDaDrugEnt = wmDaDrugEntService.getById(sysDept.getLevel1Id());
                if (null != wmDaDrugEnt) {
                    if (!"".equals(buildName)) {
                        buildName = buildName + "-" + wmDaDrugEnt.getEntname();
                    } else {
                        buildName = wmDaDrugEnt.getEntname();
                    }
                }
            }
        }
        return R.ok(buildName);
    }

    @GetMapping("/page")
    public R<List<SysDept>> getSysDeptPage() {
        return R.ok(sysDeptService.list());
    }

    @GetMapping("/selectSysDeptList")
    public R<Page<Map<String, Object>>> selectSysDeptList(Page<?> page, SysDept sysDept) {
        return R.ok(relationService.selectSysDeptList(page, sysDept));
    }

    /**
     * 查询关联服务企业列表
     *
     * @return 获得所有父企业
     */
    @GetMapping("/getAllParentDept")
    public R<List<SysDept>> getAllParentDept() {
        SysDept sysDept = new SysDept();
        sysDept.setDeptId(SecurityUtils.getUser().getDeptId());
        List<SysDept> list = sysDeptService.getAllParentDept(sysDept);
        return R.ok(list);
    }

    /**
     * 查询服务上级
     *
     * @return list
     */
    @GetMapping("/getParentDept")
    public R<List<SysDept>> getParentDept() {
        SysDept sysDept = new SysDept();
        sysDept.setDeptId(SecurityUtils.getUser().getDeptId());
        List<SysDept> list = sysDeptService.getParentDept(sysDept);
        return R.ok(list);
    }

    /**
     * 查询服务上级
     *
     * @return list
     */
    @GetMapping("/getChildDept")
    public R<List<Map<String, Object>>> getChildDept() {
        SysDept sysDept = new SysDept();
        sysDept.setDeptId(SecurityUtils.getUser().getDeptId());
        List<Map<String, Object>> list = sysDeptService.getChildDept(sysDept);
        return R.ok(list);
    }


    /**
     * Integer类型deptId -> name
     * 自定义sql 包含停用的机构
     *
     * @return list
     */
    @GetMapping("/convertToName")
    public R<List<Map<String, Object>>> convertToName() {
        return R.ok(sysDeptService.convertToName());
    }

    /**
     * 分页查询药企和CSO代理商
     *
     * @param deptName 企业名称
     * @return 分页集合
     */
    @GetMapping("/list-ent-and-agent")
    public R<?> listEntAndAgent(Page<SysDept> page, String deptName) {
        return R.ok(sysDeptService.listEntAndAgent(page, deptName));
    }

    /**
     * 查询部门规章制度列表
     *
     * @return {@link R}<{@link ?}> 返回结果
     */
    @GetMapping("/dept-regulations/list")
    public R<List<WmDeptRegulations>> listDeptRegulations() {
        Integer deptId = SecurityUtils.getUser().getDeptId();
        List<WmDeptRegulations> list = wmDeptRegulationsService.list(Wrappers.<WmDeptRegulations>lambdaQuery()
                .eq(WmDeptRegulations::getDeptId, deptId)
                .eq(WmDeptRegulations::getDelFlag, DelEnum.NOT_DEL.getVal()));
        list.sort((o1, o2) -> {
            if (o1.getSort() > 0 && o2.getSort() > 0) {
                return o1.getSort() - o2.getSort();
            }
            return o1.getId() - o2.getId();
        });

        return R.ok(list);
    }

    /**
     * 分页查询企业-用于BC总负责人
     *
     * @param name    企业名称
     * @param csmName 负责人
     * @param level   企业类型(2-药企,3-CSO,4-CSO,5-其他)
     * @return {@link R}<{@link ?}> 结果
     */
    @GetMapping("/page-dept-for-bc")
    public R<IPage<SysDept>> pageDeptForCsm(Page<SysDept> page, String name, String csmName, Integer level) {
        LambdaQueryWrapper<SysDept> queryWrapper = Wrappers.lambdaQuery();
        if (StrUtil.isNotBlank(name)) {
            queryWrapper.like(SysDept::getName, name);
        }
        if (StrUtil.isNotBlank(csmName)) {
            List<SysUser> users = sysUserMapper.selectList(Wrappers.<SysUser>lambdaQuery()
                    .eq(SysUser::getRealname, csmName));
            if (CollUtil.isEmpty(users)) {
                return R.ok(page);
            }
            Set<Integer> userIds = users.stream().map(SysUser::getUserId).collect(Collectors.toSet());
            List<SysDeptCsm> deptCsms = sysDeptCsmMapper.selectList(Wrappers.<SysDeptCsm>lambdaQuery().in(SysDeptCsm::getUserId, userIds));
            if (CollUtil.isEmpty(deptCsms)) {
                return R.ok(page);
            }
            Set<Integer> deptIds = deptCsms.stream().map(SysDeptCsm::getDeptId).collect(Collectors.toSet());
            queryWrapper.in(SysDept::getDeptId, deptIds);
        }
        if (level != null) {
            queryWrapper.eq(SysDept::getLevel, level);
        }

        if (SecurityUtils.getRoles().contains(19)) {
            // CSM只能查看自己维护的企业下的用户, 以及下属维护的企业下的用户
            Integer userId = SecurityUtils.getUser().getId();
            // 递归查询下属关联的userId
            Set<Integer> allUserIds = sysCsmUserRelationService.recurseCsmLowerUserId(Collections.singleton(userId), new AtomicInteger(10));
            // 查询企业id
            List<SysDeptCsm> deptCsms = sysDeptCsmMapper.selectList(Wrappers.<SysDeptCsm>lambdaQuery()
                    .in(SysDeptCsm::getUserId, allUserIds));
            if (CollUtil.isEmpty(deptCsms)) {
                return R.ok(page);
            }

            Set<Integer> deptIds = deptCsms.stream().map(SysDeptCsm::getDeptId).collect(Collectors.toSet());
            queryWrapper.in(SysDept::getDeptId, deptIds);
        }

        queryWrapper.ne(SysDept::getLevel, DeptLevelEnum.ADMIN.getVal());
        sysDeptMapper.pageAllDept(page, queryWrapper);

        List<SysDept> records = page.getRecords();
        if (CollUtil.isEmpty(records)) {
            return R.ok(page);
        }

        records.forEach(record -> {
            // 负责人
            List<SysDeptCsm> deptCsms = sysDeptCsmMapper.selectList(Wrappers.<SysDeptCsm>lambdaQuery()
                    .eq(SysDeptCsm::getDeptId, record.getDeptId()));
            if (CollUtil.isNotEmpty(deptCsms)) {
                Set<Integer> userIds = deptCsms.stream().map(SysDeptCsm::getUserId).collect(Collectors.toSet());
                List<SysUser> users = sysUserMapper.selectBatchIds(userIds);
                record.setCsmNames(users.stream().map(SysUser::getRealname).collect(Collectors.joining(",")));
                record.setCsmUserIds(userIds.toArray(new Integer[0]));
            }
            // 是否启用
            if (StrUtil.isNotBlank(record.getDelFlag())) {
                record.setDelFlagName(Objects.requireNonNull(EnableEnum.getByVal(record.getDelFlag())).getDesc());
            }
            // 企业类型
            if (record.getLevel() != null) {
                DeptLevelEnum enumByVal = DeptLevelEnum.getEnumByVal(record.getLevel());
                if (enumByVal != null) {
                    record.setLevelName(enumByVal.getName());
                }
            }
            // 上级企业
            List<SysDeptRelation> deptRelations = sysDeptRelationMapper.selectList(Wrappers.<SysDeptRelation>lambdaQuery()
                    .eq(SysDeptRelation::getDescendant, record.getDeptId())
                    .notIn(SysDeptRelation::getAncestor, record.getDeptId(), 1));
            if (CollUtil.isNotEmpty(deptRelations)) {
                Set<Integer> superDeptIds = deptRelations.stream().map(SysDeptRelation::getAncestor).collect(Collectors.toSet());
                List<SysDept> depts = sysDeptService.listByIds(superDeptIds);
                record.setSuperDeptName(depts.stream().map(SysDept::getName).collect(Collectors.joining(",")));

                // 根据上级企业获取上级企业的csm
//				List<SysDeptCsm> csms = sysDeptCsmMapper.selectList(Wrappers.<SysDeptCsm>lambdaQuery()
//						.in(SysDeptCsm::getDeptId, superDeptIds));
//				if (CollUtil.isNotEmpty(csms)) {
//					Set<Integer> userIds = csms.stream().map(SysDeptCsm::getUserId).collect(Collectors.toSet());
//					record.setCsmUserIds(ArrayUtil.addAll(userIds.toArray(new Integer[0]), record.getCsmUserIds()));
//
//					List<SysUser> users = sysUserMapper.selectBatchIds(userIds);
//					if (record.getCsmNames() != null) {
//						record.setCsmNames(StrUtil.join(",", users.stream().map(SysUser::getRealname).collect(Collectors.joining(",")), record.getCsmNames()));
//					} else {
//						record.setCsmNames(users.stream().map(SysUser::getRealname).collect(Collectors.joining(",")));
//					}
//				}
            }
        });

        return R.ok(page);
    }

    /**
     * 将CMS分配给部门
     *
     * @param params 参数
     * @return {@link R}<{@link ?}> 结果
     */
    @SuppressWarnings("unchecked")
    @SysLog("将CMS分配给部门")
    @PostMapping("/assign-cms-to-dept")
    public R<?> assignCmsToDept(@RequestBody Map<String, Object> params) {
        List<Integer> cmsUserIds = (List<Integer>) params.get("userIds");
        List<Integer> deptIds = (List<Integer>) params.get("deptIds");
        if (CollUtil.isEmpty(deptIds)) {
            return R.failed("deptIds必填");
        }


        // 删除关联关系
        sysDeptCsmMapper.delete(Wrappers.<SysDeptCsm>lambdaUpdate()
                .in(SysDeptCsm::getDeptId, deptIds));

        if (CollUtil.isNotEmpty(cmsUserIds)) {
            // 查询已有的csm的配置信息, 用于复制到新增的数据
            Map<Integer, SysDeptCsm> deptCsmMap = new HashMap<>();
            List<SysDeptCsm> sysDeptCsms = sysDeptCsmMapper.listDistinctUserCsm(cmsUserIds);
            if (CollUtil.isNotEmpty(sysDeptCsms)) {
                deptCsmMap.putAll(sysDeptCsms.stream().collect(Collectors.toMap(SysDeptCsm::getUserId, Function.identity())));
            }

            // 建立关联关系
            deptIds.forEach(deptId -> {
                cmsUserIds.forEach(userId -> {
                    Integer count = sysDeptCsmMapper.selectCount(Wrappers.<SysDeptCsm>lambdaQuery()
                            .eq(SysDeptCsm::getDeptId, deptId)
                            .eq(SysDeptCsm::getUserId, userId));
                    if (count <= 0) {
                        SysUser user = sysUserMapper.selectById(userId);
                        SysDeptCsm insert = new SysDeptCsm();
                        insert.setDeptId(deptId);
                        insert.setUserId(userId);
                        insert.setCsmName(user.getRealname());
                        SysDeptCsm sysDeptCsm = deptCsmMap.get(userId);
                        if (sysDeptCsm == null) {
                            insert.setQrCodeId(" ");
                            insert.setCorpId(" ");
                        } else {
                            // 复制该csm的原有配置信息
                            insert.setQrCodeId(StrUtil.isNotBlank(sysDeptCsm.getQrCodeId()) ? sysDeptCsm.getQrCodeId() : " ");
                            insert.setCorpId(StrUtil.isNotBlank(sysDeptCsm.getCorpId()) ? sysDeptCsm.getQrCodeId() : " ");
                        }
                        sysDeptCsmMapper.insert(insert);
                    }
                });
            });
        }

        return R.ok(Boolean.TRUE);
    }

    /**
     * 保存部门规章制度
     *
     * @param list 列表
     * @return {@link R}<{@link ?}>
     */
    @SysLog("保存部门规章制度")
    @PostMapping("/dept-regulations/save")
    public R<?> saveDeptRegulations(@RequestBody List<WmDeptRegulations> list) {
        if (CollUtil.isEmpty(list)) {
            throw new RuntimeException("必填参数为空");
        }

        Integer deptId = SecurityUtils.getUser().getDeptId();

        // 先删除
        wmDeptRegulationsService.remove(Wrappers.<WmDeptRegulations>lambdaQuery()
                .eq(WmDeptRegulations::getDeptId, deptId));
        // 后保存
        list.forEach(regulations -> {
            regulations.setDeptId(deptId);
            regulations.setDelFlag(DelEnum.NOT_DEL.getVal());
        });

        return R.ok(wmDeptRegulationsService.saveBatch(list));
    }

    /**
     * 更新企业截止时间
     *
     * @param params 参数
     * @return {@link R}<{@link ?}> 结果
     */
    @SuppressWarnings("unchecked")
    @SysLog("更新企业截止时间")
    @PostMapping("/update-expiry-date")
    public R<?> updateExpiryDate(@RequestBody Map<String, Object> params) {
        List<Integer> deptIds = (List<Integer>) params.get("deptIds");
        String expiryDate = (String) params.get("expiryDate");
        if (CollUtil.isEmpty(deptIds)) {
            return R.failed("deptIds必填");
        }

        for (Integer deptId : deptIds) {
            SysDept update = new SysDept();
            update.setDeptId(deptId);
            if (expiryDate != null) {
                update.setExpiryDate(LocalDate.parse(expiryDate, DateTimeFormatter.ofPattern("yyyy-MM-dd")));
            } else {
                // 默认延长一年
                SysDept dept = sysDeptService.getById(deptId);
                if (dept.getExpiryDate() != null) {
                    update.setExpiryDate(dept.getExpiryDate().plusYears(1L));
                } else {
                    update.setExpiryDate(LocalDate.now().plusYears(1L));
                }
            }
            sysDeptService.updateById(update);
        }

        return R.ok();
    }
}
