package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.model.input.DeptSubLimitAmountInput;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptSub;
import com.qunzhixinxi.hnqz.admin.service.SysDeptSubService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 机构支付渠道管理
 *
 * @author ryz
 * @date 2021-04-19 16:02:30
 */
@RestController
@AllArgsConstructor
@RequestMapping("/sysdeptsub")
public class SysDeptSubController {

    private final SysDeptSubService sysDeptSubService;


    /**
     * 修改限额
     *
     * @param input
     * @return
     */
    @PostMapping("/updateSubLimitAmount")
    public R updateSubLimitAmount(@RequestBody DeptSubLimitAmountInput input) {
        if (CollectionUtils.isEmpty(input.getSubList())) {
            return R.failed("参数异常");
        }
        return sysDeptSubService.updateSubLimitAmount(input);
    }


    /**
     * 分页查询
     *
     * @param page       分页对象
     * @param sysDeptSub 部门管理
     * @return
     */
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('admin_sysdeptsub_view')")
    public R getSysDeptSubPage(Page page, SysDeptSub sysDeptSub) {
        return R.ok(sysDeptSubService.page(page, Wrappers.query(sysDeptSub)));
    }


    /**
     * 通过id查询部门管理
     *
     * @param deptId id
     * @return R
     */
    @GetMapping("/{deptId}")
    @PreAuthorize("@pms.hasPermission('admin_sysdeptsub_view')")
    public R getById(@PathVariable("deptId") Integer deptId) {
        return R.ok(sysDeptSubService.getById(deptId));
    }

    /**
     * 新增部门管理
     *
     * @param sysDeptSub 部门管理
     * @return R
     */
    @SysLog("新增部门管理")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('admin_sysdeptsub_add')")
    public R save(@RequestBody SysDeptSub sysDeptSub) {
        return R.ok(sysDeptSubService.save(sysDeptSub));
    }

    /**
     * 修改部门管理
     *
     * @param sysDeptSub 部门管理
     * @return R
     */
    @SysLog("修改部门管理")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('admin_sysdeptsub_edit')")
    public R updateById(@RequestBody SysDeptSub sysDeptSub) {
        return R.ok(sysDeptSubService.updateById(sysDeptSub));
    }

    /**
     * 通过id删除部门管理
     *
     * @param deptId id
     * @return R
     */
    @SysLog("通过id删除部门管理")
    @DeleteMapping("/{deptId}")
    @PreAuthorize("@pms.hasPermission('admin_sysdeptsub_del')")
    public R removeById(@PathVariable Integer deptId) {
        return R.ok(sysDeptSubService.removeById(deptId));
    }

}
