package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.pig4cloud.plugin.excel.annotation.RequestExcel;
import com.pig4cloud.plugin.excel.annotation.ResponseExcel;
import com.pig4cloud.plugin.excel.annotation.Sheet;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.dto.SysImplementPlanDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.SysImplementPlanDetailsDTO;
import com.qunzhixinxi.hnqz.admin.api.model.excel.PlanDetailsExportModel;
import com.qunzhixinxi.hnqz.admin.api.model.excel.PlanInfoExportModel;
import com.qunzhixinxi.hnqz.admin.api.vo.SysImplementPlanVO;
import com.qunzhixinxi.hnqz.admin.manager.SysImplementPlanManager;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 执行计划前端控制器
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/19 11:42
 */
@RestController
@AllArgsConstructor
public class SysImplementPlanController {

    private final SysImplementPlanManager implementPlanManager;


    @SysLog("创建执行计划(批量)")
    @PostMapping("/plan/batch/create")
    public R<List<Map<String, String>>> createBatch(@RequestExcel(ignoreEmptyRow = true) List<SysImplementPlanDTO.OnBatchCreate> resource) {

        return R.ok(implementPlanManager.createBatch(resource, SecurityUtils.getUser()));

    }


    @SysLog("更新执行计划")
    @PostMapping("/plan/update")
    public R<Boolean> updatePlan(@Validated @RequestBody SysImplementPlanDTO.OnUpdate resource) {
        return R.ok(implementPlanManager.updatePlan(resource, SecurityUtils.getUser()));
    }


    @ResponseExcel(name = "planInfoExport", sheets = {@Sheet(sheetName = "目标（计划）信息")})
    @GetMapping(value = "/plan/info/export")
    public List<PlanInfoExportModel> exportPlan(@Validated SysImplementPlanDTO.OnPage query) {
        return implementPlanManager.exportPlan(query, SecurityUtils.getUser());
    }


    @SysLog(value = "创建执行计划")
    @PostMapping(value = "/plan/create")
    public R<Boolean> createPlan(@Validated @RequestBody SysImplementPlanDTO.OnCreate resource) {

        return R.ok(implementPlanManager.createPlan(resource, SecurityUtils.getUser()));

    }

    @GetMapping(value = "/plan/info/{plan_id}")
    public R<SysImplementPlanVO.ToList> getPlanById(@PathVariable(value = "plan_id") Long planId) {
        return R.ok(implementPlanManager.findById(planId));
    }


    /**
     * 获取执行计划分页（web使用）
     *
     * @param query 分页条件
     * @return 分页结果
     */
    @GetMapping(value = "/plan/page")
    public R<Page<?>> pagePlans(@Validated SysImplementPlanDTO.OnPage query) {

        return R.ok(implementPlanManager.pagePlans(query, SecurityUtils.getUser()));
    }

    /**
     * 获取执行计划分页（web使用）
     *
     * @param query 分页条件
     * @return 分页结果
     */
    @PostMapping(value = "/plan/app/page")
    public R<Page<?>> pagePlans1(@Validated @RequestBody SysImplementPlanDTO.OnPage query) {

        return R.ok(implementPlanManager.pagePlans(query, SecurityUtils.getUser()));
    }

    @SuppressWarnings("unchecked")
    @GetMapping(value = "/plan/children/list")
    public R<List<Tree<Integer>>> listPlanChildren(@Validated SysImplementPlanDTO.OnPage query) {

        Page<?> page = implementPlanManager.pagePlans(query, SecurityUtils.getUser());

        if (CollUtil.isEmpty(page.getRecords())) {
            return R.ok(Collections.emptyList());
        }

        List<Tree<Integer>> records = (List<Tree<Integer>>) page.getRecords();

        for (Tree<Integer> record : records) {
            Integer id = record.getId();
            if (!Objects.equals(id, query.getPlanId())) {
                continue;
            }

            List<Tree<Integer>> children = record.getChildren();
            return R.ok(children);

        }

        return R.ok(Collections.emptyList());
    }

    @GetMapping(value = "/plan/list")
    public R<List<SysImplementPlanVO.ToList>> listPlans(@RequestParam(value = "planType", required = false) UpmsType.ImplementPlanType planType,
                                                        @RequestParam(value = "pkgId") String pkgId,
                                                        @RequestParam(value = "planState", required = false, defaultValue = "PASSED") UpmsState.ImplementPlanState planState) {
        return R.ok(implementPlanManager.listPlans(planState, planType, pkgId, SecurityUtils.getUser()));
    }


    @SysLog(value = "审核计划")
    @PostMapping(value = "/plan/check")
    public R<Boolean> checkPlans(@Validated @RequestBody SysImplementPlanDTO.OnCheck resource) {

        return R.ok(implementPlanManager.checkPlan(resource, SecurityUtils.getUser()));
    }


    @SysLog(value = "审核计划（一键审核）")
    @PostMapping(value = "/plan/check/batch")
    public R<Boolean> batchCheckPlans(@Validated @RequestBody SysImplementPlanDTO.OnBatchCheck resource) {

        return R.ok(implementPlanManager.batchCheckPlans(resource, SecurityUtils.getUser()));
    }

    @SysLog(value = "批量审核计划")
    @PostMapping(value = "/plan/info/check/batch")
    public R<List<Map<String, String>>> batchCheckPlansInfo(@RequestExcel(ignoreEmptyRow = true) List<SysImplementPlanDTO.OnBatchCheckInfo> resource) {


        return R.ok(implementPlanManager.batchCheckPlansInfo(resource, SecurityUtils.getUser()));
    }


    @SysLog(value = "创建计划详情")
    @PostMapping(value = "/plan/details/create")
    public R<Boolean> createDetails(@Validated @RequestBody SysImplementPlanDetailsDTO.OnCreate resource) {
        return R.ok(implementPlanManager.createPlanDetails(resource, SecurityUtils.getUser()));
    }

    @SysLog(value = "审核计划详情")
    @PostMapping(value = "/plan/details/check")
    public R<Boolean> checkPlanDetails(@Validated @RequestBody SysImplementPlanDetailsDTO.OnCheck resource) {

        return R.ok(implementPlanManager.checkPlanDetails(resource, SecurityUtils.getUser()));
    }


    @SysLog(value = "审核计划详情(批量)")
    @PostMapping(value = "/plan/details/check/batch")
    public R<List<Map<String, String>>> batchCheckPlanDetails(@RequestExcel(ignoreEmptyRow = true) List<SysImplementPlanDetailsDTO.OnBatchCheck> resource) {


        return R.ok(implementPlanManager.batchCheckPlanDetails(resource, SecurityUtils.getUser()));
    }

    @GetMapping(value = "/plan/details/{details_id}")
    public R<Map<String, Object>> getPlanDetails(@PathVariable(value = "details_id") Integer detailsId) {
        return R.ok(implementPlanManager.getPlanDetails(detailsId));
    }

    @GetMapping(value = "/plan/details/export")
    @ResponseExcel(name = "exportPlanDetails", sheets = {@Sheet(sheetName = "计划详情")})
    public List<PlanDetailsExportModel> exportPlanDetails(@Validated SysImplementPlanDetailsDTO.OnExport query) {

        return implementPlanManager.exportPlanDetails(query, SecurityUtils.getUser());
    }

}
