package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.dto.SysStatisticDto;
import com.qunzhixinxi.hnqz.admin.manager.SysStatisticManager;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * 统计前端控制器
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/26 15:21
 */
@Slf4j
@RestController
@AllArgsConstructor
public class SysStatisticController {

    private final SysStatisticManager statisticManager;


    @GetMapping(value = "/stat/pkg")
    public R<Map<String, Object>> getPkgStat() {
        HnqzUser operator = SecurityUtils.getUser();
        Map<String, Object> res = statisticManager.getPkgStat(operator);
        return R.ok(res);
    }


    @GetMapping(value = "/stat/task")
    public R<Map<String, Object>> getTaskStat() {

        HnqzUser operator = SecurityUtils.getUser();
        Map<String, Object> res = statisticManager.getTaskStat(operator);
        return R.ok(res);
    }

    /**
     * 目标数据统计
     *
     * @param query 查询条件
     * @return 分页结果
     */
    @GetMapping(value = "/stat/plan/tree")
    public R<Page<?>> statPlanTree(@Validated SysStatisticDto.OnPlanTree query) {
        return R.ok(statisticManager.statPlanTree(query, SecurityUtils.getUser()));
    }

    /**
     * 执行包数据统计
     *
     * @param query 查询条件
     * @return 分页结果
     */
    @GetMapping(value = "/stat/pkg/tree")
    public R<Page<?>> statPkgTree(@Validated SysStatisticDto.OnPkgTree query) {
        return R.ok(statisticManager.statPkgTree(query, SecurityUtils.getUser()));
    }

    /**
     * 服务商评分统计
     *
     * @param query 查询条件
     * @return 分页结果
     */
    @GetMapping(value = "/stat/provider/data")
    public R<Page<?>> statProviderData(@Validated SysStatisticDto.OnProviderQccData query) {
        return R.ok(statisticManager.statProviderDataNew(query, SecurityUtils.getUser()));
    }

    /**
     * 服务商员工信息汇总
     *
     * @param query 查询条件
     * @return 分页结果
     */
    @GetMapping(value = "/stat/provider/data/tree")
    public R<Page<?>> statProviderDataTree(SysStatisticDto.OnProviderTree query) {
        return R.ok(statisticManager.statProviderDataTree(query, SecurityUtils.getUser()));
    }

    /**
     * 监督信息统计
     *
     * @param query 查询条件
     * @return 分页结果
     */
    @GetMapping(value = "/stat/supervision/data")
    public R<Page<?>> statSupData(@Validated SysStatisticDto.OnSupData query) {
        return R.ok(statisticManager.statSupData(query, SecurityUtils.getUser()));
    }


}
