package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.nacos.common.utils.Md5Utils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.pig4cloud.plugin.excel.annotation.RequestExcel;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.dto.SysUserDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.UserDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.UserInfo;
import com.qunzhixinxi.hnqz.admin.api.dto.UserPwd;
import com.qunzhixinxi.hnqz.admin.api.entity.SysAreaEntity;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysRole;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUserRole;
import com.qunzhixinxi.hnqz.admin.api.entity.WmPlatformQuizTestResult;
import com.qunzhixinxi.hnqz.admin.api.model.WmQuizResultModel;
import com.qunzhixinxi.hnqz.admin.api.vo.UserVO;
import com.qunzhixinxi.hnqz.admin.api.dto.CommonUserDTO;
import com.qunzhixinxi.hnqz.admin.api.model.input.UserCertificationInput;
import com.qunzhixinxi.hnqz.admin.api.model.excel.CommonUserExcelModel;
import com.qunzhixinxi.hnqz.admin.manager.SysUserManager;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserRoleMapper;
import com.qunzhixinxi.hnqz.admin.service.SysAreaEntityService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysRoleService;
import com.qunzhixinxi.hnqz.admin.service.SysUserAreaService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.admin.service.WmPlatformQuizTestResultService;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.entity.BaseEntity;
import com.qunzhixinxi.hnqz.common.core.exception.BizException;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.annotation.Inner;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants.USER_FORGET_PWD_KEY;

/**
 * 用户管理
 *
 * @version 1.0.0
 * @date 2023-10-26 15:32
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/user")
public class SysUserController {

    private final SysUserService userService;
    private final SysUserMapper sysUserMapper;
    private final SysDeptService sysDeptService;
    private final RedisTemplate<String, Object> redisTemplate;
    private final SysUserRoleMapper sysUserRoleMapper;
    private final SysUserAreaService userAreaService;
    private final WmPlatformQuizTestResultService platformQuizTestResultService;
    private final SysAreaEntityService areaEntityService;
    private final SysUserManager userManager;
    private final SysRoleService roleService;

    private static final List<Integer> SALESMAN_ROLE_ID = ListUtil.of(5, 6);
    private static final List<Integer> ADMIN_ROLE_ID = ListUtil.of(1, 2, 7, 19, 27, 34, 35, 44, 45, 46, 47, 48, 49);


    /**
     * 判断可以登陆的角色
     * 只有单纯是5 和 6的角色才不能登陆
     *
     * @param roles 角色合集
     * @return 判断结果
     */
    private boolean isZbOrDbUsers(Integer[] roles) {

        if (ArrayUtil.isEmpty(roles)) {
            return false;
        }

        Set<Integer> roleIds = Arrays.stream(roles).collect(Collectors.toSet());

        if (roleIds.size() == 1) {
            return !CollUtil.containsAny(roleIds, SALESMAN_ROLE_ID);
        }

        if (roleIds.size() == 2) {
            return !CollUtil.containsAll(roleIds, SALESMAN_ROLE_ID);
        }

        return true;
    }

    /**
     * 判断可以登录小程序的角色
     * 只有单纯是
     * 1   系统管理员,
     * 2   平台管理员,
     * 7   财务管理员,
     * 19  csm管理员,
     * 27  平台财务管理员,
     * 34  财务复核管理员,
     * 35  csm总负责人,
     * 44  安全管理员,
     * 45  审计管理员,
     * 46  （无）,
     * 47  事业部医药代表备案管理员,
     * 48  平台监督员,
     * 49  生产企业监督员
     * 的角色才不能登陆
     *
     * @param roles 角色合集
     * @return 判断结果
     */
    private boolean allowAdminRole2LoginApp(Integer[] roles) {

        if (ArrayUtil.isEmpty(roles)) {
            return false;
        }

        Set<Integer> roleIds = Arrays.stream(roles).collect(Collectors.toSet());


        return !CollUtil.containsAny(roleIds, ADMIN_ROLE_ID);
    }

    /**
     * 用户管理-认证
     *
     * @param input 输入信息
     * @return R
     */
    @SysLog("用户管理-认证")
    @GetMapping("/getAuthInfo")
    public R<?> getAuthInfo(UserCertificationInput input) {
        // 参数校验
        return userService.getAuthInfo(input);
    }


    /**
     * 获取指定用户全部信息
     *
     * @return 用户信息
     */
    @Inner
    @GetMapping("/info/{username}")
    public R<?> info(@PathVariable String username) {

        boolean b = username.endsWith("@mp");
        if (b) {
            username = username.split("@")[0];
        }

        // 根据用户名查询用户时，过滤掉已删除和已禁用的用户
        List<SysUser> user = userService.list(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username)
                .eq(SysUser::getDelFlag, 0).eq(SysUser::getLockFlag, 0));

        if (user == null) {
            return R.failed(null, String.format("用户信息为空 %s", username));
        }

        if (user.size() == 1) {
            UserInfo userInfo = userService.findUserInfo(user.get(0));

            boolean zbOrDbUsers = isZbOrDbUsers(userInfo.getRoles());
            // 校验用户角色是否包含众包用户和医药代表用户
            if ((b && !zbOrDbUsers) || (!b && zbOrDbUsers)) {
                // 角色是小程序角色，则直接返回
                return R.ok(userInfo);
            } else {
                return R.failed(null, String.format("用户信息错误 %s", username));
            }
        } else {
            // 小程序多账号情况
            if (b) {
                for (SysUser u : user) {
                    UserInfo userInfo = userService.findUserInfo(u);
                    if (!isZbOrDbUsers(userInfo.getRoles())) {
                        // 角色是小程序角色，则直接返回
                        return R.ok(userInfo);
                    }
                }
            }
            String password = redisTemplate.opsForValue().get(username) + "";
            if (null != password) {
                for (SysUser sysUser : user) {
                    UserInfo userInfo = userService.findUserInfo(sysUser);
                    if (Md5Utils.getMD5(password.getBytes()).equals(sysUser.getW1())) {
                        if (isZbOrDbUsers(userInfo.getRoles())) {
                            redisTemplate.delete(username);
                            return R.ok(userInfo);
                        }
                    }
                }
            }
        }
        return R.failed(null, String.format("用户信息为空 %s", username));
    }

    /**
     * 获取指定用户全部信息
     *
     * @return 用户信息
     */
    @Inner
    @GetMapping("/info/inner")
    R<UserInfo> info1(@RequestParam("username") String username,
                      @RequestParam("appLogin") Boolean isApp) {

        // 根据用户名查询用户时，过滤掉已删除和已禁用的用户
        List<SysUser> users = userService.list(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username)
                .eq(SysUser::getDelFlag, 0).eq(SysUser::getLockFlag, 0));

        if (CollUtil.isEmpty(users)) {
            return R.failed(null, String.format("用户信息为空 %s", username));
        }

        // 移动端登录
        if (isApp) {
            return appUserInfo(users);
        }
        // pc端登录
        else {
            return webUserInfo(username, users);
        }
    }

    // pc端登录
    private R<UserInfo> webUserInfo(String username, List<SysUser> users) {
        // 单一账号情况
        if (users.size() == 1) {
            SysUser sysUser = users.get(0);
            UserInfo userInfo = userService.findUserInfo(sysUser);
            // 校验用户角色是否包含众包用户和医药代表用户, 兼职和全职不能登录pc
            return isZbOrDbUsers(userInfo.getRoles()) ? R.ok(userInfo) : R.failed(null, String.format("用户信息错误 %s", username));
        }
        // 多账号情况
        else {
            String password = (String) redisTemplate.opsForValue().get(username);
            if (StrUtil.isNotBlank(password)) {
                for (SysUser sysUser : users) {
                    UserInfo userInfo = userService.findUserInfo(sysUser);
                    if (Md5Utils.getMD5(password.getBytes()).equals(sysUser.getW1())) {
                        if (isZbOrDbUsers(userInfo.getRoles())) {
                            redisTemplate.delete(username);
                            return R.ok(userInfo);
                        }
                    }
                }
            }
        }
        return R.failed(null, String.format("用户信息为空 %s", username));

    }

    // 移动端登录
    private R<UserInfo> appUserInfo(List<SysUser> users) {

        SysUser sysUser = users.get(0);
        UserInfo userInfo = userService.findUserInfo(sysUser);
        return allowAdminRole2LoginApp(userInfo.getRoles()) ? R.ok(userInfo) : R.failed(null, String.format("用户信息错误或角色不支持 %s", sysUser.getUsername()));

    }

    /**
     * 通过ID查询用户信息
     *
     * @param id ID
     * @return 用户信息
     */
    @GetMapping("/{id}")
    public R<?> user(@PathVariable Integer id) {
        return R.ok(userService.selectUserVoById(id));
    }

    /**
     * 根据用户名查询用户信息
     *
     * @param username 用户名
     * @return 用户详情
     */
    @GetMapping("/details/{username}")
    public R<SysUser> user(@PathVariable String username) {

        SysUser one = userService.getOne(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getUsername, username));
        if (one != null) {
            List<Long> areaCodes = userAreaService.listUserAreas(Long.valueOf(one.getUserId()));
            one.setAreaCodes(areaCodes);
        }

        return R.ok(one);
    }

    /**
     * 根据用户名查询用户信息
     *
     * @param username 用户名
     * @return 用户信息
     */
    @GetMapping("/getDetails")
    public R<SysUser> getuser(@RequestParam(value = "deptId") Integer deptId,
                              @RequestParam(value = "username") String username) {

        SysUser one = userService.getOne(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getUsername, username).eq(SysUser::getDeptId, deptId));
        if (one != null) {
            List<Long> areaCodes = userAreaService.listUserAreas(Long.valueOf(one.getUserId()));
            one.setAreaCodes(areaCodes);
        }

        return R.ok(one);
    }

    /**
     * 删除用户信息
     *
     * @param id ID
     * @return R
     */
    @SysLog("删除用户信息")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('sys_user_del')")
    public R<Boolean> userDel(@PathVariable Integer id) {
        SysUser sysUser = userService.getById(id);
        return R.ok(userService.deleteUserById(sysUser));
    }

    /**
     * 忘记密码（更新）
     *
     * @param userPwd 用户信息
     * @return success/false
     */
    @Inner(value = false)
    @SysLog("忘记密码（更新）")
    @PostMapping("/upd")
    public R<?> updatePwd(@RequestBody @Validated UserPwd userPwd) {

        // 校验验证码是否准确
        final String key = USER_FORGET_PWD_KEY + userPwd.getUserName();
        if (Boolean.TRUE.equals(redisTemplate.hasKey(key))) {
            Object o = redisTemplate.opsForValue().get(key);
            if (Objects.isNull(o)) {
                throw new RuntimeException("验证码不存在或者已过期");
            }
            String code = (String) o;

            if (!StrUtil.equals(code, userPwd.getCode())) {
                throw new RuntimeException("验证码错误");
            }

        } else {
            throw new RuntimeException("验证码不存在或者已过期");
        }


        return userService.updatePwd(userPwd);
    }

    /**
     * 更新密码
     *
     * @param userPwd 用户信息
     * @return success/false
     */
    @SysLog("更新密码")
    @PostMapping("/pwd/renew")
    public R<?> renewPwd(@RequestBody @Validated(BaseEntity.Renew.class) UserPwd userPwd) {

        userPwd.setUserName(SecurityUtils.getUser().getUsername());

        return userService.updatePwd(userPwd);
    }

    /**
     * 暂不修改密码
     *
     * @return success/false
     */
    @SysLog("暂不修改密码")
    @PostMapping("/pwd/exceeding")
    public R<?> exceedingPwdChangeTime() {

        return R.ok(userService.exceedingPwdChangeTime(SecurityUtils.getUser()));
    }


    /**
     * 添加管理员
     *
     * @param userDto 管理员信息
     * @return success/false
     */
    @SysLog("添加管理员信息")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('sys_user_add')")
    public R<Boolean> createAdministrator(@Validated(value = BaseEntity.Create.class) @RequestBody UserDTO userDto) {

        if (CollUtil.containsAny(userDto.getRole(), SALESMAN_ROLE_ID)) {
            return R.failed("当前功能仅支持添加非业务员角色");
        }

        // 姓名去除空格（包括字符之间的所有空格都会去除）
        userDto.setRealname(StrUtil.cleanBlank(userDto.getRealname()));

        // SysDept sysDept = sysDeptService.getById(userDto.getDeptId());

        List<SysUser> queryUserByPhoneList = userService.list(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getUsername, userDto.getUsername()));
        if (CollUtil.isNotEmpty(queryUserByPhoneList)) {
            SysUser queryUserByPhone = queryUserByPhoneList.get(0);
            if (!StrUtil.equals(queryUserByPhone.getRealname(), userDto.getRealname())) {
                return R.failed(String.format("创建失败，已存在该手机号，姓名为%s，请核对信息", queryUserByPhone.getRealname()));
            }
        }


//         if (userDto.getRole().contains(2)) {
//             UserDTO userDTO = new UserDTO();
//             userDTO.setUsername(userDto.getUsername());
//             userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
//             List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
//
//             if (CollUtil.isNotEmpty(userDTOList)) {
//                 for (UserVO userVO : userDTOList) {
//                     if (null != userVO.getRoleList()) {
//                         for (SysRole sysRole : userVO.getRoleList()) {
//                             if (sysRole.getRoleId() == 3) {
//                                 return R.failed("用户已存在平台管理员角色,不能同时为药企管理员");
//                             }
//                             if (sysRole.getRoleId() == 4) {
//                                 return R.failed("用户已存在平台管理员角色,不能同时为CSO管理员");
//                             }
//                             if (sysRole.getRoleId() == 5) {
//                                 return R.failed("用户已存在平台管理员角色,不能同时为全职学术推广员");
//                             }
//                             if (sysRole.getRoleId() == 6) {
//                                 return R.failed("用户已存在平台管理员角色,不能同时为兼职学术推广员");
//                             }
//                             if (sysRole.getRoleId() == 7) {
//                                 return R.failed("用户已存在平台管理员角色,不能同时为财务管理员");
//                             }
//                             if (sysRole.getRoleId() == 31) {
//                                 return R.failed("用户已存在平台管理员角色,不能同时为招商经理");
//                             }
//                         }
//                     }
//                 }
//             }
//
//         } else if (userDto.getRole().contains(3)) {
//             UserDTO userDTO = new UserDTO();
//             userDTO.setUsername(userDto.getUsername());
//             userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
//             List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
//             if (CollUtil.isNotEmpty(userDTOList)) {
//                 for (UserVO userVO : userDTOList) {
//                     if (null != userVO.getRoleList()) {
//                         for (SysRole sysRole : userVO.getRoleList()) {
//                             if (sysRole.getRoleId() == 2) {
//                                 return R.failed("用户已存在药企管理员角色,不能同时为平台管理员");
//                             }
//
//                             if (sysRole.getRoleId() == 3) {
//                                 return R.failed("用户已存在药企管理员角色,不能同时为药企管理员");
//                             }
//
//                             if (sysRole.getRoleId() == 4) {
//                                 return R.failed("用户已存在药企管理员角色,不能同时为CSO管理员");
//                             }
// //							SysDept sysDept = sysDeptService.getById(userVO.getDeptId());
//                             if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
//                                 if (sysRole.getRoleId() == 5) {
//                                     return R.failed("用户已存在CSO关系角色,不能同时为药企管理员");
//                                 }
//                             }
//                         }
//                     }
//                 }
//             }
//
//         } else if (userDto.getRole().contains(4) || userDto.getRole().contains(37)) {
//             UserDTO userDTO = new UserDTO();
//             userDTO.setUsername(userDto.getUsername());
//             userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
//             List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
//             if (CollUtil.isNotEmpty(userDTOList)) {
//                 for (UserVO userVO : userDTOList) {
//                     if (null != userVO.getRoleList()) {
//                         for (SysRole sysRole : userVO.getRoleList()) {
//                             if (sysRole.getRoleId() == 2) {
//                                 return R.failed("用户已存在平台管理员角色,不能同时为CSO管理员");
//                             }
//                             if (sysRole.getRoleId() == 4) {
//                                 return R.failed("用户已存在CSO管理员角色,不能为其他机构CSO管理员");
//                             }
//                             if (sysRole.getRoleId() == 9) {
//                                 return R.failed("用户已存在CRO管理员,不能同时为CSO管理员");
//                             }
//
//                             if (sysRole.getRoleId() == 3) {
//                                 return R.failed("用户已存在药企管理员角色,不能同时为CSO管理员");
//                             }
//                             if (sysRole.getRoleId() == 5) {
//                                 SysUserRole sysUserRole = new SysUserRole();
//                                 sysUserRole.setUserId(userVO.getUserId());
//                                 List<SysUserRole> sysUserRoleList = sysUserRoleMapper.selectById(sysUserRole);
//                                 for (SysUserRole userRole : sysUserRoleList) {
//                                     SysUser sysUser = userService.getById(userRole.getUserId());
//                                     if (!sysUser.getDeptId().equals(userDto.getDeptId())) {
//                                         return R.failed("用户已存在全职学术推广员角色,不能为其他机构CSO管理员");
//                                     }
//                                 }
//                             }
//                             if (sysRole.getRoleId() == 31) {
//                                 SysUserRole sysUserRole = new SysUserRole();
//                                 sysUserRole.setUserId(userVO.getUserId());
//                                 List<SysUserRole> sysUserRoleList = sysUserRoleMapper.selectById(sysUserRole);
//                                 for (SysUserRole userRole : sysUserRoleList) {
//                                     SysUser sysUser = userService.getById(userRole.getUserId());
//                                     if (!sysUser.getDeptId().equals(userDto.getDeptId())) {
//                                         return R.failed("用户已存在招商经理角色,不能为其他机构CSO管理员");
//                                     }
//                                 }
//                             }
//                         }
//                     }
//                 }
//             }
//
//         } else if (userDto.getRole().contains(7)) {
//             UserDTO userDTO = new UserDTO();
//             userDTO.setUsername(userDto.getUsername());
//             userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
//             List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
//             if (CollUtil.isNotEmpty(userDTOList)) {
//                 for (UserVO userVO : userDTOList) {
//                     if (null != userVO.getRoleList()) {
//                         for (SysRole sysRole : userVO.getRoleList()) {
//                             if (sysRole.getRoleId() == 2) {
//                                 return R.failed("用户已存在平台管理员,不能同时为财务管理员");
//                             }
//                             if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
//                                 if (sysRole.getRoleId() == 3) {
//                                     return R.failed("用户已存在药企管理员,不能同时为CSO机构财务管理员");
//                                 }
//                             }
//                         }
//                     }
//                 }
//             }
//
//         } else if (userDto.getRole().contains(9)) {
//             UserDTO userDTO = new UserDTO();
//             userDTO.setUsername(userDto.getUsername());
//             userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
//             List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
//             if (CollUtil.isNotEmpty(userDTOList)) {
//                 for (UserVO userVO : userDTOList) {
//                     if (null != userVO.getRoleList()) {
//                         for (SysRole sysRole : userVO.getRoleList()) {
//                             if (sysRole.getRoleId() == 2) {
//                                 return R.failed("用户已存在平台管理员,不能同时为CRO管理员");
//                             }
//                             if (sysRole.getRoleId() == 3) {
//                                 return R.failed("用户已存在药企管理员,不能同时为CRO管理员");
//                             }
//                             if (sysRole.getRoleId() == 4) {
//                                 return R.failed("用户已存在CSO管理员角色,不能同时为CRO管理员");
//                             }
//                             if (sysRole.getRoleId() == 9) {
//                                 return R.failed("用户已存在CRO管理员角色");
//                             }
//                         }
//                     }
//                 }
//             }
//
//         } else if (userDto.getRole().contains(31)) {
//             UserDTO userDTO = new UserDTO();
//             userDTO.setUsername(userDto.getUsername());
//             userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
//             List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
//             if (CollUtil.isNotEmpty(userDTOList)) {
//                 for (UserVO userVO : userDTOList) {
//                     for (SysRole sysRole : userVO.getRoleList()) {
//                         if (sysRole.getRoleId() == 2) {
//                             return R.failed("用户已存在平台管理员,不能同时为招商经理");
//                         }
//                         if (sysRole.getRoleId() == 3) {
//                             if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
//                                 return R.failed("用户已存在药企管理员,不能为CSO的招商经理");
//                             }
//                         }
//                         if (sysRole.getRoleId() == 4) {
//                             if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
//                                 return R.failed("用户已存在CSO管理员,不能为其他CSO的招商经理");
//                             }
//                         }
//
//                     }
//                 }
//             }
//         }
//
//         if (userDto.getRole().contains(3)) {
//             if (sysDept.getLevel() != 2) {
//                 return R.failed("角色选为药企管理员时,组织架构必须为药企");
//             }
//         }
//         // }
//         if (!userDto.getDeptId().equals(SecurityUtils.getUser().getDeptId())) {
//             if (userDto.getRole().contains(5) || userDto.getRole().contains(6)) {
//                 return R.failed("不能跨机构维护学术推广员");
//             }
//             if (userDto.getRole().contains(31)) {
//                 return R.failed("不能跨机构维护招商经理");
//             }
//         }
        SysUser query = new SysUser();
        query.setUsername(userDto.getUsername());
        query.setDeptId(userDto.getDeptId());
        Integer count = sysUserMapper.selectCount(Wrappers.query(query));

        if (count != 0) {
            return R.failed("当前机构已存在该账号");
        }


        // // 校验身份证上的年龄
        // if (StringUtils.isNotEmpty(userDto.getIdCardNumber())) {
        //     // 获取更新操作的用户角色
        //     List<Integer> roles = userDto.getRole();
        //     // 如果角色中包含全职和兼职就需要校验身份证年龄
        //     if (roles.contains(5) || roles.contains(6)) {
        //         int ageByIdCard = IdcardUtil.getAgeByIdCard(userDto.getIdCardNumber());
        //         if (ageByIdCard < 18) {
        //             return R.failed("年龄未满十八");
        //         } else if (ageByIdCard > 65) {
        //             return R.failed("年龄已超过六十五");
        //         }
        //     }
        // }

        userService.saveUser(userDto);
        return R.ok(Boolean.TRUE);
    }


    /**
     * 添加业务员
     *
     * @param resource 业务员信息
     * @return success/false
     */
    @SysLog("添加业务员")
    @PostMapping(value = "/salesman/create")
    @PreAuthorize("@pms.hasPermission('sys_user_add')")
    public R<Boolean> createSalesman(@Validated @RequestBody SysUserDTO.OnSalesmanCreate resource) {

        List<Integer> role = resource.getRole();

        // if (role.size() != 1 || !CollUtil.containsAny(role, SALESMAN_ROLE_ID) || !role.contains(37)) {
        //     return R.failed("当前功能仅支持添加单一人员角色，仅支持全职学术沟通专员或服务商管理员");
        // }

        // 校验服务商
        HnqzUser operator = SecurityUtils.getUser();
        Integer deptId = resource.getDeptId();
        // if (!deptId.equals(operator.getDeptId())) {
        //
        //     return R.failed("不能跨机构维护学术推广员");
        // }


        // 校验用户名
        String username = resource.getUsername().trim();
        boolean availUsername = userService.count(Wrappers.<SysUser>lambdaQuery()
                .eq(SysUser::getDelFlag, CommonConstants.STATUS_NORMAL)
                .eq(SysUser::getUsername, username)) == 0;
        if (!availUsername) {
            return R.failed("用户名已存在");
        }


        // 校验身份证上的年龄
        String idCardNumber = resource.getIdCardNumber().trim();
        // if (CharSequenceUtil.isNotEmpty(idCardNumber)) {
        //     int ageByIdCard = IdcardUtil.getAgeByIdCard(idCardNumber);
        //     if (ageByIdCard < 18 || ageByIdCard > 65) {
        //         return R.failed("业务员年龄应在18～65周岁之间");
        //     }
        //
        // }

        UserDTO userDto = new UserDTO();
        userDto.setRole(role);
        userDto.setDeptId(deptId);
        userDto.setAreaCodes(resource.getAreaCodes());
        userDto.setUsername(username);
        userDto.setIdCardNumber(idCardNumber);
        userDto.setLockFlag(resource.getLockFlag());
        userDto.setPhone(resource.getPhone().trim());
        userDto.setRealname(resource.getRealName());

        Integer userId = userService.saveUser(userDto);

        // 创建备案记录
        SysUserDTO.OnCertificateCreate certResource = new SysUserDTO.OnCertificateCreate();
        certResource.setUserId(userId);
        certResource.setType("REG");
        certResource.setFileIds(resource.getCertFileIds());

        userManager.createUserCertificate(certResource, operator);

        return R.ok(Boolean.TRUE);
    }


    /**
     * 批量导入用户
     *
     * @param commonUserExcelModelList 用户信息列表
     * @return success/false
     */
    @SysLog("批量导入用户")
    @PostMapping(value = "/batch")
    @PreAuthorize("@pms.hasPermission('sys_user_add')")
    public R<?> batchUser(@RequestExcel List<CommonUserExcelModel> commonUserExcelModelList) {

        // 参数校验
        if (CollectionUtils.isEmpty(commonUserExcelModelList)) {
            log.error("空报表，不能上传");
            return R.failed("空报表，不能上传");
        }

        Map<String, List<CommonUserDTO>> collect = commonUserExcelModelList.stream()
                .filter(commonUserExcelModel -> StrUtil.isNotBlank(commonUserExcelModel.getUsername())).map(commonUserExcelModel ->
                        BeanUtil.copyProperties(commonUserExcelModel, CommonUserDTO.class)
                ).collect(Collectors.groupingBy(CommonUserDTO::getUsername));


        // 实际的处理业务
        Map<String, String> errorMap = userService.batchByExcel(collect);

        // 返回上传结果
        Map<String, Object> result = new HashMap<>(3);
        if (CollectionUtil.isEmpty(errorMap)) {
            result.put("code", "SUCCESS");
            result.put("message", "上传成功");
            result.put("data", null);
            return R.ok(result);
        } else {
            log.error("上传数据错误，{}", JSONUtil.toJsonStr(errorMap));
            result.put("code", "FAILURE");
            result.put("message", "上传数据错误");
            result.put("data", errorMap);
            return R.failed(result);

        }
    }

    /**
     * 更新用户信息
     *
     * @param userDto 用户信息
     * @return R
     */
    @SysLog("更新用户信息")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('sys_user_edit')")
    public R<?> updateUser(@Valid @RequestBody UserDTO userDto) {


        // if (CollUtil.containsAny(userDto.getRole(), SALESMAN_ROLE_ID)) {
        //     return R.failed("当前业务规定不支持修改（全职/兼职）业务员信息");
        // }

        // SysDept sysDept = sysDeptService.getById(userDto.getDeptId());

        if (null != userDto.getUserId()) {

            SysUser oldData = userService.getById(userDto.getUserId());

            // if (StrUtil.isNotBlank(oldData.getRealname()) && StrUtil.isNotBlank(userDto.getRealname()) && !StrUtil.equals(oldData.getRealname(), userDto.getRealname())) {
            //     return R.failed("用户不能修改姓名");
            // }
            if (StrUtil.isNotBlank(oldData.getIdCardNumber()) && StrUtil.isNotBlank(userDto.getIdCardNumber()) && !StrUtil.equals(oldData.getIdCardNumber(), userDto.getIdCardNumber())) {
                return R.failed("用户不能修改身份证号");
            }

        }


        // if (SecurityUtils.getRoles().contains(2) || SecurityUtils.getRoles().contains(1)) {
        //     if (userDto.getRole().contains(2)) {
        //         UserDTO userDTO = new UserDTO();
        //         userDTO.setPlatId(userDto.getUsername());
        //         userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
        //         List<UserVO> userDTOList = userService.getUserVoByPlatId(userDTO);
        //         if (CollUtil.isNotEmpty(userDTOList)) {
        //             for (UserVO userVO : userDTOList) {
        //                 if (null != userVO.getRoleList()) {
        //                     for (SysRole sysRole : userVO.getRoleList()) {
        //                         if (sysRole.getRoleId() == 3) {
        //                             return R.failed("用户不能同时为平台管理员和药企管理员");
        //                         }
        //                     }
        //                 }
        //             }
        //         }
        //
        //     } else if (userDto.getRole().contains(3)) {
        //         UserDTO userDTO = new UserDTO();
        //         userDTO.setPlatId(userDto.getUsername());
        //         userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
        //         List<UserVO> userDTOList = userService.getUserVoByPlatId(userDTO);
        //         if (CollUtil.isNotEmpty(userDTOList)) {
        //             for (UserVO userVO : userDTOList) {
        //                 if (null != userVO.getRoleList()) {
        //                     for (SysRole sysRole : userVO.getRoleList()) {
        //                         if (sysRole.getRoleId() == 2) {
        //                             return R.failed("用户不能同时为平台管理员和药企管理员");
        //                         }
        //
        //                         if (userDto.getDeptId().equals(userVO.getDeptId())) {
        //                             if (sysRole.getRoleId() == 6) {
        //                                 return R.failed("用户已存在本机构药企管理员,不能同时为兼职学术推广员");
        //                             }
        //                         }
        //                     }
        //                 }
        //             }
        //         }
        //         SysUser query = new SysUser();
        //         query.setUsername(userDto.getUsername());
        //         query.setDeptId(userDto.getDeptId());
        //         SysUser sysUser = sysUserMapper.selectUserByName(query);
        //         if (sysUser != null) {
        //             if (sysDept.getLevel() != 2) {
        //                 return R.failed("角色选为药企管理员时,组织架构必须为药企");
        //             }
        //         }
        //
        //     } else if (userDto.getRole().contains(4) || userDto.getRole().contains(37)) {
        //         UserDTO userDTO = new UserDTO();
        //         userDTO.setPlatId(userDto.getUsername());
        //         userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
        //         List<UserVO> userDTOList = userService.getUserVoByPlatId(userDTO);
        //         if (CollUtil.isNotEmpty(userDTOList)) {
        //             for (UserVO userVO : userDTOList) {
        //                 if (null != userVO.getRoleList()) {
        //                     for (SysRole sysRole : userVO.getRoleList()) {
        //                         if (sysRole.getRoleId() == 2) {
        //                             return R.failed("用户不能同时为平台管理员和经销商管理员");
        //                         }
        //
        //                         if (sysRole.getRoleId() == 3) {
        //                             return R.failed("用户不能同时为药企管理员和经销商管理员");
        //                         }
        //                         if (!userDto.getUserId().equals(userVO.getUserId()) && sysRole.getRoleId() == 4) {
        //                             return R.failed("用户已存在CSO管理员角色,不能维护为其他经销商组织的CSO管理员角色");
        //                         }
        //                         if (!userDto.getUserId().equals(userVO.getUserId()) && sysRole.getRoleId() == 9) {
        //                             return R.failed("用户已存在CRO管理员角色,不能维护为其他经销商组织的CSO管理员角色");
        //                         }
        //                     }
        //                 }
        //             }
        //         }
        //
        //     } else if (userDto.getRole().contains(9)) {
        //         UserDTO userDTO = new UserDTO();
        //         userDTO.setPlatId(userDto.getUsername());
        //         userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
        //         List<UserVO> userDTOList = userService.getUserVoByPlatId(userDTO);
        //         if (CollUtil.isNotEmpty(userDTOList)) {
        //             for (UserVO userVO : userDTOList) {
        //                 if (null != userVO.getRoleList()) {
        //                     for (SysRole sysRole : userVO.getRoleList()) {
        //                         if (sysRole.getRoleId() == 2) {
        //                             return R.failed("用户不能同时为平台管理员和CRO管理员");
        //                         }
        //
        //                         if (sysRole.getRoleId() == 3) {
        //                             return R.failed("用户不能同时为药企管理员和CRO管理员");
        //                         }
        //                         if (!userDto.getUserId().equals(userVO.getUserId()) && sysRole.getRoleId() == 4) {
        //                             return R.failed("用户已存在CSO管理员角色,不能维护为其他经销商组织的CRO管理员角色");
        //                         }
        //                         if (!userDto.getUserId().equals(userVO.getUserId()) && sysRole.getRoleId() == 9) {
        //                             return R.failed("用户已存在CRO管理员角色,不能维护为其他经销商组织的CRO管理员角色");
        //                         }
        //                     }
        //                 }
        //             }
        //         }
        //
        //     } else if (userDto.getRole().contains(6)) {
        //         UserDTO userDTO = new UserDTO();
        //         userDTO.setPlatId(userDto.getUsername());
        //         userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
        //         List<UserVO> userDTOList = userService.getUserVoByPlatId(userDTO);
        //         if (CollUtil.isNotEmpty(userDTOList)) {
        //             for (UserVO userVO : userDTOList) {
        //                 if (null != userVO.getRoleList()) {
        //                     for (SysRole sysRole : userVO.getRoleList()) {
        //                         if (sysRole.getRoleId() == 2) {
        //                             return R.failed("用户不能同时为平台管理员和兼职学术推广员");
        //                         }
        //                         if (userDto.getDeptId().equals(userVO.getDeptId())) {
        //                             if (sysRole.getRoleId() == 3) {
        //                                 return R.failed("用户已存在本药企药企管理员,不能同时为兼职学术推广员");
        //                             }
        //                         }
        //                     }
        //                 }
        //             }
        //         }
        //
        //     } else {
        //         if (!userDto.getDeptId().equals(SecurityUtils.getUser().getDeptId())) {
        //             if (userDto.getRole().contains(5) || userDto.getRole().contains(6)) {
        //                 return R.failed("不能跨机构维护学术推广员");
        //             }
        //         }
        //     }
        //
        // } else {
        //     if (!userDto.getDeptId().equals(SecurityUtils.getUser().getDeptId())) {
        //         if (userDto.getRole().contains(5) || userDto.getRole().contains(6)) {
        //             return R.failed("不能跨机构维护学术推广员");
        //         }
        //     }
        //
        //     Integer count = sysUserMapper.selectCount(Wrappers.<SysUser>lambdaQuery()
        //             .eq(SysUser::getUsername, userDto.getUsername())
        //             .eq(SysUser::getDeptId, userDto.getDeptId())
        //             .eq(SysUser::getDelFlag, DelEnum.NOT_DEL.getVal()));
        //     if (count > 0) {
        //         if (CollUtil.containsAll(userDto.getRole(), ListUtil.of(6, 37))) {
        //             return R.failed("用户不能同时为经销商管理员角色和兼职学术推广员角色");
        //         }
        //         if (userDto.getRole().contains(7) && userDto.getRole().contains(6)) {
        //             return R.failed("用户不能同时为财务管理员角色和兼职学术推广员角色");
        //         }
        //         if (userDto.getRole().contains(3)) {
        //             if (sysDept.getLevel() != 2) {
        //                 return R.failed("角色选为药企管理员时,组织架构必须为药企");
        //             }
        //         }
        //     }
        // }
        //
        // // 校验身份证上的年龄
        // if (StringUtils.isNotEmpty(userDto.getIdCardNumber())) {
        //     // 获取更新操作的用户角色
        //     List<Integer> roles = userDto.getRole();
        //     // 如果角色中包含全职和兼职就需要校验身份证年龄
        //     if (roles.contains(5) || roles.contains(6)) {
        //         int ageByIdCard = IdcardUtil.getAgeByIdCard(userDto.getIdCardNumber());
        //         if (ageByIdCard < 18) {
        //             return R.failed("年龄未满十八");
        //         } else if (ageByIdCard > 65) {
        //             return R.failed("年龄已超过六十五");
        //         }
        //     }
        // }

        return R.ok(userService.updateUser(userDto));
    }

    /**
     * 分页查询用户
     *
     * @param page    参数集
     * @param userDTO 查询参数列表
     * @return 用户集合
     */
    @GetMapping("/page")
    public R<Page<UserVO>> getUserPage(Page<UserVO> page, UserDTO userDTO) {

        return R.ok(userService.getUsersWithRolePageNoLock(page, userDTO));
    }

    /**
     * 分页查询用户
     *
     * @param userDTO 查询参数列表
     * @return 用户集合
     */
    // @PostMapping(value = "/selectUserList")
    public R<?> selectUserList(@RequestBody UserDTO userDTO) {

        HnqzUser user = SecurityUtils.getUser();

        Page<UserVO> userVosPage = userService.selectUserList(new Page<>(userDTO.getCurrent(), userDTO.getSize()), userDTO, user);

        Set<Integer> userIds = userVosPage.getRecords().stream().map(UserVO::getUserId).collect(Collectors.toSet());

        if (CollUtil.isNotEmpty(userIds)) {

            List<WmPlatformQuizTestResult> quizTestResults = platformQuizTestResultService.listPltQuizResults(userIds);

            LocalDate today = LocalDate.now();
            Map<Integer, List<WmPlatformQuizTestResult>> quizTestResult = quizTestResults.stream().filter(res -> res.getExpiry().isAfter(today)).collect(Collectors.groupingBy(WmPlatformQuizTestResult::getUserId));

            userVosPage.getRecords().forEach(vo -> {
                List<WmPlatformQuizTestResult> testResults = quizTestResult.get(vo.getUserId());
                List<WmQuizResultModel> pure = null;
                if (CollUtil.isNotEmpty(testResults)) {
                    Map<Integer, List<WmPlatformQuizTestResult>> collect = testResults.stream().collect(Collectors.groupingBy(WmPlatformQuizTestResult::getQuizId));
                    pure = new ArrayList<>(collect.size());

                    for (Map.Entry<Integer, List<WmPlatformQuizTestResult>> entry : collect.entrySet()) {
                        pure.add(entry.getValue().get(0).getQuizResult());
                    }
                }

                vo.setQuizResult(CollUtil.isEmpty(pure) ? Collections.emptyList() : pure);

                // 所属区域
                List<Long> areaCodes = userAreaService.listUserAreas(Long.valueOf(vo.getUserId()));
                vo.setAreaCodes(areaCodes);

            });
        }

        //
        // Map<Integer, List<SysCsmUserRelation>> csmParentUserMap = new HashMap<>();
        // if (CollUtil.isNotEmpty(userIds)) {
        //     // 上级csm
        //     List<SysCsmUserRelation> csmUserRelations = sysCsmUserRelationMapper.selectList(Wrappers.<SysCsmUserRelation>lambdaQuery()
        //             .in(SysCsmUserRelation::getUserId, userIds));
        //     if (CollUtil.isNotEmpty(csmUserRelations)) {
        //         csmParentUserMap.putAll(csmUserRelations.stream().collect(Collectors.groupingBy(SysCsmUserRelation::getUserId)));
        //     }
        // }
        //
        // userVosPage.getRecords().forEach(userVO -> {
        //     // 上级csm
        //     List<SysCsmUserRelation> csmUserRelations = csmParentUserMap.get(userVO.getUserId());
        //     if (CollUtil.isNotEmpty(csmUserRelations)) {
        //         userVO.setParentIdList(csmUserRelations.stream().map(SysCsmUserRelation::getParentId).collect(Collectors.toList()));
        //     }
        // });

        return R.ok(userVosPage);
    }

    /**
     * 分页查询用户
     *
     * @param query 查询参数列表
     * @return 用户集合
     */
    @PostMapping(value = "/selectUserList")
    public R<?> pageUsers(@Validated @RequestBody SysUserDTO.OnPage query) {

        HnqzUser user = SecurityUtils.getUser();

        //  获取全部的可用角色
        Map<Integer, SysRole> roleId2RoleMap = roleService.list().stream().collect(Collectors.toMap(SysRole::getRoleId, Function.identity()));

        // 获取操作人的角色
        Set<Integer> operatorRoleIds = new HashSet<>(SecurityUtils.getRoles());

        Set<Integer> targetRoleIds = new HashSet<>();


        SysRole role = null;
        boolean needArea = false;

        // 系统管理员
        if (operatorRoleIds.contains(1)) {
            role = roleId2RoleMap.get(1);
        }
        // 中生平台管理员
        else if (operatorRoleIds.contains(2)) {
            role = roleId2RoleMap.get(2);
        }
        // 事业部系统管理员
        else if (operatorRoleIds.contains(50)) {
            role = roleId2RoleMap.get(50);
        }
        // 运营管理员
        else if (operatorRoleIds.contains(3)) {
            role = roleId2RoleMap.get(3);
        }
        // 区域管理员
        else if (operatorRoleIds.contains(4)) {
            role = roleId2RoleMap.get(4);
            query.setDeptId(user.getDeptId());
            needArea = true;
        }
        // 服务商管理员
        else if (operatorRoleIds.contains(37)) {
            role = roleId2RoleMap.get(37);
            query.setDeptId(user.getDeptId());
            needArea = true;
        }

        if (role == null) {
            throw new BizException("当前用户角色不可用");
        } else {
            String visible = role.getVisible();
            if (StrUtil.isBlank(visible)) {
                throw new BizException("当前角色未配置查询角色");
            }

            Set<Integer> visibles = Arrays.stream(visible.split(StrUtil.COMMA)).mapToInt(Integer::valueOf).boxed().collect(Collectors.toSet());

            Integer targetRoleId = CollUtil.isEmpty(query.getRole()) ? null : query.getRole().get(0);

            if (targetRoleId == null) {
                targetRoleIds.addAll(visibles);
            } else if (visibles.contains(targetRoleId)) {
                targetRoleIds.add(targetRoleId);
            } else {
                throw new BizException("当前角色禁止查看其他角色");
            }
        }

        // 获取所有三级区域
        String areaCacheKey = "sys:area:lv3";
        Set<Long> areaEntities;
        if (Boolean.TRUE.equals(redisTemplate.hasKey(areaCacheKey))) {
            areaEntities = (Set<Long>) redisTemplate.opsForValue().get(areaCacheKey);
        } else {
            areaEntities = areaEntityService.list(Wrappers.<SysAreaEntity>lambdaQuery().eq(SysAreaEntity::getAreaType, UpmsType.AreaType.DISTRICT))
                    .stream().map(SysAreaEntity::getAreaId).collect(Collectors.toSet());
            redisTemplate.opsForValue().set(areaCacheKey, areaEntities, 12, TimeUnit.HOURS);
        }

        List<Long> areas = query.getAreaCodes();

        if (CollUtil.isEmpty(areas) && needArea) {
            areas = userAreaService.listUserAreas(Long.valueOf(user.getId()));
        }

        // 如果实际三级区域于查询三级区域相等，也就是全国的时候，默认直接查询全国
        query.setAreaCodes((areaEntities.size() == (CollUtil.isNotEmpty(areas) ? areas.size() : 0)) ? Collections.emptyList() : new LinkedList<>(CollUtil.intersectionDistinct(areaEntities, areas)));
        query.setRole(new LinkedList<>(targetRoleIds));

        Page<?> userVosPage = userService.pageUser(query, user);


        if (CollUtil.isNotEmpty(userVosPage.getRecords())) {

            List<UserVO> collect1 = userVosPage.getRecords().stream().map(o -> (UserVO) o).collect(Collectors.toList());


            Set<Integer> userIds = collect1.stream().mapToInt(UserVO::getUserId).boxed().collect(Collectors.toSet());

            List<WmPlatformQuizTestResult> quizTestResults = platformQuizTestResultService.listPltQuizResults(userIds);

            LocalDate today = LocalDate.now();
            Map<Integer, List<WmPlatformQuizTestResult>> quizTestResult = quizTestResults.stream().filter(res -> res.getExpiry().isAfter(today)).collect(Collectors.groupingBy(WmPlatformQuizTestResult::getUserId));

            collect1.forEach(vo -> {
                List<WmPlatformQuizTestResult> testResults = quizTestResult.get(vo.getUserId());
                List<WmQuizResultModel> pure = null;
                if (CollUtil.isNotEmpty(testResults)) {
                    Map<Integer, List<WmPlatformQuizTestResult>> collect = testResults.stream().collect(Collectors.groupingBy(WmPlatformQuizTestResult::getQuizId));
                    pure = new ArrayList<>(collect.size());

                    for (Map.Entry<Integer, List<WmPlatformQuizTestResult>> entry : collect.entrySet()) {
                        pure.add(entry.getValue().get(0).getQuizResult());
                    }
                }

                vo.setQuizResult(CollUtil.isEmpty(pure) ? Collections.emptyList() : pure);

                // 所属区域
                List<Long> areaCodes = userAreaService.listUserAreas(Long.valueOf(vo.getUserId()));
                vo.setAreaCodes(areaCodes);

                //  角色
                List<SysRole> roles = new LinkedList<>();
                for (SysRole r : vo.getRoleList()) {
                    SysRole role1 = roleId2RoleMap.get(r.getRoleId());
                    if (role1 == null) {
                        continue;
                    }
                    roles.add(role1);
                }
                vo.setRoleList(roles);
            });

            Page<UserVO> page = new Page<>(userVosPage.getCurrent(), userVosPage.getSize(), userVosPage.getTotal());
            page.setPages(userVosPage.getPages());
            page.setRecords(collect1);
        }

        return R.ok(userVosPage);
    }

    /**
     * 修改个人信息
     *
     * @param userDto userDto
     * @return success/false
     */
    @SysLog("修改个人信息")
    @PutMapping("/edit")
    public R<?> updateUserInfo(@Valid @RequestBody UserDTO userDto) {
        userDto.setUserId(SecurityUtils.getUser().getId());
        return userService.updateUserInfo(userDto);
    }

    /**
     * @param username 用户名称
     * @return 上级部门用户列表
     */
    @GetMapping("/ancestor/{username}")
    public R<?> listAncestorUsers(@PathVariable String username) {
        return R.ok(userService.listAncestorUsers(username));
    }

    /**
     * 修改个人信息
     *
     * @param userDTO userDTO
     * @return success/false
     */
    @GetMapping("/list")
    public R<?> listScope(Page page, UserDTO userDTO) {
        if (CollUtil.isNotEmpty(userDTO.getRole())) {
            userDTO.setDeptId(SecurityUtils.getUser().getDeptId());
            page.setSize(10000);
            List<UserVO> sysUserList = userService.getUsersWithRolePage(page, userDTO).getRecords();
            List<Map<String, Object>> answerList = new ArrayList<>();
            if (CollUtil.isNotEmpty(sysUserList)) {
                for (UserVO sysUser : sysUserList) {
                    Map<String, Object> map = new HashMap<>(2);
                    map.put("id", sysUser.getUserId());
                    map.put("realname", sysUser.getRealname());
                    answerList.add(map);
                }

            }
            return R.ok(answerList);
        } else {
            page.setSize(10000);
            List<Integer> list = new ArrayList<>();
            list.add(5);
            list.add(6);
            userDTO.setRole(list);

            List<UserVO> sysUserList = userService.getUsersWithRolePage(page, userDTO).getRecords();
            List<Map<String, Object>> answerList = new ArrayList<>(sysUserList.size());
            if (CollUtil.isNotEmpty(sysUserList)) {
                for (UserVO sysUser : sysUserList) {
                    Map<String, Object> map = new HashMap<>(2);
                    map.put("id", sysUser.getUserId());
                    map.put("realname", sysUser.getRealname());
                    answerList.add(map);
                }

            }
            return R.ok(answerList);
        }
    }

    /**
     * 查询全可用的用户
     *
     * @param userDTO userDTO
     * @return success/false
     */
    @SysLog("查询全可用的用户")
    @GetMapping("/list/available")
    public R<?> listScope1(Page page, UserDTO userDTO) {
        userDTO.setLockFlag(CommonConstants.STATUS_NORMAL);
        if (CollUtil.isNotEmpty(userDTO.getRole())) {
            userDTO.setDeptId(SecurityUtils.getUser().getDeptId());
            page.setSize(10000);
            List<UserVO> sysUserList = userService.getUsersWithRolePage(page, userDTO).getRecords();
            List<Map<String, Object>> answerList = new ArrayList<>(sysUserList.size());
            if (CollUtil.isNotEmpty(sysUserList)) {
                for (UserVO sysUser : sysUserList) {
                    Map<String, Object> map = new HashMap<>(3);
                    map.put("id", sysUser.getUserId());
                    map.put("username", sysUser.getUsername());
                    map.put("realname", sysUser.getRealname());
                    answerList.add(map);
                }

            }
            return R.ok(answerList);
        } else {
            page.setSize(10000);
            List<Integer> list = new ArrayList<>(2);
            list.add(5);
            list.add(6);
            userDTO.setRole(list);

            List<UserVO> sysUserList = userService.getUsersWithRolePage(page, userDTO).getRecords();
            List<Map<String, Object>> answerList = new ArrayList<>(sysUserList.size());
            if (CollUtil.isNotEmpty(sysUserList)) {
                for (UserVO sysUser : sysUserList) {
                    Map<String, Object> map = new HashMap<>(3);
                    map.put("id", sysUser.getUserId());
                    map.put("username", sysUser.getUsername());
                    map.put("realname", sysUser.getRealname());
                    answerList.add(map);
                }

            }
            return R.ok(answerList);
        }
    }

    /**
     * 查询用户列表
     *
     * @param userDTO
     * @return
     */
    @GetMapping("/listNoScope")
    public R<?> listNoScope(UserDTO userDTO) {

        SysUser sysUserSel = new SysUser();

        if (SecurityUtils.getRoles().contains(2) || SecurityUtils.getRoles().contains(1)) {

        } else {
            sysUserSel.setDrugEntId(SecurityUtils.getUser().getDrugEntId());
        }
        QueryWrapper<SysUser> query = Wrappers.query(sysUserSel);
        if (StringUtils.isNotEmpty(userDTO.getRealname())) {
            query.lambda().like(SysUser::getRealname, userDTO.getRealname());
        }
        List<SysUser> sysUserList = userService.list(query);
        List<Map<String, Object>> answerList = new ArrayList<>(sysUserList.size());
        if (CollUtil.isNotEmpty(sysUserList)) {
            for (SysUser sysUser : sysUserList) {
                Map<String, Object> map = new HashMap<>(3);
                map.put("id", sysUser.getUserId());
                map.put("realname", sysUser.getRealname());
                map.put("name", sysUser.getUsername());

                answerList.add(map);
            }

        }
        return R.ok(answerList);
    }

    /**
     * 修改个人信息
     *
     * @param userDTO userDTO
     * @return success/false
     */
    @GetMapping("/listNoScopeAll")
    public R<?> listNoScopeAll(UserDTO userDTO) {

        List<SysUser> sysUserList = userService.list(Wrappers.<SysUser>lambdaQuery().eq(userDTO.getDeptId() != null, SysUser::getDeptId, userDTO.getDeptId()));
        List<Map<String, Object>> answerList = new ArrayList<>(sysUserList.size());
        if (CollUtil.isNotEmpty(sysUserList)) {
            for (SysUser sysUser : sysUserList) {
                Map<String, Object> map = new HashMap<>(3);
                map.put("id", sysUser.getUserId());
                map.put("realname", sysUser.getRealname());
                map.put("name", sysUser.getUsername());

                answerList.add(map);
            }

        }
        return R.ok(answerList);
    }

    /**
     * 查询CSM列表
     *
     * @return {@link R}<{@link ?}>
     */
    @GetMapping("/list-csm")
    public R<?> listCsm() {
        UserDTO userDTO = new UserDTO();
        userDTO.setRole(Stream.of(19).collect(Collectors.toList()));
        List<UserVO> users = sysUserMapper.listUserByRole(userDTO);

        List<Map<String, Object>> result = users.stream().map(user -> {
            Map<String, Object> map = new HashMap<>(2);
            map.put("userId", user.getUserId());
            map.put("realname", user.getRealname());
            return map;
        }).collect(Collectors.toList());

        return R.ok(result);
    }


    /**
     * 获取用户区域树
     *
     * @return 用户区域树
     */
    @GetMapping(value = "/area/tree")
    public R<List<Tree<Long>>> getUserAreaTree() {

        HnqzUser user = SecurityUtils.getUser();
        List<Integer> roles = SecurityUtils.getRoles();

        // admin || 平台管理员
        List<Tree<Long>> userAreaTree;
        if (roles.contains(2) || roles.contains(1) || roles.contains(50)) {
            userAreaTree = areaEntityService.selectTree(new SysAreaEntity());
        }
        //  药企管理员（事业部） || 区域管理员（一级cso管理员） || etc,.
        else if (roles.contains(3) || roles.contains(4) || roles.contains(39) || roles.contains(40) || roles.contains(41) || roles.contains(42) || roles.contains(43)) {
            userAreaTree = userService.getUserAreaTree(Long.valueOf(user.getId()));
        }
        // 服务商管理员（二级cso管理员）
        else if (roles.contains(37)) {
            userAreaTree = sysDeptService.getDeptAreaTree(Long.valueOf(user.getDeptId()));
        } else {
            userAreaTree = Collections.emptyList();
        }


        return R.ok(userAreaTree);
    }

    @SysLog(value = "创建用户证书记录")
    @PostMapping(value = "/cert/create")
    public R<Boolean> createUserCertificate(@Validated @RequestBody SysUserDTO.OnCertificateCreate resource) {

        return R.ok(userManager.createUserCertificate(resource, SecurityUtils.getUser()));

    }

    @GetMapping(value = "/cert/details/{user_id}")
    public R<List<Map<String, Object>>> listUserCertificates(@PathVariable(value = "user_id") Integer userId) {

        return R.ok(userManager.listUserCertificates(userId));

    }

    @SysLog("备案管理员绑定备案产品")
    @PostMapping(value = "/cert/prod/bind")
    public R<Boolean> bindProdToUserCert(@Validated @RequestBody SysUserDTO.OnProdBind resource) {

        return R.ok(userManager.bindProdToUserCert(resource, SecurityUtils.getUser()));
    }

    @GetMapping(value = "/cert/list")
    public R<List<Map<String, Object>>> listUserCerts(@RequestParam(value = "userId") Integer userId) {
        return R.ok(userManager.listUserCerts(userId));
    }

    @SysLog("给监督员绑定产品")
    @PostMapping(value = "/prod/bind")
    public R<Boolean> bindDrug(@Validated @RequestBody SysUserDTO.OnProdBind resource) {
        return R.ok(userManager.bindDrug(resource, SecurityUtils.getUser()));
    }

    @GetMapping(value = "/list/bind/prod")
    public R<List<String>> listBondProds(@RequestParam(value = "userId") Integer userId) {
        return R.ok(userManager.listBondProds(userId));
    }

}
