package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.dto.UserInfo;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.EnableEnum;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaAgent;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEnt;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserMapper;
import com.qunzhixinxi.hnqz.admin.service.SysCsmUserRelationService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysEnterpriseAreaService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.admin.service.WmDaAgentService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.common.core.constant.CacheConstants;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.entity.BaseEntity;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;


/**
 * 档案-药企
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:42
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/wmdadrugent")
public class WmDaDrugEntController {

    private final WmDaDrugEntService wmDaDrugEntService;
    private final WmDaAgentService wmDaAgentService;
    private final SysUserService userService;
    private final SysDeptService sysDeptService;
    private final SysDeptMapper sysDeptMapper;
    private final SysUserMapper sysUserMapper;
    private final SysCsmUserRelationService sysCsmUserRelationService;
    private final RedisTemplate<String, Object> redisTemplate;
    private final SysEnterpriseAreaService enterpriseAreaService;


    /**
     * 分页查询
     *
     * @param wmDaDrugEnt 档案-药企
     * @return 分页结果
     */
    @PostMapping("/page")
    @PreAuthorize("@pms.hasPermission('admin_wmdadrugent_view')")
    public R<?> getWmDaDrugEntPage(@RequestBody WmDaDrugEnt wmDaDrugEnt) {
        if (SecurityUtils.getRoles().contains(19)) {
            // CSM只能查看自己维护的企业下的用户, 以及下属维护的企业下的用户
            Integer userId = SecurityUtils.getUser().getId();
            // 递归查询下属关联的userId
            Set<Integer> allUserIds = sysCsmUserRelationService.recurseCsmLowerUserId(Collections.singleton(userId), new AtomicInteger(10));
            wmDaDrugEnt.setCsmUserIdList(allUserIds);
        }
        Page<WmDaDrugEnt> page = new Page<>(wmDaDrugEnt.getCurrent(), wmDaDrugEnt.getSize());
        wmDaDrugEntService.selectPage(page, wmDaDrugEnt);
        return R.ok(page);
    }


    /**
     * 通过id查询档案-药企
     *
     * @param id id
     * @return R
     */
    @GetMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('admin_wmdadrugent_view')")
    public R<WmDaDrugEnt> getById(@PathVariable("id") Integer id) {
        return R.ok(wmDaDrugEntService.getById(id));
    }

    /**
     * 新增档案-药企
     *
     * @param entity 企业信息
     * @return 保存结果
     */
    @SysLog("新增档案-药企")
    @PostMapping("/saves")
    @PreAuthorize("@pms.hasPermission('admin_wmdadrugent_add')")
    public R<?> saves(@Validated(value = BaseEntity.Create.class) @RequestBody WmDaDrugEnt entity) {


        int count = wmDaDrugEntService.count(Wrappers.<WmDaDrugEnt>lambdaQuery()
                .eq(WmDaDrugEnt::getEntname, entity.getEntname())
                .or()
                .eq(WmDaDrugEnt::getEntcode, entity.getEntcode())
        );

        if (count != 0) {
            return R.failed("药企公司名称或统一社会信用代码已存在");
        }

        return R.ok(wmDaDrugEntService.saveDrugEnt(entity, SecurityUtils.getUser()));
    }

    /**
     * 修改档案-药企
     *
     * @param wmDaDrugEnt 档案-药企
     * @return R
     */
    @SysLog("修改档案-药企")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('admin_wmdadrugent_edit')")
    @Transactional(rollbackFor = Exception.class)
    public R<?> updateById(@Validated(value = BaseEntity.Update.class) @RequestBody WmDaDrugEnt wmDaDrugEnt) {

        SysDept sysDept = new SysDept();
        sysDept.setDelFlag(wmDaDrugEnt.getEnableFlag());
        sysDept.setName(wmDaDrugEnt.getEntname());
        sysDept.setDeptId(wmDaDrugEnt.getDeptId());
        sysDept.setTaxCode(wmDaDrugEnt.getEntcode());
        sysDept.setDeptPermissions(wmDaDrugEnt.getDeptPermissions());
        sysDept.setFinancialReviewType(wmDaDrugEnt.getFinancialReviewType());
        sysDeptMapper.updateByPrimaryKeySelective(sysDept);

        SysUser sysUserCondition = new SysUser();
        sysUserCondition.setDeptId(wmDaDrugEnt.getDeptId());
        List<SysUser> sysUserList = sysUserMapper.selectUserById(sysUserCondition);

        for (SysUser item : sysUserList) {
            item.setDelFlag(wmDaDrugEnt.getEnableFlag());
            sysUserMapper.updateByPrimaryKeySelective(item);
        }

        if (EnableEnum.DISABLE.val().equals(wmDaDrugEnt.getEnableFlag())) {
            // 移除user的缓存
            Set<String> userNameKeys = sysUserList.stream()
                    .map(sysUser -> String.format("%s%s", CacheConstants.USER_DETAILS, sysUser.getUsername()))
                    .collect(Collectors.toSet());
            redisTemplate.delete(userNameKeys);
        }

        wmDaDrugEnt.setUpdateTime(LocalDateTime.now());
        wmDaDrugEnt.setUpdateUser(SecurityUtils.getUser().getId());
        wmDaDrugEntService.updateById(wmDaDrugEnt);

        if (CollUtil.isNotEmpty(wmDaDrugEnt.getAreaCodes())) {
            // 删除历史的
            enterpriseAreaService.delRelation(Long.valueOf(wmDaDrugEnt.getDeptId()));
            // 保存信息
            enterpriseAreaService.saveRelation(Long.valueOf(wmDaDrugEnt.getDeptId()), wmDaDrugEnt.getAreaCodes());
        }


        return R.ok(Boolean.TRUE);
    }

    /**
     * 通过id删除档案-药企
     *
     * @param id id
     * @return R
     */
    @SysLog("通过id删除档案-药企")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('admin_wmdadrugent_del')")
    public R<Boolean> removeById(@PathVariable Integer id) {
        return R.ok(wmDaDrugEntService.removeById(id));
    }


    /**
     * 获取药企列表
     *
     * @return 角色列表
     */
    @GetMapping("/list")
    public R<List<WmDaDrugEnt>> listDrugEnt(WmDaDrugEnt wmDaDrugEnt) {
        wmDaDrugEnt.setDelFlag("0");
        wmDaDrugEnt.setEnableFlag("0");
        if (null != wmDaDrugEnt.getEntname()) {
            String entname = wmDaDrugEnt.getEntname();
            wmDaDrugEnt.setEntname(null);
            return R.ok(wmDaDrugEntService.list(Wrappers.query(wmDaDrugEnt).lambda().like(WmDaDrugEnt::getEntname, entname)));
        } else {
            return R.ok(wmDaDrugEntService.list(Wrappers.query(wmDaDrugEnt)));
        }
    }

    /**
     * 获取药企列表
     *
     * @return 角色列表
     */
    @GetMapping("/listScope")
    public R<List<WmDaDrugEnt>> listScope(WmDaDrugEnt wmDaDrugEnt) {
        wmDaDrugEnt.setDelFlag(CommonConstants.STATUS_NORMAL);
        wmDaDrugEnt.setEnableFlag(CommonConstants.STATUS_NORMAL);

        if (null != wmDaDrugEnt.getEntname()) {
            String entname = wmDaDrugEnt.getEntname();
            wmDaDrugEnt.setEntname(null);
            return R.ok(wmDaDrugEntService.selectByScope(Wrappers.query(wmDaDrugEnt).lambda().like(WmDaDrugEnt::getEntname, entname)));
        } else {
            return R.ok(wmDaDrugEntService.selectByScope(Wrappers.query(wmDaDrugEnt)));
        }
    }

    /**
     * 获取药企列表
     *
     * @return 角色列表
     */
    @GetMapping("/listBySel")
    public R listBySel(WmDaDrugEnt wmDaDrugEnt) {
        wmDaDrugEnt.setDelFlag("0");
        wmDaDrugEnt.setEnableFlag("0");
        if (!SecurityUtils.getRoles().contains(1) && !SecurityUtils.getRoles().contains(2)) {
            wmDaDrugEnt.setId(SecurityUtils.getUser().getDrugEntId());
            return R.ok(wmDaDrugEntService.list(Wrappers.query(wmDaDrugEnt)));
        } else {
            return R.ok(wmDaDrugEntService.list(Wrappers.query(wmDaDrugEnt)));
        }
    }


    private boolean isZbOrDbUsers(Integer[] roles) {
        if (null == roles) {
            return false;
        }

        if (Arrays.asList(roles).contains(1)
                || Arrays.asList(roles).contains(2)
                || Arrays.asList(roles).contains(3)
                || Arrays.asList(roles).contains(4)
                || Arrays.asList(roles).contains(37)) {
            return true;
        }
        return false;
    }

    /**
     * 通过ID查询
     *
     * @param username 用户名
     * @return SysDept
     */
    @GetMapping("/getDrugListByUsername")
    public R<?> getDrugListByUsername(String username) {

        List<SysUser> userList = userService.list(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username).eq(SysUser::getDelFlag, "0"));
        List<Integer> list = new ArrayList<>();
        if (CollUtil.isNotEmpty(userList)) {
            for (SysUser s : userList) {
                if (null != s.getDrugEntId()) {
                    UserInfo userInfo = userService.findUserInfo(s);
                    if (isZbOrDbUsers(userInfo.getRoles())) {
                        list.add(s.getDrugEntId());
                    }
                }
            }
        }
        if (CollUtil.isNotEmpty(list)) {
            return R.ok(wmDaDrugEntService.listByIds(list));
        } else {
            return R.ok();
        }
    }


    /**
     * 通过ID查询
     *
     * @return SysDept
     */
    @GetMapping("/getDrugListByAgentServiceEnt")
    public R<?> getDrugListByAgentServiceEnt() {

        List<String> stringList = new ArrayList<>();

        SysDept sysDept = sysDeptService.getById(SecurityUtils.getUser().getDeptId());
        if (null != sysDept) {
            String levelId = sysDept.getLevel() == 3 ? sysDept.getLevel2Id().toString() : sysDept.getLevel3Id().toString();

            WmDaAgent wmDaAgent = wmDaAgentService.getById(levelId);
            if (null != wmDaAgent && null != wmDaAgent.getExtDrugEnt() && StrUtil.isNotBlank(wmDaAgent.getExtDrugEnt())) {
                String extDrugEnt = wmDaAgent.getExtDrugEnt();
                String[] ext = extDrugEnt.split(",");
                for (String args : ext) {
                    if (!(sysDept.getLevel1Id() + "").equals(args)) {
                        stringList.add(args);
                    }
                }
            }
        }
        stringList.add(SecurityUtils.getUser().getDrugEntId().toString());

        if (CollUtil.isNotEmpty(stringList)) {
            return R.ok(wmDaDrugEntService.listByIds(stringList));
        }

        return R.ok();

    }


}
