package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.dto.WmQuizDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDrugQuizRelation;
import com.qunzhixinxi.hnqz.admin.service.WmDrugQuizRelationService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 药品试卷前端控制器
 *
 * @author jimmy
 * @date 07/07/2024 15:41
 */
@RestController
@AllArgsConstructor
public class WmDrugQuizController {

    private final WmDrugQuizRelationService drugQuizRelationService;

    @SysLog("药品问卷绑定")
    @PostMapping(value = "/drug/quiz/bind")
    public R<Boolean> bind(@Validated @RequestBody WmQuizDTO.OnDrugBind resource) {
        return R.ok(drugQuizRelationService.bind(resource, SecurityUtils.getUser()));
    }


    @GetMapping(value = "/drug/quiz/list")
    public R<List<Integer>> listDrugQuizzes(@RequestParam(value = "drugId") String drugId) {

        List<WmDrugQuizRelation> relations = drugQuizRelationService.list(Wrappers.<WmDrugQuizRelation>lambdaQuery().eq(WmDrugQuizRelation::getDrugId, Integer.valueOf(drugId)));

        if (CollUtil.isEmpty(relations)) {
            return R.ok(Collections.emptyList());
        }

        List<Integer> collect = relations.stream().mapToInt(WmDrugQuizRelation::getQuizId).boxed().distinct().collect(Collectors.toList());

        return R.ok(CollUtil.isEmpty(collect) ? Collections.emptyList() : collect);

    }


}
