package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.util.CreditCodeUtil;
import com.qunzhixinxi.hnqz.admin.api.dto.WmQccInfoDto;
import com.qunzhixinxi.hnqz.admin.api.entity.WmQccInfo;
import com.qunzhixinxi.hnqz.admin.service.WmQccInfoService;
import com.qunzhixinxi.hnqz.common.core.exception.BizException;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 企查查信息
 *
 * @author jimmy
 * @date 10/07/2024 11:10
 */
@RestController
@AllArgsConstructor
public class WmQccInfoController {

    private final RedisTemplate<String, Object> redisTemplate;
    private final WmQccInfoService qccInfoService;

    @SysLog(value = "刷新企业信息")
    @PostMapping(value = "/qcc/info/refresh")
    public R<Map<String, Object>> refreshQccInfo(@Validated @RequestBody WmQccInfoDto.OnRefresh resource) {

        boolean creditCode = CreditCodeUtil.isCreditCode(resource.getRegCode());
        if (!creditCode) {
            throw new BizException("请核实填写【" + resource.getRegCode() + "】的社会用代码是否正确");
        }

        final String cacheKey = "qcc:info:refresh:" + resource.getRegCode();


        if (Boolean.FALSE.equals(redisTemplate.opsForValue().setIfAbsent(cacheKey, resource.getRegCode(), 3, TimeUnit.MINUTES))) {
            return R.failed("当前企业存在在途刷新动作，请稍后重试");
        }


        return R.ok(qccInfoService.refresh(resource.getRegCode(), SecurityUtils.getUser()));

    }

    @GetMapping(value = "/qcc/info/details/{reg_code}")
    public R<WmQccInfo> getQccInfo(@PathVariable(value = "reg_code") String regCode) {

        return R.ok(qccInfoService.getQccInfo(regCode));
    }
}
