package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.dto.WmSupervisionDto;
import com.qunzhixinxi.hnqz.admin.api.entity.WmPackageSupervision;
import com.qunzhixinxi.hnqz.admin.manager.WmSupervisionManager;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.time.LocalDate;
import java.util.List;
import java.util.Map;

/**
 * 监督前端控制器
 *
 * @author jimmy
 * @date 11/07/2024 16:53
 */
@RestController
@AllArgsConstructor
public class WmSupervisionController {

    private final WmSupervisionManager supervisionManager;

    /**
     * 监督页面获取
     *
     * @param query 查询条件
     * @return 分页结果
     */
    @PostMapping(value = "/supervision/pkg/page")
    public R<Page<Map<String, Object>>> pagePackage(@Validated @RequestBody WmSupervisionDto.OnB2BPkgPage query) {

        // return R.ok(supervisionManager.pagePkg(query, SecurityUtils.getUser()));
        // return R.ok(supervisionManager.pagePkgV2(query, SecurityUtils.getUser()));
        return R.ok(supervisionManager.pagePkgV3(query, SecurityUtils.getUser()));
    }

    @SysLog(value = "执行包监督")
    @PostMapping(value = "/supervision/pkg/create")
    public R<Boolean> createPkgSupervision(@Validated @RequestBody WmSupervisionDto.OnPkgSupervision resource) {
        return R.ok(supervisionManager.createPkgSupervision(resource, SecurityUtils.getUser()));
    }

    @GetMapping(value = "/supervision/pkg/res/{sup_id}")
    public R<WmPackageSupervision> getPkgSupervision(@PathVariable(value = "sup_id") Integer supId) {
        return R.ok(supervisionManager.getPkgSupervision(supId));

    }

    @SysLog(value = "执行包监督的审核")
    @PostMapping(value = "/supervision/pkg/res/check")
    public R<Boolean> checkPkgSupervision(@Validated @RequestBody WmSupervisionDto.OnPkgSupervisionCheck resource) {
        return R.ok(supervisionManager.checkPkgSupervision(resource, SecurityUtils.getUser()));

    }


    @GetMapping(value = "/supervision/pkg/stat")
    public R<Map<String, String>> pkgSupervisionStat(@RequestParam(value = "all", required = false) Boolean all,
                                                     @RequestParam(value = "season", required = false) Integer season) {
        return R.ok(supervisionManager.pkgSupervisionStat(all, season, SecurityUtils.getUser()));

    }

    // @SysLog(value = "服务任务监督")
    // @PostMapping(value = "/supervision/task/create")
    // public R<Boolean> createTaskSupervision(@Validated @RequestBody WmSupervisionDto.OnTaskSupervision resource) {
    //     return R.ok(supervisionManager.createTaskSupervision(resource, SecurityUtils.getUser()));
    // }

    @SysLog(value = "服务任务监督v1")
    @PostMapping(value = "/supervision/task/create")
    public R<Boolean> createTaskSupervision(@Validated @RequestBody WmSupervisionDto.OnTaskSupervision resource) {

        return R.ok(supervisionManager.doTaskSupervision(resource, SecurityUtils.getUser()));

    }

    @SysLog(value = "管理员批量进行服务任务监督")
    @PostMapping(value = "/supervision/task/create/batch")
    public R<Boolean> createTaskSupervisionBatch(@Validated @RequestBody WmSupervisionDto.OnTaskSupervisionBatch resource) {

        return R.ok(supervisionManager.doTaskSupervisionBatch(resource, SecurityUtils.getUser()));

    }



    // @SysLog(value = "服务任务监督的审核")
    // @PostMapping(value = "/supervision/task/res/check")
    // public R<Boolean> checkTaskSupervision(@Validated @RequestBody WmSupervisionDto.OnTaskSupervisionCheck resource) {
    //     return R.ok(supervisionManager.checkTaskSupervision(resource, SecurityUtils.getUser()));
    //
    // }

    // 获取当前积分包任务监督结果信息
    @GetMapping(value = "/supervision/task/result")
    public R<String> getCurrentPkgTaskSupResult(@RequestParam(value = "pkgId") Integer pkg) {
        return R.ok(supervisionManager.getCurrentPkgTaskSupResult(pkg, SecurityUtils.getUser()));
    }

    @GetMapping(value = "/supervision/task/sup/page")
    public R<Page<Map<String, Object>>> pageTaskSupervision(@Validated WmSupervisionDto.OnTaskSupervisionPage query) {

        return R.ok(supervisionManager.pageTaskSupervision(query));
    }

    // @GetMapping(value = "/supervision/task/avail/list")
    // public R<List<Map<String, Object>>> listAvailableTask(@RequestParam(value = "pkgId") Integer pkgId,
    //                                                       @RequestParam(value = "review", defaultValue = "false") Boolean review) {
    //
    //     return R.ok(supervisionManager.listAvailableTask(pkgId, review));
    //
    // }

    @GetMapping(value = "/supervision/warn/rule/page")
    public R<Page<Map<String, Object>>> pageWarnRule(@Validated WmSupervisionDto.OnWarnRulePage query) {
        return R.ok(supervisionManager.pageWarnRule(query));
    }

    @SysLog("创建预警规则")
    @PostMapping(value = "/supervision/warn/rule/create")
    public R<Boolean> createWarnRule(@Validated @RequestBody WmSupervisionDto.OnWarnRulCreate resource) {
        return R.ok(supervisionManager.createWarnRule(resource, SecurityUtils.getUser()));
    }


    @SysLog("修改预警规则")
    @PostMapping(value = "/supervision/warn/rule/update")
    public R<Boolean> updateWarnRule(@Validated @RequestBody WmSupervisionDto.OnWarnRulUpdate resource) {
        return R.ok(supervisionManager.updateWarnRule(resource, SecurityUtils.getUser()));
    }

    @GetMapping(value = "/supervision/warn/page")
    public R<Page<Map<String, Object>>> pageRoleSupervisionWarn(@Validated WmSupervisionDto.OnWarnPage query) {

        return R.ok(supervisionManager.pageRoleSupervisionWarn(query, SecurityUtils.getUser()));

    }

    @SysLog("警告信息阅读")
    @PostMapping(value = "/supervision/warn/read")
    public R<Boolean> readWarn(@Validated @RequestBody WmSupervisionDto.OnWarnCreate resource) {

        return R.ok(supervisionManager.readWarn(resource, SecurityUtils.getUser()));

    }

    /**
     * 导出监察报告
     *
     * @param entId  企业ID
     * @param period 周期
     * @return 上传结果
     * @throws Exception 异常
     */
    @GetMapping(value = "/supervision/report/export")
    public R<Map<String, String>> exportSupervisionReportPDF(@RequestParam(value = "entId") Integer entId,
                                                             @RequestParam(value = "period", required = false) LocalDate[] period) throws Exception {

        return R.ok(supervisionManager.exportSupervisionReportPDF(entId, period, SecurityUtils.getUser()));


    }


}
