package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaAgent;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEnt;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.WmDaAgentService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskTypeEntRefService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskTypeService;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskTypeEntRef;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;


/**
 * 药企给代理商设定积分规则
 *
 * @author gaoyang
 * @date 2020-07-02 23:35:21
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmtasktypeentref")
public class WmTaskTypeEntRefController {

    private final WmTaskTypeEntRefService wmTaskTypeEntRefService;
    private final WmTaskTypeService wmTaskTypeService;
    private final WmDaAgentService wmDaAgentService;
    private final SysDeptService sysDeptService;
    private final WmDaDrugEntService wmDaDrugEntService;

    /**
     * 分页查询
     *
     * @param page             分页对象
     * @param wmTaskTypeEntRef 药企给代理商设定积分规则
     * @return 分页结果
     */
    @GetMapping("/page")
    public R<Page<WmTaskTypeEntRef>> getWmTaskTypeEntRefPage(Page<WmTaskTypeEntRef> page, WmTaskTypeEntRef wmTaskTypeEntRef) {
        OrderItem orderItem = new OrderItem();
        orderItem.setColumn("create_time");
        orderItem.setAsc(false);
        page.addOrder(orderItem);
        WmDaDrugEnt drId = wmDaDrugEntService.selectByDeptId(SecurityUtils.getUser().getDeptId());
        wmTaskTypeEntRef.setDrugentId((drId.getId()));
        return R.ok(wmTaskTypeEntRefService.page(page, Wrappers.query(wmTaskTypeEntRef)));
    }

    /**
     * 分页查询积分规则
     *
     * @param page    分页对象
     * @param agentId 药企给代理商Id
     * @return 分页结果
     */
    @GetMapping("/pages")
    public R<Page<WmTaskTypeEntRef>> getWmTaskTypeEntRefPages(Page<WmTaskTypeEntRef> page,
                                                              @RequestParam(value = "agentId", required = false) Integer agentId) {

        if (Objects.isNull(agentId)) {
            WmDaAgent anId = wmDaAgentService.selectByDeptId(SecurityUtils.getUser().getDeptId());
            if (anId == null) {
                return R.ok(page);
            }
            agentId = anId.getId();
        }
        return R.ok(wmTaskTypeEntRefService.page(page, Wrappers.<WmTaskTypeEntRef>lambdaQuery()
                .eq(WmTaskTypeEntRef::getAgentId, agentId)
                .isNotNull(WmTaskTypeEntRef::getDrugentId)
                .orderByDesc(WmTaskTypeEntRef::getCreateTime)));
    }


    /**
     * 通过id查询药企给代理商设定积分规则
     *
     * @param id id
     * @return R
     */
    @GetMapping("/{id}")
    public R<WmTaskTypeEntRef> getById(@PathVariable("id") Integer id) {
        return R.ok(wmTaskTypeEntRefService.getById(id));
    }


    /**
     * 新增药企给代理商设定积分规则
     *
     * @param wmTaskTypeEntRef 药企给代理商设定积分规则
     * @return R
     */
    @SysLog("新增药企给代理商设定积分规则")
    @PostMapping("/saves")
    @Transactional(rollbackFor = Exception.class)
    public R<Boolean> saves(@RequestBody WmTaskTypeEntRef wmTaskTypeEntRef) {

        SysDept deptId = sysDeptService.getById(SecurityUtils.getUser().getDeptId());
        WmDaDrugEnt wmDaDrugEnt = wmDaDrugEntService.selectByDeptId(deptId.getDeptId());

        wmTaskTypeEntRef.setDrugentId(wmDaDrugEnt.getId());

        WmTaskType wmTaskType = new WmTaskType();
        wmTaskType.setDurgentId(wmTaskTypeEntRef.getDrugentId());
        wmTaskType.setLevel1Id(wmTaskTypeEntRef.getAgentId());
        wmTaskType.setDelFlag("0");
        List<WmTaskType> list = wmTaskTypeService.list(Wrappers.query(wmTaskType));
        WmDaAgent wmDaAgent = wmDaAgentService.getById(wmTaskTypeEntRef.getAgentId());
        WmDaDrugEnt wmDaDrugEnt1 = wmDaDrugEntService.getById(wmTaskTypeEntRef.getDrugentId());
        SysDept DrugEntDeptId = sysDeptService.getById(wmDaDrugEnt1.getDeptId());

        if (CollUtil.isNotEmpty(list)) {

            SysDept AgentDeptId = sysDeptService.getById(wmDaAgent.getDeptId());

            if (AgentDeptId.getLevel() == 3) {
                return R.failed("药企已分配一代积分规则");
            }
        } else if (null != wmDaAgent) {
            wmTaskTypeEntRef.setDeptId(wmDaAgent.getDeptId().toString());
        } else if (DrugEntDeptId.getLevel() == 2) {
            return R.failed("药企只能分配积分规则");
        }

        wmTaskTypeEntRefService.save(wmTaskTypeEntRef);
        WmTaskType wmTaskTypeQ = new WmTaskType();
        wmTaskTypeQ.setTaskTypeLevel("1");
        wmTaskTypeQ.setDelFlag("0");
        wmTaskTypeQ.setStatus("0");
        List<WmTaskType> tempList = wmTaskTypeService.list(Wrappers.query(wmTaskTypeQ));
        List<WmTaskType> newList = new ArrayList<>();

        for (WmTaskType wmTaskType1 : tempList) {
            wmTaskType1.setId(null);
            wmTaskType1.setLevel1Id(wmTaskTypeEntRef.getAgentId());
            wmTaskType1.setDurgentId(wmTaskTypeEntRef.getDrugentId());
            wmTaskType1.setTaskTypeLevel("2");
            wmTaskType1.setTaskEntId(wmTaskTypeEntRef.getAgentId());
            wmTaskType1.setDeptId(wmDaAgent.getDeptId());
            wmTaskType1.setCreateTime(LocalDateTime.now());
            wmTaskType1.setStatus("0");
            wmTaskType1.setBaseId(wmTaskType1.getId());
            wmTaskType1.setRuleId("" + wmTaskType1.getDurgentId() + wmTaskType1.getLevel1Id());
            wmTaskType1.setParentId(wmTaskType1.getParentId());
            newList.add(wmTaskType1);
        }


        wmTaskTypeService.saveBatch(newList);

        return R.ok(Boolean.TRUE);
    }


    /**
     * 修改药企给代理商设定积分规则
     *
     * @param wmTaskTypeEntRef 药企给代理商设定积分规则
     * @return R
     */
    @SysLog("修改药企给代理商设定积分规则")
    @PutMapping
    public R<Boolean> updateById(@RequestBody WmTaskTypeEntRef wmTaskTypeEntRef) {
        return R.ok(wmTaskTypeEntRefService.updateById(wmTaskTypeEntRef));
    }

    /**
     * 通过id删除药企给代理商设定积分规则
     *
     * @param id id
     * @return R
     */
    @SysLog("通过id删除药企给代理商设定积分规则")
    @DeleteMapping("/{id}")
    public R<Boolean> removeById(@PathVariable Integer id) {
        return R.ok(wmTaskTypeEntRefService.removeById(id));
    }


    /**
     * 获取药企列表
     *
     * @return 角色列表
     */
    @GetMapping("/list")
    public R<List<WmTaskTypeEntRef>> listWmScorePackage(WmTaskTypeEntRef wmTaskTypeEntRef) {
        wmTaskTypeEntRef.setDelFlag(CommonConstants.STATUS_NORMAL);
        wmTaskTypeEntRef.setEnableFlag(CommonConstants.STATUS_NORMAL);
        wmTaskTypeEntRef.setDrugentId(SecurityUtils.getUser().getDrugEntId());
        return R.ok(wmTaskTypeEntRefService.list(Wrappers.query(wmTaskTypeEntRef)));
    }

}
