package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.dto.WmUserSignDTO;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.api.entity.WmUserSign;
import com.qunzhixinxi.hnqz.admin.service.WmUserSignService;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;


/**
 * 用户签到表
 *
 * @author gaoyang
 * @date 2020-07-14 20:10:02
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmusersign")
public class WmUserSignController {

    private final WmUserSignService wmUserSignService;
    private final RedisTemplate<String, Object> redisTemplate;


    /**
     * 分页查询
     *
     * @param page       分页对象
     * @param wmUserSign 用户签到表
     * @return 分页结果
     */
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('admin_wmusersign_view')")
    public R<?> getWmUserSignPage(Page<WmUserSign> page, WmUserSign wmUserSign) {

        return R.ok(wmUserSignService.page(page, Wrappers.query(wmUserSign).orderByDesc("create_time")));
    }


    /**
     * 通过id查询用户签到表
     *
     * @param id id
     * @return R
     */
    @GetMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('admin_wmusersign_view')")
    public R<?> getById(@PathVariable("id") Integer id) {
        return R.ok(wmUserSignService.getById(id));
    }

    /**
     * 新增用户签到表
     *
     * @param wmUserSign 用户签到表
     * @return R
     */
    @SysLog("新增用户签到表")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('admin_wmusersign_add')")
    public R<?> save(@RequestBody WmUserSign wmUserSign) {
        return R.ok(wmUserSignService.save(wmUserSign));
    }

    /**
     * 修改用户签到表
     *
     * @param wmUserSign 用户签到表
     * @return R
     */
    @SysLog("修改用户签到表")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('admin_wmusersign_edit')")
    public R<?> updateById(@RequestBody WmUserSign wmUserSign) {
        return R.ok(wmUserSignService.updateById(wmUserSign));
    }

    /**
     * 通过id删除用户签到表
     *
     * @param id id
     * @return R
     */
    @SysLog("通过id删除用户签到表")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('admin_wmusersign_del')")
    public R<?> removeById(@PathVariable Integer id) {
        return R.ok(wmUserSignService.removeById(id));
    }

    /**
     * 会员临时签到
     *
     * @param resource 签到信息
     * @return 签到时间
     */
    @SysLog("临时签到")
    @PostMapping(value = "/temp/sign")
    public R<String> doTempSign(@Validated @RequestBody WmUserSignDTO.OnTempSign resource) {
        return R.ok(wmUserSignService.doTempSign(resource, SecurityUtils.getUser()));
    }

    /**
     * 会员临时签到
     *
     * @param resource 签到信息
     * @return 签到时间
     */
    @PostMapping(value = "/temp/sign/result")
    public R<?> getTempSign(@Validated @RequestBody WmUserSignDTO.OnTempSign resource) {
        Object o = redisTemplate.opsForValue().get(String.format("user:sign:temp:%s:%s", SecurityUtils.getUser().getId(), resource.getLocationId()));
        return R.ok(o);
    }

}
