package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEntDrugtable;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.service.SysCsmUserRelationService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntDrugtableService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.api.entity.WmWkAlbum;
import com.qunzhixinxi.hnqz.admin.service.WmWkAlbumService;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;


/**
 * 文库-相册
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:24
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmwkalbum")
public class WmWkAlbumController {

    private final WmWkAlbumService wmWkAlbumService;
    private final WmDaDrugEntDrugtableService wmDaDrugEntDrugtableService;
    private final SysCsmUserRelationService sysCsmUserRelationService;

    /**
     * 分页查询
     *
     * @param page      分页对象
     * @param wmWkAlbum 文库-相册
     * @return 分页结果
     */
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('admin_wmwkalbum_view')")
    public R<IPage<WmWkAlbum>> getWmWkAlbumPage(Page<WmWkAlbum> page, WmWkAlbum wmWkAlbum) {
        OrderItem orderItem = new OrderItem();
        orderItem.setColumn("create_time");
        orderItem.setAsc(false);
        page.addOrder(orderItem);

        QueryWrapper<WmWkAlbum> queryWrapper = Wrappers.query();
        if (null != wmWkAlbum.getXcmc()) {
            queryWrapper.like(true, "xcmc", wmWkAlbum.getXcmc());
        }

        if (null != wmWkAlbum.getGlyp()) {
            queryWrapper.apply(true, "FIND_IN_SET (" + wmWkAlbum.getGlyp() + ",glyp)");
        }

        queryWrapper.eq("del_flag", DelEnum.NOT_DEL.getVal());
        queryWrapper.orderByDesc("create_time");

        if (SecurityUtils.getRoles().contains(19)) {
            // CSM只能查看自己维护的企业下的数据, 以及下属维护的企业下的数据
            Integer userId = SecurityUtils.getUser().getId();
            // 递归查询下属关联的userId
            Set<Integer> allUserIds = sysCsmUserRelationService.recurseCsmLowerUserId(Collections.singleton(userId), new AtomicInteger(10));
            queryWrapper.and(wrapper -> {
                wrapper.in("create_user", allUserIds)
                        .or(wrapper2 -> wrapper2.isNull("create_user"));
            });
        }

        Page<WmWkAlbum> pageAnswer = wmWkAlbumService.page(page, queryWrapper);
        List<WmWkAlbum> wmWkAlbumList = pageAnswer.getRecords();
        if (null != wmWkAlbumList) {
            for (WmWkAlbum wmW : wmWkAlbumList) {
                if (null != wmW.getGlyp()) {
                    String[] ar = wmW.getGlyp().split(StrUtil.COMMA);
                    List<String> list = new ArrayList<>(Arrays.asList(ar));
                    List<WmDaDrugEntDrugtable> wmDaDrugEntDrugtableList = wmDaDrugEntDrugtableService.listByIds(list);
                    if (CollUtil.isNotEmpty(wmDaDrugEntDrugtableList)) {
                        StringBuilder temp1 = new StringBuilder();
                        for (int i = 0; i < wmDaDrugEntDrugtableList.size(); i++) {
                            temp1.append(wmDaDrugEntDrugtableList.get(i).getDrugname());
                            if (i < wmDaDrugEntDrugtableList.size() - 1) {
                                temp1.append(",");
                            }
                        }
                        wmW.setTemp1(temp1.toString());
                    }

                }
            }
        }
        pageAnswer.setRecords(wmWkAlbumList);
        return R.ok(pageAnswer);
    }


    /**
     * 通过id查询文库-相册
     *
     * @param id id
     * @return R
     */
    @GetMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('admin_wmwkalbum_view')")
    public R<?> getById(@PathVariable("id") Integer id) {
        return R.ok(wmWkAlbumService.getById(id));
    }

    /**
     * 新增文库-相册
     *
     * @param wmWkAlbum 文库-相册
     * @return R
     */
    @SysLog("新增文库-相册")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('admin_wmwkalbum_add')")
    public R<?> save(@RequestBody WmWkAlbum wmWkAlbum) {
        wmWkAlbum.setForwardNum(0);
        wmWkAlbum.setCreateTime(LocalDateTime.now());
        wmWkAlbum.setCreateUser(SecurityUtils.getUser().getId());
        wmWkAlbumService.saveWmWkAlbum(wmWkAlbum);
        return R.ok();
    }

    /**
     * 修改文库-相册
     *
     * @param wmWkAlbum 文库-相册
     * @return R
     */
    @SysLog("修改文库-相册")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('admin_wmwkalbum_edit')")
    public R<?> updateById(@RequestBody WmWkAlbum wmWkAlbum) {
        wmWkAlbum.setUpdateTime(LocalDateTime.now());
        wmWkAlbum.setUpdateUser(SecurityUtils.getUser().getId());
        wmWkAlbumService.updateWmWkAlbumById(wmWkAlbum);
        return R.ok();
    }

    /**
     * 通过id删除文库-相册
     *
     * @param id id
     * @return R
     */
    @SysLog("通过id删除文库-相册")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('admin_wmwkalbum_del')")
    public R<?> removeById(@PathVariable Integer id) {
        return R.ok(wmWkAlbumService.removeById(id));
    }

}
