package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.util.ArrayUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.entity.WmWkAlbum;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.service.SysCsmUserRelationService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.api.entity.WmWkArticle;
import com.qunzhixinxi.hnqz.admin.service.WmWkArticleService;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;


/**
 * 文库-文章
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:24
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmwkarticle")
public class WmWkArticleController {

	private final WmWkArticleService wmWkArticleService;
	private final SysCsmUserRelationService sysCsmUserRelationService;

	/**
	 * 分页查询
	 *
	 * @param page        分页对象
	 * @param wmWkArticle 文库-文章 测试合并分支
	 * @return
	 */
	@GetMapping("/page")
	@PreAuthorize("@pms.hasPermission('admin_wmwkarticle_view')")
	public R<IPage<WmWkArticle>> getWmWkArticlePage(Page<WmWkArticle> page, WmWkArticle wmWkArticle) {

		OrderItem orderItem = new OrderItem();
		orderItem.setColumn("create_time");
		orderItem.setAsc(false);
		page.addOrder(orderItem);

		LocalDateTime startTime = wmWkArticle.getCreateTime();
		LocalDateTime endTime = wmWkArticle.getUpdateTime();
		QueryWrapper<WmWkArticle> wrappers = Wrappers.query();
		if (null != startTime) {
			wrappers.ge(true, "create_time", startTime);
		}

		if (null != startTime) {
			wrappers.le(true, "create_time", endTime);
		}

		if (null != wmWkArticle.getTitle()) {
			wrappers.like(true, "title", wmWkArticle.getTitle());
		}
		if (StringUtils.isNotEmpty(wmWkArticle.getArea())) {
			wrappers.like(true, "area", wmWkArticle.getArea());
		}
		if (StringUtils.isNotEmpty(wmWkArticle.getAuthor())) {
			wrappers.like(true, "author", wmWkArticle.getAuthor());
		}

		if (null != wmWkArticle.getSource()) {
			wrappers.eq(true, "source", wmWkArticle.getSource());
		}
		if (ArrayUtil.isNotEmpty(wmWkArticle.getSourceEnt())) {
			wrappers.apply("FIND_IN_SET({0}, source_ent)", wmWkArticle.getSourceEnt()[0]);
		}

		wrappers.eq("del_flag", DelEnum.NOT_DEL.getVal());
		wrappers.orderByDesc("create_time");

		if (SecurityUtils.getRoles().contains(19)) {
			// CSM只能查看自己维护的企业下的数据, 以及下属维护的企业下的数据
			Integer userId = SecurityUtils.getUser().getId();
			// 递归查询下属关联的userId
			Set<Integer> allUserIds = sysCsmUserRelationService.recurseCsmLowerUserId(Collections.singleton(userId), new AtomicInteger(10));
			wrappers.and(wrapper -> {
				wrapper.in("create_user", allUserIds)
						.or(wrapper2 -> wrapper2.isNull("create_user"));
			});
		}

		return R.ok(wmWkArticleService.page(page, wrappers));
	}


	/**
	 * 通过id查询文库-文章
	 *
	 * @param id id
	 * @return R
	 */
	@GetMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('admin_wmwkarticle_view')")
	public R getById(@PathVariable("id") Integer id) {

		return R.ok(wmWkArticleService.getById(id));
	}

	/**
	 * 新增文库-文章
	 *
	 * @param wmWkArticle 文库-文章
	 * @return R
	 */
	@SysLog("新增文库-文章")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('admin_wmwkarticle_add')")
	public R save(@RequestBody WmWkArticle wmWkArticle) {
		wmWkArticle.setForwardNum(0);
		wmWkArticle.setCreateTime(LocalDateTime.now());
		return R.ok(wmWkArticleService.save(wmWkArticle));
	}

	/**
	 * 修改文库-文章
	 *
	 * @param wmWkArticle 文库-文章
	 * @return R
	 */
	@SysLog("修改文库-文章")
	@PutMapping
	@PreAuthorize("@pms.hasPermission('admin_wmwkarticle_edit')")
	public R updateById(@RequestBody WmWkArticle wmWkArticle) {
		return R.ok(wmWkArticleService.updateById(wmWkArticle));
	}

	/**
	 * 通过id删除文库-文章
	 *
	 * @param id id
	 * @return R
	 */
	@SysLog("通过id删除文库-文章")
	@DeleteMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('admin_wmwkarticle_del')")
	public R removeById(@PathVariable Integer id) {
		return R.ok(wmWkArticleService.removeById(id));
	}

}
