package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.api.entity.WmWkArticleShare;
import com.qunzhixinxi.hnqz.admin.service.WmWkArticleShareService;
import org.springframework.security.access.prepost.PreAuthorize;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;


/**
 * 文章分享
 *
 * @author gaoyang
 * @date 2020-07-09 22:34:30
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmwkarticleshare")
public class WmWkArticleShareController {

    private final WmWkArticleShareService wmWkArticleShareService;

    /**
     * 分页查询
     *
     * @param page             分页对象
     * @param wmWkArticleShare 文章分享
     * @return 分页结果
     */
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('admin_wmwkarticleshare_view')")
    public R<Page<WmWkArticleShare>> getWmWkArticleSharePage(Page<WmWkArticleShare> page, WmWkArticleShare wmWkArticleShare) {


        return R.ok(wmWkArticleShareService.page(page, Wrappers.query(wmWkArticleShare).orderByDesc("create_time")));
    }


    /**
     * 通过id查询文章分享
     *
     * @param id id
     * @return R
     */
    @GetMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('admin_wmwkarticleshare_view')")
    public R<WmWkArticleShare> getById(@PathVariable("id") Integer id) {
        return R.ok(wmWkArticleShareService.getById(id));
    }

    /**
     * 新增文章分享
     *
     * @param wmWkArticleShare 文章分享
     * @return R
     */
    @SysLog("新增文章分享")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('admin_wmwkarticleshare_add')")
    public R<Boolean> save(@RequestBody WmWkArticleShare wmWkArticleShare) {
        wmWkArticleShare.setCreateTime(LocalDateTime.now());
        return R.ok(wmWkArticleShareService.save(wmWkArticleShare));
    }

    /**
     * 修改文章分享
     *
     * @param wmWkArticleShare 文章分享
     * @return R
     */
    @SysLog("修改文章分享")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('admin_wmwkarticleshare_edit')")
    public R<Boolean> updateById(@RequestBody WmWkArticleShare wmWkArticleShare) {
        return R.ok(wmWkArticleShareService.updateById(wmWkArticleShare));
    }

    /**
     * 通过id删除文章分享
     *
     * @param id id
     * @return R
     */
    @SysLog("通过id删除文章分享")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('admin_wmwkarticleshare_del')")
    public R<Boolean> removeById(@PathVariable Integer id) {
        return R.ok(wmWkArticleShareService.removeById(id));
    }

}
