package com.qunzhixinxi.hnqz.admin.listener;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysEntContractInfo;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEntDrugtable;
import com.qunzhixinxi.hnqz.admin.api.entity.WmPackageContract;
import com.qunzhixinxi.hnqz.admin.api.entity.WmPackageTaskTypeQty;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.event.PackageContractEvent;
import com.qunzhixinxi.hnqz.admin.mapper.SysEntContractInfoMapper;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysFileService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntDrugtableService;
import com.qunzhixinxi.hnqz.admin.service.WmPackageContractService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.admin.util.OsEnvUtils;
import com.qunzhixinxi.hnqz.common.core.util.PdfUtils;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.event.EventListener;
import org.springframework.core.annotation.Order;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.scheduling.annotation.Async;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 积分包合同生成
 *
 * @author jimmy
 * @date 2024-08-17 21:33
 */
@Slf4j
@AllArgsConstructor
@Configuration
public class PackageContractEventListener {

    @Getter
    @AllArgsConstructor
    private enum MAH {

        ENT_1("长春祈健生物制品有限公司", "长春祈健", "长春市高新开发区火炬路1号", "蔡岩", "", "0431-85089081", "", BankInfo.E1),
        ENT_2("上海生物制品研究所有限责任公司", "上海公司", "上海市奉贤区广丰路758号", "李秀玲", "张光中", "021-22132888", "", BankInfo.E2),
        ENT_3("武汉生物制品研究所有限责任公司", "武汉公司", "武汉市江夏区郑店黄金工业园路1号", "段凯", "", "027-88840477", "027-88913353", BankInfo.E3),
        ENT_4("北京生物制品研究所有限责任公司", "北京公司", "北京市北京经济技术开发区博兴二路9号院4号楼2层205室", "张云涛", "赵增虎", "010-60963039", "010-60963311", BankInfo.E4),
        ENT_5("长春生物制品研究所有限责任公司", "长春公司", "长春市高新技术开发区创新路1616号", "顾建阳", "马俊刚", "0431-87914066", "", BankInfo.E5),
        ENT_6("成都生物制品研究所有限责任公司", "成都公司", "成都市锦江区锦华路三段379号", "葛永红", "徐澜方", "028-84418827", "028-84418201", BankInfo.E6),
        ENT_7("兰州生物制品研究所有限责任公司", "兰州公司", "甘肃省兰州市城关区盐场路888号", "邹勇", "马忠山", "0931-8316777", "0931-8343199", BankInfo.E7);


        public static MAH get(String entName) {
            return Arrays.stream(MAH.values()).filter(ent -> ent.name.equals(entName)).findFirst().orElse(null);
        }


        private final String name;
        private final String abbr;
        private final String address;
        private final String respName;
        private final String contractName;
        private final String contractPhone;
        private final String fax;
        private final BankInfo bankInfo;
    }


    @Getter
    @AllArgsConstructor
    private enum BankInfo {


        E1("长春祈健生物制品有限公司", "91220101748449895N", "长春市高新技术开发区火炬路1号", "0431-85089080", "", "中国农业银行长春汽车城支行", "07191901040015396"),
        E2("上海生物制品研究所有限责任公司", "91310105132708043G", "上海市奉贤区广丰路758号", "021-22132888", "", "中国工商银行上海市延安西路支行", "1001278609004650185"),
        E3("武汉生物制品研究所有限责任公司", "91420115177683863Q", "武汉市江夏区郑店街黄金工业园路1号", "027-86637107", "", "中国工商银行武汉江夏支行", "3202009319200287191"),
        E4("北京生物制品研究所有限责任公司", "91110302MA007H95X1", "北京市北京经济技术开发区博兴二路9号院4号楼2层205室", "010-60963038", "", "中国工商银行股份有限公司北京管庄支行", "0200006809200111871"),
        E5("长春生物制品研究所有限责任公司", "91220101123998130E", "长春市高新技术开发区创新路1616号", "0431-85854537", "", "中国工商银行长春迎宾路支行", "4200220909000001253"),
        E6("成都生物制品研究所有限责任公司", "9151000020181000XC", "成都市锦江区锦华路三段379号", "028-84418815", "", "中国工商银行股份有限公司成都芷泉支行", "4402205009004600681"),
        E7("兰州生物制品研究所有限责任公司", "916200002243483486", "甘肃省兰州市城关区盐场路888号", "0931-8316403", "", "中国建设银行股份有限公司兰州大砂坪支行", "62001360015051500920");

        private final String entName;
        private final String regCode;
        private final String address;
        private final String phone;
        private final String bankName;
        private final String bankNo;
        private final String bankAccountNumber;
    }


    private final WmPackageContractService packageContractService;
    private final SysFileService fileService;
    private final WmScorePackageService packageService;
    private final WmDaDrugEntDrugtableService drugEntDrugtableService;
    private final SysEntContractInfoMapper entContractInfoMapper;
    private final SysDeptService deptService;


    /**
     * 推送事件
     *
     * @param event 事件
     */
    @Async
    @Order
    @EventListener(PackageContractEvent.class)
    public void pushEvent(PackageContractEvent event) throws IOException {

        Integer pkgId = event.getPackageId();
        LocalDateTime now = LocalDateTime.now();

        log.info("生成市场服务包(id:{})合同", pkgId);

        // 获取环境变量
        Map<String, String> envPaths = OsEnvUtils.getEachEnvPaths();

        // 生成合同
        String templatePath = envPaths.get(OsEnvUtils.TargetFile.TEMPLATE.getName()) + "pkg_tmpl_v19_1.pdf";
        final String outFilePath =
                // 根据环境获取临时目录
                envPaths.get(OsEnvUtils.TargetFile.CONTRACT.getName())
                        // 拼接包ID
                        + "PID" + pkgId
                        // 拼接时间
                        + "_" + now.format(DateTimeFormatter.ofPattern(DatePattern.PURE_DATETIME_PATTERN))
                        + ".pdf";

        Map<String, String> params = buildContractParams(event, now);

        PdfUtils.resumeGenerate(templatePath, outFilePath, envPaths.get(OsEnvUtils.TargetFile.FONT.getName()) + "SimSun.ttc", params);


        // 上传到oss
        File contractPdf = new File(outFilePath);
        MockMultipartFile multipartFile = new MockMultipartFile("file", contractPdf.getName(), null, Files.newInputStream(contractPdf.toPath()));
        Map<String, String> uploaded = fileService.upload(multipartFile, event.getCreator());
        String fileId = uploaded.get("fileId");

        // 保存合同对象
        WmPackageContract pc = new WmPackageContract();
        pc.setPackageId(pkgId);
        pc.setFileId(Integer.valueOf(fileId));
        pc.setSn(params.get("contract_no"));
        pc.setCreateTime(now);

        packageContractService.save(pc);

    }

    private Map<String, String> buildContractParams(PackageContractEvent event, LocalDateTime now) {

        Map<String, String> params = new HashMap<>(80);

        // 获取积分包
        WmScorePackage pkg = packageService.getById(event.getPackageId().toString());
        log.info("pkg_info:【{}】", pkg);

        // 获取区域信息
        // SysDept cso1 = deptService.getById(Integer.valueOf(pkg.getSendPackageDeptId()));

        // 获取服务商信息
        SysDept service = deptService.getById(Integer.getInteger(pkg.getDeptId()));

        // 通过药品获取甲方信息
        String[] drugtable = pkg.getDrugtable();
        List<WmDaDrugEntDrugtable> drugtables = drugEntDrugtableService.listByIds(Arrays.stream(drugtable).collect(Collectors.toSet()));
        WmDaDrugEntDrugtable drug = drugtables.get(0);
        String drugEntName = drug.getDrugEntName();
        MAH mah = MAH.get(drugEntName);


        // 甲方信息
        if (mah != null) {
            params.put("party_a_name", mah.getName());
            params.put("party_a_address", mah.getAddress());
            params.put("party_a_resp_name", mah.getRespName());
            params.put("party_a_contact_name", mah.getContractName());
            params.put("party_a_contact_phone", mah.getContractPhone());
            BankInfo bankInfo = mah.bankInfo;
            params.put("party_a_bank_name", bankInfo.getBankName());
            params.put("party_a_bank_account", bankInfo.getBankAccountNumber());
        }


        // 乙方信息
        SysEntContractInfo contractInfo = entContractInfoMapper.selectOne(Wrappers.<SysEntContractInfo>lambdaQuery().eq(SysEntContractInfo::getDeptId, pkg.getDeptId()));
        if (contractInfo != null) {
            params.put("party_b_name", contractInfo.getEntName());
            params.put("party_b_address", contractInfo.getAddress());
            params.put("party_b_resp_name", contractInfo.getRespName());
            params.put("party_b_contact_name", contractInfo.getContactName());
            params.put("party_b_contact_phone", contractInfo.getContactPhone());
            params.put("party_b_bank_name", contractInfo.getBankName());
            params.put("party_b_bank_no", contractInfo.getBankNo());
            params.put("party_b_bank_account", contractInfo.getBankAccount());
            params.put("party_b_reg_code", contractInfo.getRegCode());
        }

        // 合同号 生产企业简称+“服务字"＋年度-【省区简称］-001（待补充映射表逻辑）-服务季度Q1\Q4-服务商简称（待补充映射表）-产品简称（待补充映射表）-若存在同服务商
        // String pkgSn = StrUtil.isNotBlank(pkg.getPackageSn()) ? pkg.getPackageSn().toUpperCase() : DateTimeFormatter.ofPattern(DatePattern.PURE_DATETIME_MS_PATTERN).format(LocalDateTime.now());
        LocalDate endTime = pkg.getEndTime();
        int month = endTime.getMonthValue();
        int quarter = (month == 3 ? 1 : month == 6 ? 2 : month == 9 ? 3 : 4);
        String pkgSn = String.format("%s服务字%s-[%s]-Q%s-%s-%s", (mah == null ? "" : mah.abbr), endTime.getYear(), pkg.getProvAbbr(), quarter, (drug == null ? "" : drug.getAbbr()), (service == null ? "" : service.getAbbr()));

        // 判断是否是补充包
        List<WmPackageContract> list = packageContractService.list(Wrappers.<WmPackageContract>lambdaQuery().likeRight(WmPackageContract::getSn, pkgSn));

        if (CollUtil.isNotEmpty(list)) {
            pkgSn += "-BC" + list.size();
        }

        params.put("contract_no", pkgSn);
        params.put("agreement_no", pkgSn);

        // 合同岂止时间
        params.put("issue_date", DateTimeFormatter.ofPattern(DatePattern.CHINESE_DATE_PATTERN).format(pkg.getCreateTime()));
        params.put("expiry_date", DateTimeFormatter.ofPattern(DatePattern.CHINESE_DATE_PATTERN).format(endTime));


        // 任务数量统计

        List<WmPackageTaskTypeQty> qtyList = event.getTaskTypeQtiyList();
        // int total = qtyList.stream().mapToInt(WmPackageTaskTypeQty::getQty).sum();

        for (int i = 0, size = 14; i < size; i++) {
            WmPackageTaskTypeQty qty = i < qtyList.size() ? qtyList.get(i) : null;
            if (qty != null) {
                Integer count = qty.getQty();
                params.put("no" + i, String.valueOf(i + 1));
                params.put("name" + i, qty.getTaskTypeName());
                params.put("qty" + i, count.toString());
                params.put("price" + i, qty.getPrice().toString());
                // params.put("rate" + i, String.format("%.2f%%", (100.0 * count) / total));
                params.put("rate" + i, String.format("%.2f%%", (100.0 * qty.getPrice()) / pkg.getScore()));
            } else {
                params.put("no" + i, StrUtil.SLASH);
                params.put("name" + i, StrUtil.SLASH);
                params.put("qty" + i, StrUtil.SLASH);
                params.put("price" + i, StrUtil.SLASH);
                params.put("rate" + i, StrUtil.SLASH);
            }

        }

        params.put("sign_date", DateTimeFormatter.ofPattern(DatePattern.CHINESE_DATE_PATTERN).format(now.toLocalDate()));
        return params;
    }
}
