package com.qunzhixinxi.hnqz.admin.manager;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.EnableEnum;
import com.qunzhixinxi.hnqz.admin.api.dto.SysStatisticDto;
import com.qunzhixinxi.hnqz.admin.api.entity.ReDeptDrug;
import com.qunzhixinxi.hnqz.admin.api.entity.SysAreaEntity;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysImplementPlan;
import com.qunzhixinxi.hnqz.admin.api.entity.SysPlanPkg;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.WmAgentEvaluation;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEntDrugtable;
import com.qunzhixinxi.hnqz.admin.api.entity.WmPackageRedemptionNotice;
import com.qunzhixinxi.hnqz.admin.api.entity.WmQccInfo;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackageStatus;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskSupervision;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.service.ReDeptDrugService;
import com.qunzhixinxi.hnqz.admin.service.SysAreaEntityService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysEnterpriseAreaService;
import com.qunzhixinxi.hnqz.admin.service.SysImplementPlanService;
import com.qunzhixinxi.hnqz.admin.service.SysPlanPkgService;
import com.qunzhixinxi.hnqz.admin.service.SysUserAreaService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.admin.service.WmAgentEvaluationService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntDrugtableService;
import com.qunzhixinxi.hnqz.admin.service.WmPackageRedemptionNoticeService;
import com.qunzhixinxi.hnqz.admin.service.WmQccInfoService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageStatusService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskSupervisionService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskTypeService;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 统计控制器
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/26 15:22
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysStatisticManager {

    private final SysPlanPkgService planPkgService;
    private final SysUserService sysUserService;
    private final SysDeptService deptService;
    private final WmScorePackageService packageService;
    private final WmTaskService taskService;
    private final WmTaskTypeService taskTypeService;
    private final WmTaskSupervisionService taskSupervisionService;
    private final SysAreaEntityService areaEntityService;
    private final SysUserAreaService userAreaService;
    private final SysEnterpriseAreaService enterpriseAreaService;
    private final SysImplementPlanService implementPlanService;
    private final WmQccInfoService qccInfoService;
    private final ReDeptDrugService deptDrugService;
    private final WmDaDrugEntDrugtableService drugTableService;
    private final WmAgentEvaluationService agentEvaluationService;
    private final WmScorePackageStatusService scorePackageStatusService;
    private final WmPackageRedemptionNoticeService packageRedemptionNoticeService;

    /**
     * 获取积分包统计
     *
     * @param operator 操作人
     * @return 包统计信息
     */
    public Map<String, Object> getPkgStat(HnqzUser operator) {
        Map<String, Object> res = new HashMap<>(3);
        Map<String, Long> total = new HashMap<>(2);
        Map<String, Long> finish = new HashMap<>(2);
        Map<String, Long> doing = new HashMap<>(2);

        // 查询全部的积分包
        List<WmScorePackage> packages = packageService.list(Wrappers.<WmScorePackage>lambdaQuery().eq(WmScorePackage::getSendPackageDeptId, operator.getDeptId()));

        if (CollUtil.isNotEmpty(packages)) {
            // 统计全部的信息
            total.put("count", (long) packages.size());
            total.put("sum", packages.stream().mapToLong(WmScorePackage::getScore).sum());

            // 统计已完成
            Map<String, List<WmScorePackage>> collect = packages.stream().collect(Collectors.groupingBy(WmScorePackage::getPackageFinishStatus));

            // 获取未完成的
            List<WmScorePackage> doingPkgs = collect.get("0");
            if (CollUtil.isNotEmpty(doingPkgs)) {
                doing.put("count", (long) doingPkgs.size());
                doing.put("sum", doingPkgs.stream().mapToLong(WmScorePackage::getScore).sum());
            }

            // 获取已完成的
            List<WmScorePackage> finishPkgs = collect.get("1");
            if (CollUtil.isNotEmpty(finishPkgs)) {
                finish.put("count", (long) finishPkgs.size());
                finish.put("sum", finishPkgs.stream().mapToLong(WmScorePackage::getScore).sum());
            }


        } else {
            total.put("count", 0L);
            total.put("sum", 0L);
            doing.put("count", 0L);
            doing.put("sum", 0L);
            finish.put("count", 0L);
            finish.put("sum", 0L);

        }

        res.put("total", total);
        res.put("finish", finish);
        res.put("doing", doing);

        return res;
    }


    /**
     * 获取任务的统计
     *
     * @param operator 操作人
     * @return 任务统计
     */
    public Map<String, Object> getTaskStat(HnqzUser operator) {

        // 查询全部的积分包
        List<WmScorePackage> packages = packageService.list(Wrappers.<WmScorePackage>lambdaQuery().eq(WmScorePackage::getSendPackageDeptId, operator.getDeptId()));

        // 过滤出有下级包的包id
        Set<String> pkgIds = packages.stream().filter(pkg -> StrUtil.isNotBlank(pkg.getRelationScoreId())).map(WmScorePackage::getId).collect(Collectors.toSet());

        if (CollUtil.isNotEmpty(pkgIds)) {
            List<WmScorePackage> pkgs = packageService.listByIds(pkgIds);
            Set<String> collect = pkgs.stream().filter(pkg -> StrUtil.isNotBlank(pkg.getRelationScoreId())).map(WmScorePackage::getId).collect(Collectors.toSet());

            if (CollUtil.isNotEmpty(collect)) {

                List<WmScorePackage> pkgs1 = packageService.listByIds(collect);
                Set<String> collect1 = pkgs1.stream().filter(pkg -> StrUtil.isNotBlank(pkg.getRelationScoreId())).map(WmScorePackage::getId).collect(Collectors.toSet());
                if (CollUtil.isNotEmpty(collect1)) {
                    List<WmScorePackage> pkgs2 = packageService.listByIds(collect1);
                    pkgIds = pkgs2.stream().filter(pkg -> StrUtil.isNotBlank(pkg.getRelationScoreId())).map(WmScorePackage::getId).collect(Collectors.toSet());

                } else {
                    pkgIds = pkgs1.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
                }

            } else {
                pkgIds = pkgs.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
            }


        } else {
            pkgIds = packages.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
        }

        Map<String, List<WmTask>> taskTypeAndTaskMap;
        if (CollUtil.isNotEmpty(pkgIds)) {
            List<WmTask> tasks = taskService.list(Wrappers.<WmTask>lambdaQuery().in(WmTask::getScorePackageId, pkgIds));

            taskTypeAndTaskMap = tasks.stream().collect(Collectors.groupingBy(WmTask::getTaskTypeId));
        } else {
            taskTypeAndTaskMap = Collections.emptyMap();
        }


        List<WmTaskType> taskTypes = taskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
                .eq(WmTaskType::getDelFlag, CommonConstants.STATUS_NORMAL)
                .lt(WmTaskType::getId, 1000));

        List<Map<String, Object>> taskTypeStatList = new ArrayList<>(taskTypes.size());

        long total = 0;
        for (WmTaskType taskType : taskTypes) {

            Map<String, Object> taskTypeAndSumMap = new HashMap<>(3);
            List<WmTask> wmTasks = taskTypeAndTaskMap.get(taskType.getId());

            long sum = CollUtil.isEmpty(wmTasks) ? 0L : wmTasks.stream().mapToLong(WmTask::getScore).sum();
            taskTypeAndSumMap.put("taskTypeName", taskType.getTaskTypeName());
            taskTypeAndSumMap.put("sum", sum);
            taskTypeAndSumMap.put("count", CollUtil.isEmpty(wmTasks) ? 0L : wmTasks.size());
            total += sum;
            taskTypeStatList.add(taskTypeAndSumMap);

        }


        Map<String, Object> res = new HashMap<>(2);
        res.put("details", taskTypeStatList);
        res.put("total", total);

        return res;
    }

    // /**
    //  * 获取服务目标统计
    //  *
    //  * @param user 操作人
    //  * @return 统计结果
    //  */
    // public List<Tree<Integer>> getImplPlanStat(HnqzUser user) {
    //
    //
    //     List<SysImplementPlan> list = new ArrayList<>(6);
    //
    //
    //
    //
    //     // List<TreeNode<Long>> collect = list.stream().map(getNodeFunction()).collect(Collectors.toList());
    //
    //
    //     // List<TreeNode<Long>> collect = baseMapper
    //     //         .selectList(Wrappers.<SysMenu>lambdaQuery()
    //     //                 .like(StrUtil.isNotBlank(menuName), SysMenu::getName, menuName)
    //     //                 .in(SysMenu::getMenuId, menuIds)
    //     //                 .eq(Objects.nonNull(type), SysMenu::getMenuType, type).orderByAsc(SysMenu::getSortOrder))
    //     //         .stream().map(getNodeFunction()).collect(Collectors.toList());
    //     //
    //     //
    //     //
    //     // return TreeUtil.build(collect, 1);
    //     return null;
    // }


    // @NotNull
    // private Function<SysMenu, TreeNode<Long>> getNodeFunction() {
    //     return menu -> {
    //         TreeNode<Long> node = new TreeNode<>();
    //         node.setId(menu.getMenuId());
    //         node.setName(menu.getName());
    //         node.setParentId(menu.getParentId());
    //         node.setWeight(menu.getSortOrder());
    //         // 扩展属性
    //         Map<String, Object> extra = new HashMap<>(5);
    //         extra.put("path", menu.getPath());
    //         extra.put("menuType", menu.getMenuType());
    //         extra.put("permission", menu.getPermission());
    //         extra.put("sortOrder", menu.getSortOrder());
    //
    //         // 适配 vue3
    //         Map<String, Object> meta = new HashMap<>(7);
    //         meta.put("title", menu.getName());
    //         meta.put("isLink", menu.getPath() != null && menu.getPath().startsWith("http") ? menu.getPath() : "");
    //         meta.put("isHide", !menu.getVisible());
    //         meta.put("isKeepAlive", menu.getKeepAlive());
    //         meta.put("isAffix", false);
    //         meta.put("isIframe", menu.getEmbedded());
    //         meta.put("icon", menu.getIcon());
    //
    //         extra.put("meta", meta);
    //         node.setExtra(extra);
    //         return node;
    //     };
    // }


    /**
     * 执行计划统计
     * 序号、目标名称、目标分值、目标局期、承接对象、创建日期、已下发机分值、已分发比例、服务完成比例
     *
     * @param query 查询条件
     * @param user  查询人
     * @return 统计信息
     */
    public Page<?> statPlanTree(SysStatisticDto.OnPlanTree query, HnqzUser user) {


        Page<SysImplementPlan> page = new Page<>(query.getCurrent(), query.getSize());

        // 获取当前操作人的角色
        List<Integer> roles = SecurityUtils.getRoles();

        LambdaQueryWrapper<SysImplementPlan> queryWrapper = Wrappers.<SysImplementPlan>lambdaQuery().eq(SysImplementPlan::getPlanState, UpmsState.ImplementPlanState.PASSED);

        boolean rootFlag = false;

        // 服务商
        if (roles.contains(37)) {
            queryWrapper.eq(SysImplementPlan::getConsigneeId, user.getDeptId());
        }
        // 区域管理员 || 地市经理
        else if (CollUtil.containsAny(roles, List.of(4, 43))) {
            queryWrapper.eq(SysImplementPlan::getConsignorId, user.getDeptId());

            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }

            queryWrapper.in(SysImplementPlan::getConsigneeId, entIds);

        }
        // 其他
        else {
            rootFlag = true;
            queryWrapper.eq(SysImplementPlan::getParentId, 0);
            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }

            queryWrapper.in(SysImplementPlan::getConsigneeId, entIds);
        }


        // 获取父级分页信息
        implementPlanService.page(page, queryWrapper);

        List<SysImplementPlan> plans = page.getRecords();

        if (CollUtil.isEmpty(plans)) {
            return page;
        }

        // 按照类型进行分类
        Map<UpmsType.ImplementPlanType, List<SysImplementPlan>> collect = plans.stream().collect(Collectors.groupingBy(SysImplementPlan::getPlanType));

        // 获取季度的信息
        List<SysImplementPlan> qrtPlanList = collect.get(UpmsType.ImplementPlanType.QRT);

        List<SysImplementPlan> cont = new LinkedList<>(plans);

        if (CollUtil.isNotEmpty(qrtPlanList)) {
            while (true) {

                // 获取子集
                List<Integer> parentQrtPlanIds = qrtPlanList.stream().mapToInt(SysImplementPlan::getPlanId).boxed().distinct().sorted().collect(Collectors.toList());

                List<SysImplementPlan> subQrtPlans = implementPlanService.list(Wrappers.<SysImplementPlan>lambdaQuery()
                        .eq(SysImplementPlan::getPlanState, UpmsState.ImplementPlanState.PASSED)
                        .in(SysImplementPlan::getParentId, parentQrtPlanIds));

                if (CollUtil.isEmpty(subQrtPlans)) {
                    break;
                }

                cont.addAll(subQrtPlans);

                qrtPlanList = subQrtPlans;

            }
        }

        // 获取承接对象信息
        List<Integer> deptIds = cont.stream().mapToInt(SysImplementPlan::getConsigneeId).distinct().boxed().sorted().collect(Collectors.toList());
        Map<Integer, String> dId2NameMap = deptService.list(Wrappers.<SysDept>lambdaQuery().in(SysDept::getDeptId, deptIds)).stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getName));


        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy/MM/dd");
        // 目标名称、目标分值、目标局期、承接对象、创建日期、已下发分值、已分发比例、服务完成比例
        List<TreeNode<Integer>> treeNodes = cont.stream().map(plan -> {
            TreeNode<Integer> node = new TreeNode<>();
            node.setId(plan.getPlanId());
            node.setParentId(plan.getParentId());
            // 目标名称
            node.setName(plan.getPlanName());
            node.setWeight(plan.getPlanId());

            Map<String, Object> extra = new HashMap<>(8);

            // 承接对象
            extra.put("providerName", dId2NameMap.get(plan.getConsigneeId()));
            // 目标分值
            extra.put("planScore", plan.getPlanScore());
            // 目标周期
            extra.put("planPeriod", String.format("%s 至 %s", formatter.format(plan.getPlanIssue()), formatter.format(plan.getPlanExpiry())));
            // 创建日期
            extra.put("createTime", plan.getCreateTime());
            // 计划类型
            extra.put("planType", plan.getPlanType());
            // 已下发积分值
            extra.put("availScore", plan.getPlanScore() - plan.getAvailScore());
            // 分发比例
            extra.put("distScoreRate", String.format("%.2f", 100.0 * (plan.getPlanScore() - plan.getAvailScore()) / plan.getPlanScore()));


            String serviceCompleteRate = "";
            List<SysPlanPkg> pp = planPkgService.list(Wrappers.<SysPlanPkg>lambdaQuery().eq(SysPlanPkg::getPlanId, plan.getPlanId()));

            if (CollUtil.isNotEmpty(pp)) {

                List<String> collect1 = pp.stream().map(r -> r.getPkgId().toString()).distinct().sorted().collect(Collectors.toList());

                List<WmTask> tasks = taskService.list(Wrappers.<WmTask>lambdaQuery()
                        .in(WmTask::getTaskStatus, UpmsState.TaskState.PASSED)
                        .in(WmTask::getScorePackageId, collect1));

                if (CollUtil.isNotEmpty(tasks)) {
                    serviceCompleteRate = String.valueOf(tasks.stream().mapToInt(WmTask::getScore).sum());
                }

            }
            // 服务完成比例
            extra.put("serviceCompleteRate", serviceCompleteRate);


            node.setExtra(extra);

            return node;
        }).collect(Collectors.toList());

        List<Tree<Integer>> temps = new ArrayList<>();

        if (rootFlag) {
            List<Tree<Integer>> builds = TreeUtil.build(treeNodes, 0);
            temps.addAll(builds);
        } else {

            for (TreeNode<Integer> node : treeNodes) {
                List<Tree<Integer>> build = TreeUtil.build(Collections.singletonList(node), node.getParentId());
                temps.addAll(build);
            }

        }

        Page<Tree<Integer>> build = new Page<>();
        build.setRecords(temps);
        build.setCurrent(page.getCurrent());
        build.setSize(page.getSize());
        build.setTotal(page.getTotal());

        return build;


    }

    /**
     * 所有执行包数据汇总树状展示
     * 序号、执行包名称、执行包分值、接包对象、创建日期、宛成分值、完成分值百分比
     *
     * @param query 查询条件
     * @param user  操作人
     * @return 统计信息
     */
    public Page<?> statPkgTree(SysStatisticDto.OnPkgTree query, HnqzUser user) {

        // 获取所有父级执行包
        Page<WmScorePackage> page = new Page<>(query.getCurrent(), query.getSize());

        // 获取当前操作人的角色
        List<Integer> roles = SecurityUtils.getRoles();

        LambdaQueryWrapper<WmScorePackage> queryWrapper = Wrappers.<WmScorePackage>lambdaQuery().eq(WmScorePackage::getEnableFlag, EnableEnum.ENABLE.val());

        boolean rootFlag = false;

        // 服务商
        if (roles.contains(37)) {
            queryWrapper.eq(WmScorePackage::getDeptId, user.getDeptId());
        }
        // 区域管理员 || 地市经理
        else if (CollUtil.containsAny(roles, List.of(4, 43))) {
            queryWrapper.eq(WmScorePackage::getSendPackageDeptId, user.getDeptId());

            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }

            queryWrapper.in(WmScorePackage::getDeptId, entIds);
        }
        // 其他
        else {
            rootFlag = true;
            queryWrapper.eq(WmScorePackage::getRelationScoreId, "");

            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }

            queryWrapper.in(WmScorePackage::getDeptId, entIds);
        }


        packageService.page(page, queryWrapper);

        List<WmScorePackage> pkgs = page.getRecords();

        if (CollUtil.isEmpty(pkgs)) {
            return page;
        }

        List<WmScorePackage> cont = new LinkedList<>(pkgs);
        if (CollUtil.isNotEmpty(pkgs)) {
            while (true) {

                // 获取子集
                List<String> parentPkgIds = pkgs.stream().map(WmScorePackage::getId).distinct().sorted().collect(Collectors.toList());

                List<WmScorePackage> subPkgs = packageService.list(Wrappers.<WmScorePackage>lambdaQuery()
                        .eq(WmScorePackage::getEnableFlag, EnableEnum.ENABLE.val())
                        .in(WmScorePackage::getRelationScoreId, parentPkgIds));

                if (CollUtil.isEmpty(subPkgs)) {
                    break;
                }

                cont.addAll(subPkgs);

                pkgs = subPkgs;
            }
        }

        // 获取承接对象信息
        List<Integer> deptIds = cont.stream().mapToInt(p -> Integer.parseInt(p.getDeptId())).distinct().boxed().sorted().collect(Collectors.toList());
        Map<Integer, String> dId2NameMap = deptService.list(Wrappers.<SysDept>lambdaQuery().in(SysDept::getDeptId, deptIds)).stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getName));

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy/MM/dd");
        List<TreeNode<Integer>> treeNodes = cont.stream().map(pkg -> {
            TreeNode<Integer> node = new TreeNode<>();
            node.setId(Integer.parseInt(pkg.getId()));
            node.setParentId(StrUtil.isBlank(pkg.getRelationScoreId()) ? 0 : Integer.parseInt(pkg.getRelationScoreId()));
            // 执行包名称
            node.setName(pkg.getScorePackageName());
            node.setWeight(pkg.getScore());

            Map<String, Object> extra = new HashMap<>(9);

            int availScore = (UpmsType.PackageType.TYPE_3.getType().equals(pkg.getTypeid())) ? 0 : pkg.getKfpjf();

            // 执行包分值
            extra.put("pkgScore", pkg.getScore());
            // 可用分值
            extra.put("availScore", availScore);
            // 下发分值
            extra.put("distScore", pkg.getScore() - availScore);
            // 下发百分比
            extra.put("distRate", String.format("%.2f", 100.0 * (pkg.getScore() - availScore) / pkg.getScore()));
            // 承接对象
            extra.put("pickerName", dId2NameMap.get(Integer.valueOf(pkg.getDeptId())) == null ? "" : dId2NameMap.get(Integer.valueOf(pkg.getDeptId())));
            // 周期
            extra.put("pkgPeriod", String.format("%s 至 %s", formatter.format(pkg.getStartTime()), formatter.format(pkg.getEndTime())));
            // 创建时间
            extra.put("createTime", pkg.getCreateTime());

            // 完成分值
            List<WmTask> tasks = taskService.list(Wrappers.<WmTask>lambdaQuery()
                    .eq(WmTask::getTaskStatus, UpmsState.TaskState.PASSED)
                    .eq(WmTask::getScorePackageId, pkg.getId()));

            int sum = 0;
            if (CollUtil.isNotEmpty(tasks)) {
                sum = tasks.stream().mapToInt(WmTask::getScore).sum();
            }
            extra.put("completedScore", sum);

            // 完成百分比
            extra.put("completedRate", String.format("%.2f", 100.0 * sum / pkg.getScore()));
            node.setExtra(extra);

            return node;
        }).collect(Collectors.toList());


        List<Tree<Integer>> temps = new ArrayList<>();

        if (rootFlag) {
            List<Tree<Integer>> builds = TreeUtil.build(treeNodes, 0);
            temps.addAll(builds);
        } else {

            for (TreeNode<Integer> node : treeNodes) {
                List<Tree<Integer>> build = TreeUtil.build(Collections.singletonList(node), node.getParentId());
                temps.addAll(build);
            }

        }

        Page<Tree<Integer>> page1 = new Page<>();
        page1.setRecords(temps);
        page1.setCurrent(page.getCurrent());
        page1.setSize(page.getSize());
        page1.setTotal(page.getTotal());

        return page1;
    }

    /**
     * 统计服务商数据
     * <p>
     * 序号、服务商名称、所属区域、服务产品、企查查评分、服务评分、服务商平均分
     *
     * @param query 查询条件
     * @param user  操作人
     * @return 统计信息
     */
    public Page<?> statProviderData(SysStatisticDto.OnProviderQccData query, HnqzUser user) {

        // 获取服务商分页
        Page<SysDept> page = new Page<>(query.getCurrent(), query.getSize());

        // 获取当前操作人的角色
        List<Integer> roles = SecurityUtils.getRoles();

        LambdaQueryWrapper<SysDept> queryWrapper = Wrappers.<SysDept>lambdaQuery().eq(SysDept::getLevel, 4).eq(SysDept::getDelFlag, CommonConstants.STATUS_NORMAL);


        // 服务商
        if (roles.contains(37)) {
            queryWrapper.eq(SysDept::getDeptId, user.getDeptId());
        }
        // 区域管理员 || 地市经理
        else if (CollUtil.containsAny(roles, List.of(4, 43))) {

            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }
            queryWrapper.in(SysDept::getDeptId, entIds);
        }
        // 其他
        else {
            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }
            queryWrapper.in(SysDept::getDeptId, entIds);
        }


        deptService.page(page, queryWrapper);

        List<SysDept> depts = page.getRecords();

        if (CollUtil.isEmpty(depts)) {
            return page;
        }

        // 获取企业的服务区域
        List<Long> deptIds = depts.stream().mapToLong(SysDept::getDeptId).boxed().distinct().sorted().collect(Collectors.toList());
        Map<Long, List<Long>> detpId2AreaCodesMap = enterpriseAreaService.listEntAreas1(deptIds);

        // 获取企查查信息
        List<String> regCodes = depts.stream().map(SysDept::getTaxCode).distinct().sorted().collect(Collectors.toList());
        Map<String, List<WmQccInfo>> qccInfo = qccInfoService.getQccInfo1(regCodes);

        // 企业评分
        Map<Integer, List<WmAgentEvaluation>> dept2EvalMap = agentEvaluationService.list(Wrappers.<WmAgentEvaluation>lambdaQuery()
                .in(WmAgentEvaluation::getDeptId, deptIds)
                .orderByDesc(WmAgentEvaluation::getCreateTime)).stream().collect(Collectors.groupingBy(WmAgentEvaluation::getDeptId));


        // 获取服务商的产品信息
        List<ReDeptDrug> deptDrugs = deptDrugService.list(Wrappers.<ReDeptDrug>lambdaQuery().in(ReDeptDrug::getDeptId, deptIds.stream().mapToInt(Long::intValue).boxed().collect(Collectors.toList())));
        Map<Integer, List<String>> drugMap;
        if (CollUtil.isNotEmpty(deptDrugs)) {
            List<String> drugIds = deptDrugs.stream().map(dd -> dd.getDrugId().toString()).distinct().sorted().collect(Collectors.toList());
            Map<String, String> drugtableMap = drugTableService.list(Wrappers.<WmDaDrugEntDrugtable>lambdaQuery()
                            .eq(WmDaDrugEntDrugtable::getDelFlag, CommonConstants.STATUS_NORMAL)
                            .eq(WmDaDrugEntDrugtable::getEnableFlag, CommonConstants.STATUS_NORMAL)
                            .in(WmDaDrugEntDrugtable::getId, drugIds))
                    .stream()
                    .collect(Collectors.toMap(WmDaDrugEntDrugtable::getId, WmDaDrugEntDrugtable::getDrugnameTy));

            drugMap = deptDrugs.stream().collect(Collectors.groupingBy(ReDeptDrug::getDeptId, Collectors.mapping(dd -> drugtableMap.get(dd.getDrugId().toString()), Collectors.toList())));

        } else {
            drugMap = Collections.emptyMap();
        }


        // 封装参数 序号、服务商名称、所属区域、服务产品、企查查评分、服务评分、服务商平均分
        List<Map<String, Object>> collect = depts.stream().map(d -> {
            Map<String, Object> map = new HashMap<>(6);

            // 企业ID
            map.put("id", d.getDeptId());
            // 服务商名称
            map.put("providerName", d.getName());
            // 区域（只显示省份）
            List<Long> areaCodes = detpId2AreaCodesMap.get((long) d.getDeptId());
            List<String> provinceNames = CollUtil.isEmpty(areaCodes)
                    ? Collections.emptyList()
                    : areaEntityService.list(Wrappers.<SysAreaEntity>lambdaQuery()
                            .in(SysAreaEntity::getAreaType, UpmsType.AreaType.PROVINCE)
                            .in(SysAreaEntity::getAreaId, areaCodes))
                    .stream()
                    .map(SysAreaEntity::getAreaName)
                    .distinct()
                    .collect(Collectors.toList());
            ;

            map.put("areaCodes", provinceNames);
            // 服务产品
            List<String> drugNames = drugMap.get(d.getDeptId());
            map.put("products", drugNames);

            // 企查查评分
            Map<String, Object> creditRatingReportGrade;
            if (StrUtil.isNotBlank(d.getTaxCode())) {
                List<WmQccInfo> wmQccInfos = qccInfo.get(d.getTaxCode());
                if (CollUtil.isNotEmpty(wmQccInfos)) {
                    creditRatingReportGrade = wmQccInfos.get(0).getCreditRatingReportGrade();
                } else {
                    creditRatingReportGrade = Collections.emptyMap();
                }
            } else {
                creditRatingReportGrade = Collections.emptyMap();
            }

            map.put("qccScore", creditRatingReportGrade);

            // 服务商评分
            List<WmAgentEvaluation> agentEvaluations = dept2EvalMap.get(d.getDeptId());
            map.put("serviceMark", CollUtil.isNotEmpty(agentEvaluations) ? agentEvaluations.get(0) : null);

            return map;
        }).collect(Collectors.toList());


        Page<Map<String, Object>> builds = new Page<>(query.getCurrent(), query.getSize());
        builds.setRecords(collect);
        builds.setTotal(page.getTotal());

        return builds;

    }


    /**
     * 统计服务商数据
     * <p>
     * 序号、服务商名称、所属区域、服务产品、企查查评分、服务评分、服务商平均分
     *
     * @param query 查询条件
     * @param user  操作人
     * @return 统计信息
     */
    public Page<?> statProviderDataNew(SysStatisticDto.OnProviderQccData query, HnqzUser user) {

        // 获取服务商分页
        Page<SysDept> page = new Page<>(query.getCurrent(), query.getSize());

        // 获取当前操作人的角色
        List<Integer> roles = SecurityUtils.getRoles();

        LambdaQueryWrapper<SysDept> queryWrapper = Wrappers.<SysDept>lambdaQuery().eq(SysDept::getLevel, 4).eq(SysDept::getDelFlag, CommonConstants.STATUS_NORMAL);


        // 服务商
        if (roles.contains(37)) {
            queryWrapper.eq(SysDept::getDeptId, user.getDeptId());
        }
        // 区域管理员 || 地市经理
        else if (CollUtil.containsAny(roles, List.of(4, 43))) {

            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }
            queryWrapper.in(SysDept::getDeptId, entIds);
        }
        // 其他
        else {
            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }
            queryWrapper.in(SysDept::getDeptId, entIds);
        }


        deptService.page(page, queryWrapper);

        List<SysDept> depts = page.getRecords();

        if (CollUtil.isEmpty(depts)) {
            return page;
        }

        // 获取企业的服务区域
        List<Long> deptIds = depts.stream().mapToLong(SysDept::getDeptId).boxed().distinct().sorted().collect(Collectors.toList());
        Map<Long, List<Long>> detpId2AreaCodesMap = enterpriseAreaService.listEntAreas1(deptIds);

        // 获取企查查信息
        List<String> regCodes = depts.stream().map(SysDept::getTaxCode).distinct().sorted().collect(Collectors.toList());
        Map<String, List<WmQccInfo>> qccInfo = qccInfoService.getQccInfo1(regCodes);

        // 获取服务评分
        List<WmScorePackage> pkgs = packageService.list(Wrappers.<WmScorePackage>lambdaQuery()
                .eq(WmScorePackage::getTypeid, UpmsType.PackageType.TYPE_2)
                .eq(WmScorePackage::getEnableFlag, EnableEnum.ENABLE.val()));

        List<Integer> pIds = pkgs.stream().mapToInt(p -> Integer.parseInt(p.getId())).boxed().distinct().sorted().collect(Collectors.toList());

        Map<String, List<WmScorePackage>> deptPackageMap = pkgs.stream().collect(Collectors.groupingBy(WmScorePackage::getDeptId));

        Map<Integer, List<WmPackageRedemptionNotice>> noticeMap = CollUtil.isEmpty(pIds) ? Collections.emptyMap() : packageRedemptionNoticeService.list(Wrappers.<WmPackageRedemptionNotice>lambdaQuery()
                .in(WmPackageRedemptionNotice::getPackageId, pIds)).stream().collect(Collectors.groupingBy(WmPackageRedemptionNotice::getPackageId));


        // 获取服务商的产品信息
        List<ReDeptDrug> deptDrugs = deptDrugService.list(Wrappers.<ReDeptDrug>lambdaQuery().in(ReDeptDrug::getDeptId, deptIds.stream().mapToInt(Long::intValue).boxed().collect(Collectors.toList())));
        Map<Integer, List<String>> drugMap;
        if (CollUtil.isNotEmpty(deptDrugs)) {
            List<String> drugIds = deptDrugs.stream().map(dd -> dd.getDrugId().toString()).distinct().sorted().collect(Collectors.toList());
            Map<String, String> drugtableMap = drugTableService.list(Wrappers.<WmDaDrugEntDrugtable>lambdaQuery()
                            .eq(WmDaDrugEntDrugtable::getDelFlag, CommonConstants.STATUS_NORMAL)
                            .eq(WmDaDrugEntDrugtable::getEnableFlag, CommonConstants.STATUS_NORMAL)
                            .in(WmDaDrugEntDrugtable::getId, drugIds))
                    .stream()
                    .collect(Collectors.toMap(WmDaDrugEntDrugtable::getId, WmDaDrugEntDrugtable::getDrugnameTy));

            drugMap = deptDrugs.stream().collect(Collectors.groupingBy(ReDeptDrug::getDeptId, Collectors.mapping(dd -> drugtableMap.get(dd.getDrugId().toString()), Collectors.toList())));

        } else {
            drugMap = Collections.emptyMap();
        }


        // 封装参数 序号、服务商名称、所属区域、服务产品、企查查评分、服务评分、服务商平均分
        List<Map<String, Object>> collect = depts.stream().map(d -> {
            Map<String, Object> map = new HashMap<>(6);

            // 企业ID
            map.put("id", d.getDeptId());
            // 服务商名称
            map.put("providerName", d.getName());
            // 区域（只显示省份）
            List<Long> areaCodes = detpId2AreaCodesMap.get((long) d.getDeptId());
            List<String> provinceNames = CollUtil.isEmpty(areaCodes)
                    ? Collections.emptyList()
                    : areaEntityService.list(Wrappers.<SysAreaEntity>lambdaQuery()
                            .in(SysAreaEntity::getAreaType, UpmsType.AreaType.PROVINCE)
                            .in(SysAreaEntity::getAreaId, areaCodes))
                    .stream()
                    .map(SysAreaEntity::getAreaName)
                    .distinct()
                    .collect(Collectors.toList());
            ;

            map.put("areaCodes", provinceNames);
            // 服务产品
            List<String> drugNames = drugMap.get(d.getDeptId());
            map.put("products", drugNames);

            // 企查查评分
            Map<String, Object> creditRatingReportGrade;
            if (StrUtil.isNotBlank(d.getTaxCode())) {
                List<WmQccInfo> wmQccInfos = qccInfo.get(d.getTaxCode());
                if (CollUtil.isNotEmpty(wmQccInfos)) {
                    creditRatingReportGrade = wmQccInfos.get(0).getCreditRatingReportGrade();
                } else {
                    creditRatingReportGrade = Collections.emptyMap();
                }
            } else {
                creditRatingReportGrade = Collections.emptyMap();
            }

            map.put("qccScore", creditRatingReportGrade);

            // 服务商评分
            List<WmScorePackage> packages = deptPackageMap.get(d.getDeptId().toString());

            List<Map<String, Object>> pMaps = CollUtil.isEmpty(packages) ? Collections.emptyList() : packages.stream().map(p -> {
                Map<String, Object> m = new HashMap<>(3);
                m.put("id", p.getId());
                m.put("pkgName", p.getScorePackageName());
                m.put("redemptionNotices", noticeMap.get(Integer.parseInt(p.getId())));

                return m;
            }).collect(Collectors.toList());


            map.put("packages", pMaps);

            return map;
        }).collect(Collectors.toList());


        Page<Map<String, Object>> builds = new Page<>(query.getCurrent(), query.getSize());
        builds.setRecords(collect);
        builds.setTotal(page.getTotal());

        return builds;

    }

    /**
     * 服务提供商数据
     *
     * @param query 查询条件
     * @param user  操作人
     * @return 统计信息
     */
    public Page<?> statProviderDataTree(SysStatisticDto.OnProviderTree query, HnqzUser user) {


        // 获取服务商分页
        Page<SysDept> page = new Page<>(query.getCurrent(), query.getSize());

        // 获取当前操作人的角色
        List<Integer> roles = SecurityUtils.getRoles();

        LambdaQueryWrapper<SysDept> queryWrapper = Wrappers.<SysDept>lambdaQuery().eq(SysDept::getLevel, 4).eq(SysDept::getDelFlag, CommonConstants.STATUS_NORMAL);


        // 服务商
        if (roles.contains(37)) {
            queryWrapper.eq(SysDept::getDeptId, user.getDeptId());
        }
        // 区域管理员 || 地市经理
        else if (CollUtil.containsAny(roles, List.of(4, 43))) {

            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }
            queryWrapper.in(SysDept::getDeptId, entIds);
        }
        // 其他
        else {
            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }
            queryWrapper.in(SysDept::getDeptId, entIds);
        }

        // 获取服务商分页
        deptService.page(page, queryWrapper);


        List<SysDept> depts = page.getRecords();

        if (CollUtil.isEmpty(depts)) {
            return page;
        }

        // 获取企业ID
        List<Integer> deptIds = depts.stream().mapToInt(SysDept::getDeptId).boxed().distinct().sorted().collect(Collectors.toList());

        // 获取员工
        List<SysUser> users = sysUserService.list(Wrappers.<SysUser>lambdaQuery().in(SysUser::getDeptId, deptIds)
                .eq(SysUser::getLockFlag, CommonConstants.STATUS_NORMAL)
                .eq(SysUser::getDelFlag, CommonConstants.STATUS_NORMAL));

        if (CollUtil.isEmpty(users)) {
            return page;
        }

        List<String> userIds = users.stream().map(u -> u.getUserId().toString()).distinct().sorted().collect(Collectors.toList());


        Map<Integer, List<SysUser>> userCollect = users.stream().collect(Collectors.groupingBy(SysUser::getDeptId));

        // 获取员工的包记录
        Map<String, List<WmScorePackageStatus>> userId2PkgMap = scorePackageStatusService.list(Wrappers.<WmScorePackageStatus>lambdaQuery().in(WmScorePackageStatus::getUserId, userIds))
                .stream().collect(Collectors.groupingBy(WmScorePackageStatus::getUserId));

        // 获取员工的任务数
        Map<String, List<WmTask>> userId2TaskMap = taskService.list(Wrappers.<WmTask>lambdaQuery().in(WmTask::getTaskUserId, userIds)).stream().collect(Collectors.groupingBy(WmTask::getTaskUserId));


        List<Map<String, Object>> buildMap = new ArrayList<>(depts.size());

        for (SysDept dept : depts) {
            Map<String, Object> map = new HashMap<>(7);
            // 服务商ID
            map.put("id", dept.getDeptId());
            // 服务商名称
            map.put("name", dept.getName());

            List<SysUser> userList = userCollect.get(dept.getDeptId());
            List<Map<String, Object>> userMap;
            int deptPkgCount = 0;
            long deptPkgScoreTotal = 0L;
            int deptTaskCount = 0;
            long deptTaskScoreTotal = 0L;
            if (CollUtil.isNotEmpty(userList)) {

                userMap = new ArrayList<>(userList.size());
                for (SysUser u : userList) {
                    String uId = u.getUserId().toString();
                    Map<String, Object> uMap = new HashMap<>(13);
                    // 用户ID
                    uMap.put("userId", u.getUserId());
                    // 用户名
                    uMap.put("userName", u.getRealname());
                    // 联系方式
                    uMap.put("phone", u.getUsername());

                    int userPkgCount = 0;
                    long userPkgScoreTotal = 0L;
                    List<WmScorePackageStatus> wmScorePackageStatuses = userId2PkgMap.get(uId);
                    if (CollUtil.isNotEmpty(wmScorePackageStatuses)) {
                        userPkgCount = wmScorePackageStatuses.size();
                        userPkgScoreTotal = wmScorePackageStatuses.stream().mapToLong(s -> Long.parseLong(s.getUserScore())).sum();
                    }

                    // 用户包数
                    uMap.put("userPkgCount", userPkgCount);
                    // 用户包值
                    uMap.put("userPkgScoreTotal", userPkgScoreTotal);
                    deptPkgCount += userPkgCount;
                    deptPkgScoreTotal += userPkgScoreTotal;

                    int taskPkgCount = 0;
                    int taskScoreTotal = 0;
                    int passedScore = 0;
                    int passedCount = 0;
                    List<WmTask> tasks = userId2TaskMap.get(uId);
                    if (CollUtil.isNotEmpty(tasks)) {
                        Map<String, List<WmTask>> statusMap = tasks.stream().collect(Collectors.groupingBy(WmTask::getTaskStatus));
                        List<WmTask> passed = statusMap.get(UpmsState.TaskState.PASSED.getState());
                        if (CollUtil.isNotEmpty(passed)) {
                            passedScore = passed.stream().mapToInt(WmTask::getScore).sum();
                            passedCount = passed.size();
                        }

                        taskPkgCount = tasks.size();
                        taskScoreTotal = tasks.stream().mapToInt(WmTask::getScore).sum();
                    }

                    // 审核通过值
                    uMap.put("passedScore", passedScore);
                    uMap.put("passedCount", passedCount);
                    uMap.put("taskPkgCount", taskPkgCount);
                    // 审核通过率
                    uMap.put("passedCountRate", taskPkgCount == 0 ? "0.00" : String.format("%.2f", 100.0 * (passedCount / taskPkgCount)));
                    // 审核通过百分比
                    uMap.put("passedRate", taskScoreTotal == 0 ? "0.00" : String.format("%.2f", 100.0 * (passedScore / taskScoreTotal)));

                    // 任务数
                    uMap.put("userTaskCount", taskPkgCount);
                    // 任务总值
                    uMap.put("taskScoreTotal", taskScoreTotal);
                    // 审核通过百分比
                    uMap.put("taskScoreRate", userPkgScoreTotal == 0 ? "0.00" : String.format("%.2f", 100.0 * (taskScoreTotal / userPkgScoreTotal)));


                    deptTaskCount += taskPkgCount;
                    deptTaskScoreTotal += taskScoreTotal;
                    userMap.add(uMap);


                }
            } else {
                userMap = Collections.emptyList();
            }

            // 用户列表
            map.put("userList", userMap);
            map.put("deptPkgCount", deptPkgCount);
            map.put("deptPkgScoreTotal", deptPkgScoreTotal);
            map.put("deptTaskCount", deptTaskCount);
            map.put("deptTaskScoreTotal", deptTaskScoreTotal);
            buildMap.add(map);

        }


        Page<Map<String, Object>> builds = new Page<>(query.getCurrent(), query.getSize());
        builds.setRecords(buildMap);
        builds.setTotal(page.getTotal());

        return builds;
    }

    /**
     * 监督信息
     *
     * @param query 查询条件
     * @param user  查询人
     * @return 统计结果
     */
    public Page<?> statSupData(SysStatisticDto.OnSupData query, HnqzUser user) {

        Page<SysDept> page = new Page<>(query.getCurrent(), query.getSize());

        // 获取当前操作人的角色
        List<Integer> roles = SecurityUtils.getRoles();

        LambdaQueryWrapper<SysDept> queryWrapper = Wrappers.<SysDept>lambdaQuery().eq(SysDept::getLevel, 4).eq(SysDept::getDelFlag, CommonConstants.STATUS_NORMAL);


        // 服务商
        if (roles.contains(37)) {
            queryWrapper.eq(SysDept::getDeptId, user.getDeptId());
        }
        // 区域管理员 || 地市经理
        else if (CollUtil.containsAny(roles, List.of(4, 43))) {

            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }
            queryWrapper.in(SysDept::getDeptId, entIds);
        }
        // 其他
        else {
            List<Integer> entIds = getEntIds(user);

            if (CollUtil.isEmpty(entIds)) {
                return page;
            }
            queryWrapper.in(SysDept::getDeptId, entIds);
        }


        // 获取服务商分页

        deptService.page(page, queryWrapper);


        List<SysDept> depts = page.getRecords();

        if (CollUtil.isEmpty(depts)) {
            return page;
        }

        // 封装参数
        List<Map<String, Object>> collect = depts.stream().map(dept -> {
            Map<String, Object> map = new HashMap<>(3);
            // 服务商名称
            map.put("deptName", dept.getName());
            // 服务商ID
            map.put("deptId", dept.getDeptId());

            // 获取当前企业当季度的包
            List<WmScorePackage> pkgs = packageService.list(Wrappers.<WmScorePackage>lambdaQuery().eq(WmScorePackage::getSendPackageDeptId, dept.getDeptId().toString()));

            int total = 0;
            int supTotal = 0;
            int passedScoreTotal = 0;
            int passedTotal = 0;
            int rejectTotal = 0;

            // 获取c端包统计信息
            if (CollUtil.isNotEmpty(pkgs)) {

                List<String> pkgIds = pkgs.stream().map(WmScorePackage::getId).distinct().sorted().collect(Collectors.toList());
                // 获取子集包中的所有任务
                List<WmTask> tasks = taskService.listAvailableTaskForSupervision1(pkgIds.stream().mapToInt(Integer::valueOf).boxed().distinct().sorted().collect(Collectors.toList()), Collections.emptyList());
                // List<WmTask> tasks = taskService.list(Wrappers.<WmTask>lambdaQuery().in(WmTask::getScorePackageId, pkgIds).eq(WmTask::getTaskStatus, "3"));
                // 获取监察记录(不可编辑的是监察过的)
                List<WmTaskSupervision> supervisions = taskSupervisionService.list(Wrappers.<WmTaskSupervision>lambdaQuery()
                        .eq(WmTaskSupervision::getModifiable, false)
                        .eq(WmTaskSupervision::getSupervisionState, UpmsState.TaskSupervisionState.PASSED)
                        .in(WmTaskSupervision::getPackageId, pkgIds.stream().mapToInt(Integer::valueOf).boxed().sorted().distinct().collect(Collectors.toList())));

                total = CollUtil.isNotEmpty(tasks) ? tasks.size() : 0;
                supTotal = CollUtil.isNotEmpty(supervisions) ? supervisions.stream().map(WmTaskSupervision::getTaskId).collect(Collectors.toSet()).size() : 0;

                if (CollUtil.isNotEmpty(supervisions)) {

                    List<Integer> collect2 = supervisions.stream().mapToInt(WmTaskSupervision::getTaskId).boxed().distinct().sorted().collect(Collectors.toList());

                    passedScoreTotal = taskService.listByIds(collect2).stream().mapToInt(WmTask::getScore).sum();

                    Map<Boolean, List<WmTaskSupervision>> collect1 = supervisions.stream().collect(Collectors.partitioningBy(WmTaskSupervision::getSupervisionResult));

                    passedTotal = CollUtil.isEmpty(collect1.get(Boolean.TRUE)) ? 0 : collect1.get(Boolean.TRUE).size();
                    rejectTotal = CollUtil.isEmpty(collect1.get(Boolean.FALSE)) ? 0 : collect1.get(Boolean.FALSE).size();
                }
            }


            Map<String, Integer> stat = new HashMap<>(5);
            // 抽查总数
            stat.put("supTotal", supTotal);
            // 任务总数
            stat.put("total", total);
            // 抽查分值
            stat.put("passedScoreTotal", passedScoreTotal);
            // 抽查通过数
            stat.put("passedTotal", passedTotal);
            // 抽查预警率
            stat.put("rejectTotal", rejectTotal);
            // 统计信息
            map.put("taskSupStat", stat);

            return map;
        }).collect(Collectors.toList());


        Page<Map<String, Object>> builds = new Page<>(page.getCurrent(), page.getSize());

        builds.setRecords(collect);
        builds.setTotal(page.getTotal());

        return builds;
    }


    private List<Integer> getEntIds(HnqzUser user) {

        // 获取当前用户的区域
        List<Long> list = userAreaService.listUserAreas((long) user.getId());

        if (CollUtil.isEmpty(list)) {
            return Collections.emptyList();
        }

        // 获取市级
        List<Long> collect = areaEntityService.list(Wrappers.<SysAreaEntity>lambdaQuery()
                        .in(SysAreaEntity::getAreaId, list)
                        .eq(SysAreaEntity::getAreaType, UpmsType.AreaType.DISTRICT))
                .stream()
                .mapToLong(SysAreaEntity::getAreaId)
                .boxed()
                .distinct()
                .sorted()
                .collect(Collectors.toList());

        if (CollUtil.isEmpty(collect)) {
            return Collections.emptyList();
        }

        // 获取企业
        return enterpriseAreaService.listAreaEnts(collect).stream().mapToInt(Long::intValue).boxed().distinct().sorted().collect(Collectors.toList());

    }


}
