package com.qunzhixinxi.hnqz.admin.mapper;

import com.baomidou.mybatisplus.annotation.SqlParser;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.dto.UserDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.vo.UserVO;
import com.qunzhixinxi.hnqz.common.data.datascope.DataScope;
import com.qunzhixinxi.hnqz.common.data.datascope.DataScopeMapper;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 用户表 Mapper 接口
 * </p>
 *
 * @author hnqz
 * @since 2017-10-29
 */
@Mapper
public interface SysUserMapper extends DataScopeMapper<SysUser> {

    /**
     * 通过用户名查询用户信息（含有角色信息）
     *
     * @param username 用户名
     * @return userVo
     */
    UserVO getUserVoByUsername(String username);

    /**
     * 分页查询用户信息（含角色）
     *
     * @param page      分页
     * @param userDTO   查询参数
     * @param dataScope 作用域
     * @return list
     */
    Page<UserVO> getUserVosPage(Page<UserVO> page, @Param("query") UserDTO userDTO, DataScope dataScope);

    /**
     * 分页查询用户信息（含角色）
     *
     * @param page      分页
     * @param userDTO   查询参数
     * @return list
     */
    Page<UserVO> getUserVosPage1(Page<UserVO> page, @Param("query") UserDTO userDTO);

    Page<UserVO> getUserVosPageNoLock(Page<UserVO> page, @Param("query") UserDTO userDTO, DataScope dataScope);

    List<UserVO> getUserVoByPlatId(@Param("query") UserDTO userDTO);

    List<UserVO> selectByPhone(@Param("query") UserDTO userDTO);

    @SqlParser(filter = true)
    List<UserVO> selectByDeptId(@Param("query") UserDTO userDTO);

    @SqlParser(filter = true)
    List<UserVO> selectByDeptId1(@Param("query") UserDTO userDTO);

    UserVO selectByUserId(@Param("query") UserDTO userDTO);


    UserVO selectByUserId1(@Param("query") UserDTO userDTO);

    void updateByPrimaryKeySelective(SysUser sysUser);


    /**
     * 通过ID查询用户信息
     *
     * @param id 用户ID
     * @return userVo
     */
    UserVO getUserVoById(Integer id);

    /**
     * 忽略逻辑删除
     * 查询数据库的用户信息（包含已逻辑删除的数据） del_flag = 0
     *
     * @param id 用户ID
     * @return 用户信息
     */
    SysUser getByIdIgnore(String id);

    SysUser selectUserByName(SysUser sysUser);

    List<SysUser> selectUserById(SysUser sysUser);

    /**
     * 根据部门id和角色id查询
     *
     * @param deptId          部门ID
     * @param realNameOrPhone 真实姓名或手机号
     * @param roleList        角色列表
     * @return 用户列表
     */
    List<SysUser> selectByDeptIdAndRoleId(@Param("deptId") Integer deptId,
										  @Param("realNameOrPhone") String realNameOrPhone,
                                          @Param("roleList") List<Integer> roleList);

    /**
     * 获取用户信息
     *
     * @param userIds 用户id列表
     * @return 用户信息集合
     */
    List<SysUser> listUserDetails(@Param(value = "userIds") List<Integer> userIds);

    /**
     * 按角色查询用户
     *
     * @param userDTO 用户 DTO
     * @return {@link List}<{@link SysUser}> 结果
     */
    List<UserVO> listUserByRole(@Param("query") UserDTO userDTO);
}
