package com.qunzhixinxi.hnqz.admin.service;

import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qunzhixinxi.hnqz.admin.api.dto.SysUserDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.UserDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.UserInfo;
import com.qunzhixinxi.hnqz.admin.api.dto.UserPwd;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.vo.UserVO;
import com.qunzhixinxi.hnqz.admin.api.dto.CommonUserDTO;
import com.qunzhixinxi.hnqz.admin.api.model.input.UserCertificationInput;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @author hnqz
 * @date 2017/10/31
 */
public interface SysUserService extends IService<SysUser> {

    /**
     * 查询用户信息
     *
     * @param sysUser 用户
     * @return userInfo
     */
    UserInfo findUserInfo(SysUser sysUser);


    /**
     * 分页查询用户信息（含有角色信息）
     *
     * @param page    分页对象
     * @param userDTO 参数列表
     * @return 分页列表
     */
    Page<UserVO> getUsersWithRolePage(Page<UserVO> page, UserDTO userDTO);

    Page<UserVO> getUsersWithRolePageNoLock(Page<UserVO> page, UserDTO userDTO);

    Page<UserVO> selectUserList(Page<UserVO> page, UserDTO userDTO, HnqzUser user);

    Page<UserVO> selectUserList1(Page<UserVO> page, UserDTO userDTO, HnqzUser user);

    Page<UserVO> selectUserList2(Page<UserVO> page, UserDTO userDTO, HnqzUser user);

    /**
     * 删除用户
     *
     * @param sysUser 用户
     * @return boolean
     */
    Boolean deleteUserById(SysUser sysUser);

    /**
     * 更新当前用户基本信息
     *
     * @param userDto 用户信息
     * @return Boolean
     */
    R<Boolean> updateUserInfo(UserDTO userDto);

    /**
     * 更新指定用户信息
     *
     * @param userDto 用户信息
     * @return 更新结果
     */
    Boolean updateUser(UserDTO userDto);

    /**
     * 通过ID查询用户信息
     *
     * @param id 用户ID
     * @return 用户信息
     */
    UserVO selectUserVoById(Integer id);

    /**
     * 查询上级部门的用户信息
     *
     * @param username 用户名
     * @return R
     */
    List<SysUser> listAncestorUsers(String username);

    /**
     * 保存用户信息
     *
     * @param userDto DTO 对象
     * @return success/fail
     */
    Integer saveUser(UserDTO userDto);

    List<SysUser> getUserByPlatId(String platId);

    List<UserVO> getUserVoByPlatId(UserDTO userDTO);


    R updatePwd(UserPwd userPwd);

    SysUser getByIdIgnore(String taskUserId);


    R getAuthInfo(UserCertificationInput input);

    /**
     * 通过excel批量创建用户
     *
     * @param commonUserDTOListMap 通用用户dto
     * @return 结果
     */
    Map<String, String> batchByExcel(Map<String, List<CommonUserDTO>> commonUserDTOListMap);


    /**
     * 校验是否超60周岁
     *
     * @param userVO 用户vo
     * @return 结果
     */
    Map<String, String> checkSixtyYearsOld(UserVO userVO);

    /**
     * 根据用户名获取用户信息
     *
     * @param usernames 用户名
     * @return 用户信息map
     */
    Map<String, List<SysUser>> mapUsersByUsername(Collection<String> usernames);

    /**
     * 获取用户的区域树
     *
     * @param id 用户ID
     * @return 区域树
     */
    List<Tree<Long>> getUserAreaTree(Long id);

    /**
     * 密码延期
     *
     * @param user 操作人
     * @return 延期结果
     */
    Boolean exceedingPwdChangeTime(HnqzUser user);

    /**
     * 查询用户分页
     *
     * @param query    查询条件
     * @param operator 操作人
     * @return 用户分页
     */
    Page<?> pageUser(SysUserDTO.OnPage query, HnqzUser operator);
}
