
package com.qunzhixinxi.hnqz.admin.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qunzhixinxi.hnqz.admin.api.dto.WmTaskDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.WmReport;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskContent;
import com.qunzhixinxi.hnqz.admin.api.dto.CommonEvidenceDTO;
import com.qunzhixinxi.hnqz.common.core.util.R;
import org.apache.ibatis.annotations.Param;

import java.sql.Timestamp;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * 任务表
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:25
 */
public interface WmTaskService extends IService<WmTask> {
    List<WmTask> listTask(QueryWrapper wrappers);

    Page<WmTask> listTaskPage(Page page, QueryWrapper wrappers);

    Page<WmTask> listTaskPage(Page page, LambdaQueryWrapper<WmTask> wrappers);


    IPage<WmTask> listTaskByExt(Page page, WmTask wmTask);

    IPage<WmTask> listTaskByExt1(Page page, WmTask wmTask);

    List<WmTask> listTaskByExtNoPage(WmTask wmTask);

    /**
     * 患者教育-处方列表
     *
     * @param scorePackageId 积分包id
     * @param prescriptionNo 处方单编号
     * @return 处方列表分页
     */
    IPage<WmTaskContent> listTaskPrescription(Page page, String scorePackageId, String prescriptionNo);

    /**
     * 患者教育-处方列表
     *
     * @param scorePackageId 积分包id
     * @param prescriptionNo 处方单编号
     * @return 处方列表
     */
    List<WmTask> listTaskPrescription(String scorePackageId, String prescriptionNo);

    List<Map<String, Object>> getTaskListByPackageIds(WmTask wmTask);

    List<Map<String, Object>> getTaskListByPackageIdsForJs(WmTask wmTask);

    R selectZbCount(WmTask wmTask);

    List<Map<String, Object>> getTaskListByPackageIdsForJsUser(WmTask wmTask);

    List<Map<String, Object>> getTaskListByPackageIdsCount(WmTask wmTask);

    List<Map<String, Object>> getTaskListByPackageIdsAndUser(WmTask wmTask);

    List<Map<String, Object>> getTaskListByPackageIdsZd(WmTask wmTask);

    List<WmTask> getTaskListByPackageIdsReturnWmtaskInfo(WmTask wmTask);

    IPage<Map<String, Object>> getH5PackageList(Page page, WmTask wmTask);

    IPage<Map<String, Object>> getH5TaskList(Page page, WmTask wmTask);

    R<?> selectByWmTask(WmTask wmTask);

    R selectByWmTaskId(WmTask wmTask);

    R saveAndUpdateWmTask(WmTask wmTask, WmReport wmReport);

    R<?> saveAndUpdateWmReport(WmTask wmTask, WmReport wmReport);

    R selectList(Page page, WmScorePackage wmScorePackage);

    List<Map<String, Object>> getTaskListByIdsCount(WmTask wmTask);

    List<Map<String, Object>> getTaskListByIds(WmTask wmTask);

    List<Map<String, Object>> getTaskListByIdsOld(WmTask wmTask);

    List<Map<String, Object>> getTaskListByIdsJs(WmTask wmTask);

    List<WmTask> getTaskListByIdsReturnWmtaskInfo(WmTask wmTask);

    List<Map<String, Object>> getTaskListByIdsZd(WmTask wmTask);

    List<Map<String, Object>> getTaskListByIdsZdJs(WmTask wmTask);

    R<?> getWmTaskList(Page page, WmTask wmTask);

    R<?> approvalTask(WmTask wmTask);

    Map<String, Timestamp> getMaxCreateTime(@Param("query") WmTask wmTask);

    R<?> saveTaskSettle(WmTask wmTask);

    /**
     * 患者教育-处方上传
     *
     * @param wmTask 任务对象
     * @return 是否成功状态
     */
    R<?> saveTaskPrescription(WmTask wmTask);

    R<?> updateTaskSettle(WmTask wmTask);

    IPage<Map<String, Object>> selectSettleTask(Page page, WmTask wmTask);

    Boolean checkUserTaskSubScore(String userId, String limitValue);

    Boolean newCheckUserTaskSubScore(String userId, LocalDate createDate);

    R<?> getTaskInfo(String taskId);

    /**
     * 查询调查信息审核数据
     *
     * @param taskId 任务id
     * @return
     */
    R<?> getSurveryTaskInfo(String taskId);

    /**
     * 患者教育-处方删除
     *
     * @param id 实际为WmTaskContent的id
     * @return 是否成功状态
     */
    Boolean deleteTaskPrescription(Integer id);

    /**
     * 证据链批量上传
     *
     * @param commonEvidenceDTOMap 通用证据链DTO map
     * @param packageName          当前积分包名称
     * @return 错误证据链编号
     */
    Map<String, String> taskBatchImport(Map<String, List<CommonEvidenceDTO>> commonEvidenceDTOMap,
                                        String packageName);

    /**
     * 查询调查信息审核列表
     *
     * @param wmTask
     * @param startDate
     * @param endDate
     * @return
     */
    IPage<WmTask> listInfoAudit(Page page, WmTask wmTask, LocalDateTime startDate, LocalDateTime endDate);

    /**
     * 查询同一家医院一个月做的任务
     *
     * @param hospitalId 医院id
     * @return
     */
    List<WmTask> listCurrentMonthHospitalTask(String hospitalId);

    /**
     * 查询会议任务个数
     *
     * @param taskTypeList 任务类型列表
     * @param taskUserId   用户id
     * @param startTime    会议时间范围-开始
     * @param endTime      会议时间范围-结束
     * @return 会议任务个数
     */
    Long countMeetingTask(List<String> taskTypeList, String taskUserId, LocalDateTime startTime, LocalDateTime endTime);

    /**
     * 个人包-任务复审
     *
     * @param id             任务id
     * @param taskStatus     审核状态(3-通过，4-不通过)
     * @param taskStatusInfo 审批说明
     * @return 复审结果
     */
    Boolean reviewTask(String id, String taskStatus, String taskStatusInfo);

    /**
     * 快速批量任务审核
     *
     * @param taskIdToNodeIdMap 任务id列表
     * @param taskStatus        审核状态(3-通过，4-不通过)
     * @param taskStatusInfo    审批说明
     * @return 审核结果
     */
    Boolean batchApprovalTask(Map<String, Integer> taskIdToNodeIdMap, String taskStatus, String taskStatusInfo);

    /**
     * 更新调查信息数据
     *
     * @param task           任务信息
     * @param taskStatus     审核状态
     * @param taskStatusInfo 审批说明
     */
    String task15_30_16_18(WmTask task, String taskStatus, String taskStatusInfo);

    /**
     * 更新调查信息数据(只入库调查信息数据,不更新任务和积分包数据,用于手动处理)
     *
     * @param task           任务信息
     * @param taskStatus     审核状态
     * @param taskStatusInfo 审批说明
     */
    void task15_30_16_18ForManualOperation(WmTask task, String taskStatus, String taskStatusInfo);

    /**
     * 获取可监督的任务
     *
     * @param pkgId     积分包ID
     * @param ignoreIds 忽略的ID
     * @return 监督任务列表
     */
    List<WmTask> listAvailableTaskForSupervision(Integer pkgId, Collection<Integer> ignoreIds);

    List<WmTask> listAvailableTaskForSupervision1(Collection<Integer> pkgIds, Collection<Integer> ignoreIds);

    Page<WmTask> listAvailableTaskForSupervision2(Page<WmTask> page, Collection<Integer> pkgIds, Collection<Integer> ignoreIds);

    List<Map<String, Object>> listTaskInfo(List<WmTask> taskList, String pkgId);


    List<WmTask> listPkgTasks(String pkgId);

    Page<WmTask> listAvailableTaskForSupervision3(Page<WmTask> page, Collection<Integer> userIds1, String taskTypeId, String taskId, String taskNumber, Collection<Integer> pkgIds, Collection<Integer> ignoreIds);
}
