package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.SysAreaEntity;
import com.qunzhixinxi.hnqz.admin.mapper.SysAreaEntityMapper;
import com.qunzhixinxi.hnqz.admin.service.SysAreaEntityService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 行政区划表服务实现
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/12 20:03
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysAreaEntityServiceImpl extends ServiceImpl<SysAreaEntityMapper, SysAreaEntity> implements SysAreaEntityService {


    /**
     * 查询行政区划树
     *
     * @param sysArea 查询条件
     * @return 树
     */
    @Override
    public List<Tree<Long>> selectTree(SysAreaEntity sysArea) {
        List<SysAreaEntity> entityList = baseMapper.selectList(Wrappers.lambdaQuery(sysArea)
                .eq(SysAreaEntity::getAreaStatus, Boolean.TRUE)
                .orderByDesc(SysAreaEntity::getAreaId));

        return CollUtil.isNotEmpty(entityList) ? getTrees(entityList) : Collections.emptyList();
    }

    /**
     * 查询行政区划树
     *
     * @param areaIds 查询条件
     * @return 树
     */
    @Override
    public List<Tree<Long>> selectTree(Collection<Long> areaIds) {

        if (CollUtil.isEmpty(areaIds)){
            return Collections.emptyList();
        }

        List<SysAreaEntity> entityList = this.list(Wrappers.<SysAreaEntity>lambdaQuery()
                .in(SysAreaEntity::getAreaId, areaIds)
                .eq(SysAreaEntity::getAreaStatus, Boolean.TRUE)
        );

        return CollUtil.isNotEmpty(entityList) ? getTrees(entityList) : Collections.emptyList();
    }

    private static List<Tree<Long>> getTrees(List<SysAreaEntity> entityList) {

        List<TreeNode<Long>> collect = entityList.stream().map(sysAreaEntity -> {
            TreeNode<Long> treeNode = new TreeNode<>(sysAreaEntity.getAreaId(), sysAreaEntity.getParentId(), sysAreaEntity.getAreaName(), sysAreaEntity.getSortOrder());

            HashMap<String, Object> extraMap = MapUtil.of("areaCode", sysAreaEntity.getAreaCode());
            extraMap.put("hot", sysAreaEntity.getHot());
            treeNode.setExtra(extraMap);
            return treeNode;
        }).collect(Collectors.toList());


        return TreeUtil.build(collect, -1L);
    }

    /**
     * 分页查询
     *
     * @param page    分页对象
     * @param sysArea 行政区划
     * @return Page
     */
    @Override
    public Page<SysAreaEntity> selectPage(Page<SysAreaEntity> page, SysAreaEntity sysArea) {
        LambdaQueryWrapper<SysAreaEntity> wrapper = Wrappers.lambdaQuery();
        wrapper.like(StrUtil.isNotBlank(sysArea.getAreaName()), SysAreaEntity::getAreaName, sysArea.getAreaName());
        wrapper.eq(Objects.nonNull(sysArea.getAreaCode()), SysAreaEntity::getParentId, sysArea.getAreaCode());
        wrapper.orderByAsc(SysAreaEntity::getAreaType);
        wrapper.orderByDesc(SysAreaEntity::getSortOrder);
        return baseMapper.selectPage(page, wrapper);
    }

    @Override
    public List<SysAreaEntity> listAreas(Long parentId) {
        return this.list(Wrappers.<SysAreaEntity>lambdaQuery()
                .eq(SysAreaEntity::getParentId, parentId)
                .eq(SysAreaEntity::getAreaStatus, Boolean.TRUE)
                .orderByAsc(SysAreaEntity::getAreaCode)
        );
    }

}




