package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.SysCsmUserRelation;
import com.qunzhixinxi.hnqz.admin.mapper.SysCsmUserRelationMapper;
import com.qunzhixinxi.hnqz.admin.service.SysCsmUserRelationService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
* @author li312
* @description 针对表【sys_csm_user_relation(csm用户关系表)】的数据库操作Service实现
* @createDate 2024-03-04 10:08:51
*/
@Service
@AllArgsConstructor
public class SysCsmUserRelationServiceImpl extends ServiceImpl<SysCsmUserRelationMapper, SysCsmUserRelation>
    implements SysCsmUserRelationService {

	private final SysCsmUserRelationMapper sysCsmUserRelationMapper;

	/**
	 * 递归查询CSM所有下级的用户ID
	 *
	 * @param parentIds  父级ID
	 * @param limitCount 递归次数限制
	 * @return {@link Set}<{@link Integer}> 结果
	 */
	@Override
	public Set<Integer> recurseCsmLowerUserId(Set<Integer> parentIds, AtomicInteger limitCount) {
		Set<Integer> allIds = new HashSet<>();
		allIds.addAll(parentIds);
		// 限制调用次数
		if (limitCount.decrementAndGet() <= 0) {
			return allIds;
		}

		List<SysCsmUserRelation> csmUserRelations = sysCsmUserRelationMapper.selectList(Wrappers.<SysCsmUserRelation>lambdaQuery()
				.in(SysCsmUserRelation::getParentId, parentIds));
		if (CollUtil.isNotEmpty(csmUserRelations)) {
			Set<Integer> userIds = csmUserRelations.stream().map(SysCsmUserRelation::getUserId).collect(Collectors.toSet());
			allIds.addAll(this.recurseCsmLowerUserId(userIds, limitCount));
		}

		return allIds;
	}
}




