package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptRelation;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptRelationMapper;
import com.qunzhixinxi.hnqz.admin.service.SysDeptRelationService;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author hnqz
 * @since 2018-02-12
 */
@Service
@AllArgsConstructor
public class SysDeptRelationServiceImpl extends ServiceImpl<SysDeptRelationMapper, SysDeptRelation> implements SysDeptRelationService {

    private final SysDeptRelationMapper sysDeptRelationMapper;
    private final SysDeptMapper sysDeptMapper;

    /**
     * 维护部门关系
     *
     * @param sysDept 部门
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertDeptRelation(SysDept sysDept) {
        // 增加部门关系表
        List<SysDeptRelation> relationList =
                sysDeptRelationMapper.selectList(Wrappers.<SysDeptRelation>query().lambda().eq(SysDeptRelation::getDescendant, sysDept.getParentId()))
                        .stream()
                        .peek(relation -> relation.setDescendant(sysDept.getDeptId())).collect(Collectors.toList());
        if (CollUtil.isNotEmpty(relationList)) {
            this.saveBatch(relationList);
        }

        // 自己也要维护到关系表中
        SysDeptRelation own = new SysDeptRelation();
        own.setDescendant(sysDept.getDeptId());
        own.setAncestor(sysDept.getDeptId());
        sysDeptRelationMapper.insert(own);
    }

    /**
     * 通过ID删除部门关系
     *
     * @param id 企业ID
     */
    @Override
    public void deleteAllDeptRelation(Integer id) {
        baseMapper.deleteDeptRelationsById(id);
    }

    /**
     * 通过ID删除部门关系
     *
     * @param id 企业ID
     */
    @Override
    public void deleteDeptById(Integer id) {
        baseMapper.deleteDeptById(id);
    }

    @Override
    public Page<Map<String, Object>> selectSysDeptList(Page<?> page, SysDept sysDept) {
        if (sysDept.getDeptId() == null) {
            sysDept.setDeptId(SecurityUtils.getUser().getDeptId());
        }
        SysDeptRelation sysDeptRelation = new SysDeptRelation();
        sysDeptRelation.setAncestor(sysDept.getDeptId());
        List<Map<String, Object>> list1 = new ArrayList<>();
        List<SysDeptRelation> sysDeptRelationList = sysDeptRelationMapper.selectListByAncestor(sysDeptRelation);
        SysDeptRelation sysDeptRelation1 = new SysDeptRelation();
        sysDeptRelation1.setDescendant(sysDept.getDeptId());
        List<SysDeptRelation> sysDeptRelationList1 = sysDeptRelationMapper.selectListByDescendant(sysDeptRelation1);
        for (SysDeptRelation deptRelation : sysDeptRelationList) {
            SysDept sysDept1 = new SysDept();
            sysDept1.setDeptId(deptRelation.getDescendant());
            List<Map<String, Object>> sysDept2 = sysDeptMapper.selectDeptById(sysDept1);
            list1.addAll(sysDept2);
        }
        for (SysDeptRelation deptRelation : sysDeptRelationList1) {
            SysDept sysDept1 = new SysDept();
            sysDept1.setDeptId(deptRelation.getAncestor());
            List<Map<String, Object>> sysDept2 = sysDeptMapper.selectDeptById(sysDept1);
            list1.addAll(sysDept2);
        }

        Page<Map<String, Object>> page1 = new Page<>();
        int current = Long.valueOf(page.getCurrent()).intValue();
        int size = Long.valueOf(page.getSize()).intValue();

        int count = list1.size();
        List<Map<String, Object>> pageList = new ArrayList<>();
        //计算当前页第一条数据的下标
        int currId = current > 1 ? (current - 1) * size : 0;
        for (int i = 0; i < size && i < count - currId; i++) {
            pageList.add(list1.get(currId + i));
        }
        page1.setSize(size);
        page1.setCurrent(current);
        page1.setTotal(count);
        //计算分页总页数
        page1.setPages(count % 10 == 0 ? count / 10 : count / 10 + 1);
        page1.setRecords(pageList);
        return page1;
    }


    /**
     * 更新部门关系
     *
     * @param relation 企业关系
     */
    @Override
    public void updateDeptRelation(SysDeptRelation relation) {
        baseMapper.updateDeptRelations(relation);
    }

}
