package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.text.StrPool;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.SysFile;
import com.qunzhixinxi.hnqz.admin.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.mapper.SysFileMapper;
import com.qunzhixinxi.hnqz.admin.service.SysFileService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.oss.OssProperties;
import com.qunzhixinxi.hnqz.common.oss.service.OssTemplate;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 文件管理
 *
 * @author Luckly
 * @date 2019-06-18 17:18:42
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysFileServiceImpl extends ServiceImpl<SysFileMapper, SysFile> implements SysFileService {

    private final OssProperties ossProperties;

    private final OssTemplate minioTemplate;

    private final UpmsConfig upmsConfig;

    /**
     * 上传文件
     *
     * @param file 文件
     * @return 上传结果
     */
    @Override
    public R<Map<String, String>> uploadFile(MultipartFile file) {
        HnqzUser user = SecurityUtils.getUser();
        return R.ok(upload(file, user.getUsername()));
    }

    /**
     * 上传文件
     *
     * @param file 文件
     * @return 上传结果
     */
    @Override
    public Map<String, String> upload(MultipartFile file, String username) {
        // 判断是否有重名的操作
        String fileName = IdUtil.simpleUUID() + StrPool.DOT + FileUtil.extName(file.getOriginalFilename());
        Map<String, String> resultMap = new HashMap<>(5);
        resultMap.put("bucketName", ossProperties.getBucketName());
        resultMap.put("orgFileName", file.getOriginalFilename());
        resultMap.put("fileName", DateTimeFormatter.ofPattern(DatePattern.PURE_DATE_PATTERN).format(LocalDateTime.now()) + File.separator + fileName);
        String url = String.format("/admin/sys-file/%s/%s", ossProperties.getBucketName(), fileName);
        resultMap.put("url", url);

        try {
            minioTemplate.putObject(ossProperties.getBucketName(), fileName, file.getInputStream());
            // 文件管理数据记录,收集管理追踪文件
            SysFile sysFile = fileLog(file, fileName, url, username);
            resultMap.put("fileId", String.valueOf(sysFile.getId()));

        } catch (Exception e) {
            log.error("上传失败", e);
            throw new RuntimeException(e.getLocalizedMessage());
        }
        return resultMap;
    }


    /**
     * 上传文件
     *
     * @param file
     * @return
     */
    @Override
    public R uploadFileAllUrl(MultipartFile file) {
//		String basePath="https://beijingc.yaoyi.net";
        String fileName = IdUtil.simpleUUID() + StrUtil.DOT + FileUtil.extName(file.getOriginalFilename());
        Map<String, String> resultMap = new HashMap<>(4);
        resultMap.put("bucketName", ossProperties.getBucketName());
        resultMap.put("fileName", fileName);
        String url = upmsConfig.getClientUrl() + String.format("/admin/sys-file/%s/%s", ossProperties.getBucketName(), fileName);
        resultMap.put("url", url);

        try {
            minioTemplate.putObject(ossProperties.getBucketName(), fileName, file.getInputStream());
            // 文件管理数据记录,收集管理追踪文件
            fileLog(file, fileName, url, SecurityUtils.getUser().getUsername());
        } catch (Exception e) {
            log.error("上传失败", e);
            return R.failed(e.getLocalizedMessage());
        }
        return R.ok(resultMap);
    }


    /**
     * 上传文件
     *
     * @param file
     * @return
     */
    @Override
    public R uploadFileAllUrlRealName(MultipartFile file) {
//		String basePath="https://beijingc.yaoyi.net";
        Date date = new Date();
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
        String idStr = format.format(date);
        String fileName = FileUtil.getName(file.getName()) + "-" + idStr + StrUtil.DOT + FileUtil.extName(file.getOriginalFilename());
        Map<String, String> resultMap = new HashMap<>(4);
        resultMap.put("bucketName", ossProperties.getBucketName());
        resultMap.put("fileName", fileName);
        String url = upmsConfig.getClientUrl() + String.format("/admin/sys-file/%s/%s", ossProperties.getBucketName(), fileName);
        resultMap.put("url", url);

        try {
            minioTemplate.putObject(ossProperties.getBucketName(), fileName, file.getInputStream());
            // 文件管理数据记录,收集管理追踪文件
            fileLog(file, fileName, url, SecurityUtils.getUser().getUsername());
        } catch (Exception e) {
            log.error("上传失败", e);
            return R.failed(e.getLocalizedMessage());
        }
        return R.ok(resultMap);
    }


    /**
     * 上传文件
     *
     * @return
     */
    @Override
    public R uploadFileAllUrlRealName(String fileName, String suffix, FileInputStream io) {

        Date date = new Date();
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
        String idStr = format.format(date);

        fileName = String.format("%s-%s.%s", fileName, idStr, suffix);

        Map<String, String> resultMap = new HashMap<>(4);
        resultMap.put("bucketName", ossProperties.getBucketName());
        resultMap.put("fileName", fileName);
        resultMap.put("url", String.format("%s/admin/sys-file/%s/%s", upmsConfig.getClientUrl(), ossProperties.getBucketName(), fileName));

        try {
            minioTemplate.putObject(ossProperties.getBucketName(), fileName, io);
        } catch (Exception e) {
            log.error("上传失败", e);
            return R.failed(e.getLocalizedMessage());
        }
        return R.ok(resultMap);
    }

    /**
     * 读取文件
     *
     * @param bucket
     * @param fileName
     * @param response
     */
    @Override
    public void getFile(String bucket, String fileName, HttpServletResponse response) {
        try (InputStream inputStream = minioTemplate.getObject(bucket, fileName)) {
            response.setContentType("application/octet-stream; charset=UTF-8");
            IoUtil.copy(inputStream, response.getOutputStream());
        } catch (Exception e) {
            log.error("文件读取异常: {}", e.getLocalizedMessage());
        }
    }

    /**
     * 删除文件
     *
     * @param id
     * @return
     */
    @Override
    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteFile(Long id) {
        SysFile file = this.getById(id);
        minioTemplate.removeObject(ossProperties.getBucketName(), file.getFileName());
        return this.removeById(id);
    }

    /**
     * 文件管理数据记录,收集管理追踪文件
     *
     * @param file     上传文件格式
     * @param fileName 文件名
     */
    private SysFile fileLog(MultipartFile file, String fileName, String url, String username) {
        SysFile sysFile = new SysFile();
        // 原文件名
        String original = file.getOriginalFilename();
        sysFile.setFileName(fileName);
        sysFile.setOriginal(original);
        sysFile.setFileSize(file.getSize());
        sysFile.setType(FileUtil.extName(original));
        sysFile.setBucketName(ossProperties.getBucketName());
        sysFile.setPath(url);
        sysFile.setCreateUser(username);
        this.save(sysFile);
        return sysFile;
    }


    @Override
    public String uploadImgByByte(InputStream inputStream) {
        String fileName = IdUtil.simpleUUID() + StrUtil.DOT + ".jpg";
        Map<String, String> resultMap = new HashMap<>(4);
        resultMap.put("bucketName", ossProperties.getBucketName());
        resultMap.put("fileName", fileName);
        resultMap.put("url", String.format("/admin/sys-file/%s/%s", ossProperties.getBucketName(), fileName));

        try {
            minioTemplate.putObject(ossProperties.getBucketName(), fileName, inputStream);
            // 文件管理数据记录,收集管理追踪文件
            // fileLog(file, fileName);
            return String.format("/admin/sys-file/%s/%s", ossProperties.getBucketName(), fileName);
        } catch (Exception e) {
            log.error("上传失败", e);
        }

        return null;
    }

    @Override
    public void getFileById(Integer fileId, HttpServletResponse response) {
        SysFile file = this.getById(fileId);

        this.getFile(file.getBucketName(), file.getFileName(), response);

    }

}
