package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import com.qunzhixinxi.hnqz.admin.api.entity.SysImplPlanDetails;
import com.qunzhixinxi.hnqz.admin.api.entity.SysImplementPlan;
import com.qunzhixinxi.hnqz.admin.mapper.SysImplPlanDetailsMapper;
import com.qunzhixinxi.hnqz.admin.service.SysImplPlanDetailsService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 执行计划详情服务实现
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/19 15:53
 */
@Service
@AllArgsConstructor
public class SysImplPlanDetailsServiceImpl extends ServiceImpl<SysImplPlanDetailsMapper, SysImplPlanDetails>
        implements SysImplPlanDetailsService {

    /**
     * 创建详情
     *
     * @param details 详情
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createDetails(SysImplPlanDetails details) {

        this.save(details);
    }

    /**
     * 更新结果
     *
     * @param details 详情
     * @return 更新结果
     */
    @Override
    public Boolean updateDetails(SysImplPlanDetails details) {
        return this.updateById(details);
    }

    /**
     * 获取计划的详情
     *
     * @param planId 计划ID
     * @return 计划详情
     */
    @Override
    public SysImplPlanDetails getDetailsByPlanId(Integer planId, UpmsState.ImplPlanDetailsState detailsState) {
        List<SysImplPlanDetails> list = this.list(Wrappers.<SysImplPlanDetails>lambdaQuery()
                .eq(SysImplPlanDetails::getDetailsState, detailsState)
                .eq(SysImplPlanDetails::getPlanId, planId)
                .orderByDesc(SysImplPlanDetails::getCreateTime)
        );
        return CollUtil.isNotEmpty(list) ? list.get(0) : null;
    }
}




