package com.qunzhixinxi.hnqz.admin.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.SysNoticeDto;
import com.qunzhixinxi.hnqz.admin.api.entity.SysNotice;
import com.qunzhixinxi.hnqz.admin.mapper.SysNoticeMapper;
import com.qunzhixinxi.hnqz.admin.service.SysNoticeService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Objects;

/**
 * 系统通知服务
 *
 * @author jimmy
 * @date 09/07/2024 15:49
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysNoticeServiceImpl extends ServiceImpl<SysNoticeMapper, SysNotice> implements SysNoticeService {
    /**
     * 分页查询通知
     *
     * @param query 查询条件
     * @param user  操作人
     * @return 分页结果
     */
    @Override
    public Page<SysNotice> pageNotices(SysNoticeDto.OnPage query, HnqzUser user) {
        return this.page(new Page<>(query.getCurrent(), query.getSize()),
                Wrappers.<SysNotice>lambdaQuery()
                        .eq(SysNotice::getUserId, user.getId())
                        .eq(Objects.nonNull(query.getRead()), SysNotice::getReadFlag, query.getRead())
                        .orderByDesc(SysNotice::getCreateTime)
        );
    }

    /**
     * 阅读通知
     *
     * @param resource 通知信息
     * @param user     阅读人
     * @return 阅读结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean readNotice(SysNoticeDto.OnRead resource, HnqzUser user) {

        SysNotice sysNotice = new SysNotice();
        sysNotice.setReadFlag(resource.getRead());
        sysNotice.setNoticeId(resource.getNoticeId());
        sysNotice.setReadTime(LocalDateTime.now());

        return this.updateById(sysNotice);
    }

    /**
     * 创建通知
     *
     * @param resource 通知信息
     * @param user     操作人
     * @return 创建结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean createNotice(SysNoticeDto.OnCreate resource, HnqzUser user) {
        SysNotice notice = new SysNotice();
        notice.setTitle(resource.getTitle());
        notice.setContent(resource.getContent());
        notice.setUserId(resource.getUserId());
        notice.setReadFlag(Boolean.FALSE);
        notice.setCreateBy(user.getUsername());
        notice.setCreateTime(LocalDateTime.now());

        return this.save(notice);
    }

    /**
     * 统计通知
     *
     * @param read 阅读状态
     * @param user
     * @return 统计结果
     */
    @Override
    public int countNotices(@Nullable Boolean read, HnqzUser user) {
        return this.count(Wrappers.<SysNotice>lambdaQuery()
                .eq(Objects.nonNull(read), SysNotice::getReadFlag, read)
                .eq(SysNotice::getUserId, user.getId()));
    }
}
