
package com.qunzhixinxi.hnqz.admin.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.SysRole;
import com.qunzhixinxi.hnqz.admin.api.entity.SysRoleMenu;
import com.qunzhixinxi.hnqz.admin.mapper.SysRoleMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysRoleMenuMapper;
import com.qunzhixinxi.hnqz.admin.service.SysRoleService;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author hnqz
 * @since 2017-10-29
 */
@Service
@AllArgsConstructor
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {

    private SysRoleMenuMapper sysRoleMenuMapper;

    /**
     * 通过用户ID，查询角色信息
     *
     * @param userId
     * @return
     */
    @Override
    public List findRolesByUserId(Integer userId) {
        return baseMapper.listRolesByUserId(userId);
    }

    /**
     * 通过角色ID，删除角色,并清空角色菜单缓存
     *
     * @param id
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean removeRoleById(Integer id) {
        sysRoleMenuMapper.delete(Wrappers.<SysRoleMenu>update().lambda().eq(SysRoleMenu::getRoleId, id));
        return this.removeById(id);
    }

    /**
     * 获取企业level
     *
     * @return 企业level
     */
    @Override
    public int getEntLevel() {

        List<Integer> roles = SecurityUtils.getRoles();
        // 服务商管理员
        if (roles.contains(37)) {
            return 2;
        }
        // 区域管理员 || 地市管理员
        else if (roles.contains(4) || roles.contains(43)) {
            return 1;
        }
        // 事业部管理员 || 事业部分管领导 || 事业部总经理 || 商务管理员 || 市场管理员
        else if (roles.contains(3) || roles.contains(41) || roles.contains(39) || roles.contains(40) || roles.contains(42)) {
            return -1;
        }
        return 0;
    }

}
