package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUserArea;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserAreaMapper;
import com.qunzhixinxi.hnqz.admin.service.SysUserAreaService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户区域关系表服务实现
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/13 13:40
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysUserAreaServiceImpl extends ServiceImpl<SysUserAreaMapper, SysUserArea> implements SysUserAreaService {


    /**
     * 保存关系
     *
     * @param userId    用户ID
     * @param areaCodes 区域列表
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveRelation(Long userId, Collection<Long> areaCodes) {

        if (CollUtil.isEmpty(areaCodes)) {
            log.info("不存在需要绑定的区域");
            return;
        }

        LocalDateTime now = LocalDateTime.now();
        List<SysUserArea> collect = areaCodes.stream().distinct().map(code -> {
            SysUserArea userArea = new SysUserArea();
            userArea.setUserId(userId);
            userArea.setAreaId(code);
            userArea.setCreateTime(now);
            return userArea;
        }).collect(Collectors.toList());


        this.saveBatch(collect);


    }

    /**
     * 查询用户的区域
     *
     * @param userId 用户id
     * @return 区域ID列表
     */
    @Override
    public List<Long> listUserAreas(Long userId) {
        return this.list(Wrappers.<SysUserArea>lambdaQuery().eq(SysUserArea::getUserId, userId)).stream().map(SysUserArea::getAreaId).distinct().sorted().collect(Collectors.toList());
    }

    /**
     * 获取区域的人员
     *
     * @param areaIds 区域列表
     * @return 人员列表
     */
    @Override
    public List<Long> listAreaUser(Collection<Long> areaIds) {
        List<Long> collect = areaIds.stream().sorted().collect(Collectors.toList());
        return this.list(Wrappers.<SysUserArea>lambdaQuery().in(SysUserArea::getAreaId, collect)).stream().map(SysUserArea::getUserId).distinct().sorted().collect(Collectors.toList());
    }

    /**
     * 获取区域的人员
     *
     * @param areaIds 区域列表
     * @return 人员列表
     */
    @Override
    public List<Long> listAreaUser1(Collection<Long> areaIds) {
        return this.baseMapper.listAreaUser(areaIds);
    }

    /**
     * 删除用户的区域
     *
     * @param userId 用户ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delUserArea(Long userId) {
        this.remove(Wrappers.<SysUserArea>lambdaQuery().eq(SysUserArea::getUserId, userId));
    }

}




