package com.qunzhixinxi.hnqz.admin.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.SysUserDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUserCertificate;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserCertificateMapper;
import com.qunzhixinxi.hnqz.admin.service.SysUserCertificateService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户证明服务实现
 *
 * @author jimmy
 * @date 13/07/2024 11:17
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysUserCertificateServiceImpl extends ServiceImpl<SysUserCertificateMapper, SysUserCertificate> implements SysUserCertificateService {


    /**
     * 保存用户证书
     *
     * @param resource 证书信息
     * @param user     操作人
     * @return 保存结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean createUserCertificate(SysUserDTO.OnCertificateCreate resource, HnqzUser user) {

        String type = resource.getType();
        Integer userId = resource.getUserId();
        LocalDateTime now = LocalDateTime.now();
        String operator = user.getUsername();

        // 删除历史
        this.remove(Wrappers.<SysUserCertificate>lambdaQuery()
                .eq(SysUserCertificate::getType, type)
                .eq(SysUserCertificate::getUserId, userId));


        List<SysUserCertificate> collect = resource.getFileIds().stream().map(fid -> {
            SysUserCertificate certificate = new SysUserCertificate();
            certificate.setUserId(userId);
            certificate.setType(type);
            certificate.setFileId(fid);
            certificate.setCreateBy(operator);
            certificate.setCreateTime(now);
            return certificate;
        }).collect(Collectors.toList());


        return this.saveBatch(collect);
    }
}
