package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.lang.Validator;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.nacos.common.utils.Md5Utils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.dto.SysUserDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.UserDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.UserInfo;
import com.qunzhixinxi.hnqz.admin.api.dto.UserPwd;
import com.qunzhixinxi.hnqz.admin.api.entity.SysAreaEntity;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptRelation;
import com.qunzhixinxi.hnqz.admin.api.entity.SysRole;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUserRole;
import com.qunzhixinxi.hnqz.admin.api.vo.MenuVO;
import com.qunzhixinxi.hnqz.admin.api.vo.UserVO;
import com.qunzhixinxi.hnqz.admin.api.entity.SysCsmUserRelation;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptSub;
import com.qunzhixinxi.hnqz.admin.api.dto.CommonUserDTO;
import com.qunzhixinxi.hnqz.admin.api.model.input.UserCertificationInput;
import com.qunzhixinxi.hnqz.admin.api.model.output.UserCertificationOutput;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptRelationMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptSubMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserRoleMapper;
import com.qunzhixinxi.hnqz.admin.service.SysAreaEntityService;
import com.qunzhixinxi.hnqz.admin.service.SysCsmUserRelationService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysMenuService;
import com.qunzhixinxi.hnqz.admin.service.SysRoleService;
import com.qunzhixinxi.hnqz.admin.service.SysUserAreaService;
import com.qunzhixinxi.hnqz.admin.service.SysUserRoleService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.common.core.constant.CacheConstants;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.data.datascope.DataScope;
import com.qunzhixinxi.hnqz.common.security.encoder.SM4PasswordEncoder;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.ListUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Caching;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Slf4j
@Service
@AllArgsConstructor
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    private static final PasswordEncoder ENCODER = new SM4PasswordEncoder();

    private final SysMenuService sysMenuService;
    private final SysRoleService sysRoleService;
    private final SysDeptService sysDeptService;
    private final SysUserRoleService sysUserRoleService;
    private final SysUserRoleMapper sysUserRoleMapper;
    private final SysUserMapper sysUserMapper;
    private final SysDeptRelationMapper sysDeptRelationMapper;
    private final RedisTemplate<String, Object> redisTemplate;
    private final SysDeptSubMapper sysDeptSubMapper;
    private final SysCsmUserRelationService sysCsmUserRelationService;
    private final SysUserAreaService userAreaService;
    private final SysAreaEntityService areaEntityService;

    @Override
    public R<UserCertificationOutput> getAuthInfo(UserCertificationInput input) {

        SysUser sysUser = sysUserMapper.selectById(input.getUserId());
        SysDeptSub sysDeptSub = new SysDeptSub();
        sysDeptSub.setDeptId(sysUser.getDeptId());
        sysDeptSub.setEnableFlag("1");
        List<SysDeptSub> subList = sysDeptSubMapper.selectList(Wrappers.query(sysDeptSub));
        UserCertificationOutput output = new UserCertificationOutput();
        output.setSubList(subList);
        output.setUserInfo(sysUser);
        return R.ok(output);
    }

    /**
     * 通过excel批量创建用户
     *
     * @param commonUserDTOListMap 通用用户dto
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Map<String, String> batchByExcel(Map<String, List<CommonUserDTO>> commonUserDTOListMap) {

        // 变量
        Map<String, String> errorMap = new HashMap<>();
        List<UserDTO> currentList = new ArrayList<>();

        // 获取操作员
        HnqzUser creator = SecurityUtils.getUser();

        // 业务值校验
        for (Map.Entry<String, List<CommonUserDTO>> entry : commonUserDTOListMap.entrySet()) {
            checkoutCommonUserDTOInfo(entry.getValue(), creator, errorMap, currentList);
        }

        // 区域
        List<SysAreaEntity> list = areaEntityService.list(Wrappers.<SysAreaEntity>lambdaQuery().eq(SysAreaEntity::getAreaStatus, true));
        List<Long> collect = list.stream().mapToLong(SysAreaEntity::getAreaId).boxed().distinct().collect(Collectors.toList());

        currentList.forEach(dto -> dto.setAreaCodes(collect));

        // 上传结果
        if (CollectionUtil.isEmpty(errorMap)) {
            currentList.forEach(this::saveUser);
        }

        return errorMap;
    }


    /**
     * 校验是否超60周岁
     *
     * @param userVO 用户vo
     * @return 结果
     */
    @Override
    public Map<String, String> checkSixtyYearsOld(UserVO userVO) {
        Map<String, String> result = new HashMap<>();

        String idCardNumber = userVO.getIdCardNumber();
        if (!IdcardUtil.isValidCard(idCardNumber)) {
            log.warn("无效的身份证号: {}", idCardNumber);
            return result;
        }

        // 当前日期
        Calendar nowCal = Calendar.getInstance();
        // 基于生日的日期,60周岁提前半年的时间
        Calendar halfYearCal = IdcardUtil.getBirthDate(idCardNumber).toCalendar();
        halfYearCal.add(Calendar.YEAR, 59);
        halfYearCal.add(Calendar.MONTH, 6);
        // 基于生日的日期,60周岁的时间
        Calendar sixtyYearsOldYearCal = IdcardUtil.getBirthDate(idCardNumber).toCalendar();
        sixtyYearsOldYearCal.add(Calendar.YEAR, 60);

        if (sixtyYearsOldYearCal.compareTo(nowCal) > 0 && halfYearCal.compareTo(nowCal) <= 0) {
            // 提前半年提醒,不满60周岁,但距离60周岁在半年内
            String format = String.format("平台要求人员年龄在年满18岁未满60岁之间，您的账号将于%s年%s月%s日被停用，停用后将无法做任务或者资金结算，请知悉！",
                    sixtyYearsOldYearCal.get(Calendar.YEAR), sixtyYearsOldYearCal.get(Calendar.MONTH) + 1, sixtyYearsOldYearCal.get(Calendar.DAY_OF_MONTH));
            result.put("HALF_YEAR", format);
        } else if (sixtyYearsOldYearCal.compareTo(nowCal) <= 0) {
            // 已超过60周岁
            String format = String.format("您的账号已于%s年%s月%s日因年龄超限被停用，请知悉",
                    sixtyYearsOldYearCal.get(Calendar.YEAR), sixtyYearsOldYearCal.get(Calendar.MONTH) + 1, sixtyYearsOldYearCal.get(Calendar.DAY_OF_MONTH));
            result.put("OVER", format);
        }

        return result;
    }

    /**
     * 根据用户名获取用户信息
     *
     * @param usernames 用户名
     * @return 用户信息map
     */
    @Override
    public Map<String, List<SysUser>> mapUsersByUsername(Collection<String> usernames) {

        if (CollUtil.isEmpty(usernames)) {
            return Collections.emptyMap();
        }

        List<SysUser> sysUsers = this.list(Wrappers.<SysUser>lambdaQuery().in(SysUser::getUsername, usernames));

        return sysUsers.stream().collect(Collectors.groupingBy(SysUser::getUsername));
    }

    /**
     * 获取用户的区域树
     *
     * @param id 用户ID
     * @return 区域树
     */
    @Override
    public List<Tree<Long>> getUserAreaTree(Long id) {
        List<Long> areas = userAreaService.listUserAreas(id);
        return areaEntityService.selectTree(areas);
    }

    /**
     * 密码延期
     *
     * @param user 操作人
     * @return 延期结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean exceedingPwdChangeTime(HnqzUser user) {

        return this.update(Wrappers.<SysUser>lambdaUpdate().eq(SysUser::getUserId, user.getId()).set(SysUser::getLatestChangePwdTime, LocalDateTime.now()));
    }

    /**
     * 查询用户分页
     *
     * @param query    查询条件
     * @param operator 操作人
     * @return 用户分页
     */
    @Override
    public Page<?> pageUser(SysUserDTO.OnPage query, HnqzUser operator) {

        Page<SysUser> toPage = new Page<>(query.getCurrent(), query.getSize());

        Set<Integer> areaUserIds = null;
        if (CollUtil.isNotEmpty(query.getAreaCodes())) {

            // List<List<Long>> partition = ListUtils.partition(query.getAreaCodes(), 30);
            // areaUserIds = new HashSet<>();
            //
            // for (List<Long> areaIds : partition) {
            //     userAreaService.list(Wrappers.<SysUserArea>)
            //     Set<Integer> collect = userAreaService.listAreaUser(areaIds).stream().map(Long::intValue).collect(Collectors.toSet());
            //     areaUserIds.addAll(collect);
            // }

            areaUserIds = userAreaService.listAreaUser(query.getAreaCodes()).stream().map(Long::intValue).collect(Collectors.toSet());
        }

        List<SysUserRole> userRoles = sysUserRoleService.list(Wrappers.<SysUserRole>lambdaQuery()
                .in(CollUtil.isNotEmpty(areaUserIds), SysUserRole::getUserId, areaUserIds)
                .in(SysUserRole::getRoleId, query.getRole()));


        Map<Integer, List<Integer>> collect = userRoles.stream().collect(Collectors.groupingBy(SysUserRole::getUserId, Collectors.mapping(SysUserRole::getRoleId, Collectors.toList())));

        // Set<Integer> userIdByRoleIdSet = sysUserRoleService.list(Wrappers.<SysUserRole>lambdaQuery()
        //         .in(SysUserRole::getUserId, areaUserIds)
        //         .in(SysUserRole::getRoleId, query.getFinalQueryRoleIds())).stream().mapToInt(SysUserRole::getUserId).boxed().collect(Collectors.toSet());

        if (CollUtil.isEmpty(collect)) {
            return toPage;
        }

        LambdaQueryWrapper<SysUser> queryWrapper = Wrappers.<SysUser>lambdaQuery()
                .like(StrUtil.isNotBlank(query.getUsername()), SysUser::getUsername, query.getUsername())
                .like(StrUtil.isNotBlank(query.getRealname()), SysUser::getRealname, query.getRealname())
                .eq(StrUtil.isNotBlank(query.getLockFlag()), SysUser::getLockFlag, query.getLockFlag())
                .eq(query.getDeptId() != null, SysUser::getDeptId, query.getDeptId())
                .in(SysUser::getUserId, collect.keySet())
                .orderByDesc(SysUser::getCreateTime);

        this.page(toPage, queryWrapper);

        List<SysUser> records = toPage.getRecords();
        if (CollUtil.isEmpty(records)) {
            return toPage;
        }

        // 拼装角色
        List<UserVO> userVOS = records.stream().map(item -> {
            UserVO userVO = BeanUtil.copyProperties(item, UserVO.class);
            List<Integer> roleIds = collect.get(item.getUserId());
            List<SysRole> roles = roleIds.stream().map(rId -> {
                SysRole role = new SysRole();
                role.setRoleId(rId);
                return role;
            }).collect(Collectors.toList());
            userVO.setRoleList(roles);
            return userVO;
        }).collect(Collectors.toList());


        Page<UserVO> userVOPage = new Page<>();
        userVOPage.setSize(toPage.getSize());
        userVOPage.setCurrent(toPage.getCurrent());
        userVOPage.setTotal(toPage.getTotal());
        userVOPage.setRecords(userVOS);

        return userVOPage;
    }

    /**
     * 封装错误信息
     *
     * @param errorMap 记录错误信息map
     * @param key      原因
     * @param idStr    错误id
     */
    private void errorMap(Map<String, String> errorMap, String key, String idStr) {
        if (errorMap.containsKey(key)) {
            String val = errorMap.get(key);
            idStr = String.join(";", val, idStr);
        }
        errorMap.put(key, idStr);
    }

    /**
     * 角色校验
     *
     * @param roleName 角色名称
     * @param userDTO  用户dto
     * @param dept     部门
     * @return 校验错误信息
     */
    private String checkoutCommonUserRole(String roleName, UserDTO userDTO, SysDept dept) {

        String errorInfo = null;

        if (StringUtils.isEmpty(roleName) || dept == null) {
            errorInfo = "角色名称为空";
        } else {


            List<String> roleNames = roleName.contains(StrUtil.SLASH) ? StrUtil.split(roleName, StrUtil.SLASH) : Collections.singletonList(roleName);

            // 根据角色名称获取角色
            List<Integer> sysRole = sysRoleService.list(Wrappers.<SysRole>lambdaQuery()
                    .in(SysRole::getRoleName, roleNames)
                    .eq(SysRole::getDelFlag, DelEnum.NOT_DEL.getVal())).stream().mapToInt(SysRole::getRoleId).boxed().distinct().collect(Collectors.toList());

            if (CollUtil.isEmpty(sysRole)) {
                errorInfo = "角色不存在或者未启用";
            } else {
                userDTO.setRole(sysRole);
                if (userDTO.getRole().contains(2)) {
                    UserDTO tmp = new UserDTO();
                    tmp.setUsername(userDTO.getUsername());
                    userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
                    List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
                    if (CollUtil.isNotEmpty(userVoList)) {
                        for (UserVO userVO : userVoList) {
                            if (null != userVO.getRoleList()) {
                                for (SysRole r : userVO.getRoleList()) {
                                    if (r.getRoleId() == 3) {
                                        errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为药企管理员！";
                                        continue;
                                    }
                                    if (r.getRoleId() == 4) {
                                        errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为CSO管理员！";
                                        continue;
                                    }
                                    if (r.getRoleId() == 5) {
                                        errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为全职学术推广员！";
                                        continue;
                                    }
                                    if (r.getRoleId() == 6) {
                                        errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为兼职学术推广员！";
                                        continue;
                                    }
                                    if (r.getRoleId() == 7) {
                                        errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为财务管理员！";
                                    }
                                    if (r.getRoleId() == 31) {
                                        errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为招商经理";
                                    }
                                }
                            }
                        }
                    }

                } else if (userDTO.getRole().contains(3)) {
                    UserDTO tmp = new UserDTO();
                    tmp.setUsername(userDTO.getUsername());
                    userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
                    List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
                    if (CollUtil.isNotEmpty(userVoList)) {
                        for (UserVO userVO : userVoList) {
                            if (null != userVO.getRoleList()) {
                                for (SysRole r : userVO.getRoleList()) {
                                    if (r.getRoleId() == 2) {
                                        errorInfo = errorInfo + "用户已存在药企管理员角色,不能同时为平台管理员！";
                                        continue;
                                    }
                                    if (r.getRoleId() == 4) {
                                        errorInfo = errorInfo + "用户已存在药企管理员角色,不能同时为CSO管理员！";
                                        continue;
                                    }
                                    if (dept.getLevel() == 3 || dept.getLevel() == 4) {
                                        if (r.getRoleId() == 5) {
                                            errorInfo = errorInfo + "用户已存在CSO关系角色,不能同时为药企管理员！";
                                        }
                                    }
                                }
                            }
                        }
                    }

                } else if (userDTO.getRole().contains(4) || userDTO.getRole().contains(37)) {
                    UserDTO tmp = new UserDTO();
                    tmp.setUsername(userDTO.getUsername());
                    userDTO.setDelFlag(CommonConstants.STATUS_NORMAL);
                    List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
                    if (CollUtil.isNotEmpty(userVoList)) {
                        for (UserVO userVO : userVoList) {
                            if (null != userVO.getRoleList()) {
                                for (SysRole r : userVO.getRoleList()) {
                                    if (r.getRoleId() == 2) {
                                        errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为！";
                                        continue;
                                    }

                                    if (r.getRoleId() == 3) {
                                        errorInfo = errorInfo + "用户已存在药企管理员角色,不能同时为CSO管理员！";
                                        continue;
                                    }
                                    if (r.getRoleId() == 5) {
                                        SysUserRole sysUserRole = new SysUserRole();
                                        sysUserRole.setUserId(userVO.getUserId());
                                        List<SysUserRole> sysUserRoleList = sysUserRoleMapper.selectById(sysUserRole);
                                        for (SysUserRole userRole : sysUserRoleList) {
                                            SysUser sysUser = getById(userRole.getUserId());
                                            if (!sysUser.getDeptId().equals(userDTO.getDeptId())) {
                                                errorInfo = errorInfo + "用户已存在全职学术推广员角色,不能为其他机构CSO管理员！";
                                            }
                                        }
                                    }
                                    if (r.getRoleId() == 31) {
                                        SysUserRole sysUserRole = new SysUserRole();
                                        sysUserRole.setUserId(userVO.getUserId());
                                        List<SysUserRole> sysUserRoleList = sysUserRoleMapper.selectById(sysUserRole);
                                        for (SysUserRole userRole : sysUserRoleList) {
                                            SysUser sysUser = getById(userRole.getUserId());
                                            if (!sysUser.getDeptId().equals(userDTO.getDeptId())) {
                                                errorInfo = errorInfo + "用户已存在招商经理角色,不能为其他机构CSO管理员";
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }

                } else if (userDTO.getRole().contains(7)) {
                    UserDTO tmp = new UserDTO();
                    tmp.setUsername(userDTO.getUsername());
                    tmp.setDelFlag("0");
                    List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
                    if (CollUtil.isNotEmpty(userVoList)) {
                        for (UserVO userVO : userVoList) {
                            if (null != userVO.getRoleList()) {
                                for (SysRole r : userVO.getRoleList()) {
                                    if (r.getRoleId() == 2) {
                                        errorInfo = errorInfo + "用户已存在平台管理员,不能同时为财务管理员！";
                                        continue;
                                    }
                                    if (dept.getLevel() == 3 || dept.getLevel() == 4) {
                                        if (r.getRoleId() == 3) {
                                            errorInfo = errorInfo + "用户已存在药企管理员,不能同时为CSO机构财务管理员！";
                                        }
                                    }
                                }
                            }
                        }
                    }

                } else if (userDTO.getRole().contains(5)) {
                    UserDTO tmp = new UserDTO();
                    tmp.setUsername(userDTO.getUsername());
                    tmp.setDelFlag("0");
                    List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
                    if (CollUtil.isNotEmpty(userVoList)) {
                        for (UserVO userVO : userVoList) {
                            if (null != userVO.getRoleList()) {
                                for (SysRole r : userVO.getRoleList()) {
                                    if (r.getRoleId() == 2) {
                                        errorInfo = errorInfo + "用户已存在平台管理员,不能同时为全职学术推广员！";
                                        continue;
                                    }
                                    if (r.getRoleId() == 3) {
                                        if (dept.getLevel() == 3 || dept.getLevel() == 4) {
                                            errorInfo = errorInfo + "用户已存在药企管理员,不能为CSO的全职学术推广员！";
                                            continue;
                                        }
                                    }
                                    if (r.getRoleId() == 4) {
                                        if (dept.getLevel() == 3 || dept.getLevel() == 4) {
                                            errorInfo = errorInfo + "用户已存在CSO管理员,不能为其他CSO的全职学术推广员！";
                                        }
                                    }

                                }
                            }
                        }
                    }

                } else if (userDTO.getRole().contains(6)) {
                    UserDTO tmp = new UserDTO();
                    tmp.setUsername(userDTO.getUsername());
                    tmp.setDelFlag("0");
                    List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
                    if (CollUtil.isNotEmpty(userVoList)) {
                        for (UserVO userVO : userVoList) {
                            if (null != userVO.getRoleList()) {
                                for (SysRole r : userVO.getRoleList()) {
                                    if (r.getRoleId() == 2) {
                                        errorInfo = errorInfo + "用户已存在平台管理员,不能同时为兼职学术推广员！";
                                        continue;
                                    }
                                    SysUserRole sysUserRole = new SysUserRole();
                                    sysUserRole.setUserId(userVO.getUserId());
                                    List<SysUserRole> sysUserRoleList = sysUserRoleMapper.selectById(sysUserRole);
                                    for (SysUserRole userRole : sysUserRoleList) {
                                        if (userRole.getRoleId().equals(7)) {
                                            SysUser sysUser = getById(userRole.getUserId());
                                            if (sysUser.getDeptId().equals(userDTO.getDeptId())) {
                                                errorInfo = errorInfo + "用户已存在财务管理员角色，不能同时为兼职学术推广员！";
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }

                } else if (userDTO.getRole().contains(9)) {
                    UserDTO tmp = new UserDTO();
                    tmp.setUsername(userDTO.getUsername());
                    tmp.setDelFlag("0");
                    List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
                    if (CollUtil.isNotEmpty(userVoList)) {
                        for (UserVO userVO : userVoList) {
                            if (null != userVO.getRoleList()) {
                                for (SysRole r : userVO.getRoleList()) {
                                    if (r.getRoleId() == 2) {
                                        errorInfo = errorInfo + "用户已存在平台管理员,不能同时为CRO管理员！";
                                        continue;
                                    }
                                    if (r.getRoleId() == 3) {
                                        errorInfo = errorInfo + "用户已存在药企管理员,不能同时为CRO管理员！";
                                        continue;
                                    }
                                }
                            }
                        }
                    }
                }
                if (userDTO.getRole().contains(3)) {
                    if (dept.getLevel() != 2) {
                        errorInfo = errorInfo + "角色选为药企管理员时,组织架构必须为药企！";
                    }
                }
            }
        }

        return errorInfo;
    }

    /**
     * 校验通用dto信息
     *
     * @param commonUserDTOList 通用用户dto信息
     * @param creator           创建人
     * @param errorMap          错误信息记录map
     * @param currentList       正确结果收集信息
     */
    private void checkoutCommonUserDTOInfo(List<CommonUserDTO> commonUserDTOList, HnqzUser creator, Map<String, String> errorMap, List<UserDTO> currentList) {

        // 行编号
        log.info("开始校验数据：{}", commonUserDTOList);

        if (commonUserDTOList.size() != 1) {
            log.error("表中手机号重复");
            Set<String> idSet = commonUserDTOList.stream().map(CommonUserDTO::getId).collect(Collectors.toSet());
            errorMap(errorMap, "表中手机号重复", JSONUtil.toJsonStr(idSet));
        }

        CommonUserDTO commonUserDTO = commonUserDTOList.get(0);

        UserDTO userDTO = new UserDTO();

        // 校验用户名称
        String realName = StrUtil.cleanBlank(commonUserDTO.getRealName());
        if (StringUtils.isEmpty(realName)) {
            log.error("用户姓名为空");
            errorMap(errorMap, "用户姓名为空", commonUserDTO.getId());
        } else {
            userDTO.setRealname(realName);
        }

        // 校验用户所属企业
        List<SysDept> list = sysDeptService.list(Wrappers.<SysDept>lambdaQuery().like(SysDept::getName, StrUtil.cleanBlank(commonUserDTO.getEntName())));
        SysDept dept = null;
        if (CollUtil.isEmpty(list)) {
            log.error("所属企业不存在");
            errorMap(errorMap, "所属企业不存在", commonUserDTO.getId());
        } else {
            dept = list.get(0);
            userDTO.setDeptId(dept.getDeptId());
        }

        // 校验身份证号
        String idCard = StrUtil.cleanBlank(commonUserDTO.getIdCard());
        if (StrUtil.isNotBlank(idCard)) {
            // log.error("身份证号不存在");
            // errorMap(errorMap, "身份证号不存在", commonUserDTO.getId());
            // } else {
            if (!Validator.isCitizenId(idCard)) {
                log.error("身份证号格式不正确");
                errorMap(errorMap, "身份证号格式不正确", commonUserDTO.getId());
            }

            int count = this.count(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getIdCardNumber, commonUserDTO.getId()));
            if (count != 0) {
                log.error("身份证号格式已经被占用");
                errorMap(errorMap, "身份证号格式已经被占用", commonUserDTO.getId());
            }

            userDTO.setIdCardNumber(idCard);
        }

        // 校验手机号
        String username = StrUtil.cleanBlank(commonUserDTO.getUsername());
        if (StringUtils.isEmpty(username)) {
            log.error("手机号为空");
            errorMap(errorMap, "用户手机号为空", commonUserDTO.getId());
        } else {
            // 校验手机号格式
            boolean isMobile = Validator.isMobile(username);
            if (!isMobile) {
                log.error("用户手机号各式不正确");
                errorMap(errorMap, "用户手机号各式不正确", commonUserDTO.getId());
            }

            // 校验部门下是否有启用的用户
            int count = count(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getDelFlag, CommonConstants.STATUS_NORMAL).eq(SysUser::getUsername, username));
            // .eq(dept != null, SysUser::getDeptId, dept.getDeptId()));
            if (count != 0) {
                log.info("手机号被占用");
                errorMap(errorMap, "手机号被占用", commonUserDTO.getId());
            }

            userDTO.setPlatId(username);
            userDTO.setPhone(username);
            userDTO.setUsername(username);
        }

        // 校验用户角色
        String roleName = StrUtil.cleanBlank(commonUserDTO.getRoleName());
        String errInfo = checkoutCommonUserRole(roleName, userDTO, dept);
        if (StringUtils.isNotEmpty(errInfo)) {
            log.error(errInfo);
            errorMap(errorMap, errInfo, commonUserDTO.getId());
        }

        // 校验用户状态
        String lockFlag = StrUtil.cleanBlank(commonUserDTO.getLockFlag());
        if (StringUtils.isEmpty(lockFlag)) {
            log.error("用户状态为空");
            errorMap(errorMap, "用户状态为空", commonUserDTO.getId());
        } else {
            if (StringUtils.equals(lockFlag, "活跃")) {
                userDTO.setLockFlag(CommonConstants.STATUS_NORMAL);
            } else if (StringUtils.equals(lockFlag, "休眠")) {
                userDTO.setLockFlag(CommonConstants.STATUS_LOCK);
            } else {
                log.error("用户状态为值不是\"活跃\"或者\"休眠\"");
                errorMap(errorMap, "用户状态为值不是\"活跃\"或者\"休眠\"", commonUserDTO.getId());
            }
        }

        // 记录正确的结果
        userDTO.setCreateTime(LocalDateTime.now());
        userDTO.setUpdateTime(LocalDateTime.now());
        currentList.add(userDTO);

    }


    /**
     * 保存用户信息
     *
     * @param userDto DTO 对象
     * @return success/fail
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer saveUser(UserDTO userDto) {

        SysUser sysUser = new SysUser();
        if (null != userDto.getUsername()) {
            sysUser.setPassword(userDto.getUsername().substring(5));
            userDto.setPassword(userDto.getUsername().substring(5));
        }
        BeanUtils.copyProperties(userDto, sysUser);
        sysUser.setDelFlag(CommonConstants.STATUS_NORMAL);
        sysUser.setW1(Md5Utils.getMD5(userDto.getPassword().getBytes()));
        sysUser.setPassword(ENCODER.encode(userDto.getPassword()));
        sysUser.setPlatId(userDto.getPhone());

        baseMapper.insert(sysUser);

        if (userDto.getRole().contains(5) || userDto.getRole().contains(6)) {
            redisTemplate.opsForValue().set("user_add_notice_key:" + sysUser.getUserId(), sysUser.getUserId());
        }

        List<SysUserRole> userRoleList = userDto.getRole().stream().map(roleId -> {
            SysUserRole userRole = new SysUserRole();
            userRole.setUserId(sysUser.getUserId());
            userRole.setRoleId(roleId);
            return userRole;
        }).collect(Collectors.toList());
        log.info("-----角色ID：{}，角色ID数：{}", userRoleList, userRoleList.size());
        sysUserRoleService.saveBatch(userRoleList);

        // 关联上级主管
        if (CollUtil.isNotEmpty(userDto.getParentIdList())) {
            userDto.getParentIdList().forEach(pId -> {
                SysCsmUserRelation csmUserRelation = new SysCsmUserRelation();
                csmUserRelation.setUserId(sysUser.getUserId());
                csmUserRelation.setParentId(pId);
                sysCsmUserRelationService.save(csmUserRelation);
            });
        }

        // 管理区域管理
        userAreaService.saveRelation(Long.valueOf(sysUser.getUserId()), userDto.getAreaCodes());

        return sysUser.getUserId();
    }

    /**
     * 通过查用户的全部信息
     *
     * @param sysUser 用户
     * @return UserInfo
     */
    @Override
    public UserInfo findUserInfo(SysUser sysUser) {

        final String errorKey = "login:error:lock:" + sysUser.getUsername();
        if (Boolean.TRUE.equals(redisTemplate.hasKey(errorKey))) {
            sysUser.setLockFlag(CommonConstants.STATUS_LOCK);
        }

        UserInfo userInfo = new UserInfo();
        userInfo.setSysUser(sysUser);
        // 设置角色列表 （ID）
        List<Integer> roleIds = sysRoleService.findRolesByUserId(sysUser.getUserId()).stream().map(SysRole::getRoleId)
                .collect(Collectors.toList());
        userInfo.setRoles(ArrayUtil.toArray(roleIds, Integer.class));

        // 设置权限列表（menu.permission）
        Set<String> permissions = new HashSet<>();
        roleIds.forEach(roleId -> {
            List<String> permissionList = sysMenuService.findMenuByRoleId(roleId).stream()
                    .map(MenuVO::getPermission).filter(StringUtils::isNotEmpty)
                    .collect(Collectors.toList());
            permissions.addAll(permissionList);
        });
        userInfo.setPermissions(ArrayUtil.toArray(permissions, String.class));
        return userInfo;
    }

    /**
     * 分页查询用户信息（含有角色信息）
     *
     * @param page    分页对象
     * @param userDTO 参数列表
     * @return 分页结果
     */
    @Override
    public Page<UserVO> getUsersWithRolePage(Page<UserVO> page, UserDTO userDTO) {
        return baseMapper.getUserVosPage(page, userDTO, new DataScope());
    }

    /**
     * 分页查询用户信息（含有角色信息）
     *
     * @param page    分页对象
     * @param userDTO 参数列表
     * @return 分页结果
     */
    @Override
    public Page<UserVO> getUsersWithRolePageNoLock(Page<UserVO> page, UserDTO userDTO) {
        return baseMapper.getUserVosPageNoLock(page, userDTO, new DataScope());
    }

    @Override
    public Page<UserVO> selectUserList(Page<UserVO> page, UserDTO userDTO, HnqzUser user) {
        List<Long> areas;
        if (CollUtil.isNotEmpty(userDTO.getAreaCodes())) {
            areas = userDTO.getAreaCodes();
        } else {
            areas = userAreaService.listUserAreas(Long.valueOf(user.getId()));
            if (CollUtil.isNotEmpty(areas)) {
                areas = areaEntityService.list(Wrappers.<SysAreaEntity>lambdaQuery()
                                .in(SysAreaEntity::getAreaId, areas)
                                .eq(SysAreaEntity::getAreaStatus, Boolean.TRUE)
                                .eq(SysAreaEntity::getAreaType, UpmsType.AreaType.DISTRICT))
                        .stream()
                        .mapToLong(SysAreaEntity::getAreaId).boxed().distinct().collect(Collectors.toList());
            }

        }

        List<Integer> userid = userAreaService.listAreaUser(areas).stream().map(Long::intValue).collect(Collectors.toList());

        userDTO.setUserIdList(userid);

        if (SecurityUtils.getRoles().contains(1)) {
            return baseMapper.getUserVosPage(page, userDTO, new DataScope());
        }
        // 系统管理员
        else if (SecurityUtils.getRoles().contains(50)) {

            // 3营管理员
            // 4域管理员
            // 5职学术推广员
            // 37服务供应商管理员
            // 38营销中心分管领导
            // 39事业部总经理
            // 40商务管理员
            // 41事业部分管领导
            // 42市场管理员
            // 43地市管理员
            // 44安全管理员
            // 45审计管理员
            // 47事业部医药代表备案管理员
            // 48平台监督员
            // 49生产企业监督员
            // 50事业部管理员

            List<Integer> role = userDTO.getRole();
            if (CollUtil.isEmpty(role)) {
                userDTO.setRole(ListUtil.of(3, 4, 5, 37, 38, 39, 40, 41, 42, 43, 44, 45, 47, 48, 49, 50));
            }

            return baseMapper.getUserVosPage(page, userDTO, new DataScope());
        } else if (SecurityUtils.getRoles().contains(2) /*|| SecurityUtils.getRoles().contains(35)*/) {
            // 人数据权限
            List<Integer> bcRoles = Stream.of(2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 19, 31, 35, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(userDTO.getRole())) {
                Collection<Integer> intersection = CollUtil.intersectionDistinct(bcRoles, userDTO.getRole());
                if (CollUtil.isEmpty(intersection)) {
                    return page;
                }
                userDTO.setRole(new ArrayList<>(intersection));
            } else {
                userDTO.setRole(bcRoles);
            }
            return baseMapper.getUserVosPage(page, userDTO, new DataScope());
        }
        // else if (SecurityUtils.getRoles().contains(19)) {
        //     // CSM只能查看自己维护的企业下的用户, 以及下属维护的企业下的用户
        //     Integer userId = SecurityUtils.getUser().getId();
        //     // 递归查询下属关联的userId
        //     Set<Integer> allUserIds = sysCsmUserRelationService.recurseCsmLowerUserId(Collections.singleton(userId), new AtomicInteger(10));
        //     userDTO.setCsmUserIdList(allUserIds);
        //     return baseMapper.getUserVosPage(page, userDTO, new DataScope());
        // } else if (SecurityUtils.getRoles().contains(35)) {
        //     // BC总负责人数据权限
        //     List<Integer> bcRoles = Stream.of(3, 4, 5, 6, 7, 9, 10, 11, 13, 19, 31, 35, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47 ,48, 49).collect(Collectors.toList());
        //     if (CollUtil.isNotEmpty(userDTO.getRole())) {
        //         Collection<Integer> intersection = CollUtil.intersectionDistinct(bcRoles, userDTO.getRole());
        //         if (CollUtil.isEmpty(intersection)) {
        //             return page;
        //         }
        //         userDTO.setRole(new ArrayList<>(intersection));
        //     } else {
        //         userDTO.setRole(bcRoles);
        //     }
        //     return baseMapper.getUserVosPage(page, userDTO, new DataScope());
        // }

        // 备案管理员
        else if (SecurityUtils.getRoles().contains(47)) {
            userDTO.setRole(ListUtil.of(5, 6));
            return baseMapper.getUserVosPage(page, userDTO, new DataScope());
        }
        Integer deptId = userDTO.getDeptId();
        userDTO.setDeptId(SecurityUtils.getUser().getDeptId());
        List<UserVO> userList = sysUserMapper.selectByDeptId(userDTO);
        SysDeptRelation sysDeptRelation = new SysDeptRelation();
        sysDeptRelation.setAncestor(userDTO.getDeptId());
        List<SysDeptRelation> sysDeptRelationList = sysDeptRelationMapper.selectListByAncestor(sysDeptRelation);
        List<Integer> ids1 = sysDeptRelationList.stream().map(SysDeptRelation::getDescendant).collect(Collectors.toList());
        List<UserVO> userList1 = new ArrayList<>();
        List<UserVO> userList2 = new ArrayList<>();
        for (Integer id : ids1) {
            userDTO.setDeptId(id);
            List<UserVO> userList3 = sysUserMapper.selectByDeptId(userDTO);
            userList1.addAll(userList3);
        }
        List<Integer> ids2 = userList1.stream().map(UserVO::getUserId).collect(Collectors.toList());
        for (int i = 0; i < userList1.size(); i++) {
            if (CollUtil.isNotEmpty(userList1.get(i).getRoleList())) {
                if (userList1.get(i).getRoleList().get(0).getRoleId().equals(4)) {
                    userDTO.setUserId(ids2.get(i));
                    UserVO userVO = sysUserMapper.selectByUserId(userDTO);
                    userList2.add(userVO);
                }
            }
        }
        userList.addAll(userList2);
        List<Integer> ids = userList.stream().map(UserVO::getUserId).collect(Collectors.toList());
        for (int i = 0; i < userList.size(); i++) {
            List<SysRole> sysRoleList1 = sysRoleService.findRolesByUserId(ids.get(i));
            userList.get(i).setRoleList(sysRoleList1);
        }

        if (null != deptId) {
            userList = userList.stream().filter(item -> deptId.equals(item.getDeptId())).collect(Collectors.toList());
        }


        // 每页数据条数
        Page<UserVO> page1 = new Page<>();
        int current = Long.valueOf(page.getCurrent()).intValue();
        int size = Long.valueOf(page.getSize()).intValue();

        int count = userList.size();
        List<UserVO> pageList = new ArrayList<>();
        // 计算当前页第一条数据的下标
        int currId = current > 1 ? (current - 1) * size : 0;
        for (int i = 0; i < size && i < count - currId; i++) {
            pageList.add(userList.get(currId + i));
        }
        page1.setSize(size);
        page1.setCurrent(current);
        page1.setTotal(count);
        // 计算分页总页数
        page1.setPages(count % 10 == 0 ? count / 10 : count / 10 + 1);
        page1.setRecords(pageList);
        return page1;
    }

    @Override
    public Page<UserVO> selectUserList2(Page<UserVO> page, UserDTO userDTO, HnqzUser user) {

        List<Long> areas;
        if (CollUtil.isNotEmpty(userDTO.getAreaCodes())) {
            areas = userDTO.getAreaCodes();
        } else {
            areas = userAreaService.listUserAreas(Long.valueOf(user.getId()));
            if (CollUtil.isNotEmpty(areas)) {
                areas = areaEntityService.list(Wrappers.<SysAreaEntity>lambdaQuery()
                                .in(SysAreaEntity::getAreaId, areas)
                                .eq(SysAreaEntity::getAreaStatus, Boolean.TRUE)
                                .eq(SysAreaEntity::getAreaType, UpmsType.AreaType.DISTRICT))
                        .stream()
                        .mapToLong(SysAreaEntity::getAreaId).boxed().distinct().collect(Collectors.toList());
            }

        }

        List<Integer> userid = userAreaService.listAreaUser(areas).stream().map(Long::intValue).collect(Collectors.toList());

        userDTO.setUserIdList(userid);

        if (SecurityUtils.getRoles().contains(1)) {
            return baseMapper.getUserVosPage(page, userDTO, new DataScope());
        }
        // 系统管理员
        else if (SecurityUtils.getRoles().contains(50)) {

            // 3营管理员
            // 4域管理员
            // 5职学术推广员
            // 37服务供应商管理员
            // 38营销中心分管领导
            // 39事业部总经理
            // 40商务管理员
            // 41事业部分管领导
            // 42市场管理员
            // 43地市管理员
            // 44安全管理员
            // 45审计管理员
            // 47事业部医药代表备案管理员
            // 48平台监督员
            // 49生产企业监督员
            // 50事业部管理员

            List<Integer> role = userDTO.getRole();
            if (CollUtil.isEmpty(role)) {
                userDTO.setRole(ListUtil.of(3, 4, 5, 37, 38, 39, 40, 41, 42, 43, 44, 45, 47, 48, 49, 50));
            }

            return baseMapper.getUserVosPage(page, userDTO, new DataScope());
        } else if (SecurityUtils.getRoles().contains(2) /*|| SecurityUtils.getRoles().contains(35)*/) {
            // 人数据权限
            List<Integer> bcRoles = Stream.of(2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 19, 31, 35, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(userDTO.getRole())) {
                Collection<Integer> intersection = CollUtil.intersectionDistinct(bcRoles, userDTO.getRole());
                if (CollUtil.isEmpty(intersection)) {
                    return page;
                }
                userDTO.setRole(new ArrayList<>(intersection));
            } else {
                userDTO.setRole(bcRoles);
            }
            return baseMapper.getUserVosPage(page, userDTO, new DataScope());
        }
        // 备案管理员
        else if (SecurityUtils.getRoles().contains(47)) {
            userDTO.setRole(ListUtil.of(5, 6));
            return baseMapper.getUserVosPage(page, userDTO, new DataScope());
        }


        Integer deptId = userDTO.getDeptId();
        userDTO.setDeptId(SecurityUtils.getUser().getDeptId());
        List<UserVO> userList = sysUserMapper.selectByDeptId1(userDTO);

        // 查询下游企业
        SysDeptRelation sysDeptRelation = new SysDeptRelation();
        sysDeptRelation.setAncestor(userDTO.getDeptId());
        List<SysDeptRelation> sysDeptRelationList = sysDeptRelationMapper.selectListByAncestor(sysDeptRelation);
        List<Integer> ids1 = sysDeptRelationList.stream().map(SysDeptRelation::getDescendant).collect(Collectors.toList());


        List<UserVO> userList2 = new ArrayList<>();

        // for (Integer id : ids1) {
        //     userDTO.setDeptId(id);
        //     List<UserVO> userList3 = sysUserMapper.selectByDeptId1(userDTO);
        //     userList1.addAll(userList3);
        // }

        List<UserVO> userList1 = ids1.parallelStream().flatMap(id -> {
            userDTO.setDeptId(id);
            return sysUserMapper.selectByDeptId1(userDTO).stream();
        }).collect(Collectors.toList());

        List<Integer> ids2 = userList1.stream().map(UserVO::getUserId).collect(Collectors.toList());

        for (int i = 0; i < userList1.size(); i++) {
            if (CollUtil.isNotEmpty(userList1.get(i).getRoleList())) {
                if (userList1.get(i).getRoleList().get(0).getRoleId().equals(4)) {
                    userDTO.setUserId(ids2.get(i));
                    UserVO userVO = sysUserMapper.selectByUserId1(userDTO);
                    userList2.add(userVO);
                }
            }
        }
        userList.addAll(userList2);

        List<Integer> ids = userList.stream().map(UserVO::getUserId).collect(Collectors.toList());
        for (int i = 0; i < userList.size(); i++) {
            List<SysRole> sysRoleList1 = sysRoleService.findRolesByUserId(ids.get(i));
            userList.get(i).setRoleList(sysRoleList1);
        }

        if (null != deptId) {
            userList = userList.stream().filter(item -> deptId.equals(item.getDeptId())).collect(Collectors.toList());
        }


        // 每页数据条数
        Page<UserVO> page1 = new Page<>();
        int current = Long.valueOf(page.getCurrent()).intValue();
        int size = Long.valueOf(page.getSize()).intValue();

        int count = userList.size();
        List<UserVO> pageList = new ArrayList<>();
        // 计算当前页第一条数据的下标
        int currId = current > 1 ? (current - 1) * size : 0;
        for (int i = 0; i < size && i < count - currId; i++) {
            pageList.add(userList.get(currId + i));
        }
        page1.setSize(size);
        page1.setCurrent(current);
        page1.setTotal(count);
        // 计算分页总页数
        page1.setPages(count % 10 == 0 ? count / 10 : count / 10 + 1);
        page1.setRecords(pageList);
        return page1;
    }

    @Override
    public Page<UserVO> selectUserList1(Page<UserVO> page, UserDTO userDTO, HnqzUser user) {


        List<Long> areas;
        if (CollUtil.isNotEmpty(userDTO.getAreaCodes())) {
            areas = userDTO.getAreaCodes();
        } else {
            areas = userAreaService.listUserAreas(Long.valueOf(user.getId()));
            if (CollUtil.isNotEmpty(areas)) {
                areas = areaEntityService.list(Wrappers.<SysAreaEntity>lambdaQuery()
                                .in(SysAreaEntity::getAreaId, areas)
                                .eq(SysAreaEntity::getAreaStatus, Boolean.TRUE)
                                .eq(SysAreaEntity::getAreaType, UpmsType.AreaType.DISTRICT))
                        .stream()
                        .mapToLong(SysAreaEntity::getAreaId).boxed().distinct().collect(Collectors.toList());
            }

        }

        List<Integer> userid = userAreaService.listAreaUser(areas).stream().map(Long::intValue).collect(Collectors.toList());

        userDTO.setUserIdList(userid);

        return baseMapper.getUserVosPage1(page, userDTO);

    }

    /**
     * 通过ID查询用户信息
     *
     * @param id 用户ID
     * @return 用户信息
     */
    @Override
    public UserVO selectUserVoById(Integer id) {
        return baseMapper.getUserVoById(id);
    }

    /**
     * 删除用户
     *
     * @param sysUser 用户
     * @return Boolean
     */
    @Override
    @CacheEvict(value = CacheConstants.USER_DETAILS, key = "#sysUser.username")
    public Boolean deleteUserById(SysUser sysUser) {
        sysUserRoleService.deleteByUserId(sysUser.getUserId());
        this.removeById(sysUser.getUserId());
        return Boolean.TRUE;
    }

    @Override
    @Caching(evict = {@CacheEvict(value = CacheConstants.CLIENT_DETAILS_KEY, key = "#userDto.username"),
            @CacheEvict(value = CacheConstants.USER_DETAILS, key = "#userDto.username")})
    public R<Boolean> updateUserInfo(UserDTO userDto) {
        UserVO userVO = baseMapper.getUserVoById(userDto.getUserId());
        SysUser sysUser = new SysUser();
        String newPwd = ENCODER.encode(userDto.getNewpassword1());
        if (StrUtil.isNotBlank(userDto.getPassword()) && StrUtil.isNotBlank(userDto.getNewpassword1())) {
            if (ENCODER.matches(userDto.getPassword(), userVO.getPassword())) {
                sysUser.setPassword(newPwd);
            } else {
                log.info("原密码错误，修改密码失败:{}", userDto.getUsername());
                return R.failed("原密码错误，修改失败");
            }
        }
        sysUser.setPhone(userDto.getPhone());
        sysUser.setUserId(userVO.getUserId());
        sysUser.setAvatar(userDto.getAvatar());
        if (null != userDto.getNewpassword1()) {
            sysUser.setW1(Md5Utils.getMD5(userDto.getNewpassword1().getBytes()));
        }


        List<SysUser> list = this.getUserByPlatId(userVO.getPlatId());

        if (this.updateById(sysUser)) {
            for (SysUser sysUser1 : list) {
                if (!sysUser.getUserId().equals(sysUser1.getUserId())) {
                    sysUser1.setPassword(newPwd);
                    this.updateById(sysUser1);
                }
            }
        }

        return R.ok();
    }


    @Override
    @CacheEvict(value = CacheConstants.USER_DETAILS, key = "#userDto.username")
    public Boolean updateUser(UserDTO userDto) {
        SysUser sysUser = new SysUser();
        BeanUtils.copyProperties(userDto, sysUser);
        sysUser.setUpdateTime(LocalDateTime.now());
        if (null != userDto.getPassword()) {
            sysUser.setW1(Md5Utils.getMD5(userDto.getPassword().getBytes()));
        }
        if (StrUtil.isNotBlank(userDto.getPassword())) {
            sysUser.setPassword(ENCODER.encode(userDto.getPassword()));
        }

        this.updateById(sysUser);

        sysUserRoleService
                .remove(Wrappers.<SysUserRole>update().lambda().eq(SysUserRole::getUserId, userDto.getUserId()));
        userDto.getRole().forEach(roleId -> {
            SysUserRole userRole = new SysUserRole();
            userRole.setUserId(sysUser.getUserId());
            userRole.setRoleId(roleId);
            userRole.insert();
        });

        // 关联上级主管
        if (CollUtil.isNotEmpty(userDto.getParentIdList())) {
            sysCsmUserRelationService.remove(Wrappers.<SysCsmUserRelation>lambdaUpdate()
                    .eq(SysCsmUserRelation::getUserId, sysUser.getUserId()));
            userDto.getParentIdList().forEach(pId -> {
                SysCsmUserRelation csmUserRelation = new SysCsmUserRelation();
                csmUserRelation.setUserId(sysUser.getUserId());
                csmUserRelation.setParentId(pId);
                sysCsmUserRelationService.save(csmUserRelation);
            });
        }

        // 更新区域
        if (CollUtil.isNotEmpty(userDto.getAreaCodes())) {
            // 删除历史的
            userAreaService.delUserArea(Long.valueOf(sysUser.getUserId()));
            // 插入新的
            userAreaService.saveRelation(Long.valueOf(sysUser.getUserId()), userDto.getAreaCodes());
        }

        return Boolean.TRUE;
    }

    /**
     * 查询上级部门的用户信息
     *
     * @param username 用户名
     * @return R
     */
    @Override
    public List<SysUser> listAncestorUsers(String username) {
        SysUser sysUser = this.getOne(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username));

        SysDept sysDept = sysDeptService.getById(sysUser.getDeptId());
        if (sysDept == null) {
            return null;
        }

        Integer parentId = sysDept.getParentId();
        return this.list(Wrappers.<SysUser>query().lambda().eq(SysUser::getDeptId, parentId));
    }


    public List<SysUser> getUserByPlatId(String platId) {
        SysUser sysUser = new SysUser();
        sysUser.setDelFlag("0");
        sysUser.setPlatId(platId);
        return super.list(Wrappers.<SysUser>query(sysUser));
    }

    @Override
    public List<UserVO> getUserVoByPlatId(UserDTO userDTO) {

        // 放开角色校验
        // List<Integer> roleList = new ArrayList<>();
        // roleList.add(5);
        // roleList.add(6);
        // roleList.add(10);
        //// 患者教育-HCP角色
        // roleList.add(13);
        //// 招商经理
        // roleList.add(31);
        //
        // userDTO.setRole(roleList);
        return baseMapper.getUserVoByPlatId(userDTO);
    }

    @Override
    public R<?> updatePwd(UserPwd userPwd) {

        String p1 = userPwd.getP1();
        String p2 = userPwd.getP2();
        String userName = userPwd.getUserName();

        List<SysUser> userList = baseMapper.selectList(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getUsername, userName));

        if (CollectionUtils.isEmpty(userList)) {
            return R.failed("用户不存在");
        }

        if (!p1.equals(p2)) {
            return R.failed("两次密码输入不一致");
        }
        LocalDateTime now = LocalDateTime.now();
        for (SysUser sysUser : userList) {
            sysUser.setUserId(sysUser.getUserId());
            sysUser.setW1(Md5Utils.getMD5(p1.getBytes()));
            sysUser.setPassword(ENCODER.encode(p1));
            sysUser.setLatestChangePwdTime(now);
            sysUser.setUpdateTime(now);
            baseMapper.updateById(sysUser);
        }

        final String lockKey = "login:error:limit:times:" + userName;
        redisTemplate.delete(lockKey);

        return R.ok();
    }

    @Override
    public SysUser getByIdIgnore(String id) {
        return baseMapper.getByIdIgnore(id);
    }
}
