package com.qunzhixinxi.hnqz.admin.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.WmDaAgentDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.WmAgentComplianceDocument;
import com.qunzhixinxi.hnqz.admin.mapper.WmAgentComplianceDocumentMapper;
import com.qunzhixinxi.hnqz.admin.service.WmAgentComplianceDocumentService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * 企业合规服务实现
 *
 * @author jimmy
 * @date 2024/07/07 12:13
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmAgentComplianceDocumentServiceImpl extends ServiceImpl<WmAgentComplianceDocumentMapper, WmAgentComplianceDocument> implements WmAgentComplianceDocumentService {
    /**
     * 添加企业合规材料
     *
     * @param resource 合规材料
     * @param user     操作人
     * @return 添加结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean create(WmDaAgentDTO.OnCompDocCreate resource, HnqzUser user) {

        WmAgentComplianceDocument document = new WmAgentComplianceDocument();
        document.setDeptId(resource.deptId);
        document.setEmployeeCodeHandbook(resource.getEmployeeCodeHandbook().toArray(new WmAgentComplianceDocument.DocumentItem[0]));
        document.setControlSystemManual(resource.getControlSystemManual().toArray(new WmAgentComplianceDocument.DocumentItem[0]));
        document.setInternalAuditDocuments(resource.getInternalAuditDocuments().toArray(new WmAgentComplianceDocument.DocumentItem[0]));
        document.setComplianceMantSystem(resource.getComplianceMantSystem().toArray(new WmAgentComplianceDocument.DocumentItem[0]));
        document.setAntiCorrAndAntiBriManSys(resource.getAntiCorrAndAntiBriManSys().toArray(new WmAgentComplianceDocument.DocumentItem[0]));
        document.setEmployeeEntertainManSys(resource.getEmployeeEntertainManSys().toArray(new WmAgentComplianceDocument.DocumentItem[0]));
        document.setComplianceCommitLet(resource.getComplianceCommitLet().toArray(new WmAgentComplianceDocument.DocumentItem[0]));
        document.setInternalComTrainMate(resource.getInternalAuditDocuments().toArray(new WmAgentComplianceDocument.DocumentItem[0]));
        document.setDocumentId(resource.getDocumentId());

        boolean updateFlag = resource.getDocumentId() != null;
        String username = user.getUsername();
        LocalDateTime now = LocalDateTime.now();

        if (!updateFlag) {
            document.setCreateBy(username);
            document.setCreateTime(now);
        }
        document.setUpdateBy(username);
        document.setUpdateTime(now);
        return this.saveOrUpdate(document);
    }

    /**
     * 获取企业合规文件记录
     *
     * @param deptId 企业ID
     * @return 文件记录
     */
    @Override
    public WmAgentComplianceDocument getDoc(Integer deptId) {
        return this.getOne(Wrappers.<WmAgentComplianceDocument>lambdaQuery().eq(WmAgentComplianceDocument::getDeptId, deptId));
    }
}
