package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.WmDaAgentDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.WmAgentEvaluation;
import com.qunzhixinxi.hnqz.admin.api.model.WmEvaluationResultItemModel;
import com.qunzhixinxi.hnqz.admin.api.model.WmEvaluationResultModel;
import com.qunzhixinxi.hnqz.admin.mapper.WmAgentEvaluationMapper;
import com.qunzhixinxi.hnqz.admin.service.WmAgentEvaluationService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

@Slf4j
@Service
@AllArgsConstructor
public class WmAgentEvaluationServiceImpl extends ServiceImpl<WmAgentEvaluationMapper, WmAgentEvaluation> implements WmAgentEvaluationService {
    /**
     * 判断是否已经评价了
     *
     * @param deptId   企业ID
     * @param evalType 评价类型
     * @return 判断结果
     */
    @Override
    public boolean evaluated(Integer deptId, String evalType) {
        return this.count(Wrappers.<WmAgentEvaluation>lambdaQuery().eq(WmAgentEvaluation::getDeptId, deptId).eq(WmAgentEvaluation::getEvalType, evalType)) > 0;
    }

    /**
     * 企业评价
     *
     * @param resource 评价信息
     * @param user     操作人
     * @return 评价操作结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean evaluate(WmDaAgentDTO.OnEval resource, HnqzUser user) {

        WmAgentEvaluation evaluation = new WmAgentEvaluation();
        evaluation.setDeptId(resource.getDeptId());
        evaluation.setEvalType(resource.getEvalType());
        evaluation.setTargetName(resource.getTargetName());
        evaluation.setEvalTime(resource.getEvalTime());

        List<WmEvaluationResultItemModel> items = resource.getItems();

        int total = 0 ;
        int mark = 0;

        for (WmEvaluationResultItemModel item : items) {
            total += item.getTotal();
            mark += item.getMark();
        }

        WmEvaluationResultModel model = new WmEvaluationResultModel();
        model.setItems(items.toArray(new WmEvaluationResultItemModel[0]));
        model.setTotal(total);
        model.setMark(mark);

        evaluation.setEvalRes(model);
        evaluation.setAdminName(resource.getAdminName());
        evaluation.setSignDate(resource.getSignDate());
        evaluation.setCreateBy(user.getUsername());
        evaluation.setUpdateBy(user.getUsername());
        LocalDateTime now = LocalDateTime.now();
        evaluation.setCreateTime(now);
        evaluation.setUpdateTime(now);


        return this.save(evaluation);
    }

    /**
     * 获取准入评价
     *
     * @param deptId   企业ID
     * @param evalType 评价类型
     * @return 评价详情
     */
    @Override
    public WmAgentEvaluation getEvalDetails(Integer deptId, String evalType) {
        List<WmAgentEvaluation> evaluations = this.list(Wrappers.<WmAgentEvaluation>lambdaQuery()
                .eq(WmAgentEvaluation::getDeptId, deptId)
                .eq(WmAgentEvaluation::getEvalType, evalType).orderByDesc(WmAgentEvaluation::getCreateTime));

        return CollUtil.isNotEmpty(evaluations) ? evaluations.get(0) : null;
    }
}
