
package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDictItem;
import com.qunzhixinxi.hnqz.admin.api.entity.ReAlbumDrug;
import com.qunzhixinxi.hnqz.admin.api.entity.ReDeptDrug;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEntDrugtable;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.mapper.ReDeptDrugMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmDaDrugEntDrugtableMapper;
import com.qunzhixinxi.hnqz.admin.service.ReDeptDrugService;
import com.qunzhixinxi.hnqz.admin.service.SysDictItemService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntDrugtableService;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 档案-药企药品
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:42
 */
@Service
@AllArgsConstructor
public class WmDaDrugEntDrugtableServiceImpl extends ServiceImpl<WmDaDrugEntDrugtableMapper, WmDaDrugEntDrugtable> implements WmDaDrugEntDrugtableService {

    private final ReDeptDrugService reDeptDrugService;
    private final SysDictItemService sysDictItemService;

    @Override
    public R<List<String>> listReDept(WmDaDrugEntDrugtable wmDaDrugEntDrugtable) {

        List<ReDeptDrug> list = reDeptDrugService.list(Wrappers.<ReDeptDrug>lambdaQuery()
                .eq(ReDeptDrug::getDrugId, wmDaDrugEntDrugtable.getId()));

        List<String> reDeptList = list.stream().map(p -> p.getDeptId() + "").collect(Collectors.toList());
        return R.ok(reDeptList);
    }

    /**
     * 获取药品
     *
     * @param page     分页信息
     * @param deptId   部门id
     * @param drugName 药品名称
     * @return 药品列表
     */
    @Override
    public IPage<WmDaDrugEntDrugtable> listDrugs(Page<WmDaDrugEntDrugtable> page, Integer deptId, String drugName) {
        LambdaQueryWrapper<WmDaDrugEntDrugtable> queryWrapper = Wrappers.lambdaQuery();
        if (StringUtils.isNotEmpty(drugName)) {
            queryWrapper.like(WmDaDrugEntDrugtable::getDrugname, drugName);
        }
        if (deptId != null) {
            final List<ReDeptDrug> reDeptDrugList = reDeptDrugService.list(Wrappers.<ReDeptDrug>lambdaQuery()
                    .eq(ReDeptDrug::getDeptId, deptId));
            final List<Integer> drugIdList = reDeptDrugList.stream()
                    .mapToInt(ReDeptDrug::getDrugId)
                    .boxed()
                    .collect(Collectors.toList());
            queryWrapper.in(WmDaDrugEntDrugtable::getId, drugIdList);
        }

        this.page(page, queryWrapper);

        if (CollUtil.isNotEmpty(page.getRecords())) {
            // 剂型字典
            List<SysDictItem> doseDictItems = sysDictItemService.list(Wrappers.<SysDictItem>lambdaQuery()
                    .eq(SysDictItem::getType, "dose")
                    .eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.getVal()));
            Map<String, String> doseMap = doseDictItems.stream().collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));
            // 剂型字典
            List<SysDictItem> guigeDictItems = sysDictItemService.list(Wrappers.<SysDictItem>lambdaQuery()
                    .eq(SysDictItem::getType, "guige")
                    .eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.getVal()));
            Map<String, String> guigeMap = guigeDictItems.stream().collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));

            page.getRecords().forEach(record -> {
                if (StrUtil.isNotBlank(record.getDoseId()) && StrUtil.isBlank(record.getDoseName())) {
                    record.setDoseName(doseMap.getOrDefault(record.getDoseId(), record.getDoseId()));
                }
                if (StrUtil.isNotBlank(record.getGuigeId()) && StrUtil.isBlank(record.getGuigeName())) {
                    record.setGuigeName(guigeMap.getOrDefault(record.getGuigeId(), record.getGuigeId()));
                }
            });
        }

        return page;
    }

    /**
     * 根据部门id获取关联药品生产生产企业名称
     *
     * @param deptId 部门id
     * @return 生产企业列表
     */
    @Override
    public Set<String> listDrugProducerByDeptId(Integer deptId) {
        Set<Integer> drugIds = reDeptDrugService.list(Wrappers.<ReDeptDrug>lambdaQuery()
                        .eq(deptId != null, ReDeptDrug::getDeptId, deptId))
                .stream()/**/
                .mapToInt(ReDeptDrug::getDrugId)
                .boxed()
                .collect(Collectors.toSet());

        if (CollectionUtil.isEmpty(drugIds)) {
            return new HashSet<>(0);
        }

        return list(Wrappers.<WmDaDrugEntDrugtable>lambdaQuery()
                .eq(WmDaDrugEntDrugtable::getDelFlag, CommonConstants.STATUS_NORMAL)
                .eq(WmDaDrugEntDrugtable::getEnableFlag, CommonConstants.STATUS_NORMAL)
                .in(WmDaDrugEntDrugtable::getId, drugIds))
                .stream()
                .map(WmDaDrugEntDrugtable::getDrugEntName)
                .collect(Collectors.toSet());
    }

    @Override
    public R<Boolean> distribute(WmDaDrugEntDrugtable wmDaDrugEntDrugtable) {
        int id = Integer.parseInt(wmDaDrugEntDrugtable.getId());
        List<String> deptList = wmDaDrugEntDrugtable.getReDeptList();

        List<ReDeptDrug> reDeptDrugs = new ArrayList<>();

        reDeptDrugService.remove(Wrappers.<ReDeptDrug>lambdaQuery().eq(ReDeptDrug::getDrugId, id));


        for (String str : deptList) {
            ReDeptDrug reDeptDrug = new ReDeptDrug();
            reDeptDrug.setDeptId(Integer.parseInt(str));
            reDeptDrug.setDrugId(id);
            reDeptDrugs.add(reDeptDrug);
        }
        reDeptDrugService.saveBatch(reDeptDrugs);

        WmDaDrugEntDrugtable updateEntity = new WmDaDrugEntDrugtable();

        updateEntity.setId(wmDaDrugEntDrugtable.getId());
        updateEntity.setDrugEntNum(deptList.size());
        updateEntity.setUpdateUser(SecurityUtils.getUser().getId());
        updateEntity.setUpdateTime(LocalDateTime.now());
        this.baseMapper.updateById(updateEntity);


        return R.ok(Boolean.TRUE);
    }

    /**
     * 获取分配给指定企业的药品列表
     *
     * @param deptId 部门id
     * @return {@link List}<{@link Map}<{@link String}, {@link String}>>
     */
    @Override
    public List<Map<String, String>> listDrugByDeptId(Collection<Integer> deptId) {
        Set<Integer> drugIds = reDeptDrugService.list(Wrappers.<ReDeptDrug>lambdaQuery()
                        .in(CollUtil.isNotEmpty(deptId), ReDeptDrug::getDeptId, deptId))
                .stream()
                .mapToInt(ReDeptDrug::getDrugId)
                .boxed()
                .collect(Collectors.toSet());

        if (CollectionUtil.isEmpty(drugIds)) {
            return Collections.emptyList();
        }

        return listDrugs(drugIds);
    }

    @Override
    public List<Map<String, String>> listDrugs(Collection<Integer> drugIds) {

        List<WmDaDrugEntDrugtable> list = this.list(Wrappers.<WmDaDrugEntDrugtable>lambdaQuery()
                .eq(WmDaDrugEntDrugtable::getDelFlag, CommonConstants.STATUS_NORMAL)
                .eq(WmDaDrugEntDrugtable::getEnableFlag, CommonConstants.STATUS_NORMAL)
                .in(WmDaDrugEntDrugtable::getId, drugIds));

        return list.stream()
                .map(drug -> {
                    Map<String, String> map = new HashMap<>(4);
                    map.put("drug_id", drug.getId());
                    map.put("drug_name", drug.getDrugnameTy());
                    map.put("drug_name1", drug.getDrugname());
                    map.put("ent_name", drug.getDrugEntName());
                    return map;
                }).collect(Collectors.toList());
    }


}
