package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsLevel;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.entity.SysAreaEntity;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptCsm;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptRelation;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEnt;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskTypeEntRef;
import com.qunzhixinxi.hnqz.admin.mapper.WmDaDrugEntMapper;
import com.qunzhixinxi.hnqz.admin.service.SysAreaEntityService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptCsmService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptRelationService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysEnterpriseAreaService;
import com.qunzhixinxi.hnqz.admin.service.SysUserAreaService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskSubmissionPercentRuleService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskSubmissionRuleService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskTypeEntRefService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskTypeService;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.util.IDUtils;
import com.qunzhixinxi.hnqz.common.data.datascope.DataScope;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 档案-药企
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:42
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmDaDrugEntServiceImpl extends ServiceImpl<WmDaDrugEntMapper, WmDaDrugEnt> implements WmDaDrugEntService {

    private final SysDeptService sysDeptService;
    private final WmDaDrugEntMapper wmDaDrugEntMapper;
    private final SysDeptRelationService sysDeptRelationService;
    private final WmTaskSubmissionRuleService taskSubmissionRuleService;
    private final SysEnterpriseAreaService enterpriseAreaService;
    private final WmTaskSubmissionPercentRuleService taskSubmissionPercentRuleService;
    private final SysUserAreaService sysUserAreaService;
    private final SysAreaEntityService areaEntityService;
    private final WmTaskTypeService taskTypeService;
    private final WmTaskTypeEntRefService taskTypeEntRefService;
    private final SysDeptCsmService deptCsmService;


    @Override
    public List<WmDaDrugEnt> selectByScope(Wrapper<WmDaDrugEnt> queryWrapper) {
        return wmDaDrugEntMapper.selectListByScope(queryWrapper, new DataScope());

    }

    /**
     * 保存药企
     *
     * @param entity   药企信息
     * @param operator 操作人
     * @return 保存结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean saveDrugEnt(WmDaDrugEnt entity, HnqzUser operator) {

        // 保存药企信息
        LocalDateTime now = LocalDateTime.now();
        Integer userId = operator.getId();
        Integer ancestorId = operator.getDeptId();
        Integer deptId = IDUtils.getId();
        entity.setDeptId(deptId);
        entity.setCreateTime(now);
        entity.setManagement("1");
        entity.setPgqx("1");
        entity.setCreateTime(now);
        entity.setUpdateTime(now);
        entity.setCreateUser(userId);
        entity.setUpdateUser(userId);

        this.save(entity);

        // 映射部门表
        SysDept sysDept = new SysDept();
        sysDept.setDeptId(deptId);
        sysDept.setName(entity.getEntname());
        sysDept.setLevel(UpmsLevel.DeptLevel.L2.getLevel());
        sysDept.setDeptPermissions(entity.getDeptPermissions());
        sysDept.setFinancialReviewType(entity.getFinancialReviewType());
        sysDept.setIssueDate(now.toLocalDate());
        sysDept.setExpiryDate(now.plusYears(99L).toLocalDate());
        sysDept.setCreateTime(now);
        sysDept.setUpdateTime(now);
        sysDeptService.save(sysDept);

        // 构建部门关联
        List<SysDeptRelation> rels = new ArrayList<>(2);
        SysDeptRelation r1 = new SysDeptRelation();
        r1.setAncestor(ancestorId);
        r1.setDescendant(deptId);
        rels.add(r1);
        SysDeptRelation r2 = new SysDeptRelation();
        r2.setAncestor(deptId);
        r2.setDescendant(deptId);
        rels.add(r2);
        sysDeptRelationService.saveBatch(rels);

        // 初始化任务规则
        taskSubmissionRuleService.initDeptTaskSubmissionRule(Collections.singleton(deptId));
        // 初始化企业的任务类型配置的信息
        taskSubmissionPercentRuleService.initTaskTypeConfigRule(Collections.singleton(deptId));

        // 企业关联任务类型
        List<WmTaskType> taskTypes = taskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
                .eq(WmTaskType::getDelFlag, CommonConstants.STATUS_NORMAL)
                .eq(WmTaskType::getDurgentId, entity.getId()));

        if (CollUtil.isNotEmpty(taskTypes)) {
            log.info("企业{}, 积分规则已经分配", entity.getId());
        }

        WmTaskTypeEntRef wmTaskTypeEntRef = new WmTaskTypeEntRef();
        wmTaskTypeEntRef.setDrugentId(entity.getId());
        wmTaskTypeEntRef.setAgentId(entity.getId());
        wmTaskTypeEntRef.setDeptId(deptId.toString());
        wmTaskTypeEntRef.setCreateTime(now);
        wmTaskTypeEntRef.setUpdateTime(now);
        wmTaskTypeEntRef.setCreateUser(userId);
        wmTaskTypeEntRef.setUpdateUser(userId);
        wmTaskTypeEntRef.setDelFlag(CommonConstants.STATUS_NORMAL);
        wmTaskTypeEntRef.setEnableFlag(CommonConstants.STATUS_NORMAL);
        wmTaskTypeEntRef.setTenantId(operator.getTenantId());
        taskTypeEntRefService.save(wmTaskTypeEntRef);


        List<WmTaskType> types = taskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
                        .eq(WmTaskType::getDelFlag, CommonConstants.STATUS_NORMAL)
                        .eq(WmTaskType::getStatus, CommonConstants.STATUS_NORMAL)
                        .eq(WmTaskType::getTaskTypeLevel, "1")).stream()
                .map(tt -> {
                    WmTaskType t = new WmTaskType();
                    t.setTaskTypeName(tt.getTaskTypeName());
                    t.setTaskTypeLevel("2");
                    t.setStatus(CommonConstants.STATUS_NORMAL);
                    t.setDurgentId(entity.getId());
                    t.setLevel1Id(tt.getLevel1Id());
                    t.setTaskEntId(entity.getId());
                    t.setScore(tt.getScore());
                    t.setTaskScoreStart(tt.getTaskScoreStart());
                    t.setTaskScoreEnd(tt.getTaskScoreEnd());
                    t.setDeptId(deptId);
                    t.setDelFlag(CommonConstants.STATUS_NORMAL);
                    t.setEnableFlag(CommonConstants.STATUS_NORMAL);
                    t.setTenantId(tt.getTenantId());
                    t.setCreateTime(now);
                    t.setCreateUser(userId);
                    t.setUpdateTime(now);
                    t.setUpdateUser(userId);
                    t.setBaseId(tt.getBaseId());
                    t.setRealFlag(tt.getRealFlag());
                    t.setRuleId(entity.getId().toString());
                    t.setParentId(tt.getParentId());
                    t.setTaskReceiverType(tt.getTaskReceiverType());
                    t.setTaskRemark(tt.getTaskRemark());
                    return t;
                }).collect(Collectors.toList());

        taskTypeService.saveBatch(types);

        // CSM创建的企业,负责人默认是该CSM
        SysDeptCsm insert = new SysDeptCsm();
        insert.setDeptId(deptId);
        insert.setUserId(userId);
        insert.setCsmName(operator.getRealName());
        deptCsmService.save(insert);

        // 区域
        enterpriseAreaService.saveRelation(Long.valueOf(deptId), entity.getAreaCodes());

        return Boolean.TRUE;


    }

    @Override
    public WmDaDrugEnt selectByDeptId(Integer deptId) {
        return wmDaDrugEntMapper.selectByDeptId(deptId);
    }

    @Override
    public List<Map<String, Object>> selectEntNameByDeptId(Integer deptId) {
        return wmDaDrugEntMapper.selectEntNameByDeptId(deptId);
    }

    @Override
    public List selectEntNameByByDeptIds(Integer upId) {
        return wmDaDrugEntMapper.selectEntNameByByDeptIds(upId);
    }

    @Override
    public List<WmDaDrugEnt> getListByCsoDeptId(Integer deptId) {
        return wmDaDrugEntMapper.getListByCsoDeptId(deptId);
    }

    /**
     * 分页查询
     *
     * @param page        分页对象
     * @param wmDaDrugEnt 实体对象
     */
    @Override
    public void selectPage(Page<WmDaDrugEnt> page, WmDaDrugEnt wmDaDrugEnt) {

        List<Long> areas;
        if (CollUtil.isNotEmpty(wmDaDrugEnt.getAreaCodes())) {
            areas = wmDaDrugEnt.getAreaCodes();
        } else {
            areas = sysUserAreaService.listUserAreas(Long.valueOf(SecurityUtils.getUser().getId()));
            if (CollUtil.isNotEmpty(areas)) {
                areas = areaEntityService.list(Wrappers.<SysAreaEntity>lambdaQuery()
                                .in(SysAreaEntity::getAreaId, areas)
                                .eq(SysAreaEntity::getAreaStatus, Boolean.TRUE)
                                .eq(SysAreaEntity::getAreaType, UpmsType.AreaType.DISTRICT))
                        .stream()
                        .mapToLong(SysAreaEntity::getAreaId).boxed().distinct().collect(Collectors.toList());
            }
        }

        List<Long> ents = enterpriseAreaService.listAreaEnts(areas);
        wmDaDrugEnt.setEntIds(ents.stream().distinct().collect(Collectors.toList()));
        wmDaDrugEntMapper.selectPage(page, wmDaDrugEnt);
        page.getRecords().forEach(ent -> {
            List<Long> areaCodes = enterpriseAreaService.listEntAreas(Collections.singletonList(Long.valueOf(ent.getDeptId())));
            ent.setAreaCodes(areaCodes);

        });

    }
}
