package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.WmQuizDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDrugQuizRelation;
import com.qunzhixinxi.hnqz.admin.mapper.WmDrugQuizRelationMapper;
import com.qunzhixinxi.hnqz.admin.service.WmDrugQuizRelationService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 药品问卷服务实现
 *
 * @author jimmy
 * @date 07/07/2024 15:39
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmDrugQuizRelationServiceImpl extends ServiceImpl<WmDrugQuizRelationMapper, WmDrugQuizRelation> implements WmDrugQuizRelationService {


    /**
     * 药品绑定问卷
     *
     * @param resource 绑定信息
     * @param user     操作人
     * @return 绑定结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean bind(WmQuizDTO.OnDrugBind resource, HnqzUser user) {

        LocalDateTime now = LocalDateTime.now();
        String username = user.getUsername();
        Integer drugId = resource.getDrugId();
        // 删除旧的绑定关系
        this.remove(Wrappers.<WmDrugQuizRelation>lambdaQuery().eq(WmDrugQuizRelation::getDrugId, drugId));

        // 如果问卷为null 表示清空
        if (CollUtil.isEmpty(resource.getQuizIds())){

            return true;

        }

        // 插入新的绑定关系
        List<WmDrugQuizRelation> collect = resource.getQuizIds().stream().distinct().map(quizId -> {
            WmDrugQuizRelation relation = new WmDrugQuizRelation();
            relation.setDrugId(drugId);
            relation.setQuizId(quizId);
            relation.setCreateBy(username);
            relation.setUpdateBy(username);
            relation.setCreateTime(now);
            relation.setUpdateTime(now);
            return relation;
        }).collect(Collectors.toList());


        return this.saveBatch(collect);
    }
}
