package com.qunzhixinxi.hnqz.admin.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import com.qunzhixinxi.hnqz.admin.api.dto.WmSupervisionDto;
import com.qunzhixinxi.hnqz.admin.api.entity.WmPackageSupervision;
import com.qunzhixinxi.hnqz.admin.mapper.WmPackageSupervisionMapper;
import com.qunzhixinxi.hnqz.admin.service.WmPackageSupervisionService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;


/**
 * 服务包监督服务实现
 *
 * @author jimmy
 * @date 11/07/2024 16:50
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmPackageSupervisionServiceImpl extends ServiceImpl<WmPackageSupervisionMapper, WmPackageSupervision> implements WmPackageSupervisionService {

    /**
     * 监督结果
     *
     * @param resource 监督信息
     * @param user     操作人
     * @return 监督结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean createPkgSupervision(WmSupervisionDto.OnPkgSupervision resource, HnqzUser user) {

        WmPackageSupervision supervision = new WmPackageSupervision();
        supervision.setPackageId(resource.getPkgId());
        supervision.setSupervisionResult(resource.getSupervision());
        supervision.setRemarks(resource.getRemarks());
        supervision.setCreateBy(user.getUsername());
        supervision.setResultInfo(resource.getResultInfo());
        supervision.setSupervisionState(UpmsState.PackageSupervisionState.INIT);
        supervision.setCreateTime(LocalDateTime.now());
        supervision.setUpdateTime(LocalDateTime.now());
        supervision.setUpdateBy(user.getUsername());

        return this.save(supervision);
    }

    /**
     * 校验积分包是否已经监督过了
     *
     * @param pkgId 积分包ID
     * @return 校验结果
     */
    @Override
    public boolean hasChecked(Integer pkgId) {
        return this.count(Wrappers.<WmPackageSupervision>lambdaQuery().eq(WmPackageSupervision::getPackageId, pkgId)) > 0;
    }
}
