package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONNull;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.WmQccInfo;
import com.qunzhixinxi.hnqz.admin.mapper.WmQccInfoMapper;
import com.qunzhixinxi.hnqz.admin.service.WmQccInfoService;
import com.qunzhixinxi.hnqz.admin.util.QccUtils;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 企查查信息服务实现
 *
 * @author jimmy
 * @date 10/07/2024 11:12
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmQccInfoServiceImpl extends ServiceImpl<WmQccInfoMapper, WmQccInfo> implements WmQccInfoService {


    /**
     * 刷新企查查信息
     *
     * @param regCode 信用代码
     * @param user    操作人
     * @return 刷新结果
     */
    @Override
    public Map<String, Object> refresh(String regCode, HnqzUser user) {

        Map<String, Object> result = new HashMap<>(3);
        WmQccInfo qccInfo = new WmQccInfo();
        qccInfo.setCreateBy(user.getUsername());
        qccInfo.setCreateTime(LocalDateTime.now());
        qccInfo.setRegCode(regCode);


        // 刷新企业信息
        log.info("获取企业【{}】刷新企业信息", regCode);
        JSONObject eciInfo = QccUtils.getECIInfo(regCode);
        boolean ok = QccUtils.ok(eciInfo);
        if (ok) {
            JSONObject resp = QccUtils.getResult(eciInfo);
            // 照面信息
            Map<String, Object> bizInfo = new HashMap<>();
            bizInfo.put("KeyNo", resp.getStr("KeyNo"));
            bizInfo.put("Name", resp.getStr("Name"));
            bizInfo.put("CreditCode", resp.getStr("CreditCode"));
            bizInfo.put("BelongOrg", resp.getStr("BelongOrg"));
            bizInfo.put("OperId", resp.getStr("OperId"));
            bizInfo.put("OperName", resp.getStr("OperName"));
            bizInfo.put("StartDate", resp.getStr("StartDate"));
            bizInfo.put("EndDate", resp.getStr("EndDate"));
            bizInfo.put("Status", resp.getStr("Status"));
            bizInfo.put("RegistCapi", resp.getStr("RegistCapi"));
            bizInfo.put("EconKind", resp.getStr("EconKind"));
            bizInfo.put("Address", resp.getStr("Address"));
            bizInfo.put("TermStart", resp.getStr("TermStart"));
            bizInfo.put("TeamEnd", resp.getStr("TeamEnd"));
            bizInfo.put("CheckDate", resp.getStr("CheckDate"));
            bizInfo.put("OrgNo", resp.getStr("OrgNo"));
            bizInfo.put("IsOnStock", resp.getStr("IsOnStock"));
            bizInfo.put("StockNumber", resp.getStr("StockNumber"));
            bizInfo.put("StockType", resp.getStr("StockType"));
            bizInfo.put("RecCap", resp.getStr("RecCap"));
            bizInfo.put("AreaCode", resp.getStr("AreaCode"));
            bizInfo.put("No", resp.getStr("No"));
            bizInfo.put("Province", resp.getStr("Province"));
            bizInfo.put("UpdatedDate", resp.getStr("UpdatedDate"));
            bizInfo.put("ImageUrl", resp.getStr("ImageUrl"));
            bizInfo.put("EntType", resp.getStr("EntType"));
            qccInfo.setBizInfo(bizInfo);

            Object revokeInfo2 = resp.get("RevokeInfo");

            Map<String, Object> revokeInfo;
            if (revokeInfo2 instanceof JSONNull) {
                revokeInfo = Collections.emptyMap();
            } else {
                JSONObject revokeInfo1 = (JSONObject) revokeInfo2;
                revokeInfo = new HashMap<>(4);
                revokeInfo.put("CancelReason", revokeInfo1.getStr("CancelReason"));
                revokeInfo.put("CancelDate", revokeInfo1.getStr("CancelDate"));
                revokeInfo.put("RevokeDate", revokeInfo1.getStr("RevokeDate"));
                revokeInfo.put("RevokeReason", revokeInfo1.getStr("RevokeReason"));
            }
            qccInfo.setRevokeInfo(revokeInfo);

            JSONArray partners = (JSONArray) resp.get("Partners");
            List<Map<String, Object>> partner1 = partners.stream().map(p -> {
                JSONObject partner = (JSONObject) p;
                Map<String, Object> partnerInfo = new HashMap<>(4);
                partnerInfo.put("StockName", partner.getStr("StockName"));
                partnerInfo.put("StockPercent", partner.getStr("StockPercent"));
                partnerInfo.put("FinalBenefitPercent", partner.getStr("FinalBenefitPercent"));
                partnerInfo.put("StockType", partner.getStr("StockType"));
                return partnerInfo;
            }).collect(Collectors.toList());
            qccInfo.setPartnerList(partner1);

            JSONArray employees = (JSONArray) resp.get("Employees");
            List<Map<String, Object>> emps = employees.stream().map(emp -> {
                JSONObject employee = (JSONObject) emp;
                Map<String, Object> empInfo = new HashMap<>(3);
                empInfo.put("Name", employee.getStr("Name"));
                empInfo.put("KeyNo", employee.getStr("KeyNo"));
                empInfo.put("Job", employee.getStr("Job"));
                return empInfo;
            }).collect(Collectors.toList());
            qccInfo.setEmployeeList(emps);

            JSONArray changeRecords = (JSONArray) resp.get("ChangeRecords");
            List<Map<String, Object>> changeRecord1 = changeRecords.stream().map(cr -> {
                JSONObject changeRecord = (JSONObject) cr;
                Map<String, Object> changeInfo = new HashMap<>(4);
                changeInfo.put("ProjectName", changeRecord.getStr("ProjectName"));
                changeInfo.put("ChangeDate", changeRecord.getStr("ChangeDate"));
                changeInfo.put("AfterContent", changeRecord.getStr("AfterContent"));
                changeInfo.put("BeforeContent", changeRecord.getStr("BeforeContent"));
                return changeInfo;
            }).collect(Collectors.toList());
            qccInfo.setChangeList(changeRecord1);


            qccInfo.setBankInfo(Collections.emptyMap());
            // qccInfo.setScale();
            qccInfo.setTaxAbnormal(Collections.emptyMap());

            // 严重违法查询
            log.info("获取企业【{}】严重违法查询", regCode);
            JSONObject seriousIllegalList = QccUtils.getSeriousIllegalList(regCode);
            if (QccUtils.ok(seriousIllegalList)) {
                JSONObject result1 = QccUtils.getResult(seriousIllegalList);

                Object data1 = result1.get("Data");

                List<Map<String, Object>> collect;
                if (data1 instanceof JSONNull) {
                    collect = Collections.emptyList();
                } else {
                    JSONArray data = (JSONArray) data1;
                    collect = data.stream().map(item -> {
                        JSONObject itemInfo = (JSONObject) item;
                        Map<String, Object> itemInfoMap = new HashMap<>(6);
                        itemInfoMap.put("Type", itemInfo.getStr("Type"));
                        itemInfoMap.put("AddReason", itemInfo.getStr("AddReason"));
                        itemInfoMap.put("AddDate", itemInfo.getStr("AddDate"));
                        itemInfoMap.put("AddOffice", itemInfo.getStr("AddOffice"));
                        itemInfoMap.put("RemoveReason", itemInfo.getStr("RemoveReason"));
                        itemInfoMap.put("RemoveDate", itemInfo.getStr("RemoveDate"));
                        itemInfoMap.put("RemoveOffice", itemInfo.getStr("RemoveOffice"));
                        return itemInfoMap;
                    }).collect(Collectors.toList());
                }


                qccInfo.setSeriousIllegal(collect);
            }

            // 失信查询
            log.info("获取企业【{}】失信查询", regCode);
            JSONObject shixinList = QccUtils.getShixinList(regCode);
            if (QccUtils.ok(shixinList)) {
                JSONObject result1 = QccUtils.getResult(shixinList);

                Object data1 = result1.get("Data");

                List<Map<String, Object>> collect;
                if (data1 instanceof JSONNull) {
                    collect = Collections.emptyList();
                } else {
                    JSONArray data = (JSONArray) data1;
                    collect = data.stream().map(sx -> {
                        JSONObject sxInfo = (JSONObject) sx;
                        Map<String, Object> sxInfoMap = new HashMap<>(9);
                        sxInfoMap.put("Id", sxInfo.getStr("Id"));
                        sxInfoMap.put("Liandate", sxInfo.getStr("Liandate"));
                        sxInfoMap.put("Anno", sxInfo.getStr("Anno"));
                        sxInfoMap.put("Executegov", sxInfo.getStr("Executegov"));
                        sxInfoMap.put("Executestatus", sxInfo.getStr("Executestatus"));
                        sxInfoMap.put("Publicdate", sxInfo.getStr("Publicdate"));
                        sxInfoMap.put("Executeno", sxInfo.getStr("Executeno"));
                        sxInfoMap.put("ActionRemark", sxInfo.getStr("ActionRemark"));
                        sxInfoMap.put("Amount", sxInfo.getStr("Amount"));
                        return sxInfoMap;
                    }).collect(Collectors.toList());
                }


                qccInfo.setShiXin(collect);
            }

            // 实际控制人
            log.info("获取企业【{}】实际控制人", regCode);
            JSONObject actualControlList = QccUtils.getActualControlList(regCode);
            if (QccUtils.ok(actualControlList)) {
                JSONArray array = QccUtils.getArray(actualControlList);
                List<Map<String, Object>> collect = array.stream().map(a -> (Map<String, Object>) a).collect(Collectors.toList());
                qccInfo.setActualControllerList(collect);
            }

            // 清算信息
            log.info("获取企业【{}】清算信息", regCode);
            JSONObject liquidationDetail = QccUtils.getLiquidationDetail(regCode);
            if (QccUtils.ok(liquidationDetail)) {
                JSONObject result1 = QccUtils.getResult(liquidationDetail);
                Object data = result1.get("Data");
                qccInfo.setLiquidation(data instanceof JSONNull ? Collections.emptyMap() : (Map<String, Object>) result1.get("Data"));
            }

            // 税务等级信息
            log.info("获取企业【{}】税务等级信息", regCode);
            JSONObject taxCreditList = QccUtils.getTaxCreditList(regCode);
            if (QccUtils.ok(taxCreditList)) {
                JSONArray array = QccUtils.getArray(taxCreditList);
                List<Map<String, Object>> collect = array.stream().map(a -> (Map<String, Object>) a).collect(Collectors.toList());
                qccInfo.setTaxCreditList(collect);
            }


            // 经营异常
            log.info("获取企业【{}】经营异常", regCode);
            JSONObject exceptionList = QccUtils.getExceptionList(regCode);
            if (QccUtils.ok(exceptionList)) {
                JSONObject result1 = QccUtils.getResult(exceptionList);

                Object data1 = result1.get("Data");

                List<Map<String, Object>> collect;
                if (data1 instanceof JSONNull) {
                    collect = Collections.emptyList();
                } else {
                    JSONArray data = (JSONArray) data1;
                    collect = data.stream().map(item -> {
                        JSONObject itemInfo = (JSONObject) item;
                        Map<String, Object> itemInfoMap = new HashMap<>(6);
                        itemInfoMap.put("AddReason", itemInfo.getStr("AddReason"));
                        itemInfoMap.put("AddDate", itemInfo.getStr("AddDate"));
                        itemInfoMap.put("RomoveReason", itemInfo.getStr("RomoveReason"));
                        itemInfoMap.put("RemoveDate", itemInfo.getStr("RemoveDate"));
                        itemInfoMap.put("DecisionOffice", itemInfo.getStr("DecisionOffice"));
                        itemInfoMap.put("RemoveDecisionOffice", itemInfo.getStr("RemoveDecisionOffice"));
                        return itemInfoMap;
                    }).collect(Collectors.toList());
                }

                qccInfo.setExp(collect);
            }

            // 执行人
            log.info("获取企业【{}】执行人", regCode);
            JSONObject zhixingList = QccUtils.getZhixingList(regCode);
            if (QccUtils.ok(zhixingList)) {
                JSONObject result1 = QccUtils.getResult(zhixingList);
                Object data1 = result1.get("Data");

                List<Map<String, Object>> collect;
                if (data1 instanceof JSONNull) {
                    collect = Collections.emptyList();
                } else {
                    JSONArray data = (JSONArray) data1;

                    collect = data.stream().map(zx -> {
                        JSONObject zxInfo = (JSONObject) zx;
                        Map<String, Object> zxInfoMap = new HashMap<>(6);
                        zxInfoMap.put("Id", zxInfo.getStr("Id"));
                        zxInfoMap.put("Liandate", zxInfo.getStr("Liandate"));
                        zxInfoMap.put("Anno", zxInfo.getStr("Anno"));
                        zxInfoMap.put("ExecuteGov", zxInfo.getStr("ExecuteGov"));
                        zxInfoMap.put("Biaodi", zxInfo.getStr("Biaodi"));
                        zxInfoMap.put("SuspectedApplicant", zxInfo.getStr("SuspectedApplicant"));
                        return zxInfoMap;
                    }).collect(Collectors.toList());
                }

                qccInfo.setZhiXing(collect);
            }

            // 税收违法
            log.info("获取企业【{}】税收违法", regCode);
            JSONObject taxIllegalList = QccUtils.getTaxIllegalList(regCode);
            if (QccUtils.ok(taxIllegalList)) {
                JSONObject result1 = QccUtils.getResult(taxIllegalList);
                Object data1 = result1.get("Data");

                List<Map<String, Object>> collect;
                if (data1 instanceof JSONNull) {
                    collect = Collections.emptyList();
                } else {
                    JSONArray data = (JSONArray) data1;
                    collect = data.stream().map(item -> {
                        JSONObject itemInfo = (JSONObject) item;
                        Map<String, Object> itemInfoMap = new HashMap<>(6);
                        itemInfoMap.put("Id", itemInfo.getStr("Id"));
                        itemInfoMap.put("PublishDate", itemInfo.getStr("PublishDate"));
                        itemInfoMap.put("CaseNature", itemInfo.getStr("CaseNature"));
                        itemInfoMap.put("TaxGov", itemInfo.getStr("TaxGov"));
                        itemInfoMap.put("IllegalContent", itemInfo.getStr("IllegalContent"));
                        itemInfoMap.put("PunishContent", itemInfo.getStr("PunishContent"));
                        return itemInfoMap;
                    }).collect(Collectors.toList());
                }
                qccInfo.setTaxIllegal(collect);
            }

            // 破产重组
            log.info("获取企业【{}】破产重组", regCode);
            JSONObject bankruptcyList = QccUtils.getBankruptcyList(regCode);
            if (QccUtils.ok(bankruptcyList)) {
                JSONObject result1 = QccUtils.getResult(bankruptcyList);
                Object data1 = result1.get("Data");

                List<Map<String, Object>> collect;
                if (data1 instanceof JSONNull) {
                    collect = Collections.emptyList();
                } else {
                    JSONArray data = (JSONArray) data1;
                    collect = data.stream().map(item -> {
                        JSONObject itemInfo = (JSONObject) item;
                        Map<String, Object> itemInfoMap = new HashMap<>(5);
                        itemInfoMap.put("Id", itemInfo.getStr("Id"));
                        itemInfoMap.put("CaseNo", itemInfo.getStr("CaseNo"));
                        itemInfoMap.put("PublicDate", itemInfo.getStr("PublicDate"));
                        itemInfoMap.put("ApplicantList", itemInfo.get("ApplicantList"));
                        itemInfoMap.put("RespondentList", itemInfo.get("RespondentList"));
                        return itemInfoMap;
                    }).collect(Collectors.toList());
                }
                qccInfo.setBankruptcy(collect);
            }

            // 动产抵押
            log.info("获取企业【{}】动产抵押", regCode);
            JSONObject chattelMortgageList = QccUtils.getChattelMortgageList(regCode);
            if (QccUtils.ok(chattelMortgageList)) {
                JSONObject result1 = QccUtils.getResult(chattelMortgageList);
                Object data1 = result1.get("Data");

                List<Map<String, Object>> collect;
                if (data1 instanceof JSONNull) {
                    collect = Collections.emptyList();
                } else {
                    JSONArray data = (JSONArray) data1;
                    collect = data.stream().map(item -> {
                        JSONObject itemInfo = (JSONObject) item;
                        Map<String, Object> itemInfoMap = new HashMap<>(6);
                        itemInfoMap.put("RegisterNo", itemInfo.getStr("RegisterNo"));
                        itemInfoMap.put("RegisterDate", itemInfo.getStr("RegisterDate"));
                        itemInfoMap.put("RegisterOffice", itemInfo.getStr("RegisterOffice"));
                        itemInfoMap.put("DebtSecuredAmount", itemInfo.getStr("DebtSecuredAmount"));
                        itemInfoMap.put("Status", itemInfo.getStr("Status"));
                        itemInfoMap.put("Detail", itemInfo.getStr("Detail"));
                        itemInfoMap.put("DebtorInfo", itemInfo.getStr("DebtorInfo"));
                        return itemInfoMap;
                    }).collect(Collectors.toList());
                }
                qccInfo.setChattelMortgage(collect);
            }

            // 行政处罚
            log.info("获取企业【{}】行政处罚", regCode);
            JSONObject adminPenaltyList = QccUtils.getAdminPenaltyList(regCode);
            if (QccUtils.ok(adminPenaltyList)) {
                JSONObject result1 = QccUtils.getResult(adminPenaltyList);
                Object data = result1.get("Data");
                List<Map<String, Object>> collect;
                if (data instanceof JSONNull) {
                    collect = Collections.emptyList();
                } else {
                    JSONArray data1 = (JSONArray) data;
                    collect = data1.stream().map(sx -> {
                        JSONObject sxInfo = (JSONObject) sx;
                        Map<String, Object> sxInfoMap = new HashMap<>(9);
                        sxInfoMap.put("Id", sxInfo.getStr("Id"));
                        sxInfoMap.put("DocNo", sxInfo.getStr("DocNo"));
                        sxInfoMap.put("PunishReason", sxInfo.getStr("PunishReason"));
                        sxInfoMap.put("PunishResult", sxInfo.getStr("PunishResult"));
                        sxInfoMap.put("PunishOffice", sxInfo.getStr("PunishOffice"));
                        sxInfoMap.put("PunishDate", sxInfo.getStr("PunishDate"));
                        sxInfoMap.put("Source", sxInfo.getStr("Source"));
                        sxInfoMap.put("SourceCode", sxInfo.getStr("SourceCode"));
                        sxInfoMap.put("PunishAmt", sxInfo.getStr("PunishAmt"));
                        return sxInfoMap;
                    }).collect(Collectors.toList());
                }

                qccInfo.setAdminPenalty(collect);
            }

            // 限制高消费
            log.info("获取企业【{}】限制高消费", regCode);
            JSONObject sumptuaryList = QccUtils.getSumptuaryList(regCode);
            if (QccUtils.ok(sumptuaryList)) {
                JSONObject result1 = QccUtils.getResult(sumptuaryList);
                Object data1 = result1.get("Data");

                List<Map<String, Object>> collect;
                if (data1 instanceof JSONNull) {
                    collect = Collections.emptyList();
                } else {
                    JSONArray data = (JSONArray) data1;
                    collect = data.stream().map(item -> {
                        JSONObject itemInfo = (JSONObject) item;
                        Map<String, Object> itemInfoMap = new HashMap<>(10);
                        itemInfoMap.put("Id", itemInfo.getStr("Id"));
                        itemInfoMap.put("CaseNo", itemInfo.getStr("CaseNo"));
                        itemInfoMap.put("PersonName", itemInfo.getStr("PersonName"));
                        itemInfoMap.put("PersonId", itemInfo.getStr("PersonId"));
                        itemInfoMap.put("JudgeDate", itemInfo.getStr("JudgeDate"));
                        itemInfoMap.put("PublishDate", itemInfo.getStr("PublishDate"));
                        itemInfoMap.put("KeyNo", itemInfo.getStr("KeyNo"));
                        itemInfoMap.put("CompanyName", itemInfo.getStr("CompanyName"));
                        itemInfoMap.put("Applicant", itemInfo.getStr("Applicant"));
                        itemInfoMap.put("ApplicantNo", itemInfo.getStr("ApplicantNo"));
                        return itemInfoMap;
                    }).collect(Collectors.toList());
                }
                qccInfo.setSumptuary(collect);
            }

            // 行政许可
            log.info("获取企业【{}】行政许可", regCode);
            JSONObject adminLicenseList = QccUtils.getAdminLicenseList(regCode);
            if (QccUtils.ok(adminLicenseList)) {
                JSONObject result1 = QccUtils.getResult(adminLicenseList);
                Object data1 = result1.get("Data");
                List<Map<String, Object>> collect;
                if (data1 instanceof JSONNull) {
                    collect = Collections.emptyList();
                } else {
                    JSONArray data = (JSONArray) data1;
                    collect = data.stream().map(item -> {
                        JSONObject itemInfo = (JSONObject) item;
                        Map<String, Object> itemInfoMap = new HashMap<>(10);
                        itemInfoMap.put("LicenseNo", itemInfo.getStr("LicenseNo"));
                        itemInfoMap.put("LicenseName", itemInfo.getStr("LicenseName"));
                        itemInfoMap.put("ValidityFrom", itemInfo.getStr("ValidityFrom"));
                        itemInfoMap.put("ValidityTo", itemInfo.getStr("ValidityTo"));
                        itemInfoMap.put("LicensOffice", itemInfo.getStr("LicensOffice"));
                        itemInfoMap.put("LicensContent", itemInfo.getStr("LicensContent"));
                        itemInfoMap.put("Id", itemInfo.getStr("Id"));
                        itemInfoMap.put("Source", itemInfo.getStr("Source"));
                        itemInfoMap.put("DocumentNo", itemInfo.getStr("DocumentNo"));
                        itemInfoMap.put("PermissionNo", itemInfo.getStr("PermissionNo"));
                        return itemInfoMap;
                    }).collect(Collectors.toList());
                }
                qccInfo.setAdminLicenseList(collect);
            }


            // 企业评分
            log.info("获取企业【{}】评分", regCode);
            JSONObject creditRatingReportGrade = QccUtils.getCreditRatingReportGrade(regCode);
            if (QccUtils.ok(creditRatingReportGrade)) {
                JSONObject result1 = QccUtils.getResult(creditRatingReportGrade);
                Object data1 = result1.get("Data");
                Map<String, Object> collect;
                if (data1 instanceof JSONNull) {
                    collect = Collections.emptyMap();
                } else {
                    JSONObject data = (JSONObject) data1;
                    collect = new HashMap<>(4);
                    collect.put("Level", data.getStr("data"));
                    collect.put("Score", data.getStr("Score"));

                    JSONObject rm = data.getJSONObject("RatingModule");
                    Map<String, Object> RatingModule = new HashMap<>(18);
                    RatingModule.put("RatingScore", rm.get("RatingScore"));
                    RatingModule.put("ShiXinScore", rm.get("ShiXinScore"));
                    RatingModule.put("ZhiXingScore", rm.get("ZhiXingScore"));
                    RatingModule.put("BlacklistScore", rm.get("BlacklistScore"));
                    RatingModule.put("SevereAdminPenaltyScore", rm.get("SevereAdminPenaltyScore"));
                    RatingModule.put("SafetyHazardScore", rm.get("SafetyHazardScore"));
                    RatingModule.put("BreakOrderScore", rm.get("BreakOrderScore"));
                    RatingModule.put("GeneralAdminPenaltyScore", rm.get("GeneralAdminPenaltyScore"));
                    RatingModule.put("AdminEnforceScore", rm.get("AdminEnforceScore"));
                    RatingModule.put("OtherShiXinScore", rm.get("OtherShiXinScore"));
                    RatingModule.put("AbnormalOperationScore", rm.get("AbnormalOperationScore"));
                    RatingModule.put("EndLevelEvaluationScore", rm.get("EndLevelEvaluationScore"));
                    RatingModule.put("ContractSXScore", rm.get("ContractSXScore"));
                    RatingModule.put("BusinessSXScore", rm.get("BusinessSXScore"));
                    RatingModule.put("NegativeOpinionScore", rm.get("NegativeOpinionScore"));
                    RatingModule.put("SeniorStaffSXScore", rm.get("SeniorStaffSXScore"));
                    RatingModule.put("OperCriminalJudgmentScore", rm.get("OperCriminalJudgmentScore"));
                    RatingModule.put("BranchCreditScore", rm.get("BranchCreditScore"));
                    collect.put("RatingModule", RatingModule);


                    JSONObject am = data.getJSONObject("AdjustModule");
                    Map<String, Object> AdjustModule = new HashMap<>(13);
                    AdjustModule.put("AdjustScore", am.get("AdjustScore"));
                    AdjustModule.put("RegistcapiScore", am.get("RegistcapiScore"));
                    AdjustModule.put("OperationTimeScore", am.get("OperationTimeScore"));
                    AdjustModule.put("EciChangeScore", am.get("EciChangeScore"));
                    AdjustModule.put("BusinessDevelop", am.get("BusinessDevelop"));
                    AdjustModule.put("MarketActivity", am.get("MarketActivity"));
                    AdjustModule.put("IntellectualProperty", am.get("IntellectualProperty"));
                    AdjustModule.put("QLicensingScore", am.get("QLicensingScore"));
                    AdjustModule.put("HonourScore", am.get("HonourScore"));
                    AdjustModule.put("RedListScore", am.get("RedListScore"));
                    AdjustModule.put("CommunityActivity", am.get("AdjustScore"));
                    AdjustModule.put("PovertyRelief", am.get("AdjustScore"));
                    AdjustModule.put("TirdEvaluation", am.get("AdjustScore"));
                    collect.put("AdjustModule", AdjustModule);
                }
                qccInfo.setCreditRatingReportGrade(collect);
            }


        }


        // 保存
        this.save(qccInfo);

        result.put("status", ok);
        result.put("msg", QccUtils.getMessage(eciInfo));
        result.put("qccInfo", qccInfo);
        return result;
    }

    /**
     * 获取企业企查查信息
     *
     * @param regCode 企业信用代码
     * @return 企查查信息
     */
    @Override
    public WmQccInfo getQccInfo(String regCode) {
        List<WmQccInfo> list = this.list(Wrappers.<WmQccInfo>lambdaQuery().eq(WmQccInfo::getRegCode, regCode).orderByDesc(WmQccInfo::getCreateTime));
        return CollUtil.isEmpty(list) ? null : list.get(0);
    }

    /**
     * 批量获取企业企查查信息
     *
     * @param regCodes 企业信用代码
     * @return 企查查信息
     */
    @Override
    public Map<String, List<WmQccInfo>> getQccInfo1(Collection<String> regCodes) {
        List<WmQccInfo> list = this.list(Wrappers.<WmQccInfo>lambdaQuery().in(WmQccInfo::getRegCode, regCodes).orderByDesc(WmQccInfo::getCreateTime));

        if (CollUtil.isEmpty(list)) {
            return Collections.emptyMap();
        }

        return list.stream().collect(Collectors.groupingBy(WmQccInfo::getRegCode));
    }
}
