package com.qunzhixinxi.hnqz.admin.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.WmSupervisionDto;
import com.qunzhixinxi.hnqz.admin.api.entity.WmRoleSupervisionWarnRule;
import com.qunzhixinxi.hnqz.admin.mapper.WmRoleSupervisionWarnRuleMapper;
import com.qunzhixinxi.hnqz.admin.service.WmRoleSupervisionWarnRuleService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * @author jimmy
 * @description 针对表【wm_role_sup_warn_rule(角色预警规则)】的数据库操作Service实现
 * @createDate 2024-07-13 12:20:00
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmRoleSupervisionWarnRuleServiceImpl extends ServiceImpl<WmRoleSupervisionWarnRuleMapper, WmRoleSupervisionWarnRule>
        implements WmRoleSupervisionWarnRuleService {

    /**
     * 创建预警规则
     *
     * @param resource 规则信息
     * @param user     创建人
     * @return 创建结果
     */
    @Override
    public Boolean createWarnRule(WmSupervisionDto.OnWarnRulCreate resource, HnqzUser user) {

        WmRoleSupervisionWarnRule rule = new WmRoleSupervisionWarnRule();
        rule.setRoleId(resource.getRoleId());
        rule.setWarnType(resource.getWarnType());
        rule.setWarnLogic(resource.getWarnLogic());
        rule.setValue(resource.getValue());
        rule.setTemplate(resource.getTemplate());
        rule.setCreateBy(user.getUsername());
        rule.setUpdateBy(user.getUsername());
        rule.setCreateTime(LocalDateTime.now());
        rule.setUpdateTime(LocalDateTime.now());

        return this.save(rule);
    }

    /**
     * 校验类型是否存在
     *
     * @param warnType 类型
     * @return 是否存在
     */
    @Override
    public boolean hasType(String warnType, Integer roleId) {
        return this.count(Wrappers.<WmRoleSupervisionWarnRule>lambdaQuery()
                .eq(WmRoleSupervisionWarnRule::getRoleId, roleId)
                .eq(WmRoleSupervisionWarnRule::getWarnType, warnType)) > 0;
    }

    /**
     * 修改预警规则
     *
     * @param resource 规则信息
     * @param user     创建人
     * @return 修改结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateWarnRule(WmSupervisionDto.OnWarnRulUpdate resource, HnqzUser user) {

        WmRoleSupervisionWarnRule rule = new WmRoleSupervisionWarnRule();
        rule.setRuleId(resource.getRuleId());
        rule.setWarnLogic(resource.getWarnLogic());
        rule.setValue(resource.getValue());
        rule.setTemplate(resource.getTemplate());
        rule.setUpdateBy(user.getUsername());
        rule.setUpdateTime(LocalDateTime.now());
        return this.updateById(rule);
    }
}




