package com.qunzhixinxi.hnqz.admin.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.WmSupervisionDto;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskSupervisionAdvice;
import com.qunzhixinxi.hnqz.admin.mapper.WmTaskSupervisionAdviceMapper;
import com.qunzhixinxi.hnqz.admin.service.WmTaskSupervisionAdviceService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * 任务监察建议服务实现
 *
 * @author jimmy
 * @date 2024-11-14 17:24
 */
@Service
@AllArgsConstructor
public class WmTaskSupervisionAdviceServiceImpl extends ServiceImpl<WmTaskSupervisionAdviceMapper, WmTaskSupervisionAdvice>
        implements WmTaskSupervisionAdviceService {

    /**
     * 创建监察意见
     *
     * @param resource           意见信息
     * @param supervisionId      监督ID
     * @param supervisionContent 监督内容
     * @param operator           操作人
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createAdvice(Long supervisionId, WmSupervisionDto.OnTaskSupervision resource, String supervisionContent, HnqzUser operator) {

        WmTaskSupervisionAdvice advice = new WmTaskSupervisionAdvice();
        advice.setTaskSupervisionId(supervisionId);
        advice.setAdviceResult(resource.getSupervision());
        advice.setRemarks(resource.getRemarks());
        advice.setCreateBy(operator.getUsername());
        advice.setTaskSupervisionContent(supervisionContent);
        advice.setCreateTime(LocalDateTime.now());
        advice.setUpdateBy(operator.getUsername());
        advice.setUpdateTime(LocalDateTime.now());
        advice.setDelFlag(false);

        this.save(advice);


    }

    /**
     * 删除历史
     *
     * @param supervisionId 监察ID
     * @param operator      操作人
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delAdvice(Long supervisionId, HnqzUser operator) {
        this.update(Wrappers.<WmTaskSupervisionAdvice>lambdaUpdate()
                .set(WmTaskSupervisionAdvice::getUpdateBy, operator.getUsername())
                .set(WmTaskSupervisionAdvice::getUpdateTime, LocalDateTime.now())
                .set(WmTaskSupervisionAdvice::getDelFlag, true)
                .eq(WmTaskSupervisionAdvice::getTaskSupervisionId, supervisionId));
    }
}
