package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import com.qunzhixinxi.hnqz.admin.api.dto.WmSupervisionDto;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskSupervision;
import com.qunzhixinxi.hnqz.admin.mapper.WmTaskSupervisionMapper;
import com.qunzhixinxi.hnqz.admin.service.WmTaskSupervisionService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * 任务监督服务实现
 *
 * @author jimmy
 * @date 11/07/2024 16:51
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmTaskSupervisionServiceImpl extends ServiceImpl<WmTaskSupervisionMapper, WmTaskSupervision> implements WmTaskSupervisionService {

    /**
     * 创建审核
     *
     * @param resource 监督信息
     * @param user     操作人
     * @return 操作结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean createTaskSupervision(WmSupervisionDto.OnTaskSupervision resource, HnqzUser user) {

        WmTaskSupervision taskSupervision = new WmTaskSupervision();
        taskSupervision.setPackageId(resource.getPkgId());
        taskSupervision.setTaskId(resource.getTaskId());
        taskSupervision.setSupervisionResult(resource.getSupervision());
        taskSupervision.setRemarks(resource.getRemarks());
        taskSupervision.setModifiable(false);
        taskSupervision.setCreateBy(user.getUsername());
        taskSupervision.setCreateTime(LocalDateTime.now());
        taskSupervision.setUpdateBy(user.getUsername());
        taskSupervision.setUpdateTime(LocalDateTime.now());
        taskSupervision.setRoleId(SecurityUtils.getRoles().get(0));
        taskSupervision.setSupervisionState(UpmsState.TaskSupervisionState.INIT);

        return this.save(taskSupervision);
    }

    /**
     * 获取任务的监察记录
     *
     * @param taskId 任务ID
     * @return 监察记录
     */
    @Override
    public WmTaskSupervision getSupervisionByTaskId(Integer taskId) {
        return this.getOne(Wrappers.<WmTaskSupervision>lambdaQuery().eq(WmTaskSupervision::getTaskId, taskId));
    }

    /**
     * 更新监察信息
     *
     * @param supervisionId 监察ID
     * @param modifiable    是否可以调整
     * @param remarks       备注
     * @param operator      操作人
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateSupervision(Long supervisionId, String remarks, Boolean modifiable, HnqzUser operator) {

        UpmsState.TaskSupervisionState state = UpmsState.TaskSupervisionState.INIT;
        if (!modifiable) {
            List<Integer> roles = SecurityUtils.getRoles();

            // 部门领导
            if (CollUtil.containsAny(roles, List.of(57, 58, 59, 60))) {
                state = UpmsState.TaskSupervisionState.CKT1;
            }
            // 分管领导
            else if (CollUtil.containsAny(roles, List.of(51, 54, 55, 61))) {
                state = UpmsState.TaskSupervisionState.CKT2;
            }
            // 事业部总经理
            else if (CollUtil.containsAny(roles, List.of(39))) {
                state = UpmsState.TaskSupervisionState.PASSED;
            }

        }

        WmTaskSupervision taskSupervision = new WmTaskSupervision();
        taskSupervision.setSupervisionId(supervisionId);
        taskSupervision.setRemarks(remarks);
        taskSupervision.setModifiable(modifiable);
        taskSupervision.setSupervisionState(state);
        taskSupervision.setUpdateBy(operator.getUsername());
        taskSupervision.setUpdateTime(LocalDateTime.now());


        this.updateById(taskSupervision);

    }


}
