package com.qunzhixinxi.hnqz.admin.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.WmSupervisionDto;
import com.qunzhixinxi.hnqz.admin.api.entity.WmWarnUserRelation;
import com.qunzhixinxi.hnqz.admin.mapper.WmWarnUserRelationMapper;
import com.qunzhixinxi.hnqz.admin.service.WmWarnUserRelationService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * 警告阅读关系
 *
 * @author jimmy
 * @date 13/07/2024 15:32
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmWarnUserRelationServiceImpl extends ServiceImpl<WmWarnUserRelationMapper, WmWarnUserRelation> implements WmWarnUserRelationService {
    /**
     * 阅读警告信息
     *
     * @param resource 警告信息
     * @param user     阅读人
     * @return 阅读结果
     */
    @Override
    public Boolean read(WmSupervisionDto.OnWarnCreate resource, HnqzUser user) {

        WmWarnUserRelation relation = new WmWarnUserRelation();
        relation.setWarnId(resource.getWarnId());
        relation.setUserId(user.getId());
        relation.setCreateTime(LocalDateTime.now());

        return this.save(relation);
    }

    /**
     * 判断是否已经阅读了
     *
     * @param warnId 警告ID
     * @param id     阅读人ID
     * @return 阅读结果
     */
    @Override
    public boolean hasRead(Integer warnId, Integer id) {
        return this.count(Wrappers.<WmWarnUserRelation>lambdaQuery().eq(WmWarnUserRelation::getUserId, id).eq(WmWarnUserRelation::getWarnId, warnId)) > 0;
    }
}
