package com.qunzhixinxi.hnqz.admin.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import lombok.experimental.UtilityClass;
import lombok.extern.slf4j.Slf4j;

import java.util.Collections;
import java.util.List;

/**
 * 区域工具类
 *
 * @author jimmy
 * @date 29/07/2024 15:04
 */
@Slf4j
@UtilityClass
public final class AreaUtils {

    // 110101006008 区域编码长度
    private static final int CODD_LENGTH = 12;

    private static final int PARTITION_LENGTH = 6_000;


    /**
     * 区域分片
     *
     * @param source 数据源
     * @return 分片结果
     */
    public static List<List<Long>> partition(List<Long> source) {

        if (CollUtil.isEmpty(source)) {
            log.info("数据源为空");
            return Collections.emptyList();
        }

        int size = calcSize(source.size());
        log.info("数据源长度：{}, 分片个数：{}", source.size(), size);
        final List<List<Long>> result;
        if (size == 1) {
            result = Collections.singletonList(source);
        } else {
            result = ListUtil.split(source, size);
        }

        return result;
    }


    private static int calcSize(int sourceSize) {

        int eleListSize = sourceSize * CODD_LENGTH + (sourceSize - 1);

        return (eleListSize / PARTITION_LENGTH) + 1;

    }


}
