package com.qunzhixinxi.hnqz.admin.util;

import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.URLUtil;
import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import lombok.experimental.UtilityClass;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;

/**
 * 中生短信工具类
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/24 12:06
 */
@UtilityClass
public class CnbgSendSmsUtils {

    // 用户账号：长度最大6个字符，统一大写
    private static final String USER_ID = "JA1105";
    // 用户密码：定长小写32位字符
    private static final String PWD = "123963";
    // 发送地址
    private static final String URL = "http://211.100.34.185:8016/sms/v2/std/send_single";
    private static final String CONTENT_TYPE = "application/json;charset=UTF-8";
    private static final String TIMESTAMP_PATTERN = "MMddHHmmss";
    private static final String CONTENT_PATTERN = "您好，%s，本次登录验证码为:%s，有效期%s分钟";
    private static final String CONTENT_UPD_PATTERN = "您好，%s，忘记密码，验证码为:%s，有效期%s分钟";

    /**
     * 发送短信验证码
     *
     * @param realName   真实名称
     * @param mobile     手机号
     * @param randomCode 随机码
     * @param expires    过期时间（分钟）
     * @return 发送结果
     */
    public static String sendSms(String realName, String mobile, String randomCode, String expires) {

        String timestamp = getTimestamp();

        Map<String, String> bodyMap = new HashMap<>();
        bodyMap.put("userid", USER_ID);
        bodyMap.put("pwd", encodePwd(timestamp));
        bodyMap.put("mobile", mobile.trim());

        String plainContent = String.format(CONTENT_PATTERN, realName, randomCode, expires);
        bodyMap.put("content", encodeContent(plainContent));
        bodyMap.put("timestamp", timestamp);

        return HttpUtil.createPost(URL).contentType(CONTENT_TYPE).body(JSONUtil.toJsonStr(bodyMap)).execute().body();

    }

    /**
     * 发送短信验证码
     *
     * @param realName   真实名称
     * @param mobile     手机号
     * @param randomCode 随机码
     * @param expires    过期时间（分钟）
     * @return 发送结果
     */
    public static String sendSms1(String realName, String mobile, String randomCode, String expires) {

        String timestamp = getTimestamp();

        Map<String, String> bodyMap = new HashMap<>();
        bodyMap.put("userid", USER_ID);
        bodyMap.put("pwd", encodePwd(timestamp));
        bodyMap.put("mobile", mobile.trim());

        String plainContent = String.format(CONTENT_UPD_PATTERN, realName, randomCode, expires);
        bodyMap.put("content", encodeContent(plainContent));
        bodyMap.put("timestamp", timestamp);

        return HttpUtil.createPost(URL).contentType(CONTENT_TYPE).body(JSONUtil.toJsonStr(bodyMap)).execute().body();

    }


    private static String getTimestamp() {
        return DateTimeFormatter.ofPattern(TIMESTAMP_PATTERN).format(LocalDateTime.now());
    }

    /**
     * pwd的密码加密方式如下：将userid值大写、固定字符串00000000、明文pwd、timestamp依次拼接成字符串后，再进行MD5加密
     *
     * @param timestamp 时间戳
     * @return 加密密码
     */
    private static String encodePwd(String timestamp) {
        String plainText = String.format("%s%s%s%s", USER_ID, "00000000", PWD, timestamp);

        return DigestUtil.md5Hex(plainText);
    }

    /**
     * 短信内容：最大支持1000个字(含签名)，发送时请预留至少10个字的签名长度，一个字母或一个汉字都视为一个字。本字段支持加密传输，若不使用加密功能编码方式：
     * UrlEncode("验证码：6666，打死都不要告诉别人哦！","UTF-8")
     *
     * @param content 内容明文
     * @return 加密后的内容
     */
    private static String encodeContent(String content) {
        return URLUtil.encode(content, CharsetUtil.CHARSET_UTF_8);
    }


}
