package com.qunzhixinxi.hnqz.admin.util;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.experimental.UtilityClass;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * {@code OsEnvUtils} 系统环境工具类
 *
 * @author Hengchen.Sun
 * @version 1.0.0
 * @date 2021-04-16
 * @since ver.1.0.0
 */
@UtilityClass
public final class OsEnvUtils {

    @Getter
    @AllArgsConstructor
    public enum TargetFile {

        TEMP("temp"),
        FONT("font"),
        TEMPLATE("tmpl"),
        EXCEL("EXCEL_FILE_PATH"),
        CONTRACT("pkg_cont");

        private final String name;
    }

    /**
     * 获取系统名称
     *
     * @return 系统名称
     */
    public static String getOSName() {
        Properties props = System.getProperties(); // 获得系统属性集
        return props.getProperty("os.name").toLowerCase();
    }


    /**
     * 根据环境获取不用的路径
     *
     * @return 路径map
     */
    public static Map<String, String> getEachEnvPaths() {
        String osName = getOSName();
        Map<String, String> pathMap = new HashMap<>(5);
        if (osName.contains("linux") || osName.contains("unix")) {
            pathMap.put(TargetFile.EXCEL.getName(), "/serving/server/cso_prod/excel/");
            pathMap.put(TargetFile.FONT.getName(), "/serving/server/cso_prod/font/");
            pathMap.put(TargetFile.CONTRACT.getName(), "/serving/server/cso_prod/contract/");
            pathMap.put(TargetFile.TEMPLATE.getName(), "/serving/server/cso_prod/template/");
            pathMap.put(TargetFile.TEMP.getName(), "/serving/server/cso_prod/temp/");
        }
        if (osName.contains("mac")) {
            pathMap.put(TargetFile.EXCEL.getName(), "~/");
            pathMap.put(TargetFile.FONT.getName(), "~/");
            pathMap.put(TargetFile.CONTRACT.getName(), "~/");
            pathMap.put(TargetFile.TEMPLATE.getName(), "~/");
            pathMap.put(TargetFile.TEMP.getName(), "~/");
        }
        if (osName.contains("windows")) {
            pathMap.put(TargetFile.EXCEL.getName(), "D:\\");
            pathMap.put(TargetFile.FONT.getName(), "D:\\");
            pathMap.put(TargetFile.CONTRACT.getName(), "D:\\");
            pathMap.put(TargetFile.TEMPLATE.getName(), "D:\\");
            pathMap.put(TargetFile.TEMP.getName(), "D:\\");
        }
        return pathMap;
    }
}
