package com.qunzhixinxi.hnqz.admin.util;

import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.qunzhixinxi.hnqz.common.core.exception.BizException;
import lombok.experimental.UtilityClass;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.HashMap;
import java.util.Map;

/**
 * 企查查工具类
 *
 * @author jimmy
 * @date 07/07/2024 16:09
 */
@UtilityClass
public class QccUtils {

    private static final String BASE_PATH = "https://pro.qcc.com";
    private static final String API_KEY = "8476fff7305911eea3f10c42a106ce72";
    private static final String SECRET_KEY = "CZMXHE8RQ59LWHFEHVM93HWPKF7GD7PM";

    /**
     * 获取企业工商信息详情
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getECIInfo(String searchKey) {

        final String METHOD_NAME = "/api/yj/ECIInfoVerify/GetInfo";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 严重违法查询
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getSeriousIllegalList(String searchKey) {


        final String METHOD_NAME = "/api/yj/SeriousIllegalCheck/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 执行人
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getZhixingList(String searchKey) {

        final String METHOD_NAME = "/api/yj/ZhixingCheck/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 失信查询
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getShixinList(String searchKey) {

        final String METHOD_NAME = "/api/yj/ShixinCheck/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 破产重组
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getBankruptcyList(String searchKey) {


        final String METHOD_NAME = "/api/yj/BankruptcyCheck/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 实际控制人
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getActualControlList(String searchKey) {


        final String METHOD_NAME = "/api/yj/ActualControl/SuspectedActualControl";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 清算信息
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getLiquidationDetail(String searchKey) {

        final String METHOD_NAME = "/api/yj/LiquidationCheck/GetDetail";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 动产抵押
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getChattelMortgageList(String searchKey) {


        final String METHOD_NAME = "/api/yj/ChattelMortgageCheck/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 税务等级
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getTaxCreditList(String searchKey) {


        final String METHOD_NAME = "/api/yj/TaxCredit/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 税收违法
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getTaxIllegalList(String searchKey) {


        final String METHOD_NAME = "/api/yj/TaxIllegalCheck/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 经营异常
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getExceptionList(String searchKey) {


        final String METHOD_NAME = "/api/yj/ExceptionCheck/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 企业评分
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getCreditRatingReportGrade(String searchKey) {

        final String METHOD_NAME = "/api/yj/CreditRatingReportGrade/GetInfo";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 行政处罚
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getAdminPenaltyList(String searchKey) {


        final String METHOD_NAME = "/api/yj/AdminPenaltyCheck/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 限制高消费
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getSumptuaryList(String searchKey) {

        final String METHOD_NAME = "/api/yj/SumptuaryCheck/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 行政许可
     *
     * @param searchKey 关键字
     * @return 信息详情
     */
    public static JSONObject getAdminLicenseList(String searchKey) {


        final String METHOD_NAME = "/api/yj/AdminLicenseCheck/GetList";

        // 校验参数
        checkSearchKey(searchKey);

        // 执行
        String result = execute(METHOD_NAME, searchKey);

        // 解析结果
        return parseResult(result);

    }

    /**
     * 执行
     *
     * @param mehtodName 执行方法
     * @param searchKey  关键字
     * @return 执行结果
     */
    private static String execute(String mehtodName, String searchKey) {

        String url = buildUrl(mehtodName);

        // 封装头
        Map<String, String> headers = buildHeader();

        // 封装参数
        Map<String, Object> params = buildParams(searchKey);

        // 发起请求
        return exec(url, headers, params);
    }


    private static String buildUrl(String methodName) {
        return BASE_PATH + methodName;
    }

    /**
     * 构建头信息
     *
     * @return 头信息
     */
    private static Map<String, String> buildHeader() {

        // 时间戳
        String timespan = String.valueOf(LocalDateTime.now().toEpochSecond(ZoneOffset.of("+8")));

        // 获取token
        String token = getToken(timespan);

        // 头信息
        Map<String, String> headers = new HashMap<>(1);
        headers.put("Token", token);
        headers.put("Timespan", timespan);
        return headers;
    }

    /**
     * 请求
     *
     * @param basePath 路径
     * @param header   头信息
     * @param params   参数
     * @return 请求结果
     */
    private static String exec(String basePath, Map<String, String> header, Map<String, Object> params) {
        return HttpUtil.createGet(basePath).addHeaders(header).form(params).execute().body();
    }

    /**
     * 构建参数
     *
     * @param searchKey 关键字
     * @return 参数列表
     */
    private static Map<String, Object> buildParams(String searchKey) {
        Map<String, Object> params = new HashMap<>(2);
        params.put("searchKey", searchKey);
        params.put("key", API_KEY);
        return params;
    }


    /**
     * 校验关键字
     *
     * @param searchKey 关键字
     */
    private void checkSearchKey(String searchKey) {
        if (CharSequenceUtil.isBlank(searchKey)) {
            throw new BizException("搜索关键字必填");
        }
    }

    /**
     * 获取token
     *
     * @param timespan 时间戳
     * @return token
     */
    private String getToken(String timespan) {
        String plainText = String.format("%s%s%s", API_KEY, timespan, SECRET_KEY);

        String md5Hex1 = DigestUtil.md5Hex(plainText);

        return md5Hex1.toUpperCase();
    }


    private static JSONObject parseResult(String result) {
        boolean json = JSONUtil.isJson(result);

        if (!json) {
            throw new BizException("结果格式不是JSON不能解析");
        }

        return JSONUtil.parseObj(result);

    }

    public static boolean ok(JSONObject parsed) {
        return "200".equals(getCode(parsed));
    }

    private static String getCode(JSONObject parsed) {
        return parsed.getStr("Status");
    }

    public static JSONObject getResult(JSONObject parsed) {
        return parsed.getJSONObject("Result");
    }

    public static JSONArray getArray(JSONObject parsed) {
        return parsed.getJSONArray("Result");
    }

    public static String getMessage(JSONObject parsed) {
        return String.format("%s，【Status】:%s", getCode(parsed), parsed.getStr("Message"));

    }


    // public static void main(String[] args) {
    //     // JSONObject jsonObject = getECIInfo("91110000100010062X");
    //     // JSONObject jsonObject = getSeriousIllegalList("91110000100010062X");
    //     // JSONObject jsonObject = getShixinList("91110000100010062X");
    //     // JSONObject jsonObject = getActualControlList("91110000100010062X");
    //     // JSONObject jsonObject = getLiquidationDetail("91110000100010062X");
    //     // JSONObject jsonObject = getTaxCreditList("91110000100010062X");
    //     // JSONObject jsonObject = getExceptionList("91110000100010062X");
    //     // JSONObject jsonObject = getBankruptcyList("91110000100010062X");
    //     // JSONObject jsonObject = getTaxIllegalList("91110000100010062X");
    //     // JSONObject jsonObject = getZhixingList("91110000100010062X");
    //     // JSONObject jsonObject = getChattelMortgageList("91110000100010062X");
    //     // JSONObject jsonObject = getAdminPenaltyList("91110000100010062X");
    //     // JSONObject jsonObject = getSumptuaryList("91110000100010062X");
    //     // JSONObject jsonObject = getAdminLicenseList("91110000100010062X");
    //     JSONObject jsonObject = getCreditRatingReportGrade("91110000100010062X");
    //
    //     System.out.println(jsonObject);
    //
    // }


}
