package com.qunzhixinxi.hnqz.daemon.quartz.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 部门管理
 * </p>
 *
 * @author hnqz
 * @since 2018-01-22
 */
@Data
@TableName(value = "sys_dept", autoResultMap = true)
public class SysDept implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 部门ID
     */
    @TableId(type = IdType.INPUT)
    private Integer deptId;
    /**
     * 部门名称
     */
    @NotBlank(message = "部门名称不能为空")
    private String name;

    /**
     * 排序
     */
    @NotNull(message = "排序值不能为空")
    private Integer sort;

    /**
     * 服务起始时间
     */
    private LocalDate issueDate;

    /**
     * 服务截止时间
     */
    private LocalDate expiryDate;

    /**
     * 创建时间
     */
    private LocalDateTime createTime;

    /**
     * 修改时间
     */
    private LocalDateTime updateTime;
    /**
     *
     */
    private Integer tenantId;
    /**
     * 父级部门id
     */
    private Integer parentId;

    /**
     * 是否删除 1：已删除 0：正常
     */
    @TableLogic(value = CommonConstants.STATUS_NORMAL, delval = CommonConstants.STATUS_DEL)
    private String delFlag;

    /**
     * 是否启用名称
     */
    @TableField(exist = false)
    private String delFlagName;

    /**
     * 部门code
     */
    private String deptCode;

    /**
     * 级别
     */
    private Integer level;

    /**
     * 级别名称(企业类型)
     */
    @TableField(exist = false)
    private String levelName;

    /**
     * 药企名称
     */
    private String level1Name;
    /**
     * 药企ID
     */
    private Integer level1Id;
    /**
     * 一级代理商名称
     */
    private String level2Name;
    /**
     * 一级代理商ID
     */
    private Integer level2Id;
    /**
     * 二级代理商名称
     */
    private String level3Name;
    /**
     * 二级代理商ID
     */
    private Integer level3Id;

    /**
     * 税号
     */
    private String taxCode;


    /**
     * 部门专用权限限制（字符串逗号分隔，0.导出）
     */
    private String[] deptPermissions;

    @TableField(exist = false)
    private List<String> subList;

    /**
     * 财务复核方式
     */
    private UpmsType.FinancialReviewType financialReviewType;

    /**
     * csm负责人
     */
    @TableField(exist = false)
    private String csmNames;

    /**
     * csm负责人用户id
     */
    @TableField(exist = false)
    private Integer[] csmUserIds;

    /**
     * 上级企业名称
     */
    @TableField(exist = false)
    private String superDeptName;

    /**
     * 区域
     */
    @TableField(exist = false)
    private List<Long> areaCodes;

}
