
package com.qunzhixinxi.hnqz.daemon.quartz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.daemon.quartz.config.QuartzAppConfig;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.WmTask;
import com.qunzhixinxi.hnqz.daemon.quartz.mapper.WmScorePackageMapper;
import com.qunzhixinxi.hnqz.daemon.quartz.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.daemon.quartz.service.WmTaskService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 任务表
 *
 * @author pigx code generator
 * @date 2020-09-27 18:11:25
 */
@Service
@Slf4j
@AllArgsConstructor
public class WmTaskServiceImpl extends ServiceImpl<WmTaskMapper, WmTask> implements WmTaskService {

	private final QuartzAppConfig quartzAppConfig;

	private final WmScorePackageMapper wmScorePackageMapper;
	private final List<Integer> taskTypeIdList = new ArrayList<>();

	{
		// 医院信息收集
		taskTypeIdList.add(15);
		// 商业公司信息收集
		taskTypeIdList.add(16);
		// 药店信息收集
		taskTypeIdList.add(18);
		// 医院信息修改
		taskTypeIdList.add(30);
	}

	/**
	 * 更新任务截图
	 *
	 * @param param 参数
	 * @return boolean
	 */
	@Override
	public boolean buildTaskImg(String param) {
		LambdaQueryWrapper<WmTask> wmTaskLambdaQueryWrapper = Wrappers.lambdaQuery();
		wmTaskLambdaQueryWrapper.isNull(WmTask::getTaskInfoImg);
		wmTaskLambdaQueryWrapper.ne(WmTask::getTaskStatus, "4");
		// 优先查询审核通过的(taskStatus == 3)
		wmTaskLambdaQueryWrapper.orderByDesc(WmTask::getTaskStatus, WmTask::getId);

		Set<String> ids = new HashSet<>();

		if (StrUtil.isNotBlank(param) && Integer.parseInt(param) > 1) {
			// 手动指定积分包id的情况
			// 查询下级积分包
			ids.add(StrUtil.trim(param));
			getAllLowerScorePackages(ids);

			wmTaskLambdaQueryWrapper.in(WmTask::getScorePackageId, ids);
		}

		List<WmTask> wmTaskList = this.list(wmTaskLambdaQueryWrapper);

		log.info("积分包id:{}, 任务个数:{}", ids, wmTaskList.size());

		if (CollUtil.isNotEmpty(wmTaskList)) {
			for (WmTask wmTask : wmTaskList) {
				String url = quartzAppConfig.getApiUrl() + "/admin/api/getH5To?taskId=" + wmTask.getId();
				log.info("当前任务id：{}，类型是：{}，url={}", wmTask.getId(), wmTask.getTaskTypeId(), url);

				try {
					String s = HttpUtil.get(url, 10 * 1000);
					log.info("taskId={} 结果:{}", wmTask.getId(), s);
				} catch (Exception ex) {
					log.error("请求getH5To接口失败", ex);
				}
			}
		}

		return true;
	}

	/**
	 * 查询所有下级的积分包id
	 *
	 * @param ids 积分包id
	 */
	private void getAllLowerScorePackages(Set<String> ids) {
		List<WmScorePackage> subList = wmScorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
				.in(WmScorePackage::getRelationScoreId, ids)
				.eq(WmScorePackage::getDelFlag, "0"));

		if (CollectionUtil.isNotEmpty(subList)) {
			Set<String> idSet = subList.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
			getAllLowerScorePackages(idSet);
			ids.addAll(idSet);
		}
	}

	@Override
	public List<Map<String, Object>> singleMemberWithoutSuchTaskTypesInSeveralMonths(LocalDateTime startTime, LocalDateTime endTime, List<String> typeList) {
		// 获取时间段内的所有任务
		List<WmTask> list = this.list(Wrappers.<WmTask>lambdaQuery().between(WmTask::getCreateTime, startTime, endTime));

		Map<String, Map<String, List<WmTask>>> collect = list.stream().collect(Collectors.groupingBy(WmTask::getTaskUserId, Collectors.groupingBy(WmTask::getTaskTypeId)));

		Set<String> packageIds = new HashSet<>(collect.size());

		for (Map.Entry<String, Map<String, List<WmTask>>> entry : collect.entrySet()) {

			Set<String> typeIds = entry.getValue().keySet();

			Set<String> result = new HashSet<>(typeList);
			result.retainAll(typeIds);

			if (result.isEmpty()) {
				for (Map.Entry<String, List<WmTask>> e : entry.getValue().entrySet()){
					Set<String> temp = e.getValue().stream().map(WmTask::getScorePackageId).collect(Collectors.toSet());
					if (CollUtil.isNotEmpty(temp)){
						packageIds.addAll(temp);
					}
				}
			}
		}

		if (CollUtil.isEmpty(packageIds)) {
			return null;
		}

		List<Map<String, Object>> packageInfo = wmScorePackageMapper.listPackageInfo(packageIds);
		return packageInfo;


	}
}
