package com.qunzhixinxi.hnqz.daemon.quartz.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.WmTaskType;
import com.qunzhixinxi.hnqz.daemon.quartz.mapper.WmTaskTypeMapper;
import com.qunzhixinxi.hnqz.daemon.quartz.service.WmTaskTypeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * 任务类型打分表
 *
 * @author gaoyang
 * @date 2020-07-01 20:20:05
 */
@Slf4j
@Service
public class WmTaskTypeServiceImpl extends ServiceImpl<WmTaskTypeMapper, WmTaskType> implements WmTaskTypeService {


    /**
     * 根据平台积分规则，修改各企业积分规则
     *
     * @param para 0-覆盖所有企业积分值，1-不覆盖符合积分规则范围的企业的
     * @return
     */
    @Override
    public int changeTaskTypeInfo(String para) {
        // 这里能查询的前提是：在平台积分规则点击了停用或编辑的数据（点击停用/启用或编辑后会新增一条status=1的相同数据，这些数据实际应该是临时数据，后边会删除）
        WmTaskType wmTaskType = new WmTaskType();
        wmTaskType.setTaskTypeLevel("1");
        wmTaskType.setStatus("1");
        List<WmTaskType> wmTaskTypes = super.list(Wrappers.query(wmTaskType));

        // 遍历平台需要修改的积分规则
        for (WmTaskType wmT : wmTaskTypes) {
            // 根据积分规则名称，查询各个企业的积分规则
            WmTaskType wmTaskType1 = new WmTaskType();
            wmTaskType1.setTaskTypeName(wmT.getTaskTypeName());
            wmTaskType1.setTaskTypeLevel("2");
            List<WmTaskType> wmTaskTypeList = super.list(Wrappers.query(wmTaskType1));
            // 修改各企业的积分规则
            LocalDateTime now = LocalDateTime.now();
            for (WmTaskType wmTaskType2 : wmTaskTypeList) {
                WmTaskType updateTaskType = new WmTaskType();
                updateTaskType.setId(wmTaskType2.getId());
                if ("1".equals(para)) {
                    // 如果企业现有的积分规则符合新的积分规则范围，则不做覆盖修改处理，否则会修改为平台的预设值
                    if (wmTaskType2.getScore() > wmT.getTaskScoreEnd() || wmTaskType2.getScore() < wmT.getTaskScoreStart()) {
                        updateTaskType.setScore(wmT.getScore());
                    }
                } else if ("0".equals(para)) {
                    // para为0时，则直接覆盖企业积分值
                    updateTaskType.setScore(wmT.getScore());
                }
                updateTaskType.setTaskScoreStart(wmT.getTaskScoreStart());
                updateTaskType.setTaskScoreEnd(wmT.getTaskScoreEnd());
                updateTaskType.setEnableFlag(wmT.getEnableFlag());
                updateTaskType.setUpdateTime(now);
                updateTaskType.setUpdateUser(0);
                this.updateById(updateTaskType);
                log.info("企业任务类型已更新：{}", updateTaskType);
            }

            // 如果baseId有对应的父级数据，则更新（原有逻辑），即更新平台任务类型的规则
            WmTaskType wmTaskTypeBase = super.getById(wmT.getBaseId());
            if (wmTaskTypeBase != null) {
                wmTaskTypeBase.setScore(wmT.getScore());
                wmTaskTypeBase.setTaskScoreEnd(wmT.getTaskScoreEnd());
                wmTaskTypeBase.setTaskScoreStart(wmT.getTaskScoreStart());
                wmTaskTypeBase.setEnableFlag(wmT.getEnableFlag());
                wmTaskTypeBase.setUpdateTime(now);
                wmTaskTypeBase.setUpdateUser(0);
                this.updateById(wmTaskTypeBase);
                log.info("平台任务类型已更新：{}", wmTaskTypeBase);
            }
        }

        // 移除这些数据（status=1的数据）
        this.remove(Wrappers.query(wmTaskType));

        return 0;
    }
}
