package com.qunzhixinxi.hnqz.report.listener;

import cn.hutool.json.JSONUtil;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.entity.WmReport;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.ReportTypeEnum;
import com.qunzhixinxi.hnqz.report.event.NbQueueEvent;
import com.qunzhixinxi.hnqz.report.service.ReportExportService;
import com.qunzhixinxi.hnqz.report.service.ReportHandlerDeptRelationService;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.util.R;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * NB事件监听
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-06-29 11:35
 */
@Slf4j
@Component
@AllArgsConstructor
public class NbQueueEventListener {

    private final ReportHandlerDeptRelationService reportHandlerDeptRelationService;
    private final StringRedisTemplate redisTemplate;

    private static final long DEF_REPORT_TTL = 7L * 24 * 60 * 60 * 1000;


    @Order
    @Async("taskExecutor")
    @EventListener(NbQueueEvent.class)
    public void handleNbaQueueEvent(NbQueueEvent event) {

        log.info("++++++ 异步处理Nb Queue操作记录[START] ++++++");

        //业务处理
        String requestBody = event.getRequestBody();
        String username = event.getUsername();
        Integer deptId = event.getDeptId();
        WmReport wmReport = JSONUtil.toBean(requestBody, WmReport.class);
        String key = String.format(CacheConstants.EXPORT_NB_REPORT_CACHE, wmReport.getId());
        String resStr = "ERROR_未知错误，请联系管理员";

        try {
            // 根据企业获取对应报告处理类
            ReportExportService reportExportService =
                    reportHandlerDeptRelationService.getHandlerClass(ReportTypeEnum.NB.getType(), deptId);


            R r = reportExportService.newExportNb(wmReport, deptId, username);

            if (r.getCode() == CommonConstants.SUCCESS) {
                // url
                Map<String, String> data = (Map<String, String>) r.getData();
                log.info("result-map:{}", data);
                resStr = data.get("url");
            } else {
                // error msg
                resStr = r.getMsg();
            }

        } catch (Exception e) {
            log.error("异步处理Nb Queue操作记录失败", e);
            e.printStackTrace();
        } finally {
            Map<String, String> res = new HashMap<>(2);
            res.put("key", key);
            res.put("res", resStr);
            log.info("异步处理Nb Queue操作记录结果:{}", res);

            redisTemplate.opsForValue().set(key, resStr, DEF_REPORT_TTL, TimeUnit.MILLISECONDS);

        }

        log.info("++++++ 异步处理Nb Queue操作记录[END] ++++++");
    }
}
