package com.qunzhixinxi.hnqz.report.mq;

import cn.hutool.json.JSONUtil;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.constant.MqConstants;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.ReportTypeEnum;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.report.service.ExcelReportExportService;
import com.qunzhixinxi.hnqz.report.service.ReportHandlerDeptRelationService;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.entity.BaseMap;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.rabbitmq.annotation.RabbitMqComponent;
import com.qunzhixinxi.hnqz.common.rabbitmq.core.BaseRabbiMqHandler;
import com.qunzhixinxi.hnqz.common.rabbitmq.listenter.MqListener;
import com.rabbitmq.client.Channel;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.messaging.handler.annotation.Header;

import java.util.HashMap;
import java.util.Map;

/**
 * zb对列监听器
 */
@Slf4j
@AllArgsConstructor
@RabbitListener(queues = MqConstants.DIRECT_MODE_EXCEL_REPORT_EXPORT_NBA_HISTORY_QUEUE)
@RabbitMqComponent(value = "excelNbaReportHistoryExportReceiver")
public class ExcelReportExportNbaHistoryReceiver extends BaseRabbiMqHandler<BaseMap> {

//    private final ReportExportService reportExportService;
//    private final WmReportExportRecordService wmReportExportRecordService;

    private final ReportHandlerDeptRelationService reportHandlerDeptRelationService;

    @RabbitHandler
    public void onMessage(BaseMap baseMap, Channel channel, @Header(AmqpHeaders.DELIVERY_TAG) long deliveryTag) {
        super.onMessageAck(baseMap, deliveryTag, channel, (MqListener<BaseMap>) (map, channel1) -> {

            //业务处理
            log.info("直接队列：nba-history报告导出，手动ACK，接收消息TAG:{},消息内容：{}", deliveryTag, baseMap);
            String requestBody = map.get("requestBody").toString();
            String username = map.get("username").toString();
            Integer deptId = Integer.valueOf(map.get("deptId").toString());
//            Integer exportRecordId = Integer.valueOf(map.get("recordId").toString());
            WmScorePackage wmScorePackage = JSONUtil.toBean(requestBody, WmScorePackage.class);
            String key = String.format(CacheConstants.EXCEL_EXPORT_NBA_HISTORY_REPORT_CACHE, wmScorePackage.getId());
            String resStr = "ERROR_未知错误，请联系管理员";


//            int exportStatus = WmReportOpt.WmReportOptStatus.GENERATED.ordinal();
//            String name = "无";
//            String title = "无";



            try {
                // 根据企业获取对应报告处理类
                ExcelReportExportService reportExportService =
                        reportHandlerDeptRelationService.getExcelHandlerClass(ReportTypeEnum.EXCEL_NBA.getType(), deptId);

                R r = reportExportService.newExportNbaHistory(wmScorePackage, deptId, username);

                if (r.getCode() == CommonConstants.SUCCESS) {
                    // url
                    Map<String,String> data = (Map<String, String>) r.getData();
                    log.info("result-map:{}",data);
                    resStr = data.get("url") ;
//                    name = data.get("fileName");
//                    title = data.get("orgFileNmae");

                } else {
                    // error msg
                    resStr = r.getMsg();
//                    exportStatus = WmReportOpt.WmReportOptStatus.ERROR.ordinal();
                }

            } catch (Exception e) {
                log.error("excel_nba_history报告导出异常", e);
            } finally {
                Map<String, String> res = new HashMap<>(2);
                res.put("key", key);
                res.put("res", resStr);

//                WmReportExportRecord byId = wmReportExportRecordService.getById(exportRecordId);
//                byId.setExportStatus(exportStatus);
//                byId.setUrl(resStr);
//                byId.setTitle(title);
//                byId.setReportNum(name);
//                byId.setFinishTime(LocalDateTime.now());
//                wmReportExportRecordService.updateById(byId);

                return res;
            }

        });
    }

}