package com.qunzhixinxi.hnqz.report.mq;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.constant.MqConstants;
import com.qunzhixinxi.hnqz.report.service.NewReportService;
import com.qunzhixinxi.hnqz.common.core.entity.BaseMap;
import com.qunzhixinxi.hnqz.common.rabbitmq.annotation.RabbitMqComponent;
import com.qunzhixinxi.hnqz.common.rabbitmq.core.BaseRabbiMqHandler;
import com.qunzhixinxi.hnqz.common.rabbitmq.listenter.MqListener;
import com.rabbitmq.client.Channel;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.messaging.handler.annotation.Header;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 新版个人报告导出队列监听器
 *
 * @author lixuesong
 * @date 2023/12/13
 */
@Slf4j
@AllArgsConstructor
@RabbitListener(queues = MqConstants.DIRECT_MODE_NEW_PERSONAL_REPORT_EXPORT_QUEUE)
@RabbitMqComponent(value = "newPersonalReportExportReceiver")
public class NewPersonalReportExportReceiver extends BaseRabbiMqHandler<BaseMap> {

    @Resource(name = "newPersonalReportServiceImpl")
    private NewReportService newReportService;
    private final RedisTemplate redisTemplate;

    private static final long DEF_REPORT_TTL = 7L * 24 * 60 * 60 * 1000;

    @RabbitHandler
    public void onMessage(BaseMap baseMap, Channel channel, @Header(AmqpHeaders.DELIVERY_TAG) long deliveryTag) {
        super.onMessageAck1(baseMap, deliveryTag, channel, (MqListener<BaseMap>) (map, channel1) -> {
            //业务处理
            log.info("直接队列：新版个人报告导出，手动ACK，接收消息TAG:{},消息内容：{}", deliveryTag, baseMap);

            String key = null;
            String resStr = null;
            try {
                String requestBody = map.get("requestBody").toString();

                JSONObject bodyJson = JSONUtil.parseObj(requestBody);
                String scorePackageId = bodyJson.getStr("id");
                String taskStatus = bodyJson.getStr("taskStatus");
                Integer deptId = bodyJson.getInt("deptId");

                key = String.format(CacheConstants.NEW_PERSONAL_REPORT_CACHE, scorePackageId);

                Map<String, String> nbaExportResult = newReportService.export(StrUtil.split(scorePackageId, ","), taskStatus, deptId);

                if (CollUtil.isNotEmpty(nbaExportResult)) {
                    resStr = nbaExportResult.get("url");
                } else {
                    resStr = "ERROR_未知错误，请联系管理员";
                }

            } catch (Exception e) {
                log.error("新版个人报告导出异常", e);
                resStr = "ERROR_" + e.getMessage();
            }

            Map<String, String> res = new HashMap<>(1);
            res.put("res", resStr);
            res.put("key", key);

            redisTemplate.opsForValue().set(key, resStr, DEF_REPORT_TTL, TimeUnit.MILLISECONDS);

            return res;
        });
    }
}