package com.qunzhixinxi.hnqz.report.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.excel.write.metadata.fill.FillWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDictItem;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEnt;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskContent;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.api.entity.WmUserSign;
import com.qunzhixinxi.hnqz.admin.api.entity.WmUserSignDetail;
import com.qunzhixinxi.hnqz.report.mapper.SysDictItemMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskContentMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignDetailMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignMapper;
import com.qunzhixinxi.hnqz.report.service.NewReportService;
import com.qunzhixinxi.hnqz.report.service.SysDeptService;
import com.qunzhixinxi.hnqz.report.service.SysUserService;
import com.qunzhixinxi.hnqz.report.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.report.service.WmScorePackageStatusService;
import com.qunzhixinxi.hnqz.report.service.WmTaskService;
import com.qunzhixinxi.hnqz.report.service.WmTaskTypeService;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 新版报告导出excel版抽象类
 *
 * @author lixuesong
 * @date 2023/12/19
 */
@Slf4j
@NoArgsConstructor
@AllArgsConstructor
public abstract class NewAbstractExcelReportService implements NewReportService {

    private WmScorePackageStatusService wmScorePackageStatusService;

    private WmTaskService wmTaskService;

    private WmTaskContentMapper wmTaskContentMapper;

    private WmUserSignMapper wmUserSignMapper;

    private WmUserSignDetailMapper wmUserSignDetailMapper;

    private WmTaskTypeService wmTaskTypeService;

    private WmDaDrugEntService wmDaDrugEntService;

    private SysDeptService sysDeptService;

    private SysUserService sysUserService;

    private SysDictItemMapper sysDictItemMapper;


    /**
     * 创建excel表格数据
     *
     * @param scorePackageList 积分包列表
     * @param taskList 任务列表
     * @return 结果
     */
    public Map<String, Object> createExcelData(List<WmScorePackage> scorePackageList, List<WmTask> taskList) {
        Map<String, Object> params = new HashMap<>();
        Map<String, Object> textParams = new HashMap<>();

        WmScorePackage wmScorePackage = scorePackageList.get(0);

        // 积分包名称
        String scorePackageNames = scorePackageList.stream().map(WmScorePackage::getScorePackageName).collect(Collectors.joining(","));
        textParams.put("score_package_name", scorePackageNames);

        // 任务承接人
        Set<String> taskUserIdSet = taskList.stream().map(WmTask::getTaskUserId).collect(Collectors.toSet());
        List<SysUser> userList = sysUserService.listByIds(taskUserIdSet);
        textParams.put("report_users", userList.stream().map(SysUser::getRealname).collect(Collectors.joining(StrUtil.COMMA)));


        // 服务周期
        Map<String, String> serviceTimeRangeMap = wmScorePackageStatusService.getServiceTimeRange(scorePackageList);
        String minDateStr = serviceTimeRangeMap.get("service_start_date");
        String maxDateStr = serviceTimeRangeMap.get("service_end_date");
        if (StrUtil.isBlank(maxDateStr)) {
            // 暂定截至时间空,则取开始时间
            maxDateStr = minDateStr;
        }
        // 服务周期
        textParams.put("service_date", String.format("%s至%s", minDateStr, maxDateStr));

        // 查询字典score_task_type
        List<SysDictItem> scoreTaskTypeDictList = sysDictItemMapper.selectList(Wrappers.<SysDictItem>lambdaQuery()
                .eq(SysDictItem::getType, "score_task_type")
                .eq(SysDictItem::getDelFlag, "0"));
        Map<String, String> scoreTaskTypeDictMap = scoreTaskTypeDictList.stream()
                .collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));

        // 查询taskType
        List<WmTaskType> taskTypeList = wmTaskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
                .eq(WmTaskType::getTaskTypeLevel, "1")
                .eq(WmTaskType::getDelFlag, "0")
//                .eq(WmTaskType::getEnableFlag, "0")
        );
        // taskType通过parentId分组（key为parentId，value为taskTypeId的set）
        Map<String, WmTaskType> taskTypeIdMap = taskTypeList.stream()
                .collect(Collectors.toMap(WmTaskType::getId, taskType -> taskType));
        // taskList添加parentId
        taskList.forEach(wmTask -> wmTask.setTaskTypeParentId(taskTypeIdMap.get(wmTask.getTaskTypeId()).getParentId()));

        // task通过taskTypeParentId分组
        Map<String, List<WmTask>> taskParantMap = taskList.stream()
                .collect(Collectors.groupingBy(WmTask::getTaskTypeParentId, Collectors.toList()));

        // 封装brief table1数据（统计每个大类任务的积分、任务数）
        List<Map<String, Object>> briefTable1List = new ArrayList<>();
        taskParantMap.forEach((parentId, wmTasks) -> {
            Map<String, Object> briefTable1Params = new HashMap<>();
            briefTable1Params.put("task_type_name", scoreTaskTypeDictMap.get(parentId));
            briefTable1Params.put("task_count", wmTasks.size());
            briefTable1Params.put("task_sum", wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum));
            briefTable1List.add(briefTable1Params);
        });
        Map<String, Object> briefTable1TotalParams = new HashMap<>();
        briefTable1TotalParams.put("task_type_name", "合计");
        briefTable1TotalParams.put("task_count", taskList.size());
        briefTable1TotalParams.put("task_sum", taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum));
        briefTable1List.add(briefTable1TotalParams);

        // task通过taskTypeId分组
        Map<String, List<WmTask>> taskTypeMap = taskList.stream()
                .collect(Collectors.groupingBy(WmTask::getTaskTypeId, Collectors.toList()));
        // 封装brief table1数据（统计每个小类任务的积分、任务数）
        List<Map<String, Object>> briefTable1bList = new ArrayList<>();
        taskTypeMap.forEach((taskTypeId, wmTasks) -> {
            Map<String, Object> briefTable1Params = new HashMap<>();
            briefTable1Params.put("task_type_name", taskTypeIdMap.get(taskTypeId).getTaskTypeName());
            briefTable1Params.put("task_count", wmTasks.size());
            briefTable1Params.put("task_sum", wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum));
            briefTable1bList.add(briefTable1Params);
        });
        Map<String, Object> briefTable11TotalParams = new HashMap<>();
        briefTable11TotalParams.put("task_type_name", "合计");
        briefTable11TotalParams.put("task_count", taskList.size());
        briefTable11TotalParams.put("task_sum", taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum));
        briefTable1bList.add(briefTable11TotalParams);

        // 封装brief table2数据（统计每个人任务积分、任务数）
        Map<String, List<WmTask>> taskUserMap = taskList.stream().collect(Collectors.groupingBy(WmTask::getTaskUserId));
        List<Map<String, Object>> briefTable2List = new ArrayList<>();
        taskUserMap.forEach((userId, wmTasks) -> {
            SysUser sysUser = sysUserService.getById(userId);
            Map<String, Object> briefTable2Map = new HashMap<>();
            briefTable2Map.put("user_name", sysUser.getRealname());
            briefTable2Map.put("user_task_count", wmTasks.size());
            briefTable2Map.put("user_task_sum", wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum));
            briefTable2List.add(briefTable2Map);
        });
        Map<String, Object> briefTable2TotalParams = new HashMap<>();
        briefTable2TotalParams.put("user_name", "合计");
        briefTable2TotalParams.put("user_task_count", taskList.size());
        briefTable2TotalParams.put("user_task_sum", taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum));
        briefTable2List.add(briefTable2TotalParams);

        // 查询所有涉及到的userId
        Set<String> userIdSet = taskList.stream().map(WmTask::getTaskUserId).collect(Collectors.toSet());
        List<SysUser> sysUsers = sysUserService.listByIds(userIdSet);
        // userMap
        Map<Integer, String> userMap = sysUsers.stream().collect(Collectors.toMap(SysUser::getUserId, SysUser::getRealname));

        // dept
        WmDaDrugEnt wmDaDrugEnt = wmDaDrugEntService.getById(wmScorePackage.getDrugEntId());
        SysDept sysDept = sysDeptService.getById(wmScorePackage.getDeptId());

        // 封装detailTable数据（任务详情）
        List<Map<String, Object>> detailTableList = new ArrayList<>();
        Map<String, Map<String, List<WmTask>>> collect = taskList.stream().collect(Collectors.groupingBy(WmTask::getTaskUserId, Collectors.groupingBy(WmTask::getTaskTypeParentId)));

        collect.forEach((k, v) -> {
            v.forEach((x, y) -> {
                Map<String, Object> detailTableMap = new HashMap<>();
                detailTableMap.put("user_name", userMap.get(Integer.valueOf(k)));
                detailTableMap.put("task_type", scoreTaskTypeDictMap.get(x));
                detailTableMap.put("score", y.stream().mapToInt(WmTask::getScore).sum());
                detailTableMap.put("finish_time", DateUtil.format(y.stream().max(Comparator.comparing(WmTask::getCreateTime)).get().getCreateTime(), DatePattern.NORM_DATE_PATTERN));
                detailTableMap.put("ent_name", (wmDaDrugEnt == null) ? ((sysDept == null) ? "无" : sysDept.getName()) : wmDaDrugEnt.getEntname());
                detailTableList.add(detailTableMap);
            });
        });

        // 封装detailTable2数据（任务详情）
        List<Map<String, Object>> detailTable2List = new ArrayList<>();
        for (int i = 0; i < taskList.size(); i++) {
            WmTask wmTask = taskList.get(i);
            Map<String, Object> detailTableMap = new HashMap<>();
            detailTableMap.put("user_name", userMap.get(Integer.valueOf(wmTask.getTaskUserId())));
            detailTableMap.put("task_type", scoreTaskTypeDictMap.get(wmTask.getTaskTypeId()));
            detailTableMap.put("score", wmTask.getScore());
            detailTableMap.put("create_time", DateUtil.format(wmTask.getCreateTime(), DatePattern.NORM_DATE_PATTERN));
            detailTableMap.put("index", i + 1);
            detailTable2List.add(detailTableMap);
        }

        // 专项汇总表数据
        // 医院/药店/商业公司打卡详情数据查询
        Set<String> taskConentIds = taskList.stream()
                .filter(task -> Arrays.asList("5", "6", "33", "73").contains(task.getTaskTypeId()))
                .map(WmTask::getTaskContentId).collect(Collectors.toSet());
        if (CollUtil.isNotEmpty(taskConentIds)) {
            List<WmTaskContent> taskContents = wmTaskContentMapper.selectBatchIds(taskConentIds);
            Set<Integer> userSignIds = taskContents.stream()
                    .map(taskContent -> Integer.parseInt(taskContent.getTemp3())).collect(Collectors.toSet());
            List<WmUserSignDetail> userSignDetails = wmUserSignDetailMapper.selectList(Wrappers.<WmUserSignDetail>lambdaQuery()
                    .in(WmUserSignDetail::getSignId, userSignIds));
            List<WmUserSign> userSigns = wmUserSignMapper.selectBatchIds(userSignIds);
            // signId和userSign的关系map
            Map<Integer, WmUserSign> userSignMap = userSigns.stream().collect(Collectors.toMap(WmUserSign::getId, Function.identity()));
            // signId和userSignDetail的关系map
            Map<Integer, WmUserSignDetail> userSignDetailMap = userSignDetails.stream()
                    .collect(Collectors.toMap(WmUserSignDetail::getSignId, Function.identity()));
            // taskConentId和signId的关系map
            Map<Integer, String> taskContSignIdMap = taskContents.stream()
                    .collect(Collectors.toMap(WmTaskContent::getId, WmTaskContent::getTemp3));
            // 查询拜访目的/拜访结果的字典值
            List<SysDictItem> dictItems = sysDictItemMapper.selectList(Wrappers.<SysDictItem>lambdaQuery()
                    .in(SysDictItem::getType, "user_sign_detail_purpose", "user_sign_detail_pharmacy_purpose",
                            "user_sign_detail_distribution_purpose", "user_sign_detail_result")
                    .eq(SysDictItem::getDelFlag, "0"));
            Map<String, List<SysDictItem>> dictItemMap = dictItems.stream()
                    .collect(Collectors.groupingBy(SysDictItem::getType));
            Map<String, String> hospitalPurposeMap = dictItemMap.get("user_sign_detail_purpose").stream()
                    .collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));
            Map<String, String> pharmacyPurposeMap = dictItemMap.get("user_sign_detail_pharmacy_purpose").stream()
                    .collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));
            Map<String, String> distributionPurposeMap = dictItemMap.get("user_sign_detail_distribution_purpose").stream()
                    .collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));
            Map<String, String> signResultMap = dictItemMap.get("user_sign_detail_result").stream()
                    .collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));

            // 专项汇总表-医院拜访汇总表
            List<Map<String, Object>> hospitalList = new ArrayList<>();
            AtomicInteger tb6Index = new AtomicInteger(1);
            taskList.stream().filter(task -> StrUtil.equals(task.getTaskTypeId(), "5")).forEach(task -> {
                // 任务编号
                String taskNumber = task.getTaskNumber();
                // 任务类型
                String taskTypeName = scoreTaskTypeDictMap.get(task.getTaskTypeId());
                // 任务积分
                String score = String.valueOf(task.getScore());
                // 完成时间
                String finishTime = DateUtil.format(task.getCreateTime(), DatePattern.NORM_DATE_PATTERN);

                String signId = taskContSignIdMap.get(Integer.parseInt(task.getTaskContentId()));
                WmUserSignDetail userSignDetail = userSignDetailMap.get(Integer.parseInt(signId));
                WmUserSign userSign = userSignMap.get(Integer.parseInt(signId));
                // 拜访地址
                String address = userSign.getSignEntName();
                String obj = "";
                String purpose = "";
                String drug = "";
                String result = "";
                if (userSignDetail != null) {
                    // 拜访对象(拜访科室)
                    obj = ArrayUtil.join(userSignDetail.getDepartment(), ",");
                    // 拜访目的
                    purpose = Arrays.stream(userSignDetail.getPurpose())
                            .map(hospitalPurposeMap::get).collect(Collectors.joining());
                    // 推广产品
                    drug = ArrayUtil.join(userSignDetail.getDrug(), ",");
                    // 结果反馈
                    result = signResultMap.get(userSignDetail.getResult());
                }
                // 序号
                int andIncrement = tb6Index.getAndIncrement();

                Map<String, Object> one = new HashMap<>();
                one.put("index", andIncrement);
                one.put("taskNumber", taskNumber);
                one.put("taskTypeName", taskTypeName);
                one.put("score", score);
                one.put("finishTime", finishTime);
                one.put("address", address);
                one.put("obj", obj);
                one.put("purpose", purpose);
                one.put("drug", drug);
                one.put("result", result);
                hospitalList.add(one);
            });
            if (CollUtil.isNotEmpty(hospitalList)) {
                params.put("hospitalList", hospitalList);
            }


            // 专项汇总表-药店拜访汇总表
            List<Map<String, Object>> pharmacyList = new ArrayList<>();
            AtomicInteger tb7Index = new AtomicInteger(1);
            taskList.stream().filter(task -> Arrays.asList("33", "73").contains(task.getTaskTypeId())).forEach(task -> {
                // 任务编号
                String taskNumber = task.getTaskNumber();
                // 任务类型
                String taskTypeName = scoreTaskTypeDictMap.get(task.getTaskTypeId());
                // 任务积分
                String score = String.valueOf(task.getScore());
                // 完成时间
                String finishTime = DateUtil.format(task.getCreateTime(), DatePattern.NORM_DATE_PATTERN);

                String signId = taskContSignIdMap.get(Integer.parseInt(task.getTaskContentId()));
                WmUserSignDetail userSignDetail = userSignDetailMap.get(Integer.parseInt(signId));
                WmUserSign userSign = userSignMap.get(Integer.parseInt(signId));
                // 拜访地址
                String address = userSign.getSignEntName();

                String obj = "";
                String purpose = "";
                String drug = "";
                String result = "";
                if (userSignDetail != null) {
                    // 拜访对象(拜访人员)
                    obj = userSignDetail.getVisitor();
                    // 拜访目的
                    purpose = Arrays.stream(userSignDetail.getPurpose())
                            .map(hospitalPurposeMap::get).collect(Collectors.joining());
                    // 推广产品
                    drug = ArrayUtil.join(userSignDetail.getDrug(), ",");
                    // 结果反馈
                    result = signResultMap.get(userSignDetail.getResult());
                }
                // 序号
                int andIncrement = tb7Index.getAndIncrement();

                Map<String, Object> one = new HashMap<>();
                one.put("index", andIncrement);
                one.put("taskNumber", taskNumber);
                one.put("taskTypeName", taskTypeName);
                one.put("score", score);
                one.put("finishTime", finishTime);
                one.put("address", address);
                one.put("obj", obj);
                one.put("purpose", purpose);
                one.put("drug", drug);
                one.put("result", result);
                pharmacyList.add(one);
            });
            if (CollUtil.isNotEmpty(pharmacyList)) {
                params.put("pharmacyList", pharmacyList);
            }


            // 专项汇总表-商业公司拜访汇总表
            List<Map<String, Object>> distributionList = new ArrayList<>();
            AtomicInteger tb8Index = new AtomicInteger(1);
            taskList.stream().filter(task -> StrUtil.equals("6", task.getTaskTypeId())).forEach(task -> {
                // 任务编号
                String taskNumber = task.getTaskNumber();
                // 任务类型
                String taskTypeName = scoreTaskTypeDictMap.get(task.getTaskTypeId());
                // 任务积分
                String score = String.valueOf(task.getScore());
                // 完成时间
                String finishTime = DateUtil.format(task.getCreateTime(), DatePattern.NORM_DATE_PATTERN);

                String signId = taskContSignIdMap.get(Integer.parseInt(task.getTaskContentId()));
                WmUserSignDetail userSignDetail = userSignDetailMap.get(Integer.parseInt(signId));
                WmUserSign userSign = userSignMap.get(Integer.parseInt(signId));
                // 拜访地址
                String address = userSign.getSignEntName();

                String purpose = "";
                String drug = "";
                String result = "";
                if (userSignDetail != null) {
                    // 拜访目的
                    purpose = Arrays.stream(userSignDetail.getPurpose())
                            .map(distributionPurposeMap::get).collect(Collectors.joining());
                    // 推广产品
                    drug = ArrayUtil.join(userSignDetail.getDrug(), ",");
                    // 结果反馈
                    result = signResultMap.get(userSignDetail.getResult());
                }
                // 序号
                int andIncrement = tb8Index.getAndIncrement();

                Map<String, Object> one = new HashMap<>();
                one.put("index", andIncrement);
                one.put("taskNumber", taskNumber);
                one.put("taskTypeName", taskTypeName);
                one.put("score", score);
                one.put("finishTime", finishTime);
                one.put("address", address);
                one.put("purpose", purpose);
                one.put("drug", drug);
                one.put("result", result);
                distributionList.add(one);
            });
            if (CollUtil.isNotEmpty(distributionList)) {
                params.put("distributionList", distributionList);
            }
        }


        // 专项汇总表-学术会议汇总表
        List<Map<String, Object>> meetingList = new ArrayList<>();
        AtomicInteger tb9Index = new AtomicInteger(1);
        Set<String> meetingTaskContIds = taskList.stream().filter(task -> Arrays.asList("1", "2", "3", "4", "64").contains(task.getTaskTypeId()))
                .map(WmTask::getTaskContentId).collect(Collectors.toSet());
        if (CollUtil.isNotEmpty(meetingTaskContIds)) {
            List<WmTaskContent> meetingTaskContents = wmTaskContentMapper.selectBatchIds(meetingTaskContIds);
            Map<Integer, WmTaskContent> meetingContentMap = meetingTaskContents.stream()
                    .collect(Collectors.toMap(WmTaskContent::getId, Function.identity()));
            taskList.stream().filter(task -> Arrays.asList("1", "2", "3", "4", "64").contains(task.getTaskTypeId())).forEach(task -> {
                // 任务编号
                String taskNumber = task.getTaskNumber();
                // 任务类型
                String taskTypeName = scoreTaskTypeDictMap.get(task.getTaskTypeId());
                // 任务积分
                String score = String.valueOf(task.getScore());
                // 完成时间
                String finishTime = DateUtil.format(task.getCreateTime(), DatePattern.NORM_DATE_PATTERN);

                WmTaskContent taskContent = meetingContentMap.get(Integer.parseInt(task.getTaskContentId()));
                // 会议主题
                String meetingTitle = taskContent.getTemp2();
                // 会议时间
                String meetingTime = taskContent.getTemp4();
                // 推广产品
                String drug = taskContent.getTemp12();
                // 会议地点
                String address = taskContent.getTemp31();
                // 会议总结
                String summary = taskContent.getTemp9();
                // 序号
                int andIncrement = tb9Index.getAndIncrement();

                Map<String, Object> one = new HashMap<>();
                one.put("index", andIncrement);
                one.put("taskNumber", taskNumber);
                one.put("taskTypeName", taskTypeName);
                one.put("score", score);
                one.put("finishTime", finishTime);
                one.put("meetingTitle", meetingTitle);
                one.put("meetingTime", meetingTime);
                one.put("drug", drug);
                one.put("address", address);
                one.put("summary", summary);
                meetingList.add(one);
            });
            if (CollUtil.isNotEmpty(meetingList)) {
                params.put("meetingList", meetingList);
            }
        }

        params.put("textParams", textParams);
        params.put("briefTable1Params", briefTable1List);
        params.put("briefTable1bParams", briefTable1bList);
        params.put("briefTable2Params", briefTable2List);
        params.put("detailTableParams", detailTableList);
        params.put("detailTable2Params", detailTable2List);

        return params;
    }

    public void createExcel(final String templateFile, final String targetFile, final Map<String, Object> params) {
        // 这里 指定文件
        try (ExcelWriter excelWriter = EasyExcel.write(targetFile).withTemplate(templateFile).build()) {

            // 填充列表开启自动换行,自动换行表示每次写入一条list数据是都会重新生成一行空行,此选项默认是关闭的,需要提前设置为true
            FillConfig fillConfig = FillConfig.builder().forceNewRow(true).build();

            // 每次都要创建writeSheet 这里注意必须指定sheetNo 而且sheetName必须不一样。
            // sheet0
            WriteSheet writeSheet = EasyExcel.writerSheet(0, "学术推广服务整体概况").build();
            // sheet0填充标题
            Map<String, Object> sheet0TitleMap = (Map<String, Object>) params.get("textParams");
            excelWriter.fill(sheet0TitleMap, writeSheet);
            // sheet0-table1填充数据
            List<Map<String, Object>> briefTable1Params = (List<Map<String, Object>>) params.get("briefTable1Params");
            excelWriter.fill(new FillWrapper("table1", briefTable1Params), fillConfig, writeSheet);
            // sheet0-table2填充数据
            List<Map<String, Object>> briefTable1bParams = (List<Map<String, Object>>) params.get("briefTable1bParams");
            excelWriter.fill(new FillWrapper("table2", briefTable1bParams), fillConfig, writeSheet);
            // sheet0-table3填充数据
            List<Map<String, Object>> briefTable2Params = (List<Map<String, Object>>) params.get("briefTable2Params");
            excelWriter.fill(new FillWrapper("table3", briefTable2Params), fillConfig, writeSheet);

            // sheet1
            WriteSheet writeSheet1 = EasyExcel.writerSheet(1, "学术推广服务明细").build();
            // sheet1填充标题
            excelWriter.fill(sheet0TitleMap, writeSheet1);
            // sheet1-table1填充数据
            List<Map<String, Object>> detailTableParams = (List<Map<String, Object>>) params.get("detailTableParams");
            excelWriter.fill(new FillWrapper("table1", detailTableParams), fillConfig, writeSheet1);

            // sheet2
            WriteSheet writeSheet2 = EasyExcel.writerSheet(2, "学术推广服务任务明细").build();
            // sheet2填充标题
            excelWriter.fill(sheet0TitleMap, writeSheet2);
            // sheet2-table1填充数据
            List<Map<String, Object>> detailTable2Params = (List<Map<String, Object>>) params.get("detailTable2Params");
            excelWriter.fill(new FillWrapper("table1", detailTable2Params), fillConfig, writeSheet2);

            // 医院拜访汇总表
            // sheet3
            WriteSheet writeSheet3 = EasyExcel.writerSheet(3, "医院拜访汇总表").build();
            // sheet3填充标题
            excelWriter.fill(sheet0TitleMap, writeSheet3);
            // sheet3-table1填充数据
            List<Map<String, Object>> hospitalList = (List<Map<String, Object>>) params.get("hospitalList");
            excelWriter.fill(new FillWrapper("table1", hospitalList), fillConfig, writeSheet3);

            // 药店拜访汇总表
            // sheet4
            WriteSheet writeSheet4 = EasyExcel.writerSheet(4, "药店拜访汇总表").build();
            // sheet4填充标题
            excelWriter.fill(sheet0TitleMap, writeSheet4);
            // sheet4-table1填充数据
            List<Map<String, Object>> pharmacyList = (List<Map<String, Object>>) params.get("pharmacyList");
            excelWriter.fill(new FillWrapper("table1", pharmacyList), fillConfig, writeSheet4);

            // 商业公司拜访汇总表
            // sheet5
            WriteSheet writeSheet5 = EasyExcel.writerSheet(5, "商业公司拜访汇总表").build();
            // sheet5填充标题
            excelWriter.fill(sheet0TitleMap, writeSheet5);
            // sheet5-table1填充数据
            List<Map<String, Object>> distributionList = (List<Map<String, Object>>) params.get("distributionList");
            excelWriter.fill(new FillWrapper("table1", distributionList), fillConfig, writeSheet5);

            // 学术会议汇总表
            // sheet6
            WriteSheet writeSheet6 = EasyExcel.writerSheet(6, "学术会议汇总表").build();
            // sheet6填充标题
            excelWriter.fill(sheet0TitleMap, writeSheet6);
            // sheet6-table1填充数据
            List<Map<String, Object>> meetingList = (List<Map<String, Object>>) params.get("meetingList");
            excelWriter.fill(new FillWrapper("table1", meetingList), fillConfig, writeSheet6);
        }
    }
}
