package com.qunzhixinxi.hnqz.report.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.data.MiniTableRenderData;
import com.deepoove.poi.data.PictureRenderData;
import com.deepoove.poi.data.RowRenderData;
import com.deepoove.poi.data.TextRenderData;
import com.deepoove.poi.data.style.TableStyle;
import com.deepoove.poi.util.BytePictureUtils;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDictItem;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.report.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEntDrugtable;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskContent;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskContentConfig;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.api.entity.WmUserSign;
import com.qunzhixinxi.hnqz.admin.api.entity.WmUserSignDetail;
import com.qunzhixinxi.hnqz.report.mapper.SysDeptMapper;
import com.qunzhixinxi.hnqz.report.mapper.SysDictItemMapper;
import com.qunzhixinxi.hnqz.report.mapper.SysUserMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmDaDrugEntDrugtableMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskContentConfigMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskContentMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskTypeMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignDetailMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignMapper;
import com.qunzhixinxi.hnqz.report.service.NewReportService;
import com.qunzhixinxi.hnqz.report.service.WmScorePackageStatusService;
import com.qunzhixinxi.hnqz.report.util.Html2ImageByJsWrapper;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.STJc;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 新版报告导出word版抽象类
 *
 * @author lixuesong
 * @date 2023/12/08
 */
@Slf4j
@NoArgsConstructor
@AllArgsConstructor
public abstract class NewAbstractReportService implements NewReportService {

    private WmTaskTypeMapper taskTypeMapper;
    private WmTaskContentMapper taskContentMapper;
    private WmTaskContentConfigMapper taskContentConfigMapper;
    private WmUserSignDetailMapper userSignDetailMapper;
    private WmUserSignMapper userSignMapper;
    private SysDeptMapper deptMapper;
    private WmDaDrugEntDrugtableMapper drugEntDrugtableMapper;
    private WmScorePackageStatusService scorePackageStatusService;
    private SysDictItemMapper dictItemMapper;
    private SysUserMapper userMapper;
    private UpmsConfig upmsConfig;

    /**
     * 任务详情截图默认宽高
     */
    private static final int DEFAULT_BASE_IMG_WITH = 550;
    private static final int DEFAULT_BASE_IMG_HEIGHT = 300;
    private static final int DEFAULT_BASE_IMG_HEIGHT2 = 750;

    /**
     * 填充模板
     *
     * @param templateFilePath 模板路径
     * @param targetFilePath   目标路径
     * @param data 数据
     */
    public File createWord(String templateFilePath, String targetFilePath, Map<String, Object> data) {
        //解析word模板
        XWPFTemplate template = XWPFTemplate.compile(templateFilePath).render(data);
        //输出文件
        FileOutputStream out = null;
        File outFile = new File(targetFilePath);
        try {
            out = new FileOutputStream(outFile);
            template.write(out);
            out.flush();
        } catch (Exception e) {
            log.error("生成word写入文件失败", e);
        } finally {
            if (template != null) {
                try {
                    template.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return outFile;
    }

    /**
     * 创建word模板所需数据
     *
     * @param scorePackageList         积分包
     * @param superApprovedTaskList    所有上级审核通过的任务列表
     * @param personalApprovedTaskList 所有提交给上级的任务列表
     * @param allTaskList              所有关联的任务列表
     * @return {@link Map}<{@link String}, {@link Object}>
     */
    public Map<String, Object> createData(List<WmScorePackage> scorePackageList, List<WmTask> superApprovedTaskList,
                                          List<WmTask> personalApprovedTaskList, List<WmTask> allTaskList) {
        // 上级积分包企业
        SysDept superDept = deptMapper.selectById(scorePackageList.get(0).getSendPackageDeptId());
        // 当前发包企业
        SysDept currentDept = deptMapper.selectById(scorePackageList.get(0).getDeptId());
        // 查询taskType
        List<WmTaskType> taskTypeList = taskTypeMapper.selectList(Wrappers.<WmTaskType>lambdaQuery()
                .eq(WmTaskType::getTaskTypeLevel, "1")
                .eq(WmTaskType::getDelFlag, "0")
                .eq(WmTaskType::getEnableFlag, "0")
                .eq(WmTaskType::getStatus, "0"));
        // 查询接单用户名称
        List<SysUser> userList = userMapper.selectBatchIds(personalApprovedTaskList.stream()
                .map(WmTask::getTaskUserId).collect(Collectors.toSet()));

        Map<String, Object> data = new HashMap<>();

        // 服务周期
        Map<String, String> serviceTimeRangeMap = scorePackageStatusService.getServiceTimeRange(scorePackageList);
        String minDateStr = serviceTimeRangeMap.get("service_start_date");
        String maxDateStr = serviceTimeRangeMap.get("service_end_date");
        if (StrUtil.isBlank(maxDateStr)) {
            // 暂定截至时间空,则取开始时间
            maxDateStr = minDateStr;
        }
        // 服务周期
        data.put("service_date", String.format("%s至%s", minDateStr, maxDateStr));

        String minYear = StrUtil.sub(minDateStr, 0, 4);
        String minMonth = StrUtil.sub(minDateStr, 5, 7);
        String maxYear = StrUtil.sub(maxDateStr, 0, 4);
        String maxMonth = StrUtil.sub(maxDateStr, 5, 7);
        // 委托项目-年月
        if (StrUtil.equals(minYear, maxYear)) {
            // 年月相同,则比较月份
            if (StrUtil.equals(minMonth, maxMonth)) {
                data.put("entrusting_project_time", String.format("%s年%s月", minYear, minMonth));
            } else {
                data.put("entrusting_project_time", String.format("%s年%s月至%s月", minYear, minMonth, maxMonth));
            }
        } else {
            // 年月都不相同
            data.put("entrusting_project_time", String.format("%s年%s月至%s年%s月", minYear, minMonth, maxYear, maxMonth));
        }

        // 委托公司
        data.put("entrusting_company", superDept.getName());
        // 服务公司
        data.put("service_company", currentDept.getName());
        // 当前积分包关联的药品
        Set<String> drugTableIds = scorePackageList.stream()
                .filter(scorePackage -> ArrayUtil.isNotEmpty(scorePackage.getDrugtable()))
                .flatMap(scorePackage -> Stream.of(scorePackage.getDrugtable()))
                .collect(Collectors.toSet());
        if (CollUtil.isNotEmpty(drugTableIds)) {
            List<WmDaDrugEntDrugtable> drugEntDrugtables = drugEntDrugtableMapper.selectBatchIds(drugTableIds);
            // 服务品种
            data.put("drugs", drugEntDrugtables.stream().map(WmDaDrugEntDrugtable::getDrugname).collect(Collectors.joining("，")));
            // 产品介绍
            data.put("drug_detail", drugEntDrugtables.stream()
                    .filter(drugTable -> StrUtil.isNotBlank(drugTable.getDetail()))
                    .map(WmDaDrugEntDrugtable::getDetail).collect(Collectors.joining("\n\n")));
        }

        // 任务完成日期
        Optional<LocalDateTime> taskMaxTime = superApprovedTaskList.stream().map(WmTask::getLookintoDate).max(LocalDateTime::compareTo);
        data.put("task_finish_time", DateUtil.format(taskMaxTime.get(), DatePattern.CHINESE_DATE_PATTERN));
        // 积分包值
        int totalPackageScore = scorePackageList.stream().mapToInt(WmScorePackage::getScore).sum();
        data.put("total_package_score", totalPackageScore);
        // 审核通过总值
        int totalApprovedScore = superApprovedTaskList.stream().mapToInt(WmTask::getScore).sum();
        data.put("total_approved_score", String.valueOf(totalApprovedScore));
        // 积分包名称
        String scorePackageNames = scorePackageList.stream().map(WmScorePackage::getScorePackageName).collect(Collectors.joining("，"));
        data.put("score_package_name", scorePackageNames);
        // 服务企业
        data.put("drug_ent_name", superDept.getName());

        // 发包日期
        String createTimes = scorePackageList.stream()
                .map(scorePackage -> DateUtil.format(scorePackage.getCreateTime(), DatePattern.NORM_DATE_PATTERN)).collect(Collectors.joining("，"));
        data.put("create_time", createTimes);
        // 最后一个审核通过的时间
        data.put("service_end_date", maxDateStr);
        // 所有任务承接人个数
        Set<String> allTaskUserIds = personalApprovedTaskList.stream().map(WmTask::getTaskUserId).collect(Collectors.toSet());
        data.put("task_user_names", allTaskUserIds.size());
        // 所有任务承接人姓名
        data.put("task_users", userList.stream().map(SysUser::getRealname).collect(Collectors.joining("，")));

        // 所有提交的任务积分
        if (CollUtil.isNotEmpty(allTaskList)) {
            int allTaskScore = allTaskList.stream().mapToInt(WmTask::getScore).sum();
            data.put("all_task_score", allTaskScore);
        }
        // 总提交的积分值(所有下级审核过的任务积分)
        int submitedTaskScore = personalApprovedTaskList.stream().mapToInt(WmTask::getScore).sum();
        data.put("submited_task_score", submitedTaskScore);
        // 审核不通过的积分值
        data.put("rejected_task_score", submitedTaskScore - totalApprovedScore);
        // 结算成功的金额
        int settleAmount = scorePackageList.stream()
                .filter(scorePackage -> scorePackage.getSettleStatus() != null && 1 == scorePackage.getSettleStatus())
                .mapToInt(WmScorePackage::getScore).sum();
        data.put("settle_amount", settleAmount);
        //// 结算成功的税源地
        //Optional<WmScorePackage> settledPackage = scorePackageList.stream()
        //        .filter(scorePackage -> scorePackage.getSettleStatus() != null && 1 == scorePackage.getSettleStatus()).findFirst();
        //settledPackage.ifPresent(scorePackage -> data.put("subject_location_company", scorePackage.getLocation() != null ? scorePackage.getLocation() : ""));

        // 审核服务费
        String auditServiceFees = scorePackageList.stream()
                .map(scorePackage -> scorePackage.getAuditManagementFeeRates()
                        .multiply(BigDecimal.valueOf(submitedTaskScore)).divide(new BigDecimal("100.00"), RoundingMode.HALF_UP).toString())
                .collect(Collectors.joining("，"));
        data.put("audit_service_fee", auditServiceFees);

        // 页眉的报告ID
        data.put("report_id",
                String.format("%s%s", scorePackageList.get(0).getDeptId(), DateUtil.format(LocalDateTime.now(), DatePattern.PURE_DATETIME_PATTERN)));


        // 创建表格数据表1~表5
        this.createTableData(superApprovedTaskList, taskTypeList, userList, currentDept, data);

        // 创建带截图的任务详细数据
//        this.createTaskDetailData(approvedTaskList, taskTypeList, userList, data);

        return data;
    }


    /**
     * 创建带截图的任务详细数据
     *
     * @param taskList 任务列表
     * @param userList
     * @param data     数据map
     */
    private void createTaskDetailData(List<WmTask> taskList, List<WmTaskType> taskTypeList, List<SysUser> userList, Map<String, Object> data) {
        // 从字典项查询支持的任务类型
        List<SysDictItem> scoreTaskTypeDictList = dictItemMapper.selectList(Wrappers.<SysDictItem>lambdaQuery()
                .eq(SysDictItem::getType, "report_export_task_type")
                .eq(SysDictItem::getDelFlag, "0"));
        List<String> supportedTaskTypeIds = scoreTaskTypeDictList.stream().map(SysDictItem::getValue).collect(Collectors.toList());

        List<WmTask> emptyImgTasks = taskList.stream()
                .filter(t -> StrUtil.isBlank(t.getTaskInfoImg()) && supportedTaskTypeIds.contains(t.getTaskTypeId()))
                .collect(Collectors.toList());

        log.warn("总任务数:{},空图片任务数:{}", taskList.size(), emptyImgTasks.size());

        List<Map<String, Object>> infoList = new ArrayList<>();
        // 任务详情转换为截图
        Map<String, BufferedImage> imgs = new HashMap<>(emptyImgTasks.size());
        for (WmTask task : emptyImgTasks) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            try {
                String taskUrl = upmsConfig.getApiUrl() + "/h5/#/pages/task/task-detail/index?id=" + task.getId();
                log.warn("获取任务图片: [{}]:[{}]", task.getId(), taskUrl);
                BufferedImage image = Html2ImageByJsWrapper.renderHtml2Image(taskUrl, upmsConfig.getServerUrl());
                if (Objects.isNull(image)) {
                    log.error("获取图片信息失败");
                    continue;
                }
                imgs.put(task.getId(), image);
            } catch (Exception e) {
                log.error("截图失败taskId={}", task.getId());
                log.error("截图失败", e);
            }
        }

        // taskType通过id分组
        Map<String, WmTaskType> taskTypeIdMap = taskTypeList.stream()
                .collect(Collectors.toMap(WmTaskType::getId, Function.identity()));
        // user通过userId分组
        Map<Integer, String> userMap = userList.stream().collect(Collectors.toMap(SysUser::getUserId, SysUser::getRealname));

        // 任务详情
        for (WmTask task : taskList) {
            Map<String, Object> map = new HashMap<>();
            WmTaskType taskType = taskTypeIdMap.get(task.getTaskTypeId());
            map.put("taskIndex", task.getTmpId());
            map.put("taskType", taskType.getTaskTypeName());
            map.put("taskTime", DateUtil.format(task.getCreateTime(), DatePattern.NORM_DATE_PATTERN));
            map.put("taskUser", userMap.get(Integer.parseInt(task.getTaskUserId())));
            map.put("taskTypeId", task.getTaskTypeId());
            map.put("taskNumber", task.getTaskNumber());

            // 任务详情的截图转换为word模板的图片类型
            // 通过配置tasktype的updateUser/createUser来控制截图大小(原有逻辑)
            int imgWidth = taskType.getUpdateUser() == null || taskType.getUpdateUser() <= 0 ? DEFAULT_BASE_IMG_WITH : taskType.getUpdateUser();
            int imgHeight = taskType.getCreateUser() == null || taskType.getCreateUser() <= 0 ? DEFAULT_BASE_IMG_HEIGHT2 : taskType.getCreateUser();
            BufferedImage bufferedImage = null;

            if (StrUtil.isNotBlank(task.getTaskInfoImg())) {
                bufferedImage = BytePictureUtils.getUrlBufferedImage(upmsConfig.getBasePath() + task.getTaskInfoImg());
            } else {
                bufferedImage = imgs.get(task.getId());
            }
            if (bufferedImage.getWidth() > bufferedImage.getHeight()) {
                imgWidth = DEFAULT_BASE_IMG_WITH;
                imgHeight = DEFAULT_BASE_IMG_HEIGHT;
            }

            PictureRenderData pictureRenderData = new PictureRenderData(imgWidth, imgHeight, ".png", bufferedImage);
            map.put("taskBaseImg", pictureRenderData);

            if (supportedTaskTypeIds.contains(task.getTaskTypeId())) {
                WmTaskContent wmTaskContent = taskContentMapper.selectById(task.getTaskContentId());
                if (null != wmTaskContent) {
                    // 任务详情的大图获取
                    List<String> stringList = new ArrayList<>();
                    WmTaskContentConfig queryWmTaskContentConfig = new WmTaskContentConfig();
                    if ("39".equals(task.getTaskTypeId()) || "40".equals(task.getTaskTypeId()) || "41".equals(task.getTaskTypeId())) {
                        // 39 应收账款回款-A类、40 应收账款回款-B类、41 应收账款回款-C类 这里需要统一设置为39（由于三种类型公用一个页面所以WmTaskContentConfig只保存了39的配置字段）
                        queryWmTaskContentConfig.setTaskTypeId("39");
                    } else if ("42".equals(task.getTaskTypeId()) || "43".equals(task.getTaskTypeId()) || "44".equals(task.getTaskTypeId())) {
                        // 手术跟台-一级/手术跟台-二级/手术跟台-三级任务类型 这里需要统一设置为42（由于三种类型公用一个页面所以WmTaskContentConfig只保存了42的配置字段）
                        queryWmTaskContentConfig.setTaskTypeId("42");
                    } else if ("14".equals(task.getTaskTypeId())) {
                        // 库存信息收集下的 库存盘点-中药颗粒剂、库存信息收集-首次 需要回显对应的详情字段(对于这几个任务类型这里的task的taskType实际存的都是14，所以需要转换下)
                        queryWmTaskContentConfig.setTaskTypeId(wmTaskContent.getTemp30());
                    } else if ("47".equals(task.getTaskTypeId()) || "48".equals(task.getTaskTypeId())
                            || "49".equals(task.getTaskTypeId()) || "50".equals(task.getTaskTypeId())) {
                        // 库存调研、竞品调研、培训、使用情况调查
                        queryWmTaskContentConfig.setTaskTypeId("47");
                    } else if ("51".equals(task.getTaskTypeId()) || "52".equals(task.getTaskTypeId())
                            || "53".equals(task.getTaskTypeId())) {
                        // 信用管理A/B/C
                        queryWmTaskContentConfig.setTaskTypeId("51");
                    } else if ("56".equals(task.getTaskTypeId())) {
                        // 首营管理-诊所/药店
                        queryWmTaskContentConfig.setTaskTypeId(wmTaskContent.getTemp3());
                    } else {
                        queryWmTaskContentConfig.setTaskTypeId(task.getTaskTypeId());
                    }
                    QueryWrapper<WmTaskContentConfig> queryWrapper = Wrappers.query(queryWmTaskContentConfig);
                    // 图片和签名都要导出
                    queryWrapper.lambda().in(WmTaskContentConfig::getTaskFiledType, "img", "sign");
                    List<WmTaskContentConfig> wmTaskContentConfigList = taskContentConfigMapper.selectList(queryWrapper);
                    if (CollUtil.isNotEmpty(wmTaskContentConfigList)) {
                        for (WmTaskContentConfig wmt : wmTaskContentConfigList) {
                            String fileName = wmt.getTaskFiledKey();
                            String imgsStr = getFieldValueByFieldName(fileName, wmTaskContent);
                            // 图片的数据格式是这样的 /admin/sys-file/wmkj/237ea87e43cc463ba4aaa295e2a28e24.png;1,/admin/sys-file/wmkj/fd7c4cd56d6748dea18ea57c809e464c.jpg
                            List<String> imgList = StrUtil.splitTrim(imgsStr, ",").stream()
                                    .filter(StrUtil::isNotBlank)
                                    .map(s -> s.split(";")[0]).collect(Collectors.toList());
                            stringList.addAll(imgList);
                        }
                    }

                    if (stringList.size() > 0) {
                        // 任务详情的大图转换为word模板图片类型
                        List<Map<String, Object>> detailPicList = new ArrayList<>();
                        stringList.forEach(url -> {
                            Map<String, Object> stringObjectMap = new HashMap<>();
                            BufferedImage detailBfImg = BytePictureUtils.getUrlBufferedImage(upmsConfig.getBasePath() + url);
                            int detailImgWidth = DEFAULT_BASE_IMG_WITH;
                            int detailImgHeight = DEFAULT_BASE_IMG_HEIGHT2;
                            if (detailBfImg.getWidth() > detailBfImg.getHeight()) {
                                detailImgWidth = DEFAULT_BASE_IMG_WITH;
                                detailImgHeight = DEFAULT_BASE_IMG_HEIGHT;
                            }
                            PictureRenderData detailPicRenderData = new PictureRenderData(detailImgWidth, detailImgHeight, ".png", detailBfImg);
                            stringObjectMap.put("taskBaseImgInfo", detailPicRenderData);
                            detailPicList.add(stringObjectMap);
                        });
                        map.put("imgList", detailPicList);
                    }

                    // 企业任务类型-回显文件地址处理
                    List<WmTaskContentConfig> fileurlTaskContentConfigList =
                            taskContentConfigMapper.selectList(Wrappers.<WmTaskContentConfig>lambdaQuery()
                                    .eq(WmTaskContentConfig::getTaskTypeId, task.getTaskTypeId())
                                    .eq(WmTaskContentConfig::getTaskFiledType, "fileurl"));
                    if (CollUtil.isNotEmpty(fileurlTaskContentConfigList)) {
                        List<Map<String, String>> fileUrlList = new ArrayList<>();
                        for (WmTaskContentConfig wmt : fileurlTaskContentConfigList) {
                            String fileName = wmt.getTaskFiledKey();
                            String taskFiledValue = wmt.getTaskFiledValue();
                            String fileUrlStr = getFieldValueByFieldName(fileName, wmTaskContent);

                            HashMap<String, String> fileUrlMap = new HashMap<>();
                            if (JSONUtil.isJsonArray(fileUrlStr)) {
                                List<String> fileUrls = new ArrayList<>();
                                JSONArray jsonArray = JSONUtil.parseArray(fileUrlStr);
                                jsonArray.jsonIter().forEach(jsonObject -> {
                                    String url = jsonObject.getStr("url");
                                    if (!StrUtil.startWith(url, "http")) {
                                        url = upmsConfig.getClientUrl() + url;
                                    }
                                    fileUrls.add(url);
                                });
                                fileUrlMap.put("fileUrl",
                                        String.format("%s: %s", taskFiledValue, String.join(",", fileUrls)));
                            } else {
                                fileUrlMap.put("fileUrl", String.format("%s: %s", taskFiledValue, fileUrlStr));
                            }
                            fileUrlList.add(fileUrlMap);
                        }
                        map.put("fileUrlList", fileUrlList);
                    }
                }
            }

            infoList.add(map);
        }

        data.put("infoList", infoList);
    }

    /**
     * 创建表格数据表1~表5
     *
     * @param taskList     任务列表
     * @param taskTypeList 任务类型列表
     * @param currentDept  当前企业
     * @param data         数据map
     */
    private void createTableData(List<WmTask> taskList, List<WmTaskType> taskTypeList, List<SysUser> userList, SysDept currentDept, Map<String, Object> data) {
        List<SysDictItem> scoreTaskTypeDictList = dictItemMapper.selectList(Wrappers.<SysDictItem>lambdaQuery()
                .eq(SysDictItem::getType, "score_task_type")
                .eq(SysDictItem::getDelFlag, "0"));
        Map<String, String> scoreTaskTypeDictMap = scoreTaskTypeDictList.stream()
                .collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));

        // taskType通过id分组
        Map<String, WmTaskType> taskTypeIdMap = taskTypeList.stream()
                .collect(Collectors.toMap(WmTaskType::getId, Function.identity()));
        // taskList添加parentId
        taskList.forEach(wmTask -> wmTask.setTaskTypeParentId(taskTypeIdMap.get(wmTask.getTaskTypeId()).getParentId()));

        // user通过userId分组
        Map<Integer, String> userMap = userList.stream().collect(Collectors.toMap(SysUser::getUserId, SysUser::getRealname));

        // =================================表格1=================================
        // 学术推广整体概况-任务大类统计
        TableStyle tableStyle = new TableStyle();
        tableStyle.setAlign(STJc.Enum.forInt(2));
        // 表头
        RowRenderData tb1Header = RowRenderData.build(new TextRenderData("任务类型"),
                new TextRenderData("活动次数"),
                new TextRenderData("合计积分"));
        tb1Header.setRowStyle(tableStyle);

        List<RowRenderData> briefTb1List = new ArrayList<>();
        // task通过taskTypeParentId分组
        Map<String, List<WmTask>> taskParantMap = taskList.stream()
                .collect(Collectors.groupingBy(WmTask::getTaskTypeParentId, Collectors.toList()));
        taskParantMap.forEach((parentId, wmTasks) -> {
            // 任务类型-大类
            String taskTypeName = scoreTaskTypeDictMap.get(parentId);
            // 活动次数
            int taskCount = wmTasks.size();
            // 合计积分
            Integer taskSum = wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum);
            RowRenderData build = RowRenderData.build(taskTypeName, String.valueOf(taskCount), String.valueOf(taskSum));
            build.setRowStyle(tableStyle);
            briefTb1List.add(build);
        });
        // 表合计
        RowRenderData tb1Bottom = RowRenderData.build(new TextRenderData("合计"),
                new TextRenderData(String.valueOf(taskList.size())),
                new TextRenderData(String.valueOf(taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum))));
        tb1Bottom.setRowStyle(tableStyle);
        briefTb1List.add(tb1Bottom);
        data.put("briefTb1", new MiniTableRenderData(tb1Header, briefTb1List));

        // =================================表格2=================================
        // 学术推广整体概况-任务小类统计
        RowRenderData tb2Header = RowRenderData.build(new TextRenderData("任务类型"),
                new TextRenderData("活动次数"),
                new TextRenderData("合计积分"));
        tb2Header.setRowStyle(tableStyle);

        List<RowRenderData> briefTb2List = new ArrayList<>();
        // task通过taskTypeId分组
        Map<String, List<WmTask>> taskTypeMap = taskList.stream()
                .collect(Collectors.groupingBy(WmTask::getTaskTypeId, Collectors.toList()));
        taskTypeMap.forEach((taskTypeId, wmTasks) -> {
            // 任务类型
            String taskTypeName = scoreTaskTypeDictMap.get(taskTypeId);
            // 活动次数
            int taskCount = wmTasks.size();
            // 合计积分
            Integer taskSum = wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum);
            RowRenderData build = RowRenderData.build(taskTypeName, String.valueOf(taskCount), String.valueOf(taskSum));
            build.setRowStyle(tableStyle);
            briefTb2List.add(build);
        });
        // 表合计
        RowRenderData tb2Bottom = RowRenderData.build(new TextRenderData("合计"),
                new TextRenderData(String.valueOf(taskList.size())),
                new TextRenderData(String.valueOf(taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum))));
        tb2Bottom.setRowStyle(tableStyle);
        briefTb2List.add(tb2Bottom);
        data.put("briefTb2", new MiniTableRenderData(tb2Header, briefTb2List));

        // =================================表格3=================================
        // 学术推广整体概况-承接方维度统计
        TableStyle tableStyle2 = new TableStyle();
        tableStyle2.setAlign(STJc.Enum.forInt(2));
        tableStyle2.setBackgroundColor("cccccc");

        RowRenderData tb3Header = RowRenderData.build(new TextRenderData("承接方"),
                new TextRenderData("任务计数"),
                new TextRenderData("累计积分"));
        tb3Header.setRowStyle(tableStyle2);

        List<RowRenderData> briefTb3List = new ArrayList<>();
        // task通过taskUserId分组
        Map<String, List<WmTask>> taskUserMap = taskList.stream().collect(Collectors.groupingBy(WmTask::getTaskUserId));
        taskUserMap.forEach((taskUserId, wmTasks) -> {
            // 承接方
            String realName = userMap.get(Integer.parseInt(taskUserId));
            // 任务计数
            int taskCount = wmTasks.size();
            // 累计积分
            Integer taskSum = wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum);
            RowRenderData build = RowRenderData.build(realName, String.valueOf(taskCount), String.valueOf(taskSum));
            build.setRowStyle(tableStyle);
            briefTb3List.add(build);
        });
        // 表合计
        RowRenderData tb3Bottom = RowRenderData.build(new TextRenderData("合计"),
                new TextRenderData(String.valueOf(taskList.size())),
                new TextRenderData(String.valueOf(taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum))));
        tb3Bottom.setRowStyle(tableStyle);
        briefTb3List.add(tb3Bottom);
        data.put("briefTb3", new MiniTableRenderData(tb3Header, briefTb3List));

        // =================================表格4=================================
        // 学术推广服务明细
        // 表头
        RowRenderData tb4Header = RowRenderData.build(new TextRenderData("完成时间"),
                new TextRenderData("任务类型"),
                new TextRenderData("分值"),
                new TextRenderData("承接方"),
                new TextRenderData("所属组织"));
        tb4Header.setRowStyle(tableStyle);

        List<RowRenderData> tb4List = new ArrayList<>();
        taskParantMap.forEach((parentId, wmTasks) -> {
            // 任务类型-大类
            String taskTypeName = scoreTaskTypeDictMap.get(parentId);
            // 根据userId二次分类
            Map<String, List<WmTask>> userTasksMap = wmTasks.stream().collect(Collectors.groupingBy(WmTask::getTaskUserId));
            userTasksMap.forEach((userId, tasks) -> {
                // 完成时间
                String finishTime = DateUtil.format(tasks.stream().map(WmTask::getCreateTime).max(LocalDateTime::compareTo).get(), DatePattern.NORM_DATE_PATTERN);
                // 合计积分
                Integer taskSum = tasks.stream().map(WmTask::getScore).reduce(0, Integer::sum);
                // 姓名
                String realName = userMap.get(Integer.parseInt(userId));
                // 所属组织
                String entName = currentDept.getName();
                RowRenderData build = RowRenderData.build(finishTime, taskTypeName, String.valueOf(taskSum), realName, entName);
                build.setRowStyle(tableStyle);
                tb4List.add(build);
            });
        });
        data.put("detailTb4", new MiniTableRenderData(tb4Header, tb4List));

        // =================================表格5=================================
        // 学术推广服务任务明细
        RowRenderData tb5Header = RowRenderData.build(new TextRenderData("序号"),
                new TextRenderData("完成时间"),
                new TextRenderData("任务名称"),
                new TextRenderData("任务编号"),
                new TextRenderData("分值"),
                new TextRenderData("承接方"),
                new TextRenderData("所属组织"));
        tb2Header.setRowStyle(tableStyle);

        List<RowRenderData> tb5List = new ArrayList<>();
        AtomicInteger index = new AtomicInteger(1);
        taskList.forEach(task -> {
            // 任务编号
            String taskNumber = task.getTaskNumber();
            // 任务类型
            String taskTypeName = scoreTaskTypeDictMap.get(task.getTaskTypeId());
            // 完成时间
            String finishTime = DateUtil.format(task.getCreateTime(), DatePattern.NORM_DATE_PATTERN);
            // 分值
            String score = String.valueOf(task.getScore());
            // 姓名
            String realName = userMap.get(Integer.parseInt(task.getTaskUserId()));
            // 所属组织
            String entName = currentDept.getName();
            // 序号
            int andIncrement = index.getAndIncrement();
            task.setTmpId(andIncrement);
            RowRenderData build = RowRenderData.build(String.valueOf(andIncrement), finishTime, taskTypeName, taskNumber,
                    score, realName, entName);
            build.setRowStyle(tableStyle);
                tb5List.add(build);
        });
        data.put("detailTb5", new MiniTableRenderData(tb5Header, tb5List));

        // 专项汇总表数据
        // 医院/药店/商业公司打卡详情数据查询
        Set<String> taskConentIds = taskList.stream()
                .filter(task -> Arrays.asList("5", "6", "33", "73").contains(task.getTaskTypeId()))
                .map(WmTask::getTaskContentId).collect(Collectors.toSet());
        if (CollUtil.isNotEmpty(taskConentIds)) {
            List<WmTaskContent> taskContents = taskContentMapper.selectBatchIds(taskConentIds);
            Set<Integer> userSignIds = taskContents.stream()
                    .map(taskContent -> Integer.parseInt(taskContent.getTemp3())).collect(Collectors.toSet());
            List<WmUserSignDetail> userSignDetails = userSignDetailMapper.selectList(Wrappers.<WmUserSignDetail>lambdaQuery()
                    .in(WmUserSignDetail::getSignId, userSignIds));
            List<WmUserSign> userSigns = userSignMapper.selectBatchIds(userSignIds);
            // signId和userSign的关系map
            Map<Integer, WmUserSign> userSignMap = userSigns.stream().collect(Collectors.toMap(WmUserSign::getId, Function.identity()));
            // signId和userSignDetail的关系map
            Map<Integer, WmUserSignDetail> userSignDetailMap = userSignDetails.stream()
                    .collect(Collectors.toMap(WmUserSignDetail::getSignId, Function.identity()));
            // taskConentId和signId的关系map
            Map<Integer, String> taskContSignIdMap = taskContents.stream()
                    .collect(Collectors.toMap(WmTaskContent::getId, WmTaskContent::getTemp3));
            // 查询拜访目的/拜访结果的字典值
            List<SysDictItem> dictItems = dictItemMapper.selectList(Wrappers.<SysDictItem>lambdaQuery()
                    .in(SysDictItem::getType, "user_sign_detail_purpose", "user_sign_detail_pharmacy_purpose",
                            "user_sign_detail_distribution_purpose", "user_sign_detail_result")
                    .eq(SysDictItem::getDelFlag, "0"));
            Map<String, List<SysDictItem>> dictItemMap = dictItems.stream()
                    .collect(Collectors.groupingBy(SysDictItem::getType));
            Map<String, String> hospitalPurposeMap = dictItemMap.get("user_sign_detail_purpose").stream()
                    .collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));
            Map<String, String> pharmacyPurposeMap = dictItemMap.get("user_sign_detail_pharmacy_purpose").stream()
                    .collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));
            Map<String, String> distributionPurposeMap = dictItemMap.get("user_sign_detail_distribution_purpose").stream()
                    .collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));
            Map<String, String> signResultMap = dictItemMap.get("user_sign_detail_result").stream()
                    .collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));

            // =================================表格6=================================
            // 专项汇总表-医院拜访汇总表
            RowRenderData tb6Header = RowRenderData.build(new TextRenderData("序号"),
                    new TextRenderData("任务编号"),
                    new TextRenderData("任务类型"),
                    new TextRenderData("任务积分"),
                    new TextRenderData("完成时间"),
                    new TextRenderData("拜访地址"),
                    new TextRenderData("拜访对象"),
                    new TextRenderData("拜访目的"),
                    new TextRenderData("推广产品"),
                    new TextRenderData("结果反馈"));
            tb2Header.setRowStyle(tableStyle);

            List<RowRenderData> tb6List = new ArrayList<>();
            AtomicInteger tb6Index = new AtomicInteger(1);
            taskList.stream().filter(task -> StrUtil.equals(task.getTaskTypeId(), "5")).forEach(task -> {
                // 任务编号
                String taskNumber = task.getTaskNumber();
                // 任务类型
                String taskTypeName = scoreTaskTypeDictMap.get(task.getTaskTypeId());
                // 任务积分
                String score = String.valueOf(task.getScore());
                // 完成时间
                String finishTime = DateUtil.format(task.getCreateTime(), DatePattern.NORM_DATE_PATTERN);

                String signId = taskContSignIdMap.get(Integer.parseInt(task.getTaskContentId()));
                WmUserSignDetail userSignDetail = userSignDetailMap.get(Integer.parseInt(signId));
                WmUserSign userSign = userSignMap.get(Integer.parseInt(signId));
                // 拜访地址
                String address = userSign.getSignEntName();
                String obj = "";
                String purpose = "";
                String drug = "";
                String result = "";
                if (userSignDetail != null) {
                    // 拜访对象(拜访科室)
                    obj = ArrayUtil.join(userSignDetail.getDepartment(), "，");
                    // 拜访目的
                    purpose = Arrays.stream(userSignDetail.getPurpose())
                            .map(hospitalPurposeMap::get).collect(Collectors.joining("，"));
                    // 推广产品
                    drug = ArrayUtil.join(userSignDetail.getDrug(), "，");
                    // 结果反馈
                    result = signResultMap.get(userSignDetail.getResult());
                }
                // 序号
                int andIncrement = tb6Index.getAndIncrement();

                RowRenderData build = RowRenderData.build(String.valueOf(andIncrement), taskNumber, taskTypeName,
                        score, finishTime, address, obj, purpose, drug, result);
                build.setRowStyle(tableStyle);
                tb6List.add(build);
            });
            if (CollUtil.isNotEmpty(tb6List)) {
                data.put("detailTb6", new MiniTableRenderData(tb6Header, tb6List));
            }

            // =================================表格7=================================
            // 专项汇总表-药店拜访汇总表
            RowRenderData tb7Header = RowRenderData.build(new TextRenderData("序号"),
                    new TextRenderData("任务编号"),
                    new TextRenderData("任务类型"),
                    new TextRenderData("任务积分"),
                    new TextRenderData("完成时间"),
                    new TextRenderData("拜访地址"),
                    new TextRenderData("拜访对象"),
                    new TextRenderData("拜访目的"),
                    new TextRenderData("推广产品"),
                    new TextRenderData("结果反馈"));
            tb2Header.setRowStyle(tableStyle);

            List<RowRenderData> tb7List = new ArrayList<>();
            AtomicInteger tb7Index = new AtomicInteger(1);
            taskList.stream().filter(task -> Arrays.asList("33", "73").contains(task.getTaskTypeId())).forEach(task -> {
                // 任务编号
                String taskNumber = task.getTaskNumber();
                // 任务类型
                String taskTypeName = scoreTaskTypeDictMap.get(task.getTaskTypeId());
                // 任务积分
                String score = String.valueOf(task.getScore());
                // 完成时间
                String finishTime = DateUtil.format(task.getCreateTime(), DatePattern.NORM_DATE_PATTERN);

                String signId = taskContSignIdMap.get(Integer.parseInt(task.getTaskContentId()));
                WmUserSignDetail userSignDetail = userSignDetailMap.get(Integer.parseInt(signId));
                WmUserSign userSign = userSignMap.get(Integer.parseInt(signId));
                // 拜访地址
                String address = userSign.getSignEntName();

                String obj = "";
                String purpose = "";
                String drug = "";
                String result = "";
                if (userSignDetail != null) {
                    // 拜访对象(拜访人员)
                    obj = userSignDetail.getVisitor();
                    // 拜访目的
                    purpose = Arrays.stream(userSignDetail.getPurpose())
                            .map(pharmacyPurposeMap::get).collect(Collectors.joining("，"));
                    // 推广产品
                    drug = ArrayUtil.join(userSignDetail.getDrug(), "，");
                    // 结果反馈
                    result = signResultMap.get(userSignDetail.getResult());
                }
                // 序号
                int andIncrement = tb7Index.getAndIncrement();

                RowRenderData build = RowRenderData.build(String.valueOf(andIncrement), taskNumber, taskTypeName,
                        score, finishTime, address, obj, purpose, drug, result);
                build.setRowStyle(tableStyle);
                tb7List.add(build);
            });
            if (CollUtil.isNotEmpty(tb7List)) {
                data.put("detailTb7", new MiniTableRenderData(tb7Header, tb7List));
            }

            // =================================表格8=================================
            // 专项汇总表-商业公司拜访汇总表
            RowRenderData tb8Header = RowRenderData.build(new TextRenderData("序号"),
                    new TextRenderData("任务编号"),
                    new TextRenderData("任务类型"),
                    new TextRenderData("任务积分"),
                    new TextRenderData("完成时间"),
                    new TextRenderData("拜访对象"),
                    new TextRenderData("拜访目的"),
                    new TextRenderData("推广产品"),
                    new TextRenderData("结果反馈"));
            tb2Header.setRowStyle(tableStyle);

            List<RowRenderData> tb8List = new ArrayList<>();
            AtomicInteger tb8Index = new AtomicInteger(1);
            taskList.stream().filter(task -> StrUtil.equals("6", task.getTaskTypeId())).forEach(task -> {
                // 任务编号
                String taskNumber = task.getTaskNumber();
                // 任务类型
                String taskTypeName = scoreTaskTypeDictMap.get(task.getTaskTypeId());
                // 任务积分
                String score = String.valueOf(task.getScore());
                // 完成时间
                String finishTime = DateUtil.format(task.getCreateTime(), DatePattern.NORM_DATE_PATTERN);

                String signId = taskContSignIdMap.get(Integer.parseInt(task.getTaskContentId()));
                WmUserSignDetail userSignDetail = userSignDetailMap.get(Integer.parseInt(signId));
                WmUserSign userSign = userSignMap.get(Integer.parseInt(signId));
                // 拜访地址
                String address = userSign.getSignEntName();

                String purpose = "";
                String drug = "";
                String result = "";
                if (userSignDetail != null) {
                    // 拜访目的
                    purpose = Arrays.stream(userSignDetail.getPurpose())
                            .map(distributionPurposeMap::get).collect(Collectors.joining("，"));
                    // 推广产品
                    drug = ArrayUtil.join(userSignDetail.getDrug(), "，");
                    // 结果反馈
                    result = signResultMap.get(userSignDetail.getResult());
                }
                // 序号
                int andIncrement = tb8Index.getAndIncrement();

                RowRenderData build = RowRenderData.build(String.valueOf(andIncrement), taskNumber, taskTypeName,
                        score, finishTime, address, purpose, drug, result);
                build.setRowStyle(tableStyle);
                tb8List.add(build);
            });
            if (CollUtil.isNotEmpty(tb8List)) {
                data.put("detailTb8", new MiniTableRenderData(tb8Header, tb8List));
            }
        }

        // =================================表格9=================================
        // 专项汇总表-学术会议汇总表
        RowRenderData tb9Header = RowRenderData.build(new TextRenderData("序号"),
                new TextRenderData("任务编号"),
                new TextRenderData("任务类型"),
                new TextRenderData("任务积分"),
                new TextRenderData("完成时间"),
                new TextRenderData("会议主题"),
                new TextRenderData("会议时间"),
                new TextRenderData("推广药品"),
                new TextRenderData("会议地点"),
                new TextRenderData("会议总结"));
        tb2Header.setRowStyle(tableStyle);

        List<RowRenderData> tb9List = new ArrayList<>();
        AtomicInteger tb9Index = new AtomicInteger(1);
        Set<String> meetingTaskContIds = taskList.stream().filter(task -> Arrays.asList("1", "2", "3", "4", "64").contains(task.getTaskTypeId()))
                .map(WmTask::getTaskContentId).collect(Collectors.toSet());
        if (CollUtil.isNotEmpty(meetingTaskContIds)) {
            List<WmTaskContent> meetingTaskContents = taskContentMapper.selectBatchIds(meetingTaskContIds);
            Map<Integer, WmTaskContent> meetingContentMap = meetingTaskContents.stream()
                    .collect(Collectors.toMap(WmTaskContent::getId, Function.identity()));
            taskList.stream().filter(task -> Arrays.asList("1", "2", "3", "4", "64").contains(task.getTaskTypeId())).forEach(task -> {
                // 任务编号
                String taskNumber = task.getTaskNumber();
                // 任务类型
                String taskTypeName = scoreTaskTypeDictMap.get(task.getTaskTypeId());
                // 任务积分
                String score = String.valueOf(task.getScore());
                // 完成时间
                String finishTime = DateUtil.format(task.getCreateTime(), DatePattern.NORM_DATE_PATTERN);

                WmTaskContent taskContent = meetingContentMap.get(Integer.parseInt(task.getTaskContentId()));
                // 会议主题
                String meetingTitle = taskContent.getTemp2();
                // 会议时间
                String meetingTime = taskContent.getTemp4();
                // 推广产品
                String drug = taskContent.getTemp12();
                // 会议地点
                String address = taskContent.getTemp31();
                // 会议总结
                String summary = taskContent.getTemp9();
                // 序号
                int andIncrement = tb9Index.getAndIncrement();

                RowRenderData build = RowRenderData.build(String.valueOf(andIncrement), taskNumber, taskTypeName,
                        score, finishTime, meetingTitle, meetingTime, drug, address, summary);
                build.setRowStyle(tableStyle);
                tb9List.add(build);
            });
            data.put("detailTb9", new MiniTableRenderData(tb9Header, tb9List));
        }

    }

    public String getFieldValueByFieldName(String fieldName, Object object) {
        try {
            Field field = object.getClass().getDeclaredField(fieldName);
            //对private的属性的访问
            field.setAccessible(true);
            return field.get(object) + "";
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
