package com.qunzhixinxi.hnqz.report.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.entity.ReportHandlerDeptRelation;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.ReportTypeEnum;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.ScorePackageStatusEnum;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTask;
import com.qunzhixinxi.hnqz.report.mapper.ReportHandlerDeptRelationMapper;
import com.qunzhixinxi.hnqz.report.mapper.SysDictItemMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmScorePackageMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskContentMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignDetailMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignMapper;
import com.qunzhixinxi.hnqz.report.service.SysDeptService;
import com.qunzhixinxi.hnqz.report.service.SysFileService;
import com.qunzhixinxi.hnqz.report.service.SysUserService;
import com.qunzhixinxi.hnqz.report.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.report.service.WmScorePackageStatusService;
import com.qunzhixinxi.hnqz.report.service.WmTaskService;
import com.qunzhixinxi.hnqz.report.service.WmTaskTypeService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.report.util.OsEnvUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 新版excel通用报告导出
 *
 * @author lixuesong
 * @date 2023/12/19
 */
@Slf4j
@Service(value = "newCommonExcelReportServiceImpl")
public class NewCommonExcelReportServiceImpl extends NewAbstractExcelReportService {

    @Resource
    private WmScorePackageMapper scorePackageMapper;
    @Resource
    private WmTaskMapper taskMapper;
    @Resource
    private ReportHandlerDeptRelationMapper reportHandlerDeptRelationMapper;
    @Resource
    private SysFileService fileService;

    public NewCommonExcelReportServiceImpl(WmScorePackageStatusService wmScorePackageStatusService, WmTaskService wmTaskService,
                                           WmTaskContentMapper wmTaskContentMapper, WmUserSignMapper wmUserSignMapper,
                                           WmUserSignDetailMapper wmUserSignDetailMapper, WmTaskTypeService wmTaskTypeService,
                                           WmDaDrugEntService wmDaDrugEntService, SysDeptService sysDeptService,
                                           SysUserService sysUserService, SysDictItemMapper sysDictItemMapper) {
        super(wmScorePackageStatusService, wmTaskService, wmTaskContentMapper, wmUserSignMapper, wmUserSignDetailMapper,
                wmTaskTypeService, wmDaDrugEntService, sysDeptService, sysUserService, sysDictItemMapper);
    }

    /**
     * @param packageIds 积分包id
     * @param taskStatus 任务状态
     * @param deptId     部门id
     * @return {@link Map}<{@link String}, {@link String}> 结果
     */
    @Override
    public Map<String, String> export(List<String> packageIds, String taskStatus, Integer deptId) throws Exception {

        List<WmScorePackage> scorePackageList = scorePackageMapper.selectBatchIds(packageIds);
        if (CollUtil.isEmpty(scorePackageList)) {
            throw new RuntimeException("积分包不存在");
        }

        List<WmTask> taskList = null;

        String typeid = scorePackageList.get(0).getTypeid();
        if (Arrays.asList("0", "1", "2").contains(typeid)) {
            // 企业版报告
            Set<String> allPackageIds = new HashSet<>();
            allPackageIds.addAll(packageIds);
            // 查询所有子包
            List<WmScorePackage> subPackageList = scorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
                    .in(WmScorePackage::getRelationScoreId, packageIds)
                    .ne(WmScorePackage::getScorePackageStatus, ScorePackageStatusEnum.TERMINATED.val())
                    .select(WmScorePackage::getId));
            if (CollUtil.isNotEmpty(subPackageList)) {
                Set<String> subPackageIds = subPackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
                allPackageIds.addAll(subPackageIds);
                // 查询所有子子包
                List<WmScorePackage> subSubPackageList = scorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
                        .in(WmScorePackage::getRelationScoreId, subPackageIds)
                        .ne(WmScorePackage::getScorePackageStatus, ScorePackageStatusEnum.TERMINATED.val())
                        .select(WmScorePackage::getId));
                if (CollUtil.isNotEmpty(subSubPackageList)) {
                    Set<String> subSubPackageIds = subSubPackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
                    allPackageIds.addAll(subSubPackageIds);
                }
            }

            // 查询所有上级审核通过的任务
            taskList = taskMapper.selectList(Wrappers.<WmTask>lambdaQuery()
                    .in(WmTask::getScorePackageId, allPackageIds)
                    .eq(WmTask::getTaskStatus, taskStatus)
                    .and(item -> item.in(WmTask::getReportDrugApprovalStatus, Arrays.asList(5, 6)).eq(WmTask::getReportDrugApprovalOpinion, 1)
                            .or(item2 -> item2.in(WmTask::getReportOneApprovalStatus, Arrays.asList(5, 6)).eq(WmTask::getReportOneApprovalOpinion, 1))));
            if (CollUtil.isEmpty(taskList)) {
                throw new RuntimeException("未查询到该报告关联的任务");
            }
        } else {
            // 个人版报告
            LambdaQueryWrapper<WmTask> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.in(WmTask::getScorePackageId, packageIds);
            queryWrapper.eq(WmTask::getDelFlag, "0");
            queryWrapper.eq(WmTask::getEnableFlag, "0");
            queryWrapper.eq(WmTask::getRealFlag, "0");
            queryWrapper.eq(WmTask::getTaskStatus, taskStatus);
            taskList = taskMapper.selectList(queryWrapper);

            if (CollUtil.isEmpty(taskList)) {
                throw new RuntimeException("未查询到该报告关联的任务");
            }
        }

        String packageIdsStr = String.join(",", packageIds);
        // 创建excel模板数据
        Map<String, Object> excelData = this.createExcelData(scorePackageList, taskList);
        // 基础路径
        String workPath = (String) OsEnvUtils.getEachEnvPaths().get("EXCEL_FILE_PATH");
        // 模板路径
        String templateFile = workPath + "new_task_summary_template.xlsx";
        // 获取企业配置的模板
        List<ReportHandlerDeptRelation> reportHandlerDeptRelations = reportHandlerDeptRelationMapper.selectList(Wrappers.<ReportHandlerDeptRelation>lambdaQuery()
                .eq(ReportHandlerDeptRelation::getDetpId, deptId)
                .eq(ReportHandlerDeptRelation::getReportType, ReportTypeEnum.NEW_EXCEL_COMMON.getType()));
        if (CollUtil.isNotEmpty(reportHandlerDeptRelations)) {
            String template = reportHandlerDeptRelations.get(0).getTemplate();
            if (StrUtil.isNotBlank(template)) {
                templateFile = workPath + template;
            }
        }

        // 目标路径
        String targetFile = workPath +
                String.format("out/任务汇总报告(%s)-%s.xlsx", packageIdsStr, DateUtil.format(LocalDateTime.now(), DatePattern.PURE_DATETIME_PATTERN));

        // 填充模板
        this.createExcel(templateFile, targetFile, excelData);

        // 上传oss
        File tgFile = new File(targetFile);
        FileInputStream input = new FileInputStream(tgFile);
        MultipartFile multipartFile = new MockMultipartFile(String.format("任务汇总报告(%s)", packageIdsStr),
                String.format("任务汇总报告(%s).xlsx", packageIdsStr),
                "text/plain", IOUtils.toByteArray(input));
        R r = fileService.uploadFileAllUrlRealName(multipartFile, null);

        input.close();
        tgFile.delete();

        if (r.getCode() == 0) {
            log.info("报告导出OSS结果:{}", r.getData());
            return (Map<String, String>) r.getData();
        }

        return null;
    }
}
