package com.qunzhixinxi.hnqz.report.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.report.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.api.entity.ReportHandlerDeptRelation;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.ReportTypeEnum;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.ScorePackageStatusEnum;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.TaskStatusEnum;
import com.qunzhixinxi.hnqz.report.mapper.ReportHandlerDeptRelationMapper;
import com.qunzhixinxi.hnqz.report.mapper.SysDeptMapper;
import com.qunzhixinxi.hnqz.report.mapper.SysDictItemMapper;
import com.qunzhixinxi.hnqz.report.mapper.SysUserMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmDaDrugEntDrugtableMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmScorePackageMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskContentConfigMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskContentMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskTypeMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignDetailMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignMapper;
import com.qunzhixinxi.hnqz.report.service.SysFileService;
import com.qunzhixinxi.hnqz.report.service.WmScorePackageStatusService;
import com.qunzhixinxi.hnqz.report.util.OsEnvUtils;
import com.qunzhixinxi.hnqz.common.core.util.R;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.context.annotation.Primary;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 新版企业报告导出
 *
 * @author lixuesong
 * @date 2023/12/08
 */
@Primary
@Slf4j
@Service(value = "newDrugEntReportServiceImpl")
public class NewDrugEntReportServiceImpl extends NewAbstractReportService {

    @Resource
    private WmScorePackageMapper scorePackageMapper;
    @Resource
    private WmTaskMapper taskMapper;
    @Resource
    private ReportHandlerDeptRelationMapper reportHandlerDeptRelationMapper;
    @Resource
    private SysFileService fileService;

    public NewDrugEntReportServiceImpl(WmTaskTypeMapper taskTypeMapper, WmTaskContentMapper taskContentMapper,
                                       WmTaskContentConfigMapper taskContentConfigMapper, WmUserSignDetailMapper userSignDetailMapper,
                                       WmUserSignMapper userSignMapper, SysDeptMapper deptMapper, WmDaDrugEntDrugtableMapper drugEntDrugtableMapper,
                                       WmScorePackageStatusService scorePackageStatusService, SysDictItemMapper dictItemMapper,
                                       SysUserMapper userMapper, UpmsConfig upmsConfig) {
        super(taskTypeMapper, taskContentMapper, taskContentConfigMapper, userSignDetailMapper, userSignMapper,
                deptMapper, drugEntDrugtableMapper, scorePackageStatusService, dictItemMapper, userMapper, upmsConfig);
    }

    /**
     * 企业承接任务的导出、任务中心单的导出
     *
     * @param packageIds 积分包id
     * @param taskStatus 任务状态
     * @param deptId     部门id
     * @return {@link Map}<{@link String}, {@link String}>
     */
    @Override
    public Map<String, String> export(List<String> packageIds, String taskStatus, Integer deptId) throws Exception {
        List<WmScorePackage> scorePackageList = scorePackageMapper.selectBatchIds(packageIds);
        if (CollUtil.isEmpty(scorePackageList)) {
            throw new RuntimeException("积分包不存在");
        }

        Set<String> allPackageIds = new HashSet<>();
        allPackageIds.addAll(packageIds);
        // 查询所有子包
        List<WmScorePackage> subPackageList = scorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
                .in(WmScorePackage::getRelationScoreId, packageIds)
                .ne(WmScorePackage::getScorePackageStatus, ScorePackageStatusEnum.TERMINATED.val())
                .select(WmScorePackage::getId));
        if (CollUtil.isNotEmpty(subPackageList)) {
            Set<String> subPackageIds = subPackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
            allPackageIds.addAll(subPackageIds);
            // 查询所有子子包
            List<WmScorePackage> subSubPackageList = scorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
                    .in(WmScorePackage::getRelationScoreId, subPackageIds)
                    .ne(WmScorePackage::getScorePackageStatus, ScorePackageStatusEnum.TERMINATED.val())
                    .select(WmScorePackage::getId));
            if (CollUtil.isNotEmpty(subSubPackageList)) {
                Set<String> subSubPackageIds = subSubPackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
                allPackageIds.addAll(subSubPackageIds);
            }
        }

        // 查询所有上级审核通过的任务
        List<WmTask> superApprovedTaskList = taskMapper.selectList(Wrappers.<WmTask>lambdaQuery()
                .in(WmTask::getScorePackageId, allPackageIds)
                .eq(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val())
                .and(item -> item.in(WmTask::getReportDrugApprovalStatus, Arrays.asList(5, 6)).eq(WmTask::getReportDrugApprovalOpinion, 1)
                        .or(item2 -> item2.in(WmTask::getReportOneApprovalStatus, Arrays.asList(5, 6)).eq(WmTask::getReportOneApprovalOpinion, 1))));
        if (CollUtil.isEmpty(superApprovedTaskList)) {
            throw new RuntimeException("未查询到该报告关联的审批通过的任务");
        }

        // 查询所有提交给上级的任务
        List<WmTask> submitedTaskList = taskMapper.selectList(Wrappers.<WmTask>lambdaQuery()
                .in(WmTask::getScorePackageId, allPackageIds)
//                .eq(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val())
                .and(item -> item.in(WmTask::getReportDrugApprovalStatus, Arrays.asList(2, 3, 5, 6))
                        .or(item2 -> item2.in(WmTask::getReportOneApprovalStatus, Arrays.asList(2, 3, 5, 6)))));
        if (CollUtil.isEmpty(submitedTaskList)) {
            throw new RuntimeException("未查询到该报告关联的任务");
        }

        // 创建模板所需数据
        Map<String, Object> data = this.createData(scorePackageList, superApprovedTaskList, submitedTaskList, submitedTaskList);

        // 基础路径
        String workPath = (String) OsEnvUtils.getEachEnvPaths().get("EXCEL_FILE_PATH");
        // 模板路径
        String templateFilePath = workPath + "drug_ent_task_report_template.docx";
        // 获取企业配置的模板
        List<ReportHandlerDeptRelation> reportHandlerDeptRelations = reportHandlerDeptRelationMapper.selectList(Wrappers.<ReportHandlerDeptRelation>lambdaQuery()
                .eq(ReportHandlerDeptRelation::getDetpId, deptId)
                .eq(ReportHandlerDeptRelation::getReportType, ReportTypeEnum.NEW_DRUG_ENT.getType()));
        if (CollUtil.isNotEmpty(reportHandlerDeptRelations)) {
            String template = reportHandlerDeptRelations.get(0).getTemplate();
            if (StrUtil.isNotBlank(template)) {
                templateFilePath = workPath + template;
            }
        }

        String packageIdsStr = String.join(",", packageIds);
        // 目标路径
        String targetFilePath = workPath +
                String.format("out/推广服务报告(%s)-%s.docx", packageIdsStr, DateUtil.format(LocalDateTime.now(), DatePattern.PURE_DATETIME_PATTERN));
        log.info("报告导出缓存路径:{}", targetFilePath);

        // 填充模板
        File resultFile = this.createWord(templateFilePath, targetFilePath, data);

        // 上传oss
        FileInputStream input = new FileInputStream(resultFile);
        MultipartFile multipartFile = new MockMultipartFile(String.format("推广服务报告(%s)", packageIdsStr),
                String.format("推广服务报告(%s).docx", packageIdsStr),
                "text/plain", IOUtils.toByteArray(input));
        R r = fileService.uploadFileAllUrlRealName(multipartFile, null);

        input.close();
        resultFile.delete();

        if (r.getCode() == 0) {
            log.info("报告导出OSS结果:{}", r.getData());
            return (Map<String, String>) r.getData();
        }

        return null;
    }

}
