package com.qunzhixinxi.hnqz.report.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.report.service.impl.NewAbstractReportService;
import com.qunzhixinxi.hnqz.report.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.api.entity.ReportHandlerDeptRelation;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.ReportTypeEnum;
import com.qunzhixinxi.hnqz.report.mapper.ReportHandlerDeptRelationMapper;
import com.qunzhixinxi.hnqz.report.mapper.SysDeptMapper;
import com.qunzhixinxi.hnqz.report.mapper.SysDictItemMapper;
import com.qunzhixinxi.hnqz.report.mapper.SysUserMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmDaDrugEntDrugtableMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmScorePackageMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskContentConfigMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskContentMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskTypeMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignDetailMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignMapper;
import com.qunzhixinxi.hnqz.report.service.SysFileService;
import com.qunzhixinxi.hnqz.report.service.WmScorePackageStatusService;
import com.qunzhixinxi.hnqz.report.util.OsEnvUtils;
import com.qunzhixinxi.hnqz.common.core.util.R;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 新版兼职/全职报告导出
 *
 * @author lixuesong
 * @date 2023/12/08
 */
@Slf4j
@Service(value = "newPersonalReportServiceImpl")
public class NewPersonalReportServiceImpl extends NewAbstractReportService {

    @Resource
    private WmScorePackageMapper scorePackageMapper;
    @Resource
    private WmTaskMapper taskMapper;
    @Resource
    private ReportHandlerDeptRelationMapper reportHandlerDeptRelationMapper;
    @Resource
    private SysFileService fileService;

    public NewPersonalReportServiceImpl(WmTaskTypeMapper taskTypeMapper, WmTaskContentMapper taskContentMapper,
                                        WmTaskContentConfigMapper taskContentConfigMapper, WmUserSignDetailMapper userSignDetailMapper,
                                        WmUserSignMapper userSignMapper, SysDeptMapper deptMapper, WmDaDrugEntDrugtableMapper drugEntDrugtableMapper,
                                        WmScorePackageStatusService scorePackageStatusService, SysDictItemMapper dictItemMapper,
                                        SysUserMapper userMapper, UpmsConfig upmsConfig) {
        super(taskTypeMapper, taskContentMapper, taskContentConfigMapper, userSignDetailMapper, userSignMapper,
                deptMapper, drugEntDrugtableMapper, scorePackageStatusService, dictItemMapper, userMapper, upmsConfig);
    }

    /**
     * 企业承接任务的导出、任务中心单的导出
     *
     * @param packageIds 积分包id
     * @param taskStatus 任务状态
     * @param deptId
     * @return {@link Map}<{@link String}, {@link String}>
     */
    @Override
    public Map<String, String> export(List<String> packageIds, String taskStatus, Integer deptId) throws Exception {
        List<WmScorePackage> scorePackageList = scorePackageMapper.selectBatchIds(packageIds);
        if (CollUtil.isEmpty(scorePackageList)) {
            throw new RuntimeException("积分包不存在");
        }

        LambdaQueryWrapper<WmTask> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(WmTask::getScorePackageId, packageIds);
        queryWrapper.eq(WmTask::getDelFlag, "0");
        queryWrapper.eq(WmTask::getEnableFlag, "0");
        queryWrapper.eq(WmTask::getRealFlag, "0");
        List<WmTask> allTaskList = taskMapper.selectList(queryWrapper);

        if (CollUtil.isEmpty(allTaskList)) {
            throw new RuntimeException("未查询到该报告关联的任务");
        }

        // 查询所有审核通过或待审核(取决于taskStatus的值)的任务
        List<WmTask> approvedTaskList = allTaskList.stream()
                .filter(task -> StrUtil.equals(task.getTaskStatus(), taskStatus)).collect(Collectors.toList());
        if (CollUtil.isEmpty(approvedTaskList)) {
            throw new RuntimeException("未查询到该报告关联的审批通过的任务");
        }

        // 创建模板所需数据
        Map<String, Object> data = this.createData(scorePackageList, approvedTaskList, allTaskList, allTaskList);

        // 基础路径
        String workPath = (String) OsEnvUtils.getEachEnvPaths().get("EXCEL_FILE_PATH");
        // 模板路径
        String templateFilePath = null;
        // 获取企业配置的模板
        String deptTemplate = null;
        List<ReportHandlerDeptRelation> reportHandlerDeptRelations = reportHandlerDeptRelationMapper.selectList(Wrappers.<ReportHandlerDeptRelation>lambdaQuery()
                .eq(ReportHandlerDeptRelation::getDetpId, deptId)
                .eq(ReportHandlerDeptRelation::getReportType, ReportTypeEnum.NEW_EXCEL_COMMON.getType()));
        if (CollUtil.isNotEmpty(reportHandlerDeptRelations)) {
            String template = reportHandlerDeptRelations.get(0).getTemplate();
            if (StrUtil.isNotBlank(template)) {
                deptTemplate = template;
            }
        }
        // 根据全职/兼职类型 获取不同模板
        String typeid = scorePackageList.get(0).getTypeid();
        switch (typeid) {
            case "3":
                // 全职
                if (StrUtil.isBlank(deptTemplate) || !StrUtil.contains(deptTemplate, "full_time")) {
                    templateFilePath = workPath + "full_time_task_report_template.docx";
                } else {
                    templateFilePath = workPath + StrUtil.splitTrim(deptTemplate, ",")
                            .stream().filter(t -> StrUtil.startWith(t, "full_time")).findFirst().get();
                }
                break;
            case "4":
                if (StrUtil.isBlank(deptTemplate) || !StrUtil.contains(deptTemplate, "part_time")) {
                    templateFilePath = workPath + "part_time_task_report_template.docx";
                } else {
                    templateFilePath = workPath + StrUtil.splitTrim(deptTemplate, ",")
                            .stream().filter(t -> StrUtil.startWith(t, "part_time")).findFirst().get();
                }
                break;
            case "5":
                if (StrUtil.isBlank(deptTemplate) || !StrUtil.contains(deptTemplate, "invest_manager")) {
                    templateFilePath = workPath + "invest_manager_task_report_template.docx";
                } else {
                    templateFilePath = workPath + StrUtil.splitTrim(deptTemplate, ",")
                            .stream().filter(t -> StrUtil.startWith(t, "invest_manager")).findFirst().get();
                }
                break;
            default:
                templateFilePath = workPath + "part_time_task_report_template.docx";
        }
        log.info("使用模板名称:{}", templateFilePath);

        String packageIdsStr = String.join(",", packageIds);
        // 目标路径
        String targetFilePath = workPath +
                String.format("out/兼职学术信息沟通专员业绩报告(%s)-%s.docx", packageIdsStr, DateUtil.format(LocalDateTime.now(), DatePattern.PURE_DATETIME_PATTERN));
        log.info("报告导出缓存路径:{}", targetFilePath);

        // 填充模板
        File resultFile = this.createWord(templateFilePath, targetFilePath, data);

        // 上传oss
        FileInputStream input = new FileInputStream(resultFile);
        MultipartFile multipartFile = new MockMultipartFile(String.format("兼职学术信息沟通专员业绩报告(%s)", packageIdsStr),
                String.format("兼职学术信息沟通专员业绩报告(%s).docx", packageIdsStr),
                "text/plain", IOUtils.toByteArray(input));
        R r = fileService.uploadFileAllUrlRealName(multipartFile, null);
        input.close();
        resultFile.delete();

        if (r.getCode() == 0) {
            log.info("报告导出OSS结果:{}", r.getData());
            return (Map<String, String>) r.getData();
        }

        return null;
    }

}
