package com.qunzhixinxi.hnqz.report.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDictItem;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskContent;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskContentConfig;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.api.entity.XlsTask;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.report.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEnt;
import com.qunzhixinxi.hnqz.admin.api.entity.WmReport;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.ReportEnum;
import com.qunzhixinxi.hnqz.report.mapper.SysDictItemMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.report.service.ReportExportService;
import com.qunzhixinxi.hnqz.report.service.SysDeptService;
import com.qunzhixinxi.hnqz.report.service.SysFileService;
import com.qunzhixinxi.hnqz.report.service.SysUserService;
import com.qunzhixinxi.hnqz.report.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.report.service.WmReportService;
import com.qunzhixinxi.hnqz.report.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.report.service.WmScorePackageStatusService;
import com.qunzhixinxi.hnqz.report.service.WmTaskContentConfigService;
import com.qunzhixinxi.hnqz.report.service.WmTaskContentService;
import com.qunzhixinxi.hnqz.report.service.WmTaskService;
import com.qunzhixinxi.hnqz.report.service.WmTaskTypeService;
import com.qunzhixinxi.hnqz.report.util.ExportReport;
import com.qunzhixinxi.hnqz.report.util.Html2ImageByJsWrapper;
import com.qunzhixinxi.hnqz.common.core.util.R;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.context.annotation.Primary;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;


/**
 * <p>
 * 报告管理 服务实现类
 * </p>
 *
 * @author hnqz
 * @since 2018-01-20
 */
@Slf4j
@Service("reportExportService")
@AllArgsConstructor
@Primary
public class ReportExportServiceImpl implements ReportExportService {

    private final WmTaskService wmTaskService;
    private final WmScorePackageService wmScorePackageService;
    private final UpmsConfig upmsConfig;
    private final SysFileService sysFileService;
    private final WmDaDrugEntService wmDaDrugEntService;
    private final WmTaskTypeService wmTaskTypeService;
    private final SysUserService sysUserService;
    private final SysDeptService sysDeptService;
    private final WmTaskContentService wmTaskContentService;
    private final WmTaskContentConfigService wmTaskContentConfigService;
    private final WmTaskMapper wmTaskMapper;
    private final WmReportService wmReportService;
    private final WmScorePackageStatusService wmScorePackageStatusService;
    private final SysDictItemMapper sysDictItemMapper;

    /**
     * 结算报告--导出汇总报告--学术推广服务报告
     *
     * @param wmReport
     * @return
     * @throws IOException
     */
    @Override
    public R<?> newExportNb(WmReport wmReport, Integer deptId, String username) throws IOException {


        wmReport = wmReportService.getById(wmReport.getId());
        List<WmTask> taskList = new ArrayList<>();// 该报告关联的所有未提交的任务
        // 更新任务表审批意见
        WmScorePackage scorePackage = wmScorePackageService.getById(wmReport.getScoreId());
        WmTask wmTask = new WmTask();
        wmTask.setDelFlag(CommonConstants.STATUS_NORMAL);
        wmTask.setEnableFlag(CommonConstants.STATUS_NORMAL);
        wmTask.setRealFlag(CommonConstants.STATUS_NORMAL);

        SysDept sysDept = sysDeptService.getById(scorePackage.getSendPackageDeptId());


        if ("0".equals(scorePackage.getTypeid())) {// 药企
            wmTask.setReportDrugId(wmReport.getId());
            wmTask.setReportDrugApprovalOpinion(ReportEnum.APPROVAL_OPINION_YES.getType());// 审核通过的状态
            taskList = wmTaskService.list(Wrappers.query(wmTask));
        } else if ("1".equals(scorePackage.getTypeid())) {// 一级代理商类型
            wmTask.setReportOneId(wmReport.getId());
            if (sysDept.getLevel() == 2) {
                wmTask.setReportDrugApprovalOpinion(ReportEnum.APPROVAL_OPINION_YES.getType());// 审核通过的状态
            } else {
                wmTask.setReportOneApprovalOpinion(ReportEnum.APPROVAL_OPINION_YES.getType());// 审核通过的状态
            }

            taskList = wmTaskService.list(Wrappers.query(wmTask));
        } else if ("2".equals(scorePackage.getTypeid())) {// 二级代理商类型
            wmTask.setReportSecondId(wmReport.getId());
            wmTask.setReportSecondApprovalOpinion(ReportEnum.APPROVAL_OPINION_YES.getType());// 审核通过的状态
            taskList = wmTaskService.list(Wrappers.query(wmTask));
        }
        if (null == taskList || taskList.size() <= 0) {
            return R.failed("ERROR_未查询到该报告关联的审批通过的任务");
        }
        String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining(","));
        wmTask.setScorePackageId(wmReport.getScoreId().toString());
        wmTask.setId(ids);

        return exportNb(wmTask, wmReport, deptId, username);// 报告汇总
    }


    /**
     * 结算报告--导出汇总报告--学术推广服务报告
     *
     * @param wmTask
     * @param wmReport
     * @return
     * @throws IOException
     */
    private R exportNb(WmTask wmTask, WmReport wmReport, Integer deptId, String username) throws IOException {

        WmTask wmTaskparam = wmTask;
        String packageId = wmTask.getScorePackageId();  // 获取积分包ID
        if (null == packageId) {
            return R.failed("积分包ID不能为空");
        }

        List<String> list = new ArrayList<>();
        list.add(packageId);

        wmTask.setExtIds(list);

        Map<String, Object> excelMap = new HashMap<>();

        WmScorePackage wmScorePackage = wmScorePackageService.getById(packageId);
        if (null == wmScorePackage) {
            return R.failed("获取积分包失败");
        }

        excelMap.put("totalscorePackageScore", wmScorePackage.getScore());
        if (wmScorePackage.getPackageType1().equals("2") || wmScorePackage.getPackageType1().equals("3")) {
            excelMap.put("totalscorePackageTaskNum", wmScorePackage.getTaskNum() * wmScorePackage.getUserNum());// 积分包值任务个数
        }


        List<Map<String, Object>> wmTaskList = wmTaskService.getTaskListByIds(wmTask);


        List<XlsTask> xlsTaskList = new ArrayList<>();// mxTable
        Integer totalMoney = 0;
//		wmTask.setScorePackageId(null);
//		wmTask.setScorePackageDrugId(null);

        // 当前登陆人的机构
        SysDept sysDept = sysDeptService.getById(deptId);

        Map<String, Object> userMap = new HashMap<>();

        // 委托公司（结算企业）
        SysDept drugEnt = sysDeptService.getById(wmReport.getDrugEntId());
        // WmDaDrugEnt wmDaDrugEnt = wmDaDrugEntService.getById(SecurityUtils.getUser().getDrugEntId());

        // 报告公司（申请结算企业）
        SysDept applydrugEnt = sysDeptService.getById(wmReport.getApplyDrugEntId());
        // WmDaAgent wmDaAgent = wmDaAgentService.getById(wmScorePackage.getLevel1Id());

        for (Map<String, Object> wmT : wmTaskList) {
            totalMoney += Integer.parseInt(wmT.get("score") + "");
            XlsTask xlsTask = new XlsTask();
            xlsTask.setTaskNum(1);
            xlsTask.setCreateTime(wmT.get("create_time") + "");

            xlsTask.setDeptName(sysDept.getName());

            // level=2是药企
            if (sysDept.getLevel() == 2) {
                xlsTask.setAgentDeptName(applydrugEnt.getName());
            }

            xlsTask.setDrugEntName(drugEnt.getName());


            xlsTask.setScore(Integer.parseInt(wmT.get("score") + ""));

            xlsTask.setUsername(wmT.get("realname") + "");
            xlsTask.setTaskType(wmT.get("base_id") + "");

            xlsTaskList.add(xlsTask);

            String buildKey = wmT.get("realname") + "_" + drugEnt.getName() + "_" + sysDept.getName();
            // 人员表格
            if (userMap.containsKey(buildKey)) {
                try {
                    userMap.put(buildKey, Integer.parseInt(wmT.get("score") + "") + Integer.parseInt(userMap.get(buildKey) + ""));
                    userMap.put(buildKey + "_total", Integer.parseInt(wmT.get("score_count") + "") + Integer.parseInt(userMap
                            .get(buildKey + "_total") + ""));

                } catch (Exception ex) {
                }
            } else {
                userMap.put(buildKey, Integer.parseInt(wmT.get("score") + ""));
                userMap.put(buildKey + "_total", Integer.parseInt(wmT.get("score_count") + ""));
            }
        }

//		List<String> list1 = new ArrayList<>();
//		list1.add(packageId);

        // 学术推广服务任务明细表格
        List<WmTask> wmTaskDetialList = wmTaskService.getTaskListByIdsReturnWmtaskInfo(wmTask);
        List<Map<String, Object>> wmTaskDetialMap = new ArrayList<>();

        if (wmTaskDetialList.size() > 0) {


            for (WmTask wmTask1 : wmTaskDetialList) {
                Map<String, Object> map = new HashMap<>();
                WmTaskType wmTaskTypeTemp1 = wmTaskTypeService.getById(wmTask1.getTaskTypeId());
                map.put("taskType", wmTaskTypeTemp1.getTaskTypeName());
                DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                String strDate2 = dtf2.format(wmTask1.getCreateTime());
                map.put("taskTime", strDate2);
                //				map.put("taskUser", sysUserService.getById(wmTask1.getTaskUserId()).getRealname());
                map.put("taskUser", sysUserService.getByIdIgnore(wmTask1.getTaskUserId()).getRealname());
                map.put("taskTypeId", wmTask1.getTaskTypeId());
                map.put("score", wmTask1.getScore());
                map.put("taskNum", 1);

                wmTaskDetialMap.add(map);
            }

        }

        excelMap.put("rwmxTable", wmTaskDetialMap);// 学术推广服务任务明细表格

        if (totalMoney > wmScorePackage.getScore()) {
            totalMoney = wmScorePackage.getScore();
        }

        excelMap.put("ryTable", userMap);// 人员表格
        excelMap.put("tMoney", totalMoney);

        if (xlsTaskList.size() > 0) {
            excelMap.put("wt_ent", xlsTaskList.get(0).getDrugEntName());// 委托公司
            excelMap.put("drugEnt", xlsTaskList.get(0).getDrugEntName());// 服务药企
            excelMap.put("mxTable", xlsTaskList);
        }

        List<Map<String, String>> ztTableList = new ArrayList<>();
        Map<String, String> ztMap = new HashMap<>();
        List<Map<String, Object>> wmTaskCount = wmTaskService.getTaskListByIdsCount(wmTask);
        if (wmTaskCount.size() > 0) {
            Integer total = 0;
            Integer sum = 0;
            for (Map<String, Object> map :
                    wmTaskCount) {
                if (null != map.get("taskCount")) {
                    total += Integer.parseInt(map.get("taskCount") + "");
                }

                if (null != map.get("taskSum")) {
                    sum += Integer.parseInt(map.get("taskSum") + "");
                }

            }
            Map<String, Object> totalMap = new HashMap<>();
            totalMap.put("taskTypeName", "合计");
            totalMap.put("taskCount", total);
            totalMap.put("taskSum", sum);
            wmTaskCount.add(totalMap);

            excelMap.put("totalSubmitScore", sum);
            excelMap.put("totalSubmitTaskNum", total);

            excelMap.put("ztTable", wmTaskCount);// 学术推广服务整体概况表格
        }

        // 任务小类及对应的任务个数和积分值
        List<WmTask> taskList = wmTaskMapper.selectBatchIds(Arrays.asList(wmTask.getId().split(",")));

        // 查询taskType
        List<WmTaskType> taskTypeList = wmTaskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
                        .eq(WmTaskType::getTaskTypeLevel, "1")
                        .eq(WmTaskType::getDelFlag, "0")
//                .eq(WmTaskType::getEnableFlag, "0")
        );
        // taskType通过id分组（key为parentId，value为taskTypeId的set）
        Map<String, String> taskTypeIdMap = taskTypeList.stream()
                .collect(Collectors.toMap(WmTaskType::getId, WmTaskType::getTaskTypeName));

        // task通过taskTypeId分组
        Map<String, List<WmTask>> taskTypeMap = taskList.stream()
                .collect(Collectors.groupingBy(WmTask::getTaskTypeId, Collectors.toList()));

        // 封装brief table1数据（统计每个小类任务的积分、任务数）
        List<Map<String, Object>> briefTable1List = new ArrayList<>();
        taskTypeMap.forEach((taskTypeId, wmTasks) -> {
            Map<String, Object> briefTable1Params = new HashMap<>();
            briefTable1Params.put("taskTypeName", taskTypeIdMap.get(taskTypeId));
            briefTable1Params.put("taskCount", wmTasks.size());
            briefTable1Params.put("taskSum", wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum));
            briefTable1List.add(briefTable1Params);
        });
        Map<String, Object> briefTable1TotalParams = new HashMap<>();
        briefTable1TotalParams.put("taskTypeName", "合计");
        briefTable1TotalParams.put("taskCount", taskList.size());
        briefTable1TotalParams.put("taskSum", taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum));
        briefTable1List.add(briefTable1TotalParams);
        excelMap.put("briefTable", briefTable1List);

        // WmTaskRule wmTaskRule = wmTaskRuleService.getById(wmScorePackage.getTaskRuleId());
        String entDate = wmScorePackage.getBelongDate() + "-31";
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            format.parse(entDate);
        } catch (Exception ex) {
            entDate = wmScorePackage.getBelongDate() + "-30";
        }
        // excelMap.put("service_date", wmTaskRule.getRuleMonth() + "-01 至 " + entDate);

        List<Map<String, Object>> wmTaskZd = wmTaskService.getTaskListByIdsZd(wmTask);
        String minDateStr = null;
        String maxDateStr = null;
        // 服务周期取 积分包领取审核通过时间~管理员审核积分包时最后审核通过的时间
        Map<String, String> serviceTimeRangeMap = wmScorePackageStatusService.getServiceTimeRange(wmScorePackage);
        if (null != wmTaskZd) {
            if (wmTaskZd.size() > 0) {
                Map<String, Object> stringMap = wmTaskZd.get(0);
                if (null == stringMap) {
                    return R.failed("未找到审核通过的任务");
                }
                excelMap.put("service_date", serviceTimeRangeMap.get("service_start_date") + "至" + serviceTimeRangeMap.get("service_end_date"));
//                excelMap.put("service_date", stringMap.get("mindate") + "至" + stringMap.get("maxdate"));

                try {
                    minDateStr = serviceTimeRangeMap.get("service_start_date").substring(5, 7);
                    maxDateStr = serviceTimeRangeMap.get("service_end_date").substring(5, 7);

                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }

        if (null != applydrugEnt) {
            excelMap.put("report_ent", applydrugEnt.getName());
        } else {
            excelMap.put("report_ent", "");
        }


        String ruleDate = wmScorePackage.getBelongDate();

        excelMap.put("rule_year", ruleDate.substring(0, 4));

        String monthStr = ruleDate.substring(5, 7);
        if (null != minDateStr && null != maxDateStr) {
            if (minDateStr.equals(maxDateStr)) {
                monthStr = minDateStr;
            } else {
                // 跨年情况判断
                if (!ruleDate.substring(0, 4).equals(serviceTimeRangeMap.get("service_end_date").substring(0, 4))) {
                    monthStr = String.format("%s月至%s年%s",
                            minDateStr, serviceTimeRangeMap.get("service_end_date").substring(0, 4), maxDateStr);
                } else {
                    monthStr = minDateStr + "月至" + maxDateStr;
                }
            }
        } else if (null != minDateStr) {
            monthStr = minDateStr;
        } else if (null != maxDateStr) {
            monthStr = maxDateStr;
        }

        excelMap.put("rule_month", monthStr);

        // 报告日期
        Map<String, Timestamp> maxMaxTime = wmTaskService.getMaxCreateTime(wmTask);
        System.out.println(maxMaxTime);
        LocalDateTime MaxTime = maxMaxTime.get("MaxTime").toLocalDateTime();
        excelMap.put("year", MaxTime.getYear() + "");
        Integer month = MaxTime.getMonthValue();
        if (month < 10) {
            excelMap.put("month", "0" + MaxTime.getMonthValue() + "");
        } else {
            excelMap.put("month", MaxTime.getMonthValue() + "");
        }
        Integer day = MaxTime.getDayOfMonth();
        if (day < 10) {
            excelMap.put("day", "0" + MaxTime.getDayOfMonth() + "");
        } else {
            excelMap.put("day", MaxTime.getDayOfMonth() + "");
        }

        excelMap.put("tMoney", totalMoney);
        excelMap.put("scoreId", wmScorePackage.getScorePackageName());

        List<Map<String, Object>> infoList = exportZbInfo(wmTaskparam);
        if (null != infoList && infoList.size() > 0) {
            excelMap.put("infoList", infoList);
        } else {
            return R.failed("未查询到审核通过的任务");
        }
        // 区分全职兼职学术推广员模版
        excelMap.put("userType", wmScorePackage.getTypeid());

        File f = null;
        if (wmScorePackage.getPackageType1().equals("1")) {// 定额
            f = ExportReport.main_bulidTemplate1(excelMap, 1, upmsConfig.getBasePath());
        } else {// 定量
            f = ExportReport.main_bulidTemplate2(excelMap, 1, upmsConfig.getBasePath());
        }

        FileInputStream input = new FileInputStream(f);

        MultipartFile multipartFile = new MockMultipartFile("学术推广服务报告(" + packageId + ")", "学术推广服务报告(" + packageId + ").doc", "text/plain", IOUtils
                .toByteArray(input));
        return sysFileService.uploadFileAllUrlRealName(multipartFile, username);
    }


    @Override
    public R<?> newExportNba(WmScorePackage wmScorePackage, Integer deptId, String username) throws IOException {

        wmScorePackage = wmScorePackageService.getById(wmScorePackage.getId());

        HashSet<String> idSet = new HashSet<>();
        idSet.add(wmScorePackage.getId());
        int size = idSet.size();
        do {
            Set<String> collect = wmScorePackageService.list(Wrappers.<WmScorePackage>lambdaQuery().in(WmScorePackage::getRelationScoreId, idSet))
                    .stream()
                    .map(WmScorePackage::getId)
                    .collect(Collectors.toSet());
            idSet.addAll(collect);
            size += collect.size();
        } while (size != idSet.size());

        List<WmTask> taskList = wmTaskMapper.selectList(Wrappers.<WmTask>lambdaQuery().in(WmTask::getScorePackageId, idSet)
                .eq(WmTask::getTaskStatus, 3)
                .and(item -> item.in(WmTask::getReportDrugApprovalStatus, Arrays.asList(5, 6)).eq(WmTask::getReportDrugApprovalOpinion, 1)
                        .or(item2 -> item2.in(WmTask::getReportOneApprovalStatus, Arrays.asList(5, 6)).eq(WmTask::getReportOneApprovalOpinion, 1))
                        .or(item3 -> item3.in(WmTask::getReportSecondApprovalStatus, Arrays.asList(5, 6)).eq(WmTask::getReportSecondApprovalOpinion, 1))
                ));

        if (CollUtil.isEmpty(taskList)) {
            return R.failed("ERROR_未查询到该报告关联的审批通过的任务");
        }

        // 更新任务表审批意见
        WmTask wmTask = new WmTask();
        wmTask.setDelFlag(CommonConstants.STATUS_NORMAL);
        wmTask.setEnableFlag(CommonConstants.STATUS_NORMAL);
        String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining(","));
        wmTask.setScorePackageId(wmScorePackage.getId());
        wmTask.setId(ids);
        return exportNba(wmTask, wmScorePackage, deptId, username);
    }

    @Override
    public R newExportNbaHistory(WmScorePackage wmScorePackage, Integer deptId, String username) throws IOException {
        wmScorePackage = wmScorePackageService.getById(wmScorePackage.getId());
        if (wmScorePackage == null) {
            return R.failed("未查询到积分包信息");
        }
        LambdaQueryWrapper<WmTask> queryWrapper = new LambdaQueryWrapper();
        queryWrapper.eq(WmTask::getScorePackageDrugId, wmScorePackage.getId());
        queryWrapper.eq(WmTask::getTaskStatus, "3");
        queryWrapper.eq(WmTask::getDelFlag, "0");
        queryWrapper.eq(WmTask::getEnableFlag, "0");
        queryWrapper.eq(WmTask::getRealFlag, "0");
        List<WmTask> taskList = wmTaskService.list(queryWrapper);
        // List<WmTask> taskList = wmTaskMapper.selectListByWmTask(wmScorePackage);
        // 更新任务表审批意见
        WmTask wmTask = new WmTask();
        wmTask.setDelFlag("0");
        wmTask.setEnableFlag("0");

        if (null == taskList || taskList.size() <= 0) {
            return R.failed("ERROR_未查询到该报告关联的审批通过的任务");
        }
        String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining(","));
        wmTask.setScorePackageId(wmScorePackage.getId());
        // wmTask.setId(ids);
        wmTask.setId("(select id from wm_task where del_flag=0 and real_flag=0 and enable_flag=0 and task_status=3  and score_package_drug_id=" + wmScorePackage.getId() + ")");

        return exportNba(wmTask, wmScorePackage, deptId, username);// 报告汇总
    }


    /**
     * 结算报告--导出汇总报告--学术推广服务报告
     *
     * @param wmTask         任务
     * @param wmScorePackage 积分包
     * @return 导出信息
     * @throws IOException 异常
     */
    private R<?> exportNba(WmTask wmTask, WmScorePackage wmScorePackage, Integer deptId, String username) throws IOException {

        WmTask wmTaskparam = wmTask;
        String packageId = wmTask.getScorePackageId();  // 获取积分包ID
        if (null == packageId) {
            return R.failed("积分包ID不能为空");
        }

        List<String> list = new ArrayList<>();
        list.add(packageId);

        wmTask.setExtIds(list);

        Map<String, Object> excelMap = new HashMap<>();

        wmScorePackage = wmScorePackageService.getById(packageId);
        if (null == wmScorePackage) {
            return R.failed("获取积分包失败");
        }

        excelMap.put("totalscorePackageScore", wmScorePackage.getScore());
        if (wmScorePackage.getPackageType1().equals("2") || wmScorePackage.getPackageType1().equals("3")) {
            excelMap.put("totalscorePackageTaskNum", wmScorePackage.getTaskNum() * wmScorePackage.getUserNum());// 积分包值任务个数
        }

        List<Map<String, Object>> wmTaskList = wmTaskService.getTaskListByIdsOld(wmTask);

        // mxTable
        List<XlsTask> xlsTaskList = new ArrayList<>();
        int totalMoney = 0;

        // 当前登陆人的机构
        SysDept sysDept = sysDeptService.getById(deptId);

        Map<String, Object> userMap = new HashMap<>();

        // 委托公司（结算企业）
        SysDept drugEnt = sysDeptService.getById(wmScorePackage.getSendPackageDeptId());

        // 报告公司（申请结算企业）
        SysDept applydrugEnt = sysDeptService.getById(wmScorePackage.getDeptId());

        for (Map<String, Object> wmT : wmTaskList) {
            totalMoney += Integer.parseInt(wmT.get("score") + "");
            XlsTask xlsTask = new XlsTask();
            xlsTask.setTaskNum(1);
            xlsTask.setCreateTime(wmT.get("create_time") + "");

            xlsTask.setDeptName(sysDept.getName());

            // level=2是药企
            if (sysDept.getLevel() == 2) {
                xlsTask.setAgentDeptName(applydrugEnt.getName());
            }

            xlsTask.setDrugEntName(drugEnt.getName());


            xlsTask.setScore(Integer.parseInt(wmT.get("score") + ""));

            xlsTask.setUsername(wmT.get("realname") + "");
            xlsTask.setTaskType(wmT.get("base_id") + "");

            xlsTaskList.add(xlsTask);

            String buildKey = wmT.get("realname") + "_" + drugEnt.getName() + "_" + sysDept.getName();
            // 人员表格
            if (userMap.containsKey(buildKey)) {
                try {
                    userMap.put(buildKey, Integer.parseInt(wmT.get("score") + "") + Integer.parseInt(userMap.get(buildKey) + ""));
                    userMap.put(buildKey + "_total", Integer.parseInt(wmT.get("score_count") + "") + Integer.parseInt(userMap
                            .get(buildKey + "_total") + ""));

                } catch (Exception ignored) {
                }
            } else {
                userMap.put(buildKey, Integer.parseInt(wmT.get("score") + ""));
                userMap.put(buildKey + "_total", Integer.parseInt(wmT.get("score_count") + ""));
            }
        }

        // 学术推广服务任务明细表格
        List<WmTask> wmTaskDetialList = wmTaskService.getTaskListByIdsReturnWmtaskInfo(wmTask);
        List<Map<String, Object>> wmTaskDetialMap = new ArrayList<>();

        if (CollUtil.isNotEmpty(wmTaskDetialList)) {
            DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            for (WmTask wmTask1 : wmTaskDetialList) {
                Map<String, Object> map = new HashMap<>(6);
                WmTaskType wmTaskTypeTemp1 = wmTaskTypeService.getById(wmTask1.getTaskTypeId());
                map.put("taskType", wmTaskTypeTemp1.getTaskTypeName());
                String strDate2 = dtf2.format(wmTask1.getCreateTime());
                map.put("taskTime", strDate2);
                map.put("taskUser", sysUserService.getByIdIgnore(wmTask1.getTaskUserId()).getRealname());
                map.put("taskTypeId", wmTask1.getTaskTypeId());
                map.put("score", wmTask1.getScore());
                map.put("taskNum", 1);

                wmTaskDetialMap.add(map);
            }

        }

        // 学术推广服务任务明细表格
        excelMap.put("rwmxTable", wmTaskDetialMap);

        if (totalMoney > wmScorePackage.getScore()) {
            totalMoney = wmScorePackage.getScore();
        }

        // 人员表格
        excelMap.put("ryTable", userMap);
        excelMap.put("tMoney", totalMoney);

        if (CollUtil.isNotEmpty(xlsTaskList)) {
            excelMap.put("wt_ent", xlsTaskList.get(0).getDrugEntName());// 委托公司
            excelMap.put("drugEnt", xlsTaskList.get(0).getDrugEntName());// 服务药企
            excelMap.put("mxTable", xlsTaskList);
        }

        List<Map<String, Object>> wmTaskCount = wmTaskService.getTaskListByIdsCount(wmTask);
        if (CollUtil.isNotEmpty(wmTaskCount)) {
            int total = 0;
            int sum = 0;
            for (Map<String, Object> map : wmTaskCount) {
                if (null != map.get("taskCount")) {
                    total += Integer.parseInt(map.get("taskCount") + "");
                }

                if (null != map.get("taskSum")) {
                    sum += Integer.parseInt(map.get("taskSum") + "");
                }

            }
            Map<String, Object> totalMap = new HashMap<>();
            totalMap.put("taskTypeName", "合计");
            totalMap.put("taskCount", total);
            totalMap.put("taskSum", sum);
            wmTaskCount.add(totalMap);

            excelMap.put("totalSubmitScore", sum);
            excelMap.put("totalSubmitTaskNum", total);

            // 学术推广服务整体概况表格
            excelMap.put("ztTable", wmTaskCount);
        }

        // 任务小类及对应的任务个数和积分值
        List<WmTask> taskList = wmTaskMapper.selectBatchIds(Arrays.asList(wmTask.getId().split(",")));

        // 查询taskType
        List<WmTaskType> taskTypeList = wmTaskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
                .eq(WmTaskType::getTaskTypeLevel, "1")
                .eq(WmTaskType::getDelFlag, CommonConstants.STATUS_NORMAL)
        );

        // taskType通过id分组（key为parentId，value为taskTypeId的set）
        Map<String, String> taskTypeIdMap = taskTypeList.stream().collect(Collectors.toMap(WmTaskType::getId, WmTaskType::getTaskTypeName));

        // task通过taskTypeId分组
        Map<String, List<WmTask>> taskTypeMap = taskList.stream().collect(Collectors.groupingBy(WmTask::getTaskTypeId, Collectors.toList()));

        // 封装brief table1数据（统计每个小类任务的积分、任务数）
        List<Map<String, Object>> briefTable1List = new ArrayList<>(taskTypeMap.size());
        taskTypeMap.forEach((taskTypeId, wmTasks) -> {
            Map<String, Object> briefTable1Params = new HashMap<>(3);
            briefTable1Params.put("taskTypeName", taskTypeIdMap.get(taskTypeId));
            briefTable1Params.put("taskCount", wmTasks.size());
            briefTable1Params.put("taskSum", wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum));
            briefTable1List.add(briefTable1Params);
        });

        Map<String, Object> briefTable1TotalParams = new HashMap<>(3);
        briefTable1TotalParams.put("taskTypeName", "合计");
        briefTable1TotalParams.put("taskCount", taskList.size());
        briefTable1TotalParams.put("taskSum", taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum));
        briefTable1List.add(briefTable1TotalParams);
        excelMap.put("briefTable", briefTable1List);

        String entDate = wmScorePackage.getBelongDate() + "-31";

        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            format.parse(entDate);
        } catch (Exception ignored) {
        }

        List<Map<String, Object>> wmTaskZd = wmTaskService.getTaskListByIdsZd(wmTask);
        String minDateStr = null;
        String maxDateStr = null;
        // 服务周期取 积分包领取审核通过时间~管理员审核积分包时最后审核通过的时间
        Map<String, String> serviceTimeRangeMap = wmScorePackageStatusService.getServiceTimeRange(wmScorePackage);
        if (CollUtil.isNotEmpty(wmTaskZd)) {
            Map<String, Object> stringMap = wmTaskZd.get(0);
            if (null == stringMap) {
                return R.failed("未找到审核通过的任务");
            }
            excelMap.put("service_date", serviceTimeRangeMap.get("service_start_date") + "至" + serviceTimeRangeMap.get("service_end_date"));

            try {
                minDateStr = serviceTimeRangeMap.get("service_start_date").substring(5, 7);
                maxDateStr = serviceTimeRangeMap.get("service_end_date").substring(5, 7);

            } catch (Exception ignored) {
            }
        }

        excelMap.put("report_ent", null != applydrugEnt ? applydrugEnt.getName() : "");

        String ruleDate = wmScorePackage.getBelongDate();
        excelMap.put("rule_year", ruleDate.substring(0, 4));

        String monthStr = ruleDate.substring(5, 7);
        if (null != minDateStr && null != maxDateStr) {
            if (minDateStr.equals(maxDateStr)) {
                monthStr = minDateStr;
            } else {
                // 跨年情况判断
                if (!ruleDate.substring(0, 4).equals(serviceTimeRangeMap.get("service_end_date").substring(0, 4))) {
                    monthStr = String.format("%s月至%s年%s",
                            minDateStr, serviceTimeRangeMap.get("service_end_date").substring(0, 4), maxDateStr);
                } else {
                    monthStr = minDateStr + "月至" + maxDateStr;
                }
            }
        } else if (null != minDateStr) {
            monthStr = minDateStr;
        }

        excelMap.put("rule_month", monthStr);

        // 报告日期
        Map<String, Timestamp> maxMaxTime = wmTaskService.getMaxCreateTime(wmTask);
        System.out.println(maxMaxTime);
        LocalDateTime MaxTime = maxMaxTime.get("MaxTime").toLocalDateTime();
        excelMap.put("year", MaxTime.getYear() + "");
        excelMap.put("month", MaxTime.getMonthValue() < 10 ? "0" + MaxTime.getMonthValue() : MaxTime.getMonthValue() + "");
        excelMap.put("day", MaxTime.getDayOfMonth() < 10 ? "0" + MaxTime.getDayOfMonth() : MaxTime.getDayOfMonth() + "");
        excelMap.put("tMoney", totalMoney);
        excelMap.put("scoreId", wmScorePackage.getScorePackageName());

        List<Map<String, Object>> infoList = exportZbInfo(wmTaskparam);

        if (CollUtil.isEmpty(infoList)) {
            return R.failed("未查询到审核通过的任务");
        }

        excelMap.put("infoList", infoList);
        // 区分全职兼职学术推广员模版
        excelMap.put("userType", wmScorePackage.getTypeid());

        // 1是定额，否则是定量
        File f = "1".equals(wmScorePackage.getPackageType1()) ? ExportReport.main_bulidTemplate1(excelMap, 1, upmsConfig.getBasePath()) : ExportReport.main_bulidTemplate2(excelMap, 1, upmsConfig.getBasePath());
        FileInputStream input = new FileInputStream(f);
        String fileName = String.format("学术推广服务报告(%s)-%s.%s", packageId, DateTimeFormatter.ofPattern(DatePattern.PURE_DATETIME_PATTERN).format(LocalDateTime.now()), FileUtil.extName(f.getName()));
        R<?> r = sysFileService.uploadFileAllUrlRealName(fileName, input);
        input.close();
        return r;
    }


    /**
     * 个人承接按积分制
     *
     * @param wmTask   参数
     * @param username 操作人
     * @return 上传参数
     * @throws IOException 异常
     */
    @Override
    public R<Map<String, String>> newExportZb(WmTask wmTask, String username) throws IOException {
        String packageId = wmTask.getScorePackageDrugId();
        if (StrUtil.isBlank(packageId)) {
            return R.failed("积分包ID不能为空");
        }
        LambdaQueryWrapper<WmTask> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(WmTask::getScorePackageId, Arrays.asList(packageId.split(StrUtil.COMMA)));
        queryWrapper.eq(WmTask::getTaskStatus, "3");
        queryWrapper.eq(WmTask::getDelFlag, CommonConstants.STATUS_NORMAL);
        queryWrapper.eq(WmTask::getEnableFlag, CommonConstants.STATUS_NORMAL);
        queryWrapper.eq(WmTask::getRealFlag, CommonConstants.STATUS_NORMAL);
        List<WmTask> taskList = wmTaskService.list(queryWrapper);
        if (taskList.isEmpty()) {
            return R.failed("未查询到审核通过的任务");
        }
        String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining(StrUtil.COMMA));
        wmTask.setScorePackageId(packageId);
        wmTask.setId(ids);
        return exportZb(wmTask, username);
    }

    /**
     * 个人报告导出--众包代表业绩报告
     *
     * @param wmTask 参数
     * @return 上传信息
     * @throws IOException 异常
     */
    private R<Map<String, String>> exportZb(WmTask wmTask, String username) throws IOException {

        WmTask wmTaskparam = wmTask;
        String packageId = wmTask.getScorePackageId();  // 获取积分包ID
        if (null == packageId) {
            packageId = wmTask.getScorePackageDrugId();
        }

        if (null == packageId) {
            return R.failed("积分包ID不能为空");
        }

        WmScorePackage wmScorePackage1 = wmScorePackageService.getById(packageId);

        Map<String, Object> excelMap = exportZbCommon(wmTask);
        if (null == excelMap) {
            return R.failed("未查询到审核通过的任务");
        }

        List<Map<String, Object>> infoList = exportZbInfo(wmTaskparam);
        if (CollUtil.isNotEmpty(infoList)) {
            excelMap.put("infoList", infoList);
        }
        // 区分全职兼职学术推广员模版
        excelMap.put("userType", wmScorePackage1.getTypeid());
        List<String> extIds = wmTask.getExtIds();
        List<WmScorePackage> wmScorePackages = wmScorePackageService.listByIds(extIds);
        String scoreIds = wmScorePackages.stream().map(WmScorePackage::getScorePackageName).collect(Collectors.joining(","));
        excelMap.put("scoreId", StrUtil.isBlank(scoreIds) ? wmScorePackage1.getScorePackageName() : scoreIds);

        // 1是定额 2是定量
        File f = "1".equals(wmScorePackage1.getPackageType1()) ? ExportReport.main_bulidTemplate1(excelMap, 2, upmsConfig.getBasePath()) : ExportReport.main_bulidTemplate2(excelMap, 2, upmsConfig.getBasePath());

        FileInputStream input = new FileInputStream(f);

        String fileName = "(" + packageId + ")";
        if (StrUtil.equals(wmScorePackage1.getTypeid(), "3")) {
            fileName = "全职学术信息沟通专员业绩报告" + fileName;
        } else if (StrUtil.equals(wmScorePackage1.getTypeid(), "5")) {
            fileName = "招商经理业绩报告" + fileName;
        } else if (StrUtil.equals(wmScorePackage1.getTypeid(), "4")) {
            fileName = "兼职学术信息沟通专员业绩报告" + fileName;
        }
        MultipartFile multipartFile = new MockMultipartFile(fileName, "兼职学术推广员业绩报告(" + packageId + ").doc", "text/plain", IOUtils.toByteArray(input));
        return sysFileService.uploadFileAllUrlRealName(multipartFile, username);

    }

    /*合并导出--汇总
     */
    public Map<String, Object> exportZbCommon(WmTask wmTask) {
        String packageId = wmTask.getScorePackageId();  // 获取积分包ID
        if (null == packageId) {
            packageId = wmTask.getScorePackageDrugId();
        }

        Map<String, Object> excelMap = new HashMap<>();
        WmScorePackage wmScorePackage1 = wmScorePackageService.getById(packageId);
        WmDaDrugEnt wmDaDrugEnt = wmDaDrugEntService.getById(wmScorePackage1.getDrugEntId());
        if (wmDaDrugEnt == null) {
            wmDaDrugEnt = new WmDaDrugEnt();
            wmDaDrugEnt.setEntname("无");
        }

        List<String> strs = new ArrayList<>();
        if (packageId.contains(",")) {
            strs.addAll(Arrays.asList(packageId.split(",")));
        } else {
            strs.add(packageId);
        }
        wmTask.setExtIds(strs);


        List<Map<String, Object>> wmTaskList = wmTaskService.getTaskListByIdsOld(wmTask);
        if (wmScorePackage1.getPackageType2().equals("2")) {
            wmTaskList = wmTaskService.getTaskListByIdsJs(wmTask);
        }
        if (wmTaskList.size() > 0) {
            List<XlsTask> xlsTaskList = new ArrayList<>();
            Map<String, Object> userMap = new HashMap<>();
            for (Map<String, Object> wmt : wmTaskList) {
                XlsTask xlsTask = new XlsTask();
                xlsTask.setTaskNum(Integer.valueOf(wmt.get("score_count") + ""));
                xlsTask.setTaskType(wmt.get("base_id") + "");
                xlsTask.setUsername(wmt.get("realname") + "");
                xlsTask.setScore(Integer.parseInt((wmt.get("score") == null ? "0" : wmt.get("score")) + ""));
                xlsTask.setDrugEntName(wmDaDrugEnt.getEntname());
                xlsTask.setDeptName(wmt.get("dept_name") + "");
                xlsTask.setCreateTime(wmt.get("create_time") + "");
                xlsTaskList.add(xlsTask);

                String buildKey = wmt.get("realname") + "_" + wmDaDrugEnt.getEntname() + "_" + wmt.get("dept_name") + "";
                if (userMap.containsKey(buildKey)) {
                    try {
                        userMap.put(buildKey, Integer.parseInt(wmt.get("score") == null ? "0" : wmt.get("score") + "") + Integer
                                .parseInt(userMap.get(buildKey) == null ? "0" : userMap.get(buildKey) + ""));
                        userMap.put(buildKey + "_total", Integer.parseInt(wmt.get("score_count") + "") + Integer.parseInt(userMap
                                .get(buildKey + "_total") + ""));

                    } catch (Exception ex) {
                    }
                } else {
                    userMap.put(buildKey, wmt.get("score"));
                    userMap.put(buildKey + "_total", Integer.parseInt(wmt.get("score_count") + ""));
                }
            }


            List<WmTask> wmTaskDetialList = wmTaskService.getTaskListByIdsReturnWmtaskInfo(wmTask);
            List<Map<String, Object>> wmTaskDetialMap = new ArrayList<>();

            List<String> listPackageIds = new ArrayList<>();

            if (wmTaskDetialList.size() > 0) {
                for (WmTask wmTask1 :
                        wmTaskDetialList) {
                    Map<String, Object> map = new HashMap<>();
                    WmTaskType wmTaskTypeTemp1 = wmTaskTypeService.getById(wmTask1.getTaskTypeId());
                    map.put("taskType", wmTaskTypeTemp1.getTaskTypeName());
                    DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                    String strDate2 = dtf2.format(wmTask1.getCreateTime());
                    map.put("taskTime", strDate2);
                    if (wmScorePackage1.getPackageType2().equals("2")) {
                        map.put("taskUser", sysUserService.getByIdIgnore(wmTask1.getTaskUserId()).getRealname());
                    } else {
                        map.put("taskUser", sysUserService.getByIdIgnore(wmTask1.getTaskUserId()).getRealname());
                    }

                    map.put("taskTypeId", wmTask1.getTaskTypeId());
                    map.put("score", wmTask1.getScore());
                    map.put("taskNum", 1);
                    wmTaskDetialMap.add(map);

                    if (listPackageIds.size() == 0) {
                        listPackageIds.add(wmTask1.getScorePackageId());
                    } else {
                        boolean isInscore = true;
                        for (String str : listPackageIds) {
                            if (str.equals(wmTask1.getScorePackageId())) {
                                isInscore = false;
                            }
                        }

                        if (isInscore) {
                            listPackageIds.add(wmTask1.getScorePackageId());
                        }
                    }
                }
            }

            if (listPackageIds.size() > 0) {
                List<WmScorePackage> wmScorePackageList = wmScorePackageService.listByIds(listPackageIds);
                Integer totalScorePackage = 0;
                BigDecimal settleAmount = new BigDecimal("0");
                if (wmScorePackageList.size() > 0) {
                    for (WmScorePackage wmScorePackage : wmScorePackageList) {
                        totalScorePackage += wmScorePackage.getScore();
                        if (wmScorePackage.getSettleStatus() != null && wmScorePackage.getSettleStatus() == 1) {
                            settleAmount = settleAmount.add(wmScorePackage.getSettleAmount());
                        }
                    }
                }
                excelMap.put("settleAmount", "0".equals(settleAmount.toString()) ? " " : settleAmount);
                excelMap.put("totalscorePackageScore", totalScorePackage);
                if (wmScorePackage1.getPackageType1().equals("2") || wmScorePackage1.getPackageType1().equals("3")) {
                    if (!wmScorePackage1.getPackageType2().equals("2")) {
                        excelMap.put("totalscorePackageTaskNum", wmScorePackage1.getTaskNum() * wmScorePackage1.getUserNum());// 积分包值任务个数
                    }
                }

                List<WmTask> tasks = wmTaskService.list(Wrappers.<WmTask>lambdaQuery()
                        .in(WmTask::getScorePackageId, listPackageIds));

                if (CollUtil.isNotEmpty(tasks)) {
                    int sum = tasks.stream()
//                            .filter(t -> "3".equals( t.getTaskStatus()) && WmTask.TaskSettleStatusEnum.SETTLED.getCode().equals(t.getTaskSettleStatus()))
                            .filter(t -> "3".equals(t.getTaskStatus()))
                            .mapToInt(WmTask::getScore).sum();
                    excelMap.put("submitAmount", sum);
                }
            }

            excelMap.put("rwmxTable", wmTaskDetialMap);
            excelMap.put("mxTable", xlsTaskList);
            excelMap.put("ryTable", userMap);

        }

        Map<String, Timestamp> maxMaxTime = wmTaskService.getMaxCreateTime(wmTask);
        System.out.println(maxMaxTime);
        LocalDateTime MaxTime = maxMaxTime.get("MaxTime").toLocalDateTime();
        excelMap.put("year", MaxTime.getYear() + "");
        Integer month = MaxTime.getMonthValue();
        if (month < 10) {
            excelMap.put("month", "0" + MaxTime.getMonthValue() + "");
        } else {
            excelMap.put("month", MaxTime.getMonthValue() + "");
        }
        Integer day = MaxTime.getDayOfMonth();
        if (day < 10) {
            excelMap.put("day", "0" + MaxTime.getDayOfMonth() + "");
        } else {
            excelMap.put("day", MaxTime.getDayOfMonth() + "");
        }

        List<Map<String, Object>> wmTaskCount = wmTaskService.getTaskListByIdsCount(wmTask);
        if (wmTaskCount.size() > 0) {
            Integer total = 0;
            Integer sum = 0;
            for (Map<String, Object> map :
                    wmTaskCount) {
                if (null != map.get("taskCount")) {
                    total += Integer.parseInt(map.get("taskCount") + "");
                }

                if (null != map.get("taskSum")) {
                    sum += Integer.parseInt(map.get("taskSum") + "");
                }

            }
            Map<String, Object> totalMap = new HashMap<>();
            totalMap.put("taskTypeName", "合计");
            totalMap.put("taskCount", total);
            totalMap.put("taskSum", sum);
            wmTaskCount.add(totalMap);

            excelMap.put("totalSubmitScore", sum);
            excelMap.put("totalSubmitTaskNum", total);
            excelMap.put("ztTable", wmTaskCount);
        }

        // 任务小类及对应的任务个数和积分值
        List<WmTask> taskList = wmTaskMapper.selectBatchIds(Arrays.asList(wmTask.getId().split(",")));

        // 查询taskType
        List<WmTaskType> taskTypeList = wmTaskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
                        .eq(WmTaskType::getTaskTypeLevel, "1")
                        .eq(WmTaskType::getDelFlag, "0")
//                .eq(WmTaskType::getEnableFlag, "0")
        );
        // taskType通过id分组（key为parentId，value为taskTypeId的set）
        Map<String, String> taskTypeIdMap = taskTypeList.stream()
                .collect(Collectors.toMap(WmTaskType::getId, WmTaskType::getTaskTypeName));

        // task通过taskTypeId分组
        Map<String, List<WmTask>> taskTypeMap = taskList.stream()
                .collect(Collectors.groupingBy(WmTask::getTaskTypeId, Collectors.toList()));

        // 封装brief table1数据（统计每个小类任务的积分、任务数）
        List<Map<String, Object>> briefTable1List = new ArrayList<>();
        taskTypeMap.forEach((taskTypeId, wmTasks) -> {
            Map<String, Object> briefTable1Params = new HashMap<>();
            briefTable1Params.put("taskTypeName", taskTypeIdMap.get(taskTypeId));
            briefTable1Params.put("taskCount", wmTasks.size());
            briefTable1Params.put("taskSum", wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum));
            briefTable1List.add(briefTable1Params);
        });
        Map<String, Object> briefTable1TotalParams = new HashMap<>();
        briefTable1TotalParams.put("taskTypeName", "合计");
        briefTable1TotalParams.put("taskCount", taskList.size());
        briefTable1TotalParams.put("taskSum", taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum));
        briefTable1List.add(briefTable1TotalParams);
        excelMap.put("briefTable", briefTable1List);

        SysDept paramDept = new SysDept();
        paramDept.setDeptId(Integer.valueOf(wmScorePackage1.getSendPackageDeptId()));
        SysDept wmDaAgent = sysDeptService.selectDeptById(paramDept);
        excelMap.put("wt_ent", wmDaAgent.getName());

        excelMap.put("drugEnt", "");


        List<Map<String, Object>> wmTaskZd = wmTaskService.getTaskListByIdsZd(wmTask);
        if (wmScorePackage1.getPackageType2().equals("2")) {
            wmTaskZd = wmTaskService.getTaskListByIdsZdJs(wmTask);
        }
        String minDateStr = null;
        String maxDateStr = null;
        if (null != wmTaskZd) {
            if (wmTaskZd.size() > 0) {
                Map<String, Object> stringMap = wmTaskZd.get(0);
                if (null == stringMap) {
                    return null;
                }
                // 服务周期取 积分包领取审核通过时间~管理员审核积分包时最后审核通过的时间
                Map<String, String> serviceTimeRangeMap = wmScorePackageStatusService.getServiceTimeRange(wmScorePackage1);
                excelMap.put("service_date", serviceTimeRangeMap.get("service_start_date") + "至" + serviceTimeRangeMap.get("service_end_date"));
                try {
                    minDateStr = serviceTimeRangeMap.get("service_start_date").substring(5, 7);
                    maxDateStr = serviceTimeRangeMap.get("service_end_date").substring(5, 7);

                } catch (Exception ignored) {
                }
                excelMap.put("tMoney", stringMap.get("sumscore") + "");
                Object realname = stringMap.get("realname");
                excelMap.put("report_users", Objects.nonNull(realname) ? realname.toString() : "");


                String ruleDate = wmScorePackage1.getBelongDate();

                excelMap.put("rule_year", ruleDate.substring(0, 4));

                String monthStr = ruleDate.substring(5, 7);
                if (null != minDateStr && null != maxDateStr) {
                    if (minDateStr.equals(maxDateStr)) {
                        monthStr = minDateStr;
                    } else {
                        // 跨年情况判断
                        if (!ruleDate.substring(0, 4).equals(serviceTimeRangeMap.get("service_end_date").substring(0, 4))) {
                            monthStr = String.format("%s月至%s年%s",
                                    minDateStr, serviceTimeRangeMap.get("service_end_date").substring(0, 4), maxDateStr);
                        } else {
                            monthStr = minDateStr + "月至" + maxDateStr;
                        }
                    }
                } else if (null != minDateStr) {
                    monthStr = minDateStr;
                } else if (null != maxDateStr) {
                    monthStr = maxDateStr;
                }

                excelMap.put("rule_month", monthStr);


            }
        }
        return excelMap;
    }

    /**
     * 个人报告导出--报告详情
     *
     * @param wmTask 参数
     * @return 导出信息
     * @throws IOException 异常
     */
    @Override
    public List<Map<String, Object>> exportZbInfo(WmTask wmTask) throws IOException {
        String packageId = wmTask.getScorePackageId();  // 获取积分包ID
        if (null == packageId) {
            packageId = wmTask.getScorePackageDrugId();
        }

        if (null == packageId) {
            return null;
        }

        wmTask.setScorePackageId(null);
        wmTask.setScorePackageDrugId(null);
        wmTask.setScorePackageId(packageId);

        List<Map<String, Object>> infoList = new ArrayList<>();

        List<String> strs = new ArrayList<>();
        if (packageId.contains(",")) {
            strs.addAll(Arrays.asList(packageId.split(",")));
        } else {
            strs.add(packageId);
        }
        wmTask.setExtIds(strs);

        List<WmTask> wmTaskList = wmTaskService.getTaskListByIdsReturnWmtaskInfo(wmTask);
        if (CollUtil.isNotEmpty(wmTaskList)) {
            // 从字典项查询支持的任务类型
            List<SysDictItem> scoreTaskTypeDictList = sysDictItemMapper.selectList(Wrappers.<SysDictItem>lambdaQuery()
                    .eq(SysDictItem::getType, "report_export_task_type")
                    .eq(SysDictItem::getDelFlag, CommonConstants.STATUS_NORMAL));
            Set<String> supportedTaskTypeIds = scoreTaskTypeDictList.stream().map(SysDictItem::getValue).collect(Collectors.toSet());

            int num = 1;

            List<WmTask> emptyImgTasks = wmTaskList.stream()
                    .filter(t -> StrUtil.isBlank(t.getTaskInfoImg()) && supportedTaskTypeIds.contains(t.getTaskTypeId()) && !"801".equals(t.getTaskTypeId()))
                    .collect(Collectors.toList());

            log.warn("总任务数:{},空图片任务数:{}", wmTaskList.size(), emptyImgTasks.size());

            Map<String, BufferedImage> imgs = new HashMap<>(emptyImgTasks.size());
            for (WmTask task : emptyImgTasks) {

                try {
                    Thread.sleep(100);
                } catch (InterruptedException ignored) {
                }
                String taskUrl = upmsConfig.getApiUrl() + "/h5/#/pages/task/task-detail/index?id=" + task.getId();
                log.warn("获取任务图片: [{}]:[{}]", task.getId(), taskUrl);
                BufferedImage image = Html2ImageByJsWrapper.renderHtml2Image(taskUrl, upmsConfig.getServerUrl());
                if (Objects.isNull(image)) {
                    log.error("获取图片信息失败");
                    continue;
                }
                imgs.put(task.getId(), image);
            }

            Set<Integer> uIds = wmTaskList.stream().map(WmTask::getTaskUserId).filter(StrUtil::isNotBlank).mapToInt(Integer::valueOf).boxed().collect(Collectors.toSet());
            Map<Integer, List<SysUser>> uId2uListMap = sysUserService.listByIds(uIds).stream().collect(Collectors.groupingBy(SysUser::getUserId));

            DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            for (WmTask wmTask1 : wmTaskList) {

                WmTaskType taskType = wmTaskTypeService.getById(wmTask1.getTaskTypeId());
                Map<String, Object> map = new HashMap<>();
                map.put("taskIndex", num++);
                map.put("taskType", taskType.getTaskTypeName());

                String strDate2 = dtf2.format(wmTask1.getCreateTime());
                map.put("taskTime", strDate2);
                String taskUserName;
                List<SysUser> users = uId2uListMap.get(Integer.valueOf(wmTask1.getTaskUserId()));
                if (CollUtil.isNotEmpty(users)) {
                    taskUserName = users.get(0).getRealname();
                } else {
                    taskUserName = "";
                }

                map.put("taskUser", taskUserName);
                map.put("taskTypeId", wmTask1.getTaskTypeId());
                map.put("taskNumber", wmTask1.getTaskNumber());
                map.put("imgWidth", taskType.getUpdateUser());
                map.put("imgHeight", taskType.getCreateUser());

                String imgUrl = wmTask1.getTaskInfoImg();
                map.put("taskBaseImg", StrUtil.isNotBlank(imgUrl) ? imgUrl : imgs.get(wmTask1.getId()));

                if (supportedTaskTypeIds.contains(wmTask1.getTaskTypeId())) {
                    WmTaskContent wmTaskContent = wmTaskContentService.getById(wmTask1.getTaskContentId());
                    if (null != wmTaskContent) {
                        List<String> stringList = new ArrayList<>();
                        WmTaskContentConfig queryWmTaskContentConfig = new WmTaskContentConfig();
                        if ("39".equals(wmTask1.getTaskTypeId()) || "40".equals(wmTask1.getTaskTypeId()) || "41".equals(wmTask1.getTaskTypeId())) {
                            // 39 应收账款回款-A类、40 应收账款回款-B类、41 应收账款回款-C类 这里需要统一设置为39（由于三种类型公用一个页面所以WmTaskContentConfig只保存了39的配置字段）
                            queryWmTaskContentConfig.setTaskTypeId("39");
                        } else if ("42".equals(wmTask1.getTaskTypeId()) || "43".equals(wmTask1.getTaskTypeId()) || "44".equals(wmTask1.getTaskTypeId())) {
                            // 手术跟台-一级/手术跟台-二级/手术跟台-三级任务类型 这里需要统一设置为42（由于三种类型公用一个页面所以WmTaskContentConfig只保存了42的配置字段）
                            queryWmTaskContentConfig.setTaskTypeId("42");
                        } else if ("14".equals(wmTask1.getTaskTypeId())) {
                            // 库存信息收集下的 库存盘点-中药颗粒剂、库存信息收集-首次 需要回显对应的详情字段(对于这几个任务类型这里的wmTask1的taskType实际存的都是14，所以需要转换下)
                            queryWmTaskContentConfig.setTaskTypeId(wmTaskContent.getTemp30());
                        } else if ("47".equals(wmTask1.getTaskTypeId()) || "48".equals(wmTask1.getTaskTypeId())
                                || "49".equals(wmTask1.getTaskTypeId()) || "50".equals(wmTask1.getTaskTypeId())) {
                            // 库存调研、竞品调研、培训、使用情况调查
                            queryWmTaskContentConfig.setTaskTypeId("47");
                        } else if ("51".equals(wmTask1.getTaskTypeId()) || "52".equals(wmTask1.getTaskTypeId())
                                || "53".equals(wmTask1.getTaskTypeId())) {
                            // 信用管理A/B/C
                            queryWmTaskContentConfig.setTaskTypeId("51");
                        } else if ("56".equals(wmTask1.getTaskTypeId())) {
                            // 首营管理-诊所/药店
                            queryWmTaskContentConfig.setTaskTypeId(wmTaskContent.getTemp3());
                        } else {
                            queryWmTaskContentConfig.setTaskTypeId(wmTask1.getTaskTypeId());
                        }
                        QueryWrapper<WmTaskContentConfig> queryWrapper = Wrappers.query(queryWmTaskContentConfig);
                        // 图片和签名都要导出
                        queryWrapper.lambda().in(WmTaskContentConfig::getTaskFiledType, "img", "sign");
                        List<WmTaskContentConfig> wmTaskContentConfigList = wmTaskContentConfigService.list(queryWrapper);
                        if (CollUtil.isNotEmpty(wmTaskContentConfigList)) {
                            for (WmTaskContentConfig wmt : wmTaskContentConfigList) {
                                String fileName = wmt.getTaskFiledKey();
                                String imgsStr = getFieldValueByFieldName(fileName, wmTaskContent);
                                // 图片的数据格式是这样的 /admin/sys-file/wmkj/237ea87e43cc463ba4aaa295e2a28e24.png;1,/admin/sys-file/wmkj/fd7c4cd56d6748dea18ea57c809e464c.jpg
                                List<String> imgList = StrUtil.splitTrim(imgsStr, ",").stream()
                                        .filter(StrUtil::isNotBlank)
                                        .map(s -> s.split(";")[0]).collect(Collectors.toList());
                                stringList.addAll(imgList);
                            }
                        }

                        if (CollUtil.isNotEmpty(stringList)) {
                            map.put("imgList", stringList);
                        }

                        // 企业任务类型-回显文件地址处理
                        List<WmTaskContentConfig> fileurlTaskContentConfigList =
                                wmTaskContentConfigService.list(Wrappers.<WmTaskContentConfig>lambdaQuery()
                                        .eq(WmTaskContentConfig::getTaskTypeId, wmTask1.getTaskTypeId())
                                        .eq(WmTaskContentConfig::getTaskFiledType, "fileurl"));
                        if (CollUtil.isNotEmpty(fileurlTaskContentConfigList)) {
                            List<Map<String, String>> fileUrlList = new ArrayList<>();
                            for (WmTaskContentConfig wmt : fileurlTaskContentConfigList) {
                                String fileName = wmt.getTaskFiledKey();
                                String taskFiledValue = wmt.getTaskFiledValue();
                                String fileUrlStr = getFieldValueByFieldName(fileName, wmTaskContent);

                                HashMap<String, String> fileUrlMap = new HashMap<>();
                                if (JSONUtil.isJsonArray(fileUrlStr)) {
                                    List<String> fileUrls = new ArrayList<>();
                                    JSONArray jsonArray = JSONUtil.parseArray(fileUrlStr);
                                    jsonArray.jsonIter().forEach(jsonObject -> {
                                        String url = jsonObject.getStr("url");
                                        if (!StrUtil.startWith(url, "http")) {
                                            url = upmsConfig.getClientUrl() + url;
                                        }
                                        fileUrls.add(url);
                                    });
                                    fileUrlMap.put("fileUrl",
                                            String.format("%s: %s", taskFiledValue, String.join(",", fileUrls)));
                                } else {
                                    fileUrlMap.put("fileUrl", String.format("%s: %s", taskFiledValue, fileUrlStr));
                                }
                                fileUrlList.add(fileUrlMap);
                            }
                            map.put("fileUrlList", fileUrlList);
                        }

                        // 问卷任务类型-回显答题结果
                        if ("801".equals(wmTask1.getTaskTypeId())) {
                            this.taskType801Detail(wmTaskContent, map);
                            // 不显示截图
                            map.put("imgList", Collections.emptyList());
                            map.put("taskBaseImg", null);
                        }
                    }
                }

                infoList.add(map);
            }


        }

        return infoList;

    }

    /**
     * 任务类型801 详细信息
     *
     * @param wmTaskContent WM 任务内容
     * @return {@link Map}<{@link String}, {@link Object}> 结果
     */
    private Map<String, Object> taskType801Detail(WmTaskContent wmTaskContent, Map<String, Object> map) {
        Map<String, Object> litBriefMap = new HashMap<>(3);
        if (JSONUtil.isJsonObj(wmTaskContent.getTemp29()) && JSONUtil.isJsonObj(wmTaskContent.getTemp28())) {
            // 问卷题干
            cn.hutool.json.JSONObject structObj = JSONUtil.parseObj(wmTaskContent.getTemp29());
            cn.hutool.json.JSONObject project = structObj.getJSONObject("project");
            JSONArray items = structObj.getJSONArray("items");

            // 问卷结果
            cn.hutool.json.JSONObject resultObj = JSONUtil.parseObj(wmTaskContent.getTemp28());
            // 问卷结果答案
            cn.hutool.json.JSONObject processData = resultObj.getJSONObject("processData");

            // 问卷名称
            if (project != null) {
                map.put("litName", project.getStr("name"));
            }
            // 问卷题目和答题结果
            if (CollUtil.isNotEmpty(items) && processData != null) {
                List<Map<String, String>> answerList = new ArrayList<>(items.size());
                for (int i = 0; i < items.size(); i++) {
                    Map<String, String> answerMap = new HashMap<>(3);
                    cn.hutool.json.JSONObject itemObj = (cn.hutool.json.JSONObject) items.get(i);
                    if (itemObj != null) {
                        // 问题
                        String question = itemObj.getStr("label");
                        // 字段id值
                        String formItemId = itemObj.getStr("formItemId");
                        // 字段值id对应的结果
                        String answer = processData.getStr("field" + formItemId);
                        answerMap.put("question", String.format("问题%s.%s", (i + 1), question));
                        answerMap.put("answer", String.format("答案:%s", answer));
                        answerList.add(answerMap);
                    }
                }
                map.put("litAnswerList", answerList);
            }
        }

        return litBriefMap;
    }

    public String getFieldValueByFieldName(String fieldName, Object object) {
        try {
            Field field = object.getClass().getDeclaredField(fieldName);
            // 对private的属性的访问
            field.setAccessible(true);
            return field.get(object) + "";
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

}
