package com.qunzhixinxi.hnqz.report.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.ReportHandlerDeptRelation;
import com.qunzhixinxi.hnqz.report.service.ExcelReportExportService;
import com.qunzhixinxi.hnqz.report.service.ReportExportService;
import com.qunzhixinxi.hnqz.report.service.ReportHandlerDeptRelationService;
import com.qunzhixinxi.hnqz.report.mapper.ReportHandlerDeptRelationMapper;
import com.qunzhixinxi.hnqz.common.core.util.SpringContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @author lixuesong
 * @description 针对表【report_handler_dept_relation(导出报告处理类和企业关系表)】的数据库操作Service实现
 * @createDate 2022-11-25 09:01:19
 */
@Slf4j
@Service
public class ReportHandlerDeptRelationServiceImpl extends ServiceImpl<ReportHandlerDeptRelationMapper, ReportHandlerDeptRelation>
        implements ReportHandlerDeptRelationService {

    /**
     * 根据企业查询对应的处理类
     *
     * @param reportType 报告类型
     * @param deptId 企业id
     * @return
     */
    @Override
    public ReportExportService getHandlerClass(String reportType, Integer deptId) {
        ReportHandlerDeptRelation reportHandlerDeptRelation = this.getOne(Wrappers.<ReportHandlerDeptRelation>lambdaQuery()
                .eq(ReportHandlerDeptRelation::getReportType, reportType)
                .eq(ReportHandlerDeptRelation::getDetpId, deptId));
        if (reportHandlerDeptRelation != null) {
            log.info("reportType={}, deptId={}, handlerClass={}", reportType, deptId, reportHandlerDeptRelation.getHandlerClass());
            return SpringContextHolder.getBean(reportHandlerDeptRelation.getHandlerClass());
        }

        // 默认返回
        return SpringContextHolder.getBean(ReportExportServiceImpl.class);
    }

    /**
     * 根据企业查询对应的处理类
     *
     * @param reportType 报告类型
     * @param deptId 企业id
     * @return
     */
    @Override
    public ReportExportService getReviewHandlerClass(String reportType, Integer deptId) {
        ReportHandlerDeptRelation reportHandlerDeptRelation = this.getOne(Wrappers.<ReportHandlerDeptRelation>lambdaQuery()
                .eq(ReportHandlerDeptRelation::getReportType, reportType)
                .eq(ReportHandlerDeptRelation::getDetpId, deptId));
        if (reportHandlerDeptRelation != null) {
            log.info("reportType={}, deptId={}, handlerClass={}", reportType, deptId, reportHandlerDeptRelation.getHandlerClass());
            return SpringContextHolder.getBean(reportHandlerDeptRelation.getHandlerClass());
        }

        // 默认返回
        return SpringContextHolder.getBean(ReportReviewExportServiceImpl.class);
    }

    /**
     * 根据企业查询对应的处理类
     *
     * @param reportType
     * @param deptId
     * @return
     */
    @Override
    public ExcelReportExportService getExcelHandlerClass(String reportType, Integer deptId) {

        // 默认返回
        return SpringContextHolder.getBean(ExcelReportExportServiceImpl.class);
    }

    /**
     * 根据企业查询对应的处理类
     *
     * @param reportType 报告类型
     * @param deptId 企业id
     * @return 处理类
     */
    @Override
    public ExcelReportExportService getExcelReviewHandlerClass(String reportType, Integer deptId) {

        // 默认返回
        return SpringContextHolder.getBean(ExcelReportReviewExportServiceImpl.class);
    }
}




