package com.qunzhixinxi.hnqz.report.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.api.entity.XlsTask;
import com.qunzhixinxi.hnqz.report.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEnt;
import com.qunzhixinxi.hnqz.admin.api.entity.WmReport;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTask;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.report.service.ReportExportService;
import com.qunzhixinxi.hnqz.report.service.SysDeptService;
import com.qunzhixinxi.hnqz.report.service.SysFileService;
import com.qunzhixinxi.hnqz.report.service.SysUserService;
import com.qunzhixinxi.hnqz.report.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.report.service.WmReportService;
import com.qunzhixinxi.hnqz.report.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.report.service.WmTaskContentConfigService;
import com.qunzhixinxi.hnqz.report.service.WmTaskContentService;
import com.qunzhixinxi.hnqz.report.service.WmTaskService;
import com.qunzhixinxi.hnqz.report.service.WmTaskTypeService;
import com.qunzhixinxi.hnqz.report.util.ExportReport;
import com.qunzhixinxi.hnqz.common.core.util.R;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 报告管理 服务实现类
 * </p>
 *
 * @author hnqz
 * @since 2018-01-20
 */
@Slf4j
@Service("ReportReviewExportServiceImpl")
@AllArgsConstructor
public class ReportReviewExportServiceImpl implements ReportExportService {

    private final WmTaskService wmTaskService;
    private final WmScorePackageService wmScorePackageService;
    private final UpmsConfig upmsConfig;
    private final SysFileService sysFileService;
    private final WmDaDrugEntService wmDaDrugEntService;
    private final WmTaskTypeService wmTaskTypeService;
    private final SysUserService sysUserService;
    private final SysDeptService sysDeptService;
    private final WmTaskContentService wmTaskContentService;
    private final WmTaskContentConfigService wmTaskContentConfigService;
    private final WmTaskMapper wmTaskMapper;
    private final WmReportService wmReportService;
    private final ReportExportService reportExportService;

    /**
     * 结算报告--导出汇总报告--学术推广服务报告
     *
     * @param wmReport
     * @return
     * @throws IOException
     */
    @Override
    public R newExportNb(WmReport wmReport, Integer deptId, String username) throws IOException {


        wmReport = wmReportService.getById(wmReport.getId());
        List<WmTask> taskList = new ArrayList<>();//该报告关联的所有未提交的任务
        //更新任务表审批意见
        WmScorePackage scorePackage = wmScorePackageService.getById(wmReport.getScoreId());
        WmTask wmTask = new WmTask();
        wmTask.setDelFlag("0");
        wmTask.setEnableFlag("0");
        wmTask.setRealFlag("0");

        SysDept sysDept = sysDeptService.getById(scorePackage.getSendPackageDeptId());


        if ("0".equals(scorePackage.getTypeid())) {//药企
            wmTask.setReportDrugId(wmReport.getId());
            // 待审核的状态
            taskList = wmTaskService.list(Wrappers.query(wmTask).lambda().isNull(WmTask::getReportDrugApprovalOpinion));
        } else if ("1".equals(scorePackage.getTypeid())) {//一级代理商类型
            wmTask.setReportOneId(wmReport.getId());
            if (sysDept.getLevel() == 2) {
                // 待审核的状态
                taskList = wmTaskService.list(Wrappers.query(wmTask).lambda().isNull(WmTask::getReportDrugApprovalOpinion));
            } else {
                // 待审核的状态
                taskList = wmTaskService.list(Wrappers.query(wmTask).lambda().isNull(WmTask::getReportOneApprovalOpinion));
            }

        } else if ("2".equals(scorePackage.getTypeid())) {//二级代理商类型
            wmTask.setReportSecondId(wmReport.getId());
            // 待审核的状态
            taskList = wmTaskService.list(Wrappers.query(wmTask).lambda().isNull(WmTask::getReportSecondApprovalOpinion));
        }
        if (null == taskList || taskList.size() <= 0) {
            return R.failed("ERROR_未查询到该报告关联的待审核的任务");
        }
        String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining(","));
        wmTask.setScorePackageId(wmReport.getScoreId().toString());
        wmTask.setId(ids);

        return exportNb(wmTask, wmReport, deptId, username);//报告汇总
    }


    /**
     * 结算报告--导出汇总报告--学术推广服务报告
     *
     * @param wmTask
     * @param wmReport
     * @return
     * @throws IOException
     */
    private R exportNb(WmTask wmTask, WmReport wmReport, Integer deptId, String username) throws IOException {

        WmTask wmTaskparam = wmTask;
        String packageId = wmTask.getScorePackageId();  // 获取积分包ID
        if (null == packageId) {
            return R.failed("积分包ID不能为空");
        }

        List<String> list = new ArrayList<>();
        list.add(packageId);

        wmTask.setExtIds(list);

        Map<String, Object> excelMap = new HashMap<>();

        WmScorePackage wmScorePackage = wmScorePackageService.getById(packageId);
        if (null == wmScorePackage) {
            return R.failed("获取积分包失败");
        }

        excelMap.put("totalscorePackageScore", wmScorePackage.getScore());
        if (wmScorePackage.getPackageType1().equals("2") || wmScorePackage.getPackageType1().equals("3")) {
            excelMap.put("totalscorePackageTaskNum", wmScorePackage.getTaskNum() * wmScorePackage.getUserNum());//积分包值任务个数
        }


        List<Map<String, Object>> wmTaskList = wmTaskService.getTaskListByIds(wmTask);


        List<XlsTask> xlsTaskList = new ArrayList<>();//mxTable
        Integer totalMoney = 0;
//		wmTask.setScorePackageId(null);
//		wmTask.setScorePackageDrugId(null);

        //当前登陆人的机构
        SysDept sysDept = sysDeptService.getById(deptId);

        Map<String, Object> userMap = new HashMap<>();

        //委托公司（结算企业）
        SysDept drugEnt = sysDeptService.getById(wmReport.getDrugEntId());

        //报告公司（申请结算企业）
        SysDept applydrugEnt = sysDeptService.getById(wmReport.getApplyDrugEntId());

        for (Map<String, Object> wmT : wmTaskList) {
            totalMoney += Integer.parseInt(wmT.get("score") + "");
            XlsTask xlsTask = new XlsTask();
            xlsTask.setTaskNum(1);
            xlsTask.setCreateTime(wmT.get("create_time") + "");

            xlsTask.setDeptName(sysDept.getName());

            //level=2是药企
            if (sysDept.getLevel() == 2) {
                xlsTask.setAgentDeptName(applydrugEnt.getName());
            }

            xlsTask.setDrugEntName(drugEnt.getName());


            xlsTask.setScore(Integer.parseInt(wmT.get("score") + ""));

            xlsTask.setUsername(wmT.get("realname") + "");
            xlsTask.setTaskType(wmT.get("base_id") + "");

            xlsTaskList.add(xlsTask);

            String buildKey = wmT.get("realname") + "_" + drugEnt.getName() + "_" + sysDept.getName();
            //人员表格
            if (userMap.containsKey(buildKey)) {
                try {
                    userMap.put(buildKey, Integer.parseInt(wmT.get("score") + "") + Integer.parseInt(userMap.get(buildKey) + ""));
                    userMap.put(buildKey + "_total", Integer.parseInt(wmT.get("score_count") + "") + Integer.parseInt(userMap
                            .get(buildKey + "_total") + ""));

                } catch (Exception ex) {
                }
            } else {
                userMap.put(buildKey, Integer.parseInt(wmT.get("score") + ""));
                userMap.put(buildKey + "_total", Integer.parseInt(wmT.get("score_count") + ""));
            }
        }

//		List<String> list1 = new ArrayList<>();
//		list1.add(packageId);

        //学术推广服务任务明细表格
        List<WmTask> wmTaskDetialList = wmTaskService.getTaskListByIdsReturnWmtaskInfo(wmTask);
        List<Map<String, Object>> wmTaskDetialMap = new ArrayList<>();

        if (wmTaskDetialList.size() > 0) {


             for (WmTask wmTask1 : wmTaskDetialList) {
                 Map<String, Object> map = new HashMap<>();
                 WmTaskType wmTaskTypeTemp1 = wmTaskTypeService.getById(wmTask1.getTaskTypeId());
                 map.put("taskType", wmTaskTypeTemp1.getTaskTypeName());
                 DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                 String strDate2 = dtf2.format(wmTask1.getCreateTime());
                 map.put("taskTime", strDate2);
 //				map.put("taskUser", sysUserService.getById(wmTask1.getTaskUserId()).getRealname());
                 map.put("taskUser", sysUserService.getByIdIgnore(wmTask1.getTaskUserId()).getRealname());
                 map.put("taskTypeId", wmTask1.getTaskTypeId());
                 map.put("score", wmTask1.getScore());
                 map.put("taskNum", 1);

                 wmTaskDetialMap.add(map);
             }

            // 待审核积分值
            int toApproveSum = wmTaskDetialList.stream()
                    .mapToInt(WmTask::getScore).sum();
            excelMap.put("toApproveScore", toApproveSum);
        }

        excelMap.put("rwmxTable", wmTaskDetialMap);//学术推广服务任务明细表格

        if (totalMoney > wmScorePackage.getScore()) {
            totalMoney = wmScorePackage.getScore();
        }

        excelMap.put("ryTable", userMap);//人员表格
        excelMap.put("tMoney", totalMoney);

        if (xlsTaskList.size() > 0) {
            excelMap.put("wt_ent", xlsTaskList.get(0).getDrugEntName());//委托公司
            excelMap.put("drugEnt", xlsTaskList.get(0).getDrugEntName());//服务药企
            excelMap.put("mxTable", xlsTaskList);
        }

        List<Map<String, String>> ztTableList = new ArrayList<>();
        Map<String, String> ztMap = new HashMap<>();
        List<Map<String, Object>> wmTaskCount = wmTaskService.getTaskListByIdsCount(wmTask);
        if (wmTaskCount.size() > 0) {
            Integer total = 0;
            Integer sum = 0;
            for (Map<String, Object> map :
                    wmTaskCount) {
                if (null != map.get("taskCount")) {
                    total += Integer.parseInt(map.get("taskCount") + "");
                }

                if (null != map.get("taskSum")) {
                    sum += Integer.parseInt(map.get("taskSum") + "");
                }

            }
            Map<String, Object> totalMap = new HashMap<>();
            totalMap.put("taskTypeName", "合计");
            totalMap.put("taskCount", total);
            totalMap.put("taskSum", sum);
            wmTaskCount.add(totalMap);

//            excelMap.put("totalSubmitScore", sum);
//            excelMap.put("totalSubmitTaskNum", total);

            excelMap.put("ztTable", wmTaskCount);//学术推广服务整体概况表格
        }

        // 任务小类及对应的任务个数和积分值
        List<WmTask> taskList = wmTaskMapper.selectBatchIds(Arrays.asList(wmTask.getId().split(",")));

        // 查询taskType
        List<WmTaskType> taskTypeList = wmTaskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
                .eq(WmTaskType::getTaskTypeLevel, "1")
                .eq(WmTaskType::getDelFlag, "0")
//                .eq(WmTaskType::getEnableFlag, "0")
        );
        // taskType通过id分组（key为parentId，value为taskTypeId的set）
        Map<String, String> taskTypeIdMap = taskTypeList.stream()
                .collect(Collectors.toMap(WmTaskType::getId, WmTaskType::getTaskTypeName));

        // task通过taskTypeId分组
        Map<String, List<WmTask>> taskTypeMap = taskList.stream()
                .collect(Collectors.groupingBy(WmTask::getTaskTypeId, Collectors.toList()));

        // 封装brief table1数据（统计每个小类任务的积分、任务数）
        List<Map<String, Object>> briefTable1List = new ArrayList<>();
        taskTypeMap.forEach((taskTypeId, wmTasks) -> {
            Map<String, Object> briefTable1Params = new HashMap<>();
            briefTable1Params.put("taskTypeName", taskTypeIdMap.get(taskTypeId));
            briefTable1Params.put("taskCount", wmTasks.size());
            briefTable1Params.put("taskSum", wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum));
            briefTable1List.add(briefTable1Params);
        });
        Map<String, Object> briefTable1TotalParams = new HashMap<>();
        briefTable1TotalParams.put("taskTypeName", "合计");
        briefTable1TotalParams.put("taskCount", taskList.size());
        briefTable1TotalParams.put("taskSum", taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum));
        briefTable1List.add(briefTable1TotalParams);
        excelMap.put("briefTable", briefTable1List);

        //WmTaskRule wmTaskRule = wmTaskRuleService.getById(wmScorePackage.getTaskRuleId());
        String entDate = wmScorePackage.getBelongDate() + "-31";
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            format.parse(entDate);
        } catch (Exception ex) {
            entDate = wmScorePackage.getBelongDate() + "-30";
        }

        if (null != applydrugEnt) {
            excelMap.put("report_ent", applydrugEnt.getName());
        } else {
            excelMap.put("report_ent", "");
        }

        String ruleDate = wmScorePackage.getBelongDate();
        excelMap.put("rule_year", ruleDate.substring(0, 4));
        excelMap.put("rule_month", ruleDate.substring(5, 7));

        //报告日期
        Map<String, Timestamp> maxMaxTime = wmTaskService.getMaxCreateTime(wmTask);
        System.out.println(maxMaxTime);
        LocalDateTime MaxTime = maxMaxTime.get("MaxTime").toLocalDateTime();
        excelMap.put("year", MaxTime.getYear() + "");
        Integer month = MaxTime.getMonthValue();
        if (month < 10) {
            excelMap.put("month", "0" + MaxTime.getMonthValue() + "");
        } else {
            excelMap.put("month", MaxTime.getMonthValue() + "");
        }
        Integer day = MaxTime.getDayOfMonth();
        if (day < 10) {
            excelMap.put("day", "0" + MaxTime.getDayOfMonth() + "");
        } else {
            excelMap.put("day", MaxTime.getDayOfMonth() + "");
        }

        excelMap.put("tMoney", totalMoney);
        excelMap.put("scoreId", wmScorePackage.getScorePackageName());

        List<Map<String, Object>> infoList = exportZbInfo(wmTaskparam);
        if (null != infoList && infoList.size() > 0) {
            excelMap.put("infoList", infoList);
        } else {
            return R.failed("未查询到待审核的任务");
        }
        //区分全职兼职学术推广员模版
        excelMap.put("userType", wmScorePackage.getTypeid());

        File f = null;
        if (wmScorePackage.getPackageType1().equals("1")) {//定额
            f = ExportReport.main_bulidTemplate1(excelMap, 1, upmsConfig.getBasePath(), "doc_template1_review.docx");
        } else {//定量
            f = ExportReport.main_bulidTemplate2(excelMap, 1, upmsConfig.getBasePath());
        }

        FileInputStream input = new FileInputStream(f);

        MultipartFile multipartFile = new MockMultipartFile("学术推广服务报告(" + packageId + ")", "学术推广服务报告(" + packageId + ").doc", "text/plain", IOUtils
                .toByteArray(input));
        return sysFileService.uploadFileAllUrlRealName(multipartFile, username);
    }


    @Override
    public R newExportNba(WmScorePackage wmScorePackage, Integer deptId, String username) throws IOException {
        return null;
    }

    @Override
    public R newExportNbaHistory(WmScorePackage wmScorePackage, Integer deptId, String username) throws IOException {
        return null;
    }

    /**
     * 个人承接按积分制
     * @param wmTask
     * @param username
     * @return
     * @throws IOException
     */
    @Override
    public R newExportZb(WmTask wmTask, String username) throws IOException {
        String packageId = wmTask.getScorePackageDrugId();
        if (null == packageId) {
            return R.failed("积分包ID不能为空");
        }
        LambdaQueryWrapper<WmTask> queryWrapper = new LambdaQueryWrapper();
        queryWrapper.in(WmTask::getScorePackageId, Arrays.asList(packageId.split(",")));
        if (StrUtil.isNotBlank(wmTask.getTaskStatus())) {
            queryWrapper.eq(WmTask::getTaskStatus, wmTask.getTaskStatus());
        } else {
            queryWrapper.eq(WmTask::getTaskStatus, "3");
        }
        queryWrapper.eq(WmTask::getDelFlag, "0");
        queryWrapper.eq(WmTask::getEnableFlag, "0");
        queryWrapper.eq(WmTask::getRealFlag, "0");
        List<WmTask> taskList = wmTaskService.list(queryWrapper);
        if (taskList.isEmpty()) {
            return R.failed("未查询到待审核的任务");
        }
        String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining(","));
        wmTask.setScorePackageId(packageId);
        wmTask.setId(ids);
        return exportZb(wmTask, username);
    }

    /**
     * 个人报告导出--众包代表业绩报告
     *
     * @param wmTask
     * @return
     * @throws IOException
     */
    private R exportZb(WmTask wmTask, String username) throws IOException {

        WmTask wmTaskparam = wmTask;
        String packageId = wmTask.getScorePackageId();  // 获取积分包ID
        if (null == packageId) {
            packageId = wmTask.getScorePackageDrugId();
        }

        if (null == packageId) {
            return R.failed("积分包ID不能为空");
        }

        WmScorePackage wmScorePackage1 = wmScorePackageService.getById(packageId);

        Map<String, Object> excelMap = exportZbCommon(wmTask);
        if (null == excelMap) {
            return R.failed("未查询到待审核的任务");
        }

        List<Map<String, Object>> infoList = exportZbInfo(wmTaskparam);
        if (null != infoList && infoList.size() > 0) {
            excelMap.put("infoList", infoList);
        }
        //区分全职兼职学术推广员模版
        excelMap.put("userType", wmScorePackage1.getTypeid());
        List<String> extIds = wmTask.getExtIds();
        List<WmScorePackage> wmScorePackages = wmScorePackageService.listByIds(extIds);
        String scoreIds = wmScorePackages.stream().map(WmScorePackage::getScorePackageName).collect(Collectors.joining(","));
        excelMap.put("scoreId", StrUtil.isBlank(scoreIds) ? wmScorePackage1.getScorePackageName() : scoreIds);

        File f = null;
        if (wmScorePackage1.getPackageType1().equals("1")) {//定额
            String templateName = "";
            if(StrUtil.equalsAny(wmScorePackage1.getTypeid(), "3", "5")){//全职学术推广员
                templateName = "doc_template12.docx";
            }else {
                templateName = "doc_template11_review.docx";
            }
            f = ExportReport.main_bulidTemplate1(excelMap, 2, upmsConfig.getBasePath(), templateName);
        } else {//定量
            f = ExportReport.main_bulidTemplate2(excelMap, 2, upmsConfig.getBasePath());
        }

        FileInputStream input = new FileInputStream(f);

        String fileNamePrefix = "";
        if (StrUtil.equals(wmScorePackage1.getTypeid(), "3")) {
            fileNamePrefix = "全职学术信息沟通专员业绩报告";
        } else if (StrUtil.equals(wmScorePackage1.getTypeid(), "5")) {
            fileNamePrefix = "招商经理业绩报告";
        } else if (StrUtil.equals(wmScorePackage1.getTypeid(), "4")) {
            fileNamePrefix = "兼职学术信息沟通专员业绩报告";
        }

        MultipartFile multipartFile = new MockMultipartFile(fileNamePrefix + "(" + packageId + ")", "兼职学术推广员业绩报告(" + packageId + ").doc", "text/plain", IOUtils
                .toByteArray(input));
        return sysFileService.uploadFileAllUrlRealName(multipartFile, username);

    }

    /*合并导出--汇总
     */
    public Map<String, Object> exportZbCommon(WmTask wmTask) {
        String packageId = wmTask.getScorePackageId();  // 获取积分包ID
        if (null == packageId) {
            packageId = wmTask.getScorePackageDrugId();
        }

        Map<String, Object> excelMap = new HashMap<>();
        WmScorePackage wmScorePackage1 = wmScorePackageService.getById(packageId);
        WmDaDrugEnt wmDaDrugEnt = wmDaDrugEntService.getById(wmScorePackage1.getDrugEntId());
        if (wmDaDrugEnt == null) {
            wmDaDrugEnt = new WmDaDrugEnt();
            wmDaDrugEnt.setEntname("无");
        }

        List<String> strs = new ArrayList<>();
        if (packageId.contains(",")) {
            strs.addAll(Arrays.asList(packageId.split(",")));
        } else {
            strs.add(packageId);
        }
        wmTask.setExtIds(strs);


        List<Map<String, Object>> wmTaskList = wmTaskService.getTaskListByIdsOld(wmTask);
        if (wmScorePackage1.getPackageType2().equals("2")) {
            wmTaskList = wmTaskService.getTaskListByIdsJs(wmTask);
        }
        if (wmTaskList.size() > 0) {
            List<XlsTask> xlsTaskList = new ArrayList<>();
            Map<String, Object> userMap = new HashMap<>();
            for (Map<String, Object> wmt : wmTaskList) {
                XlsTask xlsTask = new XlsTask();
                xlsTask.setTaskNum(Integer.valueOf(wmt.get("score_count") + ""));
                xlsTask.setTaskType(wmt.get("base_id") + "");
                xlsTask.setUsername(wmt.get("realname") + "");
                xlsTask.setScore(Integer.parseInt((wmt.get("score") == null ? "0" : wmt.get("score")) + ""));
                xlsTask.setDrugEntName(wmDaDrugEnt.getEntname());
                xlsTask.setDeptName(wmt.get("dept_name") + "");
                xlsTask.setCreateTime(wmt.get("create_time") + "");
                xlsTaskList.add(xlsTask);

                String buildKey = wmt.get("realname") + "_" + wmDaDrugEnt.getEntname() + "_" + wmt.get("dept_name") + "";
                if (userMap.containsKey(buildKey)) {
                    try {
                        userMap.put(buildKey, Integer.parseInt(wmt.get("score") == null ? "0" : wmt.get("score") + "") + Integer
                                .parseInt(userMap.get(buildKey) == null ? "0" : userMap.get(buildKey) + ""));
                        userMap.put(buildKey + "_total", Integer.parseInt(wmt.get("score_count") + "") + Integer.parseInt(userMap
                                .get(buildKey + "_total") + ""));

                    } catch (Exception ex) {
                    }
                } else {
                    userMap.put(buildKey, wmt.get("score"));
                    userMap.put(buildKey + "_total", Integer.parseInt(wmt.get("score_count") + ""));
                }
            }


            List<WmTask> wmTaskDetialList = wmTaskService.getTaskListByIdsReturnWmtaskInfo(wmTask);
            List<Map<String, Object>> wmTaskDetialMap = new ArrayList<>();

            List<String> listPackageIds = new ArrayList<>();

            if (wmTaskDetialList.size() > 0) {
                for (WmTask wmTask1 :
                        wmTaskDetialList) {
                    Map<String, Object> map = new HashMap<>();
                    WmTaskType wmTaskTypeTemp1 = wmTaskTypeService.getById(wmTask1.getTaskTypeId());
                    map.put("taskType", wmTaskTypeTemp1.getTaskTypeName());
                    DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                    String strDate2 = dtf2.format(wmTask1.getCreateTime());
                    map.put("taskTime", strDate2);
                    SysUser user = sysUserService.getByIdIgnore(wmTask1.getTaskUserId());
                    if (user != null) {
                        map.put("taskUser", user.getRealname());
                    }

                    map.put("taskTypeId", wmTask1.getTaskTypeId());
                    map.put("score", wmTask1.getScore());
                    map.put("taskNum", 1);
                    wmTaskDetialMap.add(map);

                    if (listPackageIds.size() == 0) {
                        listPackageIds.add(wmTask1.getScorePackageId());
                    } else {
                        boolean isInscore = true;
                        for (String str : listPackageIds) {
                            if (str.equals(wmTask1.getScorePackageId())) {
                                isInscore = false;
                                break;
                            }
                        }

                        if (isInscore) {
                            listPackageIds.add(wmTask1.getScorePackageId());
                        }
                    }
                }

                // 待审核积分值
                int toApproveSum = wmTaskDetialList.stream()
                        .mapToInt(WmTask::getScore).sum();
                excelMap.put("toApproveScore", toApproveSum);
            }

            if (listPackageIds.size() > 0) {
                List<WmScorePackage> wmScorePackageList = wmScorePackageService.listByIds(listPackageIds);
                Integer totalScorePackage = 0;
                BigDecimal settleAmount = new BigDecimal("0");
                if (wmScorePackageList.size() > 0) {
                    for (WmScorePackage wmScorePackage : wmScorePackageList) {
                        totalScorePackage += wmScorePackage.getScore();
                        if (wmScorePackage.getSettleStatus() != null && wmScorePackage.getSettleStatus() == 1) {
                            settleAmount = settleAmount.add(wmScorePackage.getSettleAmount());
                        }
                    }
                }
                excelMap.put("settleAmount", "0".equals(settleAmount.toString()) ? " " : settleAmount);
                excelMap.put("totalscorePackageScore", totalScorePackage);
                if (wmScorePackage1.getPackageType1().equals("2") || wmScorePackage1.getPackageType1().equals("3")) {
                    if (!wmScorePackage1.getPackageType2().equals("2")) {
                        excelMap.put("totalscorePackageTaskNum", wmScorePackage1.getTaskNum() * wmScorePackage1.getUserNum());//积分包值任务个数
                    }
                }

            }

            excelMap.put("rwmxTable", wmTaskDetialMap);
            excelMap.put("mxTable", xlsTaskList);
            excelMap.put("ryTable", userMap);

        }

        Map<String, Timestamp> maxMaxTime = wmTaskService.getMaxCreateTime(wmTask);
        System.out.println(maxMaxTime);
        LocalDateTime MaxTime = maxMaxTime.get("MaxTime").toLocalDateTime();
        excelMap.put("year", MaxTime.getYear() + "");
        Integer month = MaxTime.getMonthValue();
        if (month < 10) {
            excelMap.put("month", "0" + MaxTime.getMonthValue() + "");
        } else {
            excelMap.put("month", MaxTime.getMonthValue() + "");
        }
        Integer day = MaxTime.getDayOfMonth();
        if (day < 10) {
            excelMap.put("day", "0" + MaxTime.getDayOfMonth() + "");
        } else {
            excelMap.put("day", MaxTime.getDayOfMonth() + "");
        }

        List<Map<String, Object>> wmTaskCount = wmTaskService.getTaskListByIdsCount(wmTask);
        if (wmTaskCount.size() > 0) {
            Integer total = 0;
            Integer sum = 0;
            for (Map<String, Object> map :
                    wmTaskCount) {
                if (null != map.get("taskCount")) {
                    total += Integer.parseInt(map.get("taskCount") + "");
                }

                if (null != map.get("taskSum")) {
                    sum += Integer.parseInt(map.get("taskSum") + "");
                }

            }
            Map<String, Object> totalMap = new HashMap<>();
            totalMap.put("taskTypeName", "合计");
            totalMap.put("taskCount", total);
            totalMap.put("taskSum", sum);
            wmTaskCount.add(totalMap);
            excelMap.put("ztTable", wmTaskCount);
        }

        // 任务小类及对应的任务个数和积分值
        List<WmTask> taskList = wmTaskMapper.selectBatchIds(Arrays.asList(wmTask.getId().split(",")));

        // 查询taskType
        List<WmTaskType> taskTypeList = wmTaskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
                .eq(WmTaskType::getTaskTypeLevel, "1")
                .eq(WmTaskType::getDelFlag, "0")
//                .eq(WmTaskType::getEnableFlag, "0")
        );
        // taskType通过id分组（key为parentId，value为taskTypeId的set）
        Map<String, String> taskTypeIdMap = taskTypeList.stream()
                .collect(Collectors.toMap(WmTaskType::getId, WmTaskType::getTaskTypeName));

        // task通过taskTypeId分组
        Map<String, List<WmTask>> taskTypeMap = taskList.stream()
                .collect(Collectors.groupingBy(WmTask::getTaskTypeId, Collectors.toList()));

        // 封装brief table1数据（统计每个小类任务的积分、任务数）
        List<Map<String, Object>> briefTable1List = new ArrayList<>();
        taskTypeMap.forEach((taskTypeId, wmTasks) -> {
            Map<String, Object> briefTable1Params = new HashMap<>();
            briefTable1Params.put("taskTypeName", taskTypeIdMap.get(taskTypeId));
            briefTable1Params.put("taskCount", wmTasks.size());
            briefTable1Params.put("taskSum", wmTasks.stream().map(WmTask::getScore).reduce(0, Integer::sum));
            briefTable1List.add(briefTable1Params);
        });
        Map<String, Object> briefTable1TotalParams = new HashMap<>();
        briefTable1TotalParams.put("taskTypeName", "合计");
        briefTable1TotalParams.put("taskCount", taskList.size());
        briefTable1TotalParams.put("taskSum", taskList.stream().map(WmTask::getScore).reduce(0, Integer::sum));
        briefTable1List.add(briefTable1TotalParams);
        excelMap.put("briefTable", briefTable1List);

        SysDept paramDept = new SysDept();
        paramDept.setDeptId(Integer.valueOf(wmScorePackage1.getSendPackageDeptId()));
        SysDept wmDaAgent = sysDeptService.selectDeptById(paramDept);
        excelMap.put("wt_ent", wmDaAgent.getName());

        excelMap.put("drugEnt", "");

        String ruleDate = wmScorePackage1.getBelongDate();
        excelMap.put("rule_year", ruleDate.substring(0, 4));
        excelMap.put("rule_month", ruleDate.substring(5, 7));

        List<Map<String, Object>> wmTaskZd = wmTaskService.getTaskListByIdsZd(wmTask);
        if (wmScorePackage1.getPackageType2().equals("2")) {
            wmTaskZd = wmTaskService.getTaskListByIdsZdJs(wmTask);
        }

        if (null != wmTaskZd) {
            if (wmTaskZd.size() > 0) {
                Map<String, Object> stringMap = wmTaskZd.get(0);
                if (null == stringMap) {
                    return null;
                }
                excelMap.put("tMoney", stringMap.get("sumscore") + "");
                excelMap.put("report_users", stringMap.get("realname") + "");
            }
        }
        return excelMap;
    }

    /**
     * 个人报告导出--报告详情
     *
     * @param wmTask
     * @return
     * @throws IOException
     */
    public List<Map<String, Object>> exportZbInfo(WmTask wmTask) throws IOException {
        return reportExportService.exportZbInfo(wmTask);
    }

    public String getFieldValueByFieldName(String fieldName, Object object) {
        try {
            Field field = object.getClass().getDeclaredField(fieldName);
            //对private的属性的访问
            field.setAccessible(true);
            return field.get(object) + "";
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

}
