package com.qunzhixinxi.hnqz.report.service.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mysql.cj.jdbc.exceptions.CommunicationsException;
import com.qunzhixinxi.hnqz.admin.api.entity.SysFile;
import com.qunzhixinxi.hnqz.report.config.UpmsConfig;
import com.qunzhixinxi.hnqz.report.mapper.SysFileMapper;
import com.qunzhixinxi.hnqz.report.service.SysFileService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.oss.OssProperties;
import com.qunzhixinxi.hnqz.common.oss.service.OssTemplate;
import io.minio.MinioClient;
import io.minio.PutObjectArgs;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;

/**
 * 文件管理
 *
 * @author Luckly
 * @date 2019-06-18 17:18:42
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysFileServiceImpl extends ServiceImpl<SysFileMapper, SysFile> implements SysFileService {

    private final OssProperties ossProperties;

    private final OssTemplate minioTemplate;

    private final MinioClient minioClient;

    private final UpmsConfig upmsConfig;

    /**
     * 上传文件
     *
     * @return
     */
    @Override
    public R uploadFileAllUrlRealName(String fileName, FileInputStream inputStream) {

        Map<String, String> resultMap = new HashMap<>(4);
        resultMap.put("bucketName", ossProperties.getBucketName());
        resultMap.put("fileName", fileName);
        resultMap.put("url", String.format("%s/admin/sys-file/%s/%s", upmsConfig.getClientUrl(), ossProperties.getBucketName(), fileName));

        try {

            minioClient.putObject(
                    PutObjectArgs.builder().bucket(ossProperties.getBucketName()).object(fileName)
                            .stream(
                                    inputStream, inputStream.available(), -1)
                            .build());

        } catch (Exception e) {
            log.error("上传失败", e);
            return R.failed(e.getLocalizedMessage());
        }
        return R.ok(resultMap);
    }

    /**
     * 上传文件
     *
     * @param file 文件
     * @return 信息
     */
    @Override
    public R<Map<String, String>> uploadFileAllUrlRealName(MultipartFile file, String username) {

        String idStr = DateTimeFormatter.ofPattern("yyyyMMddHHmmss").format(LocalDateTime.now());
        String fileName = FileUtil.getName(file.getName()) + "-" + idStr + StrUtil.DOT + FileUtil.extName(file.getOriginalFilename());

        Map<String, String> resultMap = new HashMap<>(4);

        resultMap.put("bucketName", ossProperties.getBucketName());
        resultMap.put("fileName", fileName);
        resultMap.put("orgFileName", file.getOriginalFilename());
        resultMap.put("url", String.format("%s/admin/sys-file/%s/%s", upmsConfig.getClientUrl(), ossProperties.getBucketName(), fileName));

        try {
            minioTemplate.putObject(ossProperties.getBucketName(), fileName, file.getInputStream());
            // 文件管理数据记录,收集管理追踪文件
            fileLog(file, fileName, username);
        } catch (Exception e) {
            log.error("上传失败", e);
            return R.failed(e.getLocalizedMessage());
        }
        return R.ok(resultMap);
    }

    /**
     * 文件管理数据记录,收集管理追踪文件
     *
     * @param file     上传文件格式
     * @param fileName 文件名
     */
    private SysFile fileLog(MultipartFile file, String fileName, String username) {
        SysFile sysFile = new SysFile();
        // 原文件名
        String original = file.getOriginalFilename();
        sysFile.setFileName(fileName);
        sysFile.setOriginal(original);
        sysFile.setFileSize(file.getSize());
        sysFile.setType(FileUtil.extName(original));
        sysFile.setBucketName(ossProperties.getBucketName());
        sysFile.setCreateUser(username);

        this.save(sysFile);
        return sysFile;
    }

}
