package com.qunzhixinxi.hnqz.report.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.PackageStatusEnum;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.TaskStatusEnum;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackageStatus;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTask;
import com.qunzhixinxi.hnqz.report.mapper.WmScorePackageMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmScorePackageStatusMapper;
import com.qunzhixinxi.hnqz.report.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.report.service.WmScorePackageStatusService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 积分包状态
 *
 * @author pigx code generator
 * @date 2020-07-17 23:32:01
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmScorePackageStatusServiceImpl extends ServiceImpl<WmScorePackageStatusMapper, WmScorePackageStatus> implements WmScorePackageStatusService {

	private final WmScorePackageMapper wmScorePackageMapper;

	private final WmTaskMapper wmTaskMapper;

	/**
	 * 查询服务周期（服务周期字段的取值 改为：积分包领取审核通过时间~管理员审核积分包时最后审核通过的时间）
	 *
	 * @param wmScorePackage
	 * @return
	 */
	@Override
	public Map<String, String> getServiceTimeRange(WmScorePackage wmScorePackage) {
		Map<String, String> resultMap = MapUtil.newHashMap(2);
		List<String> packageIdList = new ArrayList<>();

		if (CollUtil.toList("0", "1", "2").contains(wmScorePackage.getTypeid())) {
			// 公对公大包
			// 取积分包发包时间
			resultMap.put("service_start_date",
					DateUtil.format(wmScorePackage.getCreateTime(), DatePattern.NORM_DATE_PATTERN));

			List<WmScorePackage> subpackageList = wmScorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
					.eq(WmScorePackage::getRelationScoreId, wmScorePackage.getId())
					.eq(WmScorePackage::getEnableFlag, "0"));
			if (CollUtil.isNotEmpty(subpackageList)) {
				List<WmScorePackage> subSubPackageList = wmScorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
						.in(WmScorePackage::getRelationScoreId, subpackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet()))
						.eq(WmScorePackage::getEnableFlag, "0"));
				if (CollUtil.isNotEmpty(subSubPackageList)) {
					packageIdList.addAll(subSubPackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet()));
				} else {
					packageIdList.addAll(subpackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet()));
				}
			}
			packageIdList.add(wmScorePackage.getId());
		} else {
			// 个人包
			packageIdList.addAll(CollUtil.toList(wmScorePackage.getId()));
			// 服务周期开始时间：取第一条 积分包领取审核通过时间
			List<WmScorePackageStatus> packageStatusList = this.list(Wrappers.<WmScorePackageStatus>lambdaQuery()
					.in(WmScorePackageStatus::getPackageId, packageIdList)
					.eq(WmScorePackageStatus::getStatus, PackageStatusEnum.APPROVED.val()));

			if (CollUtil.isNotEmpty(packageStatusList)) {
				WmScorePackageStatus scorePackageStatus = packageStatusList.stream().min((o1, o2) -> {
					LocalDateTime o1Time = o1.getCreateTime();
					LocalDateTime o2Time = o2.getCreateTime();
					return o1Time.compareTo(o2Time);
				}).get();
				// 兼职取更新时间，全职取创建时间（全职的更新时间为空）
				resultMap.put("service_start_date",
						DateUtil.format(scorePackageStatus.getCreateTime(), DatePattern.NORM_DATE_PATTERN));
			}
		}

		// 服务周期结束时间：取最后一条任务的审核通过时间（个人的）
		List<WmTask> taskList = wmTaskMapper.selectList(Wrappers.<WmTask>lambdaQuery()
				.eq(WmTask::getEnableFlag, "0")
				.eq(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val())
				.in(WmTask::getScorePackageId, packageIdList)
				.orderByDesc(WmTask::getUpdateTime));
		if (CollUtil.isNotEmpty(taskList)) {
			WmTask task = taskList.get(0);
			resultMap.put("service_end_date", DateUtil.format(task.getUpdateTime(), DatePattern.NORM_DATE_PATTERN));
		}

		return resultMap;
	}

	/**
	 * 查询服务周期（服务周期字段的取值 改为：积分包领取审核通过时间~管理员审核积分包时最后审核通过的时间）
	 *
	 * @param scorePackageList 积分包列表
	 * @return 结果
	 */
	@Override
	public Map<String, String> getServiceTimeRange(List<WmScorePackage> scorePackageList) {
		Map<String, String> resultMap = MapUtil.newHashMap(2);
		Set<String> packageIdList = new HashSet<>();

		Set<String> packageIds = scorePackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
		packageIdList.addAll(packageIds);

		String typeid = scorePackageList.get(0).getTypeid();
		if (Arrays.asList("0", "1", "2").contains(typeid)) {
			// 公对公大包
			// 取积分包发包时间
			resultMap.put("service_start_date",
					DateUtil.format(
							scorePackageList.stream().map(WmScorePackage::getCreateTime).min(LocalDateTime::compareTo).get(),
							DatePattern.NORM_DATE_PATTERN
					));

			List<WmScorePackage> subpackageList = wmScorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
					.in(WmScorePackage::getRelationScoreId, packageIds)
					.eq(WmScorePackage::getEnableFlag, "0"));
			if (CollUtil.isNotEmpty(subpackageList)) {
				List<WmScorePackage> subSubPackageList = wmScorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
						.in(WmScorePackage::getRelationScoreId, subpackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet()))
						.eq(WmScorePackage::getEnableFlag, "0"));
				if (CollUtil.isNotEmpty(subSubPackageList)) {
					packageIdList.addAll(subSubPackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet()));
				} else {
					packageIdList.addAll(subpackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet()));
				}
			}
		} else {
			// 个人包
			// 服务周期开始时间：取第一条 积分包领取审核通过时间
			List<WmScorePackageStatus> packageStatusList = this.list(Wrappers.<WmScorePackageStatus>lambdaQuery()
					.in(WmScorePackageStatus::getPackageId, packageIdList)
					.eq(WmScorePackageStatus::getStatus, PackageStatusEnum.APPROVED.val()));

			if (CollUtil.isNotEmpty(packageStatusList)) {
				WmScorePackageStatus scorePackageStatus = packageStatusList.stream().min((o1, o2) -> {
					LocalDateTime o1Time = o1.getCreateTime();
					LocalDateTime o2Time = o2.getCreateTime();
					return o1Time.compareTo(o2Time);
				}).get();
				// 兼职取更新时间，全职取创建时间（全职的更新时间为空）
				resultMap.put("service_start_date",
						DateUtil.format(scorePackageStatus.getCreateTime(), DatePattern.NORM_DATE_PATTERN));
			}
		}

		// 服务周期结束时间：取最后一条任务的审核通过时间（个人的）
		List<WmTask> taskList = wmTaskMapper.selectList(Wrappers.<WmTask>lambdaQuery()
				.eq(WmTask::getEnableFlag, "0")
				.eq(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val())
				.in(WmTask::getScorePackageId, packageIdList)
				.orderByDesc(WmTask::getUpdateTime));
		if (CollUtil.isNotEmpty(taskList)) {
			WmTask task = taskList.get(0);
			resultMap.put("service_end_date", DateUtil.format(task.getUpdateTime(), DatePattern.NORM_DATE_PATTERN));
		}

		return resultMap;
	}
}
