package com.qunzhixinxi.hnqz.report.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.UserSignPurposeEnum;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.UserSignResultEnum;
import com.qunzhixinxi.hnqz.admin.api.entity.WmUserSign;
import com.qunzhixinxi.hnqz.admin.api.entity.WmUserSignDetail;
import com.qunzhixinxi.hnqz.report.mapper.WmUserSignMapper;
import com.qunzhixinxi.hnqz.report.service.WmUserSignDetailService;
import com.qunzhixinxi.hnqz.report.service.WmUserSignService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;
import java.util.stream.Stream;

/**
 * 用户签到表
 *
 * @author pigx code generator
 * @date 2020-07-14 20:10:02
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmUserSignServiceImpl extends ServiceImpl<WmUserSignMapper, WmUserSign> implements WmUserSignService {

    private final WmUserSignDetailService wmUserSignDetailService;

    /**
     * 根据signId查询打卡详情list
     *
     * @param signIdList
     * @return
     */
    @Override
    public List<WmUserSign> listUserSign(List<Integer> signIdList) {
        List<WmUserSign> signList = this.listByIds(signIdList);
        if (CollUtil.isNotEmpty(signList)) {
            signList.forEach(wmUserSign -> {
                WmUserSignDetail userSignDetail = wmUserSignDetailService.getOne(Wrappers.<WmUserSignDetail>lambdaQuery()
                        .eq(WmUserSignDetail::getSignId, wmUserSign.getId()));
                if (userSignDetail != null) {
                    if (ArrayUtil.isNotEmpty(userSignDetail.getPurpose())) {
                        String[] purposeNameArr = Stream.of(userSignDetail.getPurpose()).map(s -> UserSignPurposeEnum.resolve(s).getName()).toArray(String[]::new);
                        userSignDetail.setPurposeName(purposeNameArr);
                    }
                    if (StrUtil.isNotBlank(userSignDetail.getResult())) {
                        userSignDetail.setResultName(UserSignResultEnum.resolve(userSignDetail.getResult()).getName());
                    }
                    wmUserSign.setUserSignDetail(userSignDetail);
                }
            });
            signList.sort(Comparator.comparing(WmUserSign::getSignDate));
        }
        return signList;
    }
}
